#include "FileManagement.h"

//#include "RendererGLUT.h"
//#include "RendererOSMesa.h"
//#include "DatabaseVisWindow.h"
#include "ModelDatabase.h"
#include "VKString.h"

#include "ShapeDistance.h"
#include "Model3D.h"
#include "ShapePointCloud.h"
#include "ShapeTriangleSoup.h"
#include "TimeProfiler.h"
#include "ShapeGrid.h"

int main(int argc, char** argv )
{
	if (argc<2)
	{
		std::cout<<"Usage: ./"<<argv[0]<<" meshfile [args]\n";
		std::cout<<"Args"<<std::endl;
		std::cout<<"   -points pntsName - point set name - required by most"<<std::endl;
		std::cout<<"   -densePnts pntsName - point set name - required by most"<<std::endl;
		std::cout<<"   -feat featType - local feature type: "<<std::endl;
		std::cout<<"        com - center of mass"<<std::endl;
		std::cout<<"        symmplane - symmetry plane"<<std::endl;
		std::cout<<"   -outfile outFile : output file\n";
		std::cout<<"   -writeTiming outFile : output file\n";
		std::cout<<"   -vis - visualize (TODO)"<<std::endl;
		exit(0);
	}
	
	VKString meshFile(argv[1]);
	VKString feat = "none";
	VKString pntsName = "none";
	VKString densePointsName = "none";
	VKString outFile = "default";
	VKString writeTiming = "none";
	VKString metricFile = "none";
	
	bool vis = false;
		
	// parse params
	for (int i=2; i<argc; i++)
	{
		if (strcmp(argv[i], "-feat")==0)
			feat = argv[++i];
		else if (strcmp(argv[i], "-points")==0)
			pntsName = argv[++i];
		else if (strcmp(argv[i], "-densePnts")==0)
			densePointsName = argv[++i];
		else if (strcmp(argv[i], "-outfile")==0)
			outFile = argv[++i];
		else if (strcmp(argv[i], "-metricFile")==0)
			metricFile = argv[++i];
		else if (strcmp(argv[i], "-randseed")==0)
			srand(atoi(argv[++i]));
		else if (strcmp(argv[i], "-writeTiming")==0)
			writeTiming = argv[++i];
		else if (strcmp(argv[i], "-vis")==0)
			vis = true;
		else
		{
			std::cout<<"[ERROR] Unknown arg: "<<argv[i]<<std::endl;
			assert(false);
		}
	}
	
	TimeProfiler timer;
	timer.startedProcess(feat);
	Model3D * model = new Model3D("model");
	ShapeMesh * mesh = new ShapeTriangleSoup(meshFile, Model3D::m_sMainGeomName);
	model->AddGeomShape(Model3D::m_sMainGeomName, mesh);
	ShapePointCloud * pnts = NULL;
	ShapePointCloud * densePoints = NULL;
	if (pntsName!="none")
	{
		GeomShape * pntsShape = model->CreateGeometry(GeomShape::SHAPE_TYPE_POINT_CLOUD, "pnts",  Model3D::m_sMainGeomName);
		assert(pntsShape!=NULL);
		pnts = pntsShape->SafeCastToPointCloud();
		assert(pnts!=NULL);
		pnts->ReadGeometryFile(pntsName);
	}
	
	if (densePointsName!="none")
	{
		GeomShape * pntsShape = model->CreateGeometry(GeomShape::SHAPE_TYPE_POINT_CLOUD, "dense",  Model3D::m_sMainGeomName);
		assert(pntsShape!=NULL);
		densePoints = pntsShape->SafeCastToPointCloud();
		assert(densePoints!=NULL);
		densePoints->ReadGeometryFile(densePointsName);
	}
	
	ShapeDistance * dist = NULL;
	if (metricFile!="none")
	{
		assert(densePoints!=NULL);
		dist = ShapeDistance::ReadMetric(metricFile, densePoints, "metric", 1.);
	}
	
	double * features = NULL;
	int nfeatures = 0;
	if (feat=="com")
	{
		nfeatures = 3;
		features = new double[nfeatures];
		R3Point com(0,0,0);
		if (densePoints!=NULL)
			com = densePoints->Centroid();
		else
			com = mesh->Centroid();
		features[0] = com[0];
		features[1] = com[1];
		features[2] = com[2];
	}
	else if (feat=="symmplane")
		assert(false);	// todo...
	else
		assert(false);

	timer.finishedProcess(feat);
	if (writeTiming!="none")
	{
		FileManagement::CreateDirsOnPathToFile(writeTiming.c_str());
		std::ofstream textStream(writeTiming.c_str());
		assert(textStream.is_open());
		textStream<<timer.profile().c_str();
		textStream.close();
	}
	
	if (outFile!="none" && features!=NULL && nfeatures>0)
	{
		FileManagement::CreateDirsOnPathToFile(outFile.c_str());
		std::ofstream textStream(outFile.c_str());
		if (!textStream.is_open())
			std::cout<<"[ERROR] Cannot write: "<<outFile.c_str()<<std::endl;
		assert(textStream.is_open());
		for (int f=0; f<nfeatures; f++)
			textStream<<features[f]<<"\n";
		textStream.close();
	}
	
//#	ifdef VISUALIZATION
//	if (vis)
//	{
//		if (pnts!=NULL && vals!=NULL)
//			pnts->AddShapeVal(vals->GetShapeValueName(), vals);
//		if (alignedMesh!=NULL)
//			model->AddGeomShape(alignedMesh->GeometryName(), alignedMesh);
//		if (pnts!=NULL)
//			model->AddGeomShape(pnts->GeometryName(), pnts);
//		if (densePoints!=NULL)
//			model->AddGeomShape(densePoints->GeometryName(), densePoints);
//
//		int listFlags = DatabaseVisWindow::LISTGP_Models | DatabaseVisWindow::LISTGP_Geoms;
//		listFlags |= DatabaseVisWindow::LISTGP_MeshFlags | DatabaseVisWindow::LISTGP_GeomVals;
//		int w=-1, h=-1;
//		ModelDatabase * db = new ModelDatabase();
//		db->AddModel(model);
//		DatabaseVisWindow * dbVis = new DatabaseVisWindow(db, NULL, listFlags, 1, 1, 0, 0, 1., w, h);
//		VisModel3D * mvis = dbVis->GetDrawableModels()[0];
//		
//#		ifdef MESA_RENDERING
//		VisWindowRenderer * wndMesa = new RendererOSMesa(dbVis);
//		wndMesa->UpdateScreen();
//#		else
//		VisWindowRenderer * wndGLUT = new RendererGLUT(argc, argv, dbVis);
//#  	 endif
//	}
//#	endif
//	
	return 0;
}



