#include "FileManagement.h"

#include "RendererGLUT.h"
#include "RendererOSMesa.h"
#include "DatabaseVisWindow.h"
#include "ModelDatabase.h"
#include "VKString.h"

#include "ShapeLocalPCA.h"
#include "ShapeTriangleSoup.h"
#include "ShapeDistance.h"
#include "DistEuclidean.h"
#include "TimeProfiler.h"
#include "ShapeGrid.h"
#include "LocFeatAlg.h"



//double * ReadPerVertexARFF(const VKString & arffFile, const ShapeMesh * r3mesh, int * nvals);
//double * ReadSidFeatures(const VKString & sidFeatureFile, ShapePointCloud * pnts, int * nvals);
void FillValuesWithPeterXYZ(ShapeMesh * mesh, ShapeMesh * alignedMesh, ShapePointCloud * densePoints,
							ShapePointCloud * pnts, double * values);

int main(int argc, char** argv )
{
	if (argc<2)
	{ 
		std::cout<<"Usage: ./"<<argv[0]<<" meshfile [args]\n";
		std::cout<<"Args"<<std::endl;
		std::cout<<"   -points pntsName - point set name - required by most"<<std::endl;
		std::cout<<"   -feat featType - local feature type: "<<std::endl;
		std::cout<<"        FullLocalPCA - local principal components"<<std::endl;
		std::cout<<"        SimplePCA - local principal components (as in Sig'13 paper)"<<std::endl;
		std::cout<<"        Height - points height"<<std::endl;
//		std::cout<<"        Primitives - primtiives from Tom"<<std::endl;
		std::cout<<"        ExternalArff - Read ARFF file (per point)"<<std::endl;
		std::cout<<"        SurfaceRatio - # points vs total # points"<<std::endl;
		std::cout<<"   -radius ratio : average radius feature radius \n";
//		std::cout<<"   -stdscale scale : scale of the mesh (after normalization, set -1 if mesh is scaled) \n";
//		std::cout<<"   -primitiveCSG file.csg: csg file \n";
//		std::cout<<"   -primitiveARFF file.arff: arff file \n";
		std::cout<<"   -arffFeatFile file2convert NFeatures name1 name2 ... : sid's format file\n";
		std::cout<<"   -r3mesh mesh.off: mesh file associated that was actually analyzed \n";
		std::cout<<"   -metricFile mesh.metric: metric file (e.g. geodesic, nn) \n";
		std::cout<<"   -outfile outFile : output file\n";
		std::cout<<"   -writeTiming outFile : output file\n";
		std::cout<<"   -addOutFile outfile offset nfeats : multiple out files (only for: ConvertSid)\n";
		std::cout<<"   -amplify param - amplify values (increase range)"<<std::endl;
		std::cout<<"   -vis - visualize"<<std::endl;
		exit(0);
	}
	
	double amplify = -1;
	VKString meshFile(argv[1]);
	VKString feat = "none";
	VKString pntsName = "none";
	VKString densePointsName = "none";
	VKString outFile = "none";
//	VKString primitiveCSG = "none";
//	VKString primitiveARFF = "none";
	VKString metricFile = "none";
	VKString writeTiming = "none";
	
	ShapeMesh * r3mesh = NULL;
	VKString arffFeatFile = "none";
//	VKStringList nsidFeatureNames;
	double radius = -1;
//	double stdscale = 1.;
	VKStringList outfiles;
	std::vector<int> outfile_offset;
	std::vector<int> outfile_ndims;
	ShapeMesh * alignedMesh = NULL;
	
	bool vis = false;
		
	// parse params
	for (int i=2; i<argc; i++)
	{
		if (strcmp(argv[i], "-feat")==0)
			feat = argv[++i];
		else if (strcmp(argv[i], "-points")==0)
			pntsName = argv[++i];
		else if (strcmp(argv[i], "-densePoints")==0)
			densePointsName = argv[++i];
		else if (strcmp(argv[i], "-radius")==0)
			radius = atof(argv[++i]);
//		else if (strcmp(argv[i], "-stdscale")==0)
//			stdscale = atof(argv[++i]);
		else if (strcmp(argv[i], "-outfile")==0)
			outFile = argv[++i];
		else if (strcmp(argv[i], "-randseed")==0)
			srand(atoi(argv[++i]));
		else if (strcmp(argv[i], "-writeTiming")==0)
			writeTiming = argv[++i];
		else if (strcmp(argv[i], "-metricFile")==0)
			metricFile = argv[++i];
		else if (strcmp(argv[i], "-addOutFile")==0)
		{
			outfiles.push_back(argv[++i]);
			outfile_offset.push_back(atoi(argv[++i]));
			outfile_ndims.push_back(atoi(argv[++i]));
		}
//		else if (strcmp(argv[i], "-primitiveCSG")==0)
//			primitiveCSG = argv[++i];
//		else if (strcmp(argv[i], "-primitiveARFF")==0)
//			primitiveARFF = argv[++i];
		else if (strcmp(argv[i], "-arffFeatFile")==0)
			arffFeatFile = argv[++i];
		else if (strcmp(argv[i], "-metricFile")==0)
			metricFile = argv[++i];
//		else if (strcmp(argv[i], "-r3mesh")==0)
//			r3mesh = new ShapeTriangleSoup(argv[++i], "r3mesh");	// does not need to use the r3 datastructure...
		else if (strcmp(argv[i], "-alignedMesh")==0)
			alignedMesh = new ShapeTriangleSoup(argv[++i], "alignedmesh");
		else if (strcmp(argv[i], "-amplify")==0)
			amplify = atof(argv[++i]);
		else if (strcmp(argv[i], "-vis")==0)
			vis = true;
		else
		{
			std::cout<<"[ERROR] Unknown arg: "<<argv[i]<<std::endl;
			assert(false);
		}
	}
	
	TimeProfiler timer;
	timer.startedProcess(feat);
	//Model3D * model = Model3D::CreateFromFile(meshFile, 0, 0);
	Model3D * model = new Model3D("model");
	ShapeMesh * mesh = new ShapeTriangleSoup(meshFile, Model3D::m_sMainGeomName);
	model->AddGeomShape(Model3D::m_sMainGeomName, mesh);
	ShapePointCloud * pnts = NULL;
	ShapePointCloud * densePoints = NULL;
	if (pntsName!="none")
	{
		GeomShape * pntsShape = model->CreateGeometry(GeomShape::SHAPE_TYPE_POINT_CLOUD, "pnts",  Model3D::m_sMainGeomName);
		assert(pntsShape!=NULL);
		pnts = pntsShape->SafeCastToPointCloud();
		assert(pnts!=NULL);
		pnts->ReadGeometryFile(pntsName);
		model->AddGeomShape("pnts", pnts);
	}
	
	if (densePointsName!="none")
	{
		GeomShape * pntsShape = model->CreateGeometry(GeomShape::SHAPE_TYPE_POINT_CLOUD, "dense",  Model3D::m_sMainGeomName);
		assert(pntsShape!=NULL);
		densePoints = pntsShape->SafeCastToPointCloud();
		assert(densePoints!=NULL);
		densePoints->ReadGeometryFile(densePointsName);
		model->AddGeomShape("dense", densePoints);
	}
	
	ShapeDistance * dist = NULL;
	if (metricFile!="none")
	{
		assert(densePoints!=NULL);
		dist = ShapeDistance::ReadMetric(metricFile, densePoints, "metric", 1.);
		densePoints->AddDistanceMetric("metric", dist);
	}
//	else
//		dist = new DistEuclidean(pnts, "metric", 1.);
	
	if (alignedMesh!=NULL)
		model->AddGeomShape(alignedMesh->GeometryName(), alignedMesh);

	
	std::vector<FeatParams*> params;
	FeatParams * newfeat = new FeatParams(FeatParams::Str2Type(feat), model, radius, arffFeatFile);
	newfeat->m_fAmplify = amplify;
	params.push_back(newfeat);
	ShapeValue * vals = FeatAlgs::ComputeFeatures(params);

	timer.finishedProcess(feat);
	if (writeTiming!="none")
	{
		FileManagement::CreateDirsOnPathToFile(writeTiming.c_str());
		std::ofstream textStream(writeTiming.c_str());
		assert(textStream.is_open());
		textStream<<timer.profile().c_str();
		textStream.close();
	}
	
	if (outFile!="none" && vals!=NULL)
		vals->WriteShapeValues(outFile);
	
#	ifdef VISUALIZATION
	if (vis)
	{
		if (pnts!=NULL && vals!=NULL)
			pnts->AddShapeVal(vals->GetShapeValueName(), vals);

		int listFlags = DatabaseVisWindow::LISTGP_Models | DatabaseVisWindow::LISTGP_Geoms;
		listFlags |= DatabaseVisWindow::LISTGP_MeshFlags | DatabaseVisWindow::LISTGP_GeomVals;
		int w=-1, h=-1;
		ModelDatabase * db = new ModelDatabase();
		db->AddModel(model);
		DatabaseVisWindow * dbVis = new DatabaseVisWindow(db, NULL, listFlags, 1, 1, 0, 0, 1., w, h);
		VisModel3D * mvis = dbVis->GetDrawableModels()[0];
		
#		ifdef MESA_RENDERING
		VisWindowRenderer * wndMesa = new RendererOSMesa(dbVis);
		wndMesa->UpdateScreen();
#		else
		VisWindowRenderer * wndGLUT = new RendererGLUT(argc, argv, dbVis);
#  	 endif
	}
#	endif
	
	return 0;
}


//double * ReadPerVertexARFF(const VKString & arffFile, const ShapeMesh * r3mesh, int * nvals)
//{
//	int lineID = 0;
//	assert(r3mesh!=NULL);
//	int nvert = r3mesh->NVertices();
//	std::ifstream textStream(arffFile.c_str());
//	VKString tempStr;
//	*nvals = 0;
//	double * vals = NULL;
//	int vtx = 0;
//	// read the arff file
//	while (!textStream.eof())
//	{
//		lineID++;
//		tempStr.readLine(textStream);
//		if (tempStr.length()==0 || tempStr.startsWith("@relation"))
//			continue;
//		else if (tempStr.startsWith("@attribute"))
//			(*nvals)++;
//		else if (tempStr.startsWith("@data"))
//		{
//			assert(vals==NULL);
//			vals = new double[*nvals * r3mesh->NVertices()];
//		}
//		else
//		{
//			assert(vals!=NULL);
//			VKStringList dataLine = tempStr.split(" ");
//			assert(dataLine.count()==*nvals);
//			bool ok;
//			if (vtx>=r3mesh->NVertices())
//			{
//				std::cout<<"[ERROR] Too many vertices: "<<r3mesh->NVertices()<<std::endl;
//				std::cout<<"LineID: "<<lineID<<": |"<<tempStr.c_str()<<std::endl;
//			}
//			assert(vtx < r3mesh->NVertices());
//			for (int d=0; d<*nvals; d++)
//			{
//				vals[vtx * (*nvals) + d] = dataLine[d].toDouble(&ok);
//				assert(ok);
//			}
//			vtx++;
//		}
//	}
//	return vals;
//}

//double * ReadSidFeatures(const VKString & sidFeatureFile, ShapePointCloud * pnts, int * nvals)
//{
//	std::ifstream textStream(sidFeatureFile.c_str());
//	assert(textStream.is_open());
//	*nvals = -1;
//	VKString tempStr;
//	bool ok;
//	std::vector<double> readvals;
//	while (!textStream.eof())
//	{
//		tempStr.readLine(textStream);
//		VKStringList strList = tempStr.split(" ");
//		if (strList.count()<=1)
//			continue;
//		assert(strList.count()>3);
//		if (*nvals==-1)
//			*nvals = (strList.count()-3);
//		else
//			assert(*nvals == (strList.count()-3));
//		double x = strList[0].toDouble(&ok); assert(ok);
//		double y = strList[1].toDouble(&ok); assert(ok);
//		double z = strList[2].toDouble(&ok); assert(ok);
//		
//		pnts->AddPoint(ShapePoint(R3Point(x, y, z)));
//		for (int d=0; d<(*nvals); d++)
//		{
//			readvals.push_back(strList[3 + d].toDouble(&ok));
//			assert(ok);
//		}
//	}
//	textStream.close();
//	
//	assert((int)readvals.size()== (pnts->NumPoints() * (*nvals)));
//	double * retval = new double[(int)readvals.size()];
//	for (int i=0; i<(int)readvals.size(); i++)
//		retval[i] = readvals[i];
//	return retval;
//}


void FillValuesWithPeterXYZ(ShapeMesh * original, ShapeMesh * deformed, ShapePointCloud * densePoints,
							ShapePointCloud * pnts, double * values)
{
	bool identicalFaces = false;
	std::vector<std::map<VKString, int> > deformed_vtx2faceMap;
	std::vector<int> subset;
	std::vector<int> densePntsMap_Orig2Deform;
	ShapePointCloud * deformedDense = new ShapePointCloud(deformed, "deformed_random");
	
	if (!identicalFaces)	// map via vertices
	{
		assert(original->NVertices()==deformed->NVertices());
		for (int v=0; v<deformed->NVertices(); v++)
			deformed_vtx2faceMap.push_back(std::map<VKString, int>());
		
		for (int f=0; f<deformed->NFaces(); f++)
		{
			int v0 = deformed->VertexOnFace(f, 0);
			int v1 = deformed->VertexOnFace(f, 1);
			int v2 = deformed->VertexOnFace(f, 2);
			VKString vstr = VKString::number(v1)+"_"+VKString::number(v2);
			deformed_vtx2faceMap[v0][vstr] = f;
		}
	}
		
	for (int i=0; i<densePoints->NumPoints(); i++)
	{
		int fid; double b1, b2; const ShapeMesh*tempmesh;
		densePoints->GetPoint(i).GetBarycentricCoords(&fid, &b1, &b2, &tempmesh);
		if (identicalFaces)
		{
			assert(fid < deformed->NFaces());
			deformedDense->AddPoint(ShapePoint(deformed, fid, b1, b2, deformedDense, deformedDense->NumPoints()));
		}
		else
		{
			int v0 = original->VertexOnFace(fid, 0);
			int v1 = original->VertexOnFace(fid, 1);
			int v2 = original->VertexOnFace(fid, 2);	// assume vertex ids are the same
			VKString vstr = VKString::number(v1)+"_"+VKString::number(v2);
			assert(v0 < (int)deformed_vtx2faceMap.size());
			std::map<VKString, int>::iterator iter = deformed_vtx2faceMap[v0].find(vstr);
			if (iter != deformed_vtx2faceMap[v0].end())
			{
				densePntsMap_Orig2Deform.push_back(deformedDense->NumPoints());
				assert(iter->second < deformed->NFaces());
				deformedDense->AddPoint(ShapePoint(deformed, iter->second, b1, b2,
												   deformedDense, deformedDense->NumPoints()));
				subset.push_back(i);
			}
			else
				densePntsMap_Orig2Deform.push_back(-1);
		}
	}
	
	const R3Kdtree<const PointInSet *> * originalKdTree = NULL;
	if (identicalFaces)
		originalKdTree = densePoints->GetKdTree();
	else
		originalKdTree = densePoints->CreateSubsetKdTree(subset);
	const R3Kdtree<const PointInSet *> * deformedKdTree = deformedDense->GetKdTree();
	
	for (int i=0; i<pnts->NumPoints(); i++)
	{
		int pntid = originalKdTree->FindClosest(pnts->PointPosition(i))->id;
		R3Point point_deformed(0,0,0);
		if (identicalFaces)
			point_deformed = deformedDense->PointPosition(pntid);
		else
		{
			assert(pntid>=0 && pntid<(int) densePntsMap_Orig2Deform.size() && densePntsMap_Orig2Deform[pntid]>=0);
			point_deformed = deformedDense->PointPosition(densePntsMap_Orig2Deform[pntid]);
		}
		
		values[i * 3 + 0] = point_deformed.X();
		values[i * 3 + 1] = point_deformed.Y();
		values[i * 3 + 2] = point_deformed.Z();
	}
}





