#include "FileManagement.h"

#include "RendererGLUT.h"
#include "RendererOSMesa.h"
#include "DatabaseVisWindow.h"
#include "ModelDatabase.h"
#include "VKString.h"

#include "ShapeLocalPCA.h"
#include "ShapeTriangleSoup.h"
#include "ShapeDistance.h"
#include "DistEuclidean.h"
#include "TimeProfiler.h"
#include "ShapeGrid.h"


class FeatParams
{
	public:
		enum FeatureType
		{
			TYPE_Invalid,
			TYPE_FullLocalPCA,
			TYPE_SimplePCA,
			TYPE_GeodesicPCA,
			TYPE_OrientedGeodesicPCA,
			TYPE_GeodesicZVariance,
			TYPE_Height,
			TYPE_CoAlignedXYZ,
			TYPE_AbsCurv,
			TYPE_AbsCurvGeodesicAvg,
			TYPE_SDF,
			TYPE_SDFGeodesicAvg,
			TYPE_DistHist,
			TYPE_ExternalArff
		};
		FeatParams(FeatureType type, Model3D * model, double radius=-1, const VKString & externalFile="none");
		static FeatureType Str2Type(const VKString & str);

		double m_fRadius;
		double m_fAmplify;
		FeatureType m_FeatureType;
		VKString m_sExternalFile;
		Model3D * m_pModel;
		int m_iHistBins;
};

class FeatAlgs
{
	public:
		static ShapeValue * ComputeFeatures(const std::vector<FeatParams*> & params);
		static double * ComputeFeature(FeatParams* params, int * nvals);
		static double * ComputePCAFeatures(FeatParams* params, int * nvals);
		static double * ComputeZVarianceFeatures(FeatParams* params, int * nvals);
		static double * ComputeHeightFeatures(FeatParams* params, int * nvals);
		static double * ComputeCoAlignedFeatures(FeatParams* params, int * nvals);
		static double * ComputeAverageFeature(const double * inFeat, int inNVals, FeatParams* params, int * nvals);
		static double * ComputeSDFFeatures(FeatParams* params, int * nvals);
		static double * ComputeCurvFeatures(FeatParams* params, int * nvals);
		static double * ComputeDistHistograms(FeatParams* params, int * nvals);
		static double * ReadExternalFeatures(FeatParams* params, int * nvals);
		static R3Vector AlignWithTheNormal(const ShapePoint & p, const R3Vector & v, ShapeGrid * insideout);
	
		static double ComputeSDFAtPoint(const ShapePoint & pnt, const R3Vector & normal, bool only_hit_interior_surfaces,
										R3Mesh * mesh, R3MeshSearchTree * searchTree);
		static void PrecomputeNormals(ShapePointCloud * pnts, std::vector<R3Vector> & pntsNormals);
};




