#include "AnalysisWriter.h"
#include "BindingManager.h"
#include "Binding.h"
#include "Interactor.h"
#include "PosePrior.h"


AnalysisWriter::AnalysisWriter()
{
	m_pBinding = NULL;
	m_Interactors[0] = NULL;
	m_Interactors[1] = NULL;
}

void AnalysisWriter::Write() const
{
	//std::cout< <"Writing:"<<std::endl;
	AnalysisWriter * ncThis = (AnalysisWriter*)this;
	
	// write actor
	{
		std::map<VKString, VKString>::iterator iter = ncThis->m_WriteMap.find("actor_analyzed");
		Interactor::InteractorID id = Interactor::ACT_Actor;
		if (iter != m_WriteMap.end())
		{
			//std::cout<<" -- "<<iter->first.c_str()<<" -> "<<iter->second.c_str()<<std::endl;
			assert(m_Interactors[id]!=NULL);
			m_Interactors[id]->WriteInteractor(iter->second);
		}
	}

	// write object
	{
		std::map<VKString, VKString>::iterator iter = ncThis->m_WriteMap.find("object_analyzed");
		Interactor::InteractorID id = Interactor::ACT_Object;
		if (iter != m_WriteMap.end())
		{
			//std::cout<<" -- "<<iter->first.c_str()<<" -> "<<iter->second.c_str()<<std::endl;
			assert(m_Interactors[id]!=NULL);
			m_Interactors[id]->WriteInteractor(iter->second);
		}
	}

	// write binding
	{
		std::map<VKString, VKString>::iterator iter = ncThis->m_WriteMap.find("bindings");
		if (iter != m_WriteMap.end())
		{
			//std::cout<<" -- "<<iter->first.c_str()<<" -> "<<iter->second.c_str()<<std::endl;
			assert(m_pBinding!=NULL);
			EnergyInfo energy;
			BindingEnergy * e = ((BindingManager*)m_pBinding)->GetEnergy();
			for (int i=0; i<m_pBinding->NBindings(); i++)
			{
				Binding * b = ((BindingManager*)m_pBinding)->GetBinding(i);
				//std::cout<<"Binding: "<<i<<" / "<<m_pBinding->NBindings()<<std::endl;
				double eval = e->ComputeBindingEnergy(b, energy);
				b->SetEnergy(eval);
			}

			m_pBinding->WriteBindings(iter->second);
		}
	}
	
	// write energy
	{
		std::map<VKString, VKString>::iterator iter = ncThis->m_WriteMap.find("energy");
		if (iter != m_WriteMap.end())
		{
			FileManagement::CreateDirsOnPathToFile(iter->second.c_str());
			std::ofstream textStream(iter->second.c_str());
			assert(textStream.is_open());
			textStream<<GetEnergyStr(-1, true).c_str();
			textStream.close();
		}
	}
	
	// write timing
	{
		std::map<VKString, VKString>::iterator iter = ncThis->m_WriteMap.find("timing");
		if (iter != m_WriteMap.end())
		{
			assert(m_pBinding!=NULL);
			m_pBinding->WriteTimer(iter->second);
		}
	}
	
	// write binding in other formats
	{
		std::map<VKString, VKString>::iterator iter = ncThis->m_WriteMap.find("txt");
		const ShapeValue * objvals = NULL;
		const ShapePointCloud * objpnts = NULL;
		if (m_Interactors[Interactor::ACT_Object]!=NULL)
			m_Interactors[Interactor::ACT_Object]->GetObjectPredictions(&objpnts, &objvals);
		assert(objpnts!=NULL && objvals!=NULL);

		if (iter != m_WriteMap.end())
		{
			std::ofstream textStream(iter->second.c_str());
			assert(textStream.is_open());
			
			for (int poseID=0; poseID<m_pBinding->NBindings(); poseID++)
			{
				Binding * b = ((BindingManager*)m_pBinding)->GetBinding(poseID);
				textStream<<"Pose: "<<poseID<<" "<<b->GetEnergy()<<"\n";
				PntCorrsPose * pose = (PntCorrsPose*)b->GetPose(Interactor::ACT_Actor);
				IKSolver * solver = ((BindingManager*)m_pBinding)->GetEnergy()->GetSolver();
				BindingEnergy * energy = ((BindingManager*)m_pBinding)->GetEnergy();
				energy->m_Params.m_wFeatures = 0.01;	// hard-coded for now...
				textStream<<"\n";
				textStream<<"Contacts:\n";
				for (int m=0; m<pose->NMatches(); m++)
				{
					VKString rname = solver->m_RegionNames[pose->Region(m)];
					double cenergy = energy->ContactPenalty(pose->GetPoseID(), pose->Region(m), pose->Prim(m), objpnts, objvals);
					//std::cout<<"E: "<<cenergy<<std::endl;
					if (pose->Prim(m).IsType(TargetPrimitive::PRIM_Point))
					{
						const ShapePoint & pnt = pose->Prim(m).GetPoint();
						R3Point pos = pnt.GetPosition();
						int fid; double bx, by; const ShapeMesh * tempmesh;
						pnt.GetBarycentricCoords(&fid, &bx, &by, &tempmesh);

						textStream<<rname.c_str()<<" point "<<cenergy<<" "<<pos.X()<<" "<<" "<<pos.Y()<<" "<<" "<<pos.Z()<<" ";
						textStream<<fid<<" "<<bx<<" "<<by<<" "<<(1-bx-by)<<"\n";
						
					}
					else if (pose->Prim(m).IsType(TargetPrimitive::PRIM_Ground))
					{
						R3Point pos = pose->Prim(m).GetPosition();
						textStream<<rname.c_str()<<" ground "<<cenergy<<"\n";
					}
					else
						assert(false);
				}
				
				textStream<<"Skeleton:\n";
				std::vector<R3Affine *> saveXForms;
				solver->ForwardKinematics(pose, &saveXForms);
				
				assert(solver->m_Joints.size()==saveXForms.size());
				for (int i=0; i<(int)solver->m_Joints.size(); i++)
				{
					textStream<<solver->m_Joints[i]->m_sJointName.c_str()<<" ";
					textStream<<pose->GetPosition(i).X()<<" "<<pose->GetPosition(i).Y()<<" "<<pose->GetPosition(i).Z()<<"\n";
					for (int r=0; r<4; r++)
					for (int c=0; c<4; c++)
						textStream<<saveXForms[i]->Matrix()[r][c]<<" ";
					textStream<<"\n";
				}
				textStream<<"\n\n";
			}
			textStream.close();
		}
	}
	
	// write binding in other formats
	{
		std::map<VKString, VKString>::iterator iter = ncThis->m_WriteMap.find("gt");
		
		if (iter != m_WriteMap.end())
		{
			IKSolver * solver = ((BindingManager*)m_pBinding)->GetEnergy()->GetSolver();
			std::vector<PntCorrsPose*> poses;
			
			for (int i=0; i<m_pBinding->NBindings(); i++)
			{
				Binding * b = ((BindingManager*)m_pBinding)->GetBinding(i);
				poses.push_back((PntCorrsPose*)b->GetPose(Interactor::ACT_Actor));
			}
			
			VKStringList regionTypes;
			std::vector<int> npnts;
			for (int i=0; i<solver->m_RegionNames.count(); i++)
			{
				regionTypes.push_back("x");
				npnts.push_back(1);
			}
			
			PntCorrsPose::WritePosesGTFile(iter->second,  poses, solver->m_RegionNames, regionTypes, npnts);
		}
	}
	
	// write binding in other formats
	{
		std::map<VKString, VKString>::iterator iter = ncThis->m_WriteMap.find("invpos");
		
		if (iter != m_WriteMap.end())
		{
			IKSolver * solver = ((BindingManager*)m_pBinding)->GetEnergy()->GetSolver();
			assert(m_pBinding->NBindings()>0);
			Binding * b = ((BindingManager*)m_pBinding)->GetBinding(0);
			PntCorrsPose * pose = (PntCorrsPose*)b->GetPose(Interactor::ACT_Actor);
			pose->Param(0) = 0;
			pose->Param(1) = 0;
			pose->Param(2) = 0;
			pose->Param(3) = 0;
			solver->ForwardKinematics(pose);
			std::ofstream textStream(iter->second.c_str());
			assert(textStream.is_open());
			textStream<<"Pose: 0 0\n";
			textStream<<"Skeleton:\n";
			for (int i=0; i<(int)solver->m_Joints.size(); i++)
			{
				textStream<<solver->m_Joints[i]->m_sJointName.c_str()<<" ";
				textStream<<pose->GetPosition(i).X()<<" "<<pose->GetPosition(i).Y()<<" "<<pose->GetPosition(i).Z()<<"\n";
			}
			textStream.close();
		}
	}
}

void AnalysisWriter::AddWriteMap(const VKString & data, const VKString & file)
{
	m_WriteMap[data] = file;
}

void AnalysisWriter::SetBindingManager(const BindingManager * binding)
{
	m_pBinding = binding;
}

void AnalysisWriter::SetInteractor(Interactor::InteractorID id, const Interactor * interactor)
{
	m_Interactors[id] = interactor;
}

const VKString & AnalysisWriter::GetWriteMap(const VKString & data) const
{
	//std::map<VKString, VKString>::iterator iter = m_WriteMap.find(data);
	assert(m_WriteMap.find(data) != m_WriteMap.end());
	return m_WriteMap.find(data)->second;
}

VKString AnalysisWriter::GetEnergyStr(int poseid, bool detailed) const
{
	return ((BindingManager*)m_pBinding)->GetEnergy()->GetEnergyStr((BindingManager*)m_pBinding, poseid, detailed);
}





