#include "AnalysisWriter.h"
#include "BindingVisApp.h"
#include "BindingManager.h"
#include "Binding.h"
#include "Interactor.h"
#include "PosePriorMixGauss.h"
//#include "ActiveRegion.h"

#ifdef INCLUDE_BULLET
	#include "btBulletDynamicsCommon.h"
	#include "BulletCollision/Gimpact/btGImpactShape.h"
	#include "BulletCollision/Gimpact/btGImpactCollisionAlgorithm.h"
#endif

#include "IKSolver.h"
#include "PntCorrsPose.h"

BindingVisApp::BindingVisApp(BindingManager * binding, AnalysisWriter * writer)
{
	m_pIK = NULL;
	m_iSelectedPntID = -1;
	m_fPointScale = .05;
	m_ARActorID = Interactor::ACT_Invalid;
	m_pBinding = binding;
	m_iCurrBinding = 0;
	int flags = VIS_ActMesh | VIS_ObjMesh | VIS_ActEnergy | VIS_MeshSurface | VIS_ActEnergyText;
	SetRenderingFlags(flags);
	m_pAnlaysisWriter = writer;
	m_Mode = INTER_None;
	m_pAR = NULL;
	m_BrushSize = 5;
	m_iCurrActorAR[0] = -1;
	m_iCurrActorAR[1] = -1;
	m_bAnimation = false;
	m_bValidLastUpdate = false;
	m_EnergyVis = ENVIS_EndEffectorDist;
	m_fPntPercentile = 0;
	
	m_iReferencePntID = -1;
	m_iDemoID = 0;
}

void BindingVisApp::SetIK(IKSolver * solver)
{
	m_pIK = solver;
}


bool BindingVisApp::KeyPressed(DatabaseVisWindow * window, const char key, int mouseX, int mouseY)
{
	char printEnergy = 0;
//	if (m_Mode == INTER_AddAR)
//	{
//		if ((key>='a' && key <='z') || (key>='A' && key <='Z') || key=='-' || key==' ')
//		{
//			assert(m_pAR!=NULL);
//			char c2[2];
//			c2[0] = key;
//			c2[1] = 0;
//			m_pAR->SetRegionName(m_pAR->GetRegionName() + VKString(c2));
//		}
//		else if (key=='\r')
//			m_Mode = INTER_SelectAR;
//		return true;
//	}
	if (key=='`')
	{
		std::cout<<"Keys (BindingVisApp):"<<std::endl;
		std::cout<<"    . , - change bindings"<<std::endl;
		std::cout<<"    v   - write variance"<<std::endl;
		std::cout<<"    e   - write energy"<<std::endl;
		std::cout<<"    E   - write energy with details"<<std::endl;
		std::cout<<"    ? / - change AR"<<std::endl;
		std::cout<<"    m - render edges / vertices"<<std::endl;
		std::cout<<"    M - render faces"<<std::endl;
		std::cout<<"    p - render candidate points"<<std::endl;
		std::cout<<"    a - render actor"<<std::endl;
		std::cout<<"    D - show distributions"<<std::endl;
		std::cout<<"    o/O - run kinematic optimiaztion (O - from scratch)"<<std::endl;
		std::cout<<"    i - init pose"<<std::endl;
		std::cout<<"    g - map left leg to ground"<<std::endl;
		std::cout<<"    G - map left leg to ground"<<std::endl;
		std::cout<<"    x - special flag"<<std::endl;
		std::cout<<"Change Binding:"<<std::endl;
		std::cout<<"    < > - change actor's poses"<<std::endl;
		std::cout<<"    r   - change rotation axis (=/- - rotate)"<<std::endl;
		std::cout<<"    t   - change translation axis (=/- - translate)"<<std::endl;
		std::cout<<"Energy Visualiaztion:"<<std::endl;
		std::cout<<"    E - toggle energy visualization"<<std::endl;
		std::cout<<"    0 - toggle render text"<<std::endl;
		std::cout<<"    1 - angle error "<<std::endl;
		std::cout<<"    2 - angle error (ang 1)"<<std::endl;
		std::cout<<"    3 - angle error (ang 2)"<<std::endl;
		std::cout<<"    4 - angle error (ang 3)"<<std::endl;
		std::cout<<"    5 - distance between end effector and surface"<<std::endl;
		std::cout<<"    6 - feature descriptor error"<<std::endl;
		std::cout<<"    7 - maximal potential error due to a point"<<std::endl;
	}
	else if (key=='x')
	{
		//Toggle(VIS_IsectGrid);
		Toggle(VIS_SymmPlanes);
		Toggle(VIS_CoMPrediction);
	}
	else if (key=='Z')
		Toggle(VIS_PrecompPoseGrids);
	else if (key=='z')
		Toggle(VIS_HighProbContacts);
	else if (key=='.'||key==',')
	{
		m_iCurrBinding += ((key=='.') ? 1 : -1);
		if (m_iCurrBinding < 0)
			m_iCurrBinding = 0;
		else if (m_iCurrBinding >= m_pBinding->NBindings())
			m_iCurrBinding = m_pBinding->NBindings() - 1;
		std::cout<<"Binding: "<<m_iCurrBinding<<" "<<m_pBinding->GetBinding(m_iCurrBinding)->GetEnergy()<<std::endl;
	}
	else if (key=='o' || key=='O' || key=='i' || key=='I')
	{
		if (m_iCurrBinding>=0 && m_iCurrBinding < m_pBinding->NBindings())
		{
			PntCorrsPose * pose = (PntCorrsPose *)m_pBinding->GetBinding(m_iCurrBinding)->GetPose(Interactor::ACT_Actor);
			if (key=='o' || key=='O')
				m_pIK->SolveIKProblem(pose, key=='O');
			else if (key=='I')
			{
				m_pIK->m_pPrior->InitPose(0, pose);
				m_pIK->FindBestRigidXForm(pose);
			}
			else if (key=='i')
			{
				int niter = m_pIK->m_iIterations;
				m_pIK->m_iIterations = 1;
				m_pIK->SolveIKProblem(pose, false);
				m_pIK->m_iIterations = niter;
			}
		}
	}
	else if (key=='v')	// variance
	{
		if (m_pIK->m_pPrior!=NULL && m_pIK->m_pPrior->PriorType() == PosePrior::PRIOR_MultiGauss)
		{
			PosePriorMixGauss * prior = (PosePriorMixGauss*)m_pIK->m_pPrior;
			std::cout<<prior->GetStr().c_str()<<std::endl;
		}
		else
			std::cout<<"[WARNING] Unknown prior"<<std::endl;
	}
	else if (key=='V')	// joints info
	{
		if (m_iCurrBinding>=0 && m_iCurrBinding < m_pBinding->NBindings())
		{
			PntCorrsPose * pose = (PntCorrsPose *)m_pBinding->GetBinding(m_iCurrBinding)->GetPose(Interactor::ACT_Actor);
			std::cout<<pose->GetStr(m_pIK).c_str()<<std::endl;
		}
		else
			std::cout<<"[WARNING] Unknown pose"<<std::endl;
	}
	else if (key=='e' || key=='E')
		printEnergy = key;
	else if (key=='E')
		Toggle(VIS_ActEnergy);
	else if (key=='m')
		Toggle(VIS_MeshDetails);
	else if (key=='M')
		Toggle(VIS_MeshSurface);
	else if (key=='w')
		m_pAnlaysisWriter->Write();
#ifdef INCLUDE_BULLET
	else if (key=='s')
		m_pBinding->SimulatePhysics(m_iCurrBinding, .01);
#endif
	else if (key=='S')
	{
		m_bAnimation = !m_bAnimation;
		m_bValidLastUpdate = false;
	}
	else if (key=='q' || key=='Q')
		m_iReferencePntID += (key=='q' ? 1 : -1);
	else if (key=='p')
		//m_pBinding->PrintPhysicsStatus(m_iCurrBinding);
		Toggle(VIS_CandidatePnts);
	else if (key=='P')
		Toggle(VIS_CandidatePntsText);
	else if (key=='0')
		Toggle(VIS_ActEnergyText);
	else if (key>='1' && key <='7')
	{
		m_EnergyVis = (EnergyVisType)(key-'0');
		switch(m_EnergyVis)
		{
			case ENVIS_AngleAll:
				std::cout<<"ENVIS: Angle Errors"<<std::endl;
				break;
			case ENVIS_Angle1:
				std::cout<<"ENVIS: Angle 1"<<std::endl;
				break;
			case ENVIS_Angle2:
				std::cout<<"ENVIS: Angle 2"<<std::endl;
				break;
			case ENVIS_Angle3:
				std::cout<<"ENVIS: Angle 3"<<std::endl;
				break;
			case ENVIS_EndEffectorDist:
				std::cout<<"ENVIS: End effector distance"<<std::endl;
				break;
			case ENVIS_ContactFeatures:
				std::cout<<"ENVIS: Contact features"<<std::endl;
				break;
			case ENVIS_PotentialTotalError:
				std::cout<<"ENVIS: Potential error"<<std::endl;
				break;
			default:
				std::cout<<"ENVIS: other"<<std::endl;
				break;
		}
	}
	else if (key=='g' || key=='G')
	{
		if (m_iCurrBinding>=0 && m_iCurrBinding < m_pBinding->NBindings())
		{
			PntCorrsPose * pose = (PntCorrsPose *)m_pBinding->GetBinding(m_iCurrBinding)->GetPose(Interactor::ACT_Actor);
			int primID = -1;
			int targetR = -1;
			for (int i=0; i<m_pIK->m_RegionNames.count(); i++)
			{
				if (m_pIK->m_RegionNames[i]=="left_toe" && key=='g')
					targetR = i;
				else if (m_pIK->m_RegionNames[i]=="right_toe" && key=='G')
					targetR = i;
			}
			assert(targetR>=0);
			for (int r=0; r<pose->NMatches(); r++)
				if (targetR == pose->Region(r))
					primID = r;
			
			if (primID>=0)
				pose->Prim(primID) = TargetPrimitive(TargetPrimitive::PRIM_Ground);
			else
				pose->AddMatch(targetR, TargetPrimitive(TargetPrimitive::PRIM_Ground));
			m_pIK->SolveIKProblem(pose, true);
		}
	}
//	else if (key=='P')
//		Toggle(VIS_Physics);
	else if (key=='D')
		Toggle(VIS_Distribution);
	else if (key=='-'||key=='+'||key=='=')
	{
		m_fPointScale += key=='-' ? -0.1 : 0.1;
		std::cout<<"Point Scale: "<<m_fPointScale<<std::endl;
	}
	else if (key=='\''||key==';')
	{
		m_fPntPercentile += (key==';' ? -1 : 1)*0.05;
		if (m_fPntPercentile<0)
			m_fPntPercentile = 0;
		else if (m_fPntPercentile>=1.)
			m_fPntPercentile = 1.;
		std::cout<<"Showing percentile: "<<m_fPntPercentile<<std::flush;
		if (m_fPntPercentile>0)
		{
			if (m_iCurrBinding>=0 && m_iCurrBinding < (int)m_pBinding->m_CandidateCorrs.size()
				&& m_iSelectedPntID>=0 && m_iSelectedPntID < (int)m_pBinding->m_CandidateCorrs[m_iCurrBinding].size())
				std::cout<<"  - "<<m_pBinding->m_CandidateCorrs[m_iCurrBinding][m_iSelectedPntID].size()*m_fPntPercentile;
		}
		std::cout<<std::endl;
	}
	else if (key=='['||key==']')
	{
		if (!IsFlagSet(VIS_CandidatePnts))
			Toggle(VIS_CandidatePnts);
		m_iSelectedPntID += key=='[' ? -1 : 1;
		if (m_iSelectedPntID<0)
			m_iSelectedPntID = -1;
		else if (m_iCurrBinding>=0 && m_iCurrBinding < (int)m_pBinding->m_CandidateCorrs.size())
		{
			std::vector<std::vector<ShapePoint> > & corrs = m_pBinding->m_CandidateCorrs[m_iCurrBinding];
			
			if (m_iSelectedPntID < (int)corrs.size())
			{
				if (m_pBinding->m_CandidateCorrValues.size()>0)
				{
					assert(m_pBinding->m_CandidateCorrValues.size()==m_pBinding->m_CandidateCorrs.size());
					std::vector<std::vector<double> > & vals = m_pBinding->m_CandidateCorrValues[m_iCurrBinding];
					assert(corrs.size()==vals.size());
					std::cout<<"Selected: "<<m_iSelectedPntID<<" values: ";
					int lid = ((int)vals[m_iSelectedPntID].size())-1;
					if (lid>=0)
						std::cout<<vals[m_iSelectedPntID][0]<<" - "<<vals[m_iSelectedPntID][lid]<<std::endl;
					else
						std::cout<<"No Points ("<<vals[m_iSelectedPntID].size()<<")"<<std::endl;
				}
				else
					std::cout<<"Selected: "<<m_iSelectedPntID<<std::endl;
			}
		}
	}
//	else if (key=='['||key==']')
//	{
//		m_iSelectedPntID += key=='[' ? -1 : 1;
//		std::cout<<"Point ID: "<<m_iSelectedPntID<<std::endl;
//	}
//	else if (key=='/'||key=='?'||key=='\''||key=='"')
//	{
//		Interactor::InteractorID aid = (key=='/'||key=='?') ? Interactor::ACT_Object : Interactor::ACT_Actor;
//		const Interactor * actor = m_pBinding->GetActor(aid);
//		m_iCurrActorAR[aid] += ((key=='?'||key=='"') ? -1 : 1);
//		if (m_iCurrActorAR[aid]<0)
//			m_iCurrActorAR[aid] = -1;
//		else if (m_iCurrActorAR[aid] >= actor->NActiveRegions())
//			m_iCurrActorAR[aid] = actor->NActiveRegions()-1;
//		VKString str = "none";
//		if (m_iCurrActorAR[aid]>=0)
//			str = actor->GetActiveRegion(m_iCurrActorAR[aid])->GetRegionName();
//		std::cout<<Interactor::ActorName(aid)<<" active region: "<<str.c_str()<<std::endl;
//	}
//	else if (key=='>'||key=='<')
//	{
//		std::cout<<"[WARNING] Disabled pose change..."<<std::endl;
//		Interactor::InteractorID aid =Interactor::ACT_Actor;
//		const Interactor * actor = m_pBinding->GetActor(aid);
//		Binding * binding = m_pBinding->GetBinding(m_iCurrBinding);
//		const InteractorPose * pose = binding->GetPose(aid);
//		
//		int pose = poseid + (key=='>' ? 1 : -1);
//		if (pose < 0)
//			pose = 0;
//		else if (pose >= actor->NPoses())
//			pose = actor->NPoses() - 1;
//		binding->SetPoseID(aid, pose);
//		std::cout<<"Pose: "<<pose<<std::endl;
//		printEnergy = true;
//	}
//	else if (key=='=' || key=='-')	// depends on interaction mode
//	{
//		printEnergy = true;
//		BindingXForm * xf = m_pBinding->GetBinding(m_iCurrBinding)->GetBindingXForm(Interactor::ACT_Actor);
//		RigidAlign * rxf = xf->SafeCastToRigidAlign();		assert(rxf!=NULL);
//
//		double scale = 1.;
//		if (m_Mode==INTER_RotX||m_Mode==INTER_RotY||m_Mode==INTER_RotZ||m_Mode==INTER_RotA)
//		{
//			R3Vector rotAxis = rxf->GetRotationAxis();
//			double rotAngle = rxf->GetRotationAngle();
//			if (m_Mode==INTER_RotA)
//				rotAngle += (key=='-'?-1:1) * vkPI/20;
//			else
//			{
//				double scale = (key=='-'?-1:1) * vkPI/20;
//				R3Affine aff = R3identity_affine;
//				if (m_Mode==INTER_RotX)
//					aff.XRotate(scale);
//				else if (m_Mode==INTER_RotY)
//					aff.YRotate(scale);
//				else if (m_Mode==INTER_RotZ)
//					aff.ZRotate(scale);
//				aff.Apply(rotAxis);
//			}
//			rxf->SetRotation(rotAxis, rotAngle);
//		}
//		else if (m_Mode==INTER_TransX||m_Mode==INTER_TransY||m_Mode==INTER_TransZ)
//		{
//			R3Vector trans = rxf->GetTranslation();
//			scale = .05;
//			trans[m_Mode-INTER_TransX] += (key=='-'?-1:1) * scale;
//			rxf->SetTranslation(trans);
//		}
//		m_pBinding->ResetPhysics();
//	}
	else if (key=='a')
		Toggle(VIS_ActMesh);
//	else if (key=='a' || key=='A')	// active region
//	{
//		if (m_Mode==INTER_SelectAR)	// add active region
//		{
//			if (key=='A')	// reset
//			{
//				if (m_pAR!=NULL)
//					delete m_pAR;
//				m_pAR = NULL;
//				std::cout<<"Reseting AR "<<std::endl;
//			}
//			else
//			{
//				std::cout<<"Adding "<<m_pAR->GetRegionName().c_str();
//				std::cout<<" to "<<Interactor::ActorName(m_ARActorID)<<std::endl;
//				Interactor * actor = (Interactor*)m_pBinding->GetActor(m_ARActorID);
//				actor->AddActiveRegion(m_pAR);
//				m_pAR = NULL;
//			}
//			m_Mode = INTER_None;
//		}
//		else
//		{
//			if (key=='a')
//				m_ARActorID = Interactor::ACT_Actor;
//			else if (key=='A')
//				m_ARActorID = Interactor::ACT_Object;
//			m_Mode = INTER_AddAR;
//			if (m_pAR!=NULL)
//				delete m_pAR;
//			Interactor * actor = (Interactor*)m_pBinding->GetActor(m_ARActorID);
//			m_pAR = new ActiveRegion(actor, "", actor->NActiveRegions());
//		}
//	}
	else if (key=='r')	// rotate
	{
		if (m_Mode==INTER_RotX)
			m_Mode=INTER_RotY;
		else if (m_Mode==INTER_RotY)
			m_Mode=INTER_RotZ;
		else if (m_Mode==INTER_RotZ)
			m_Mode=INTER_RotA;
		else
			m_Mode=INTER_RotX;
		std::cout<<"SETMODE  Rotate: "<<m_Mode<<std::endl;
	}
	else if (key=='t')	// translate
	{
		if (m_Mode==INTER_TransX)
			m_Mode=INTER_TransY;
		else if (m_Mode==INTER_TransY)
			m_Mode=INTER_TransZ;
		else
			m_Mode=INTER_TransX;
		std::cout<<"SETMODE  Translate: "<<(m_Mode-INTER_TransX)<<std::endl;
	}
	else if (key==27)
		exit(0);
	if (printEnergy!=0)
	{
//		const Binding * binding = m_pBinding->GetBinding(m_iCurrBinding);
//		EnergyInfo e;
//		m_pBinding->GetEnergy()->ComputeBindingEnergy(binding, e);
//		std::cout<<"Energy: "<<e.m_fTotalEnergy<<std::endl;
//		std::cout<<"   Pose:     "<<e.m_fPosePriorEnergy<<std::endl;
//		std::cout<<"   Features: "<<e.m_fFeatPointsEnergy<<std::endl;
//
//		((Binding*)binding)->SetEnergy(e.m_fTotalEnergy);
		std::cout<<m_pAnlaysisWriter->GetEnergyStr(m_iCurrBinding, printEnergy=='E').c_str()<<std::flush;
	}
	return true;
}

bool BindingVisApp::MouseButtonEvent(DatabaseVisWindow * window, int x, int y,
									 VisWindow::MouseButton mouseButtonID, VisWindow::MouseButtonState newState)
{
	return false;
}

bool BindingVisApp::MouseMotionEvent(DatabaseVisWindow * window, int x, int y, int dx, int dy)
{
	if (window->IsMouseButtonDown(VisWindow::MOUSE_LEFT))
	{
		Interactor::InteractorID cid = Interactor::ACT_Object;
		Binding * binding = m_pBinding->GetBinding(m_iCurrBinding);
		const InteractorPose * pose = binding->GetPose(cid);
		VisModel3D * visModel = GetVisModel(window, binding->GetActor(cid), pose);
		if (!window->Ctrl() && !window->Alt() && !window->Shift())	// rotate
			visModel->Rotate(0, x, y, dx, dy, window);
		else if (window->Ctrl())
			visModel->Scale(0, x, y, dx, dy, window);
		else if (window->Shift())
			visModel->Translate(0, x, y, dx, dy, window);
	}
	else if (window->IsMouseButtonDown(VisWindow::MOUSE_RIGHT))
	{
		std::cout<<"[WARNING] Right-click not defined..."<<std::endl;
//		if (m_Mode==INTER_SelectAR)
//		{
//			assert(m_pAR!=NULL);
//			int fid;
//			double b1, b2;
//			const ShapeMesh * meshptr;
//			
//			const Binding * binding = m_pBinding->GetBinding(m_iCurrBinding);
//			int poseid = binding->GetPose(m_ARActorID);
//
//			int s = m_BrushSize;
//			std::vector<int> fids;
//			m_aVisModels[m_ARActorID][poseid]->QuickIntersectTriangleID(0, x-s, y-s, 2*s, 2*s, fids, window);
//			for (int i=0; i<(int)fids.size(); i++)
//				m_pAR->AddActiveFace(fids[i]);
//
//		}
		
	}
	return true;
}

bool BindingVisApp::Draw(DatabaseVisWindow * window)
{
	double br = 0, bg = 0, bb = 0;
	window->GetBackgroundColor(&br, &bg, &bb);
	glClearColor(br, bg, bb, 1.0);
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT | GL_ACCUM_BUFFER_BIT | GL_STENCIL_BUFFER_BIT);

	// prepare visualiaztion transforms
	R3Affine xforms[3];
	
	int currbinding = m_iCurrBinding;
	if (IsFlagSet(VIS_ScreenshotMode) && IsFlagSet(VIS_CandidatePnts))
		currbinding = 0;
		
	const Binding * binding = m_pBinding->GetBinding(currbinding);
	Interactor::InteractorID cid = Interactor::ACT_Object;
	R3Affine cam = GetVisModel(window, binding->GetActor(cid), binding->GetPose(cid))->GetCameraTransform(0);
	FillTransforms(window, cam, xforms);

	// things to draw later
	std::vector<RenderableFace> faces;
	VKStringList texts;
	std::vector<R3Point> points;

	DrawGround(window, xforms);
	DrawSymmPlanes(window, xforms[cid], faces);
	DrawPoses(window, xforms, faces, texts, points);
	int nblue = (int)points.size();
	DrawIsectGrid(window, xforms[cid], faces);
	DrawCandidatePnts(window, xforms, texts, points);
	
	
//	// draw energy: active points, etc...
//	DrawEnergy(window, xforms);
//	
//	// draw initial pose distributions
//	DrawDistribution(window, xforms);
//
//	// draw active regions
//	DrawActiveRegions(window, xforms);
//
//	// draw physics
//	DrawPhysics(window, xforms);
	
	// final: draw transparent faces (if any)
	window->LoadStandardSceneProjectionCamera();
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	window->GetRenderer()->DrawTransparentFaces(faces);

//#ifdef MESA_RENDERING
//	// draw poses
//	DrawPoses(window, xforms, faces, texts, points);
//	// draw candidate points
//	DrawCandidatePnts(window, xforms, texts, points);
//#endif
	
	// draw text
	for (int i=0; i<texts.count(); i++)
		window->GetRenderer()->RenderText(points[i], texts[i], i<nblue?.2:.8, i<nblue?.4:.2, .2, VisWindowRenderer::TEXT_SIZE_LARGE);
	
	// update physics
#ifdef INCLUDE_BULLET
	UpdatePhysics(window);
#else
	if (m_bAnimation)
	{
		int nframes = 300;
		Interactor::InteractorID cid = Interactor::ACT_Object;
		Binding * binding = m_pBinding->GetBinding(m_iCurrBinding);
		const InteractorPose * pose = binding->GetPose(cid);
		VisModel3D * visModel = GetVisModel(window, binding->GetActor(cid), pose);
		R3Vector axis(0, 1, 0.5);
		axis.Normalize();
		visModel->Rotate(0, axis,  M_PI/nframes, window);
		if (m_iDemoID==-1)
		{
			visModel->Scale(0, 1.3, window);
			m_iDemoID = 0;
		}
		m_iDemoID++;
		if (m_iDemoID>=2*nframes && m_iDemoID<4*nframes)
		{
			if (m_iCurrBinding==0)
				window->GetRenderer()->RenderText(R2Point(.3, .9), "Best Predicted Pose", 0, 0, 0, VisWindowRenderer::TEXT_SIZE_LARGE);
			window->GetRenderer()->RenderText(R2Point(.3, .8), "Candidate Contact Points", 0, 0, 0, VisWindowRenderer::TEXT_SIZE_LARGE);
			if (m_iDemoID==2*nframes)
				Toggle(VIS_CandidatePnts);
		}
		else
		{
			if (m_iCurrBinding==0)
				window->GetRenderer()->RenderText(R2Point(.3, .9), "Best Predicted Pose", 0, 0, 0, VisWindowRenderer::TEXT_SIZE_LARGE);
			if (m_iDemoID==4*nframes)
			{
				Toggle(VIS_CandidatePnts);
				m_iDemoID = 0;
			}
		}
			
		window->GetRenderer()->UpdateScreen();
	}
#endif

	
	return true;
}

void BindingVisApp::StartDemo()
{
	m_bAnimation = true;
	m_iDemoID =-1;
	//Toggle(VIS_CandidatePnts);
	//Toggle(VIS_CandidatePntsText);
	
}

void BindingVisApp::FillTransforms(DatabaseVisWindow * window, const R3Affine & cam, R3Affine * outxforms) const
{
	int currbinding = m_iCurrBinding;
	if (IsFlagSet(VIS_ScreenshotMode) && IsFlagSet(VIS_CandidatePnts))
		currbinding = 0;
	Binding * binding = m_pBinding->GetBinding(currbinding);
	
	for (int i=0; i<2; i++)
	{
		const Interactor * actor = binding->GetActor((Interactor::InteractorID)i);
		const RigidAlign * rxf = binding->GetBindingXForm((Interactor::InteractorID)i)->SafeCastToRigidAlign();
		assert(rxf!=NULL);
		//const R3Affine & canonical = actor->GetCanonical(binding->GetPose((Interactor::InteractorID)i));
		
		R3Affine xform = R3identity_affine;
		
		xform.Transform(cam);
		xform.Transform(rxf->GetXForm());
		//xform.Transform(canonical);
		outxforms[i] = xform;
	}
	outxforms[2] = cam;
}

void BindingVisApp::DrawPhysics(DatabaseVisWindow * window, const R3Affine * xforms) const
{
#ifdef INCLUDE_BULLET
	if (IsFlagSet(VIS_Physics))
	{
		window->LoadStandardSceneProjectionCamera();
		glMatrixMode(GL_MODELVIEW);
		glLoadIdentity();

		glDisable(GL_LIGHTING);
		std::vector<btManifoldPoint> pnts;
		m_pBinding->GetContactPoints(m_iCurrBinding, pnts);
		for (int i=0; i<(int)pnts.size(); i++)
		{
			btVector3 ptA = pnts[i].getPositionWorldOnA();
			btVector3 ptB = pnts[i].getPositionWorldOnB();
			
			R3Point p1(ptA.x(), ptA.y(), ptA.z());
			R3Point p2(ptB.x(), ptB.y(), ptB.z());
			xforms[2].Apply(p1);
			xforms[2].Apply(p2);
			glColor3d(1., 0., 0.);
			R3Sphere(p1, 0.01).Draw();
			glColor3d(1., 0.5, 0.5);
			R3Sphere(p2, 0.01).Draw();
			glBegin(GL_LINES);
			glVertex3d(p1.X(), p1.Y(), p1.Z());
			glVertex3d(p2.X(), p2.Y(), p2.Z());
			glEnd();
		}
	}
#endif
}

void BindingVisApp::DrawDistribution(DatabaseVisWindow * window, const R3Affine * xforms) const
{
	if (IsFlagSet(VIS_Distribution))
	{
		window->LoadStandardSceneProjectionCamera();
		glMatrixMode(GL_MODELVIEW);
		glLoadIdentity();
	
		// compute min/max energy
		double minEnergy = 0;
		double maxEnergy = 0;
		for (int i=0; i<m_pBinding->NBindings(); i++)
		{
			double energy = m_pBinding->GetBinding(i)->GetEnergy();
			if (i==0 || energy > maxEnergy)
				maxEnergy = energy;
			if (i==0 || energy < minEnergy)
				minEnergy = energy;
		}

		// space for orientations
		//int nposes = m_pBinding->GetActor(Interactor::ACT_Actor)->NPoses();
		int nposes = 1;
		glColor3d(0.4, 0.4, 0.4);
		glBegin(GL_QUADS);
		glVertex3d(-.8, 0.44, -1.1);
		glVertex3d(-.59+nposes*0.043, 0.44, -1.1);
		glVertex3d(-.59+nposes*0.043, 0.7, -1.1);
		glVertex3d(-.8, 0.7, -1.1);
		glEnd();

		// draw translations / orientations + energy affect sphere sizes		
		for (int i=0; i<m_pBinding->NBindings(); i++)
		{
			R3Point p1(0,0,0);
			R3Point p2(0,0, 1.);
			m_pBinding->GetBinding(i)->GetBindingXForm(Interactor::ACT_Actor)->Apply(p1);
			m_pBinding->GetBinding(i)->GetBindingXForm(Interactor::ACT_Actor)->Apply(p2);

			// choose radius + color based on energy
			double energy = maxEnergy - m_pBinding->GetBinding(i)->GetEnergy();
			double rad = 0;
			if ( minEnergy < maxEnergy)
				rad = (energy)/(maxEnergy - minEnergy);

			double r,g,b;
			window->MapScalarToColor(energy, 0, maxEnergy-minEnergy, r, g, b);
			if (i==m_iCurrBinding)
			{
				glColor3d(1., 0, 0);
				glLineWidth(4.);
			}
			else
			{
				glColor3d(r, g, b);
				glLineWidth(1.);
			}

			// draw sphere orientations
			//double pose = (double)m_pBinding->GetBinding(i)->GetPoseID(Interactor::ACT_Actor);
			double pose = 0;
			R3Vector offset(-.59 + pose * .06, .43, -1.);
			xforms[2].Apply(p1);	xforms[2].Apply(p2);
			R3Vector vec = p2-p1;	vec.Normalize(); vec *= 0.025; //vec *= rad;
			
			glBegin(GL_LINES);
			glVertex3d((offset).X(), (offset).Y(), (offset).Z());
			glVertex3d((offset+vec).X(), (offset+vec).Y(), (offset+vec).Z());
			glEnd();
			
			// draw sphere for translation
			R3Sphere(p1, 0.001 + 0.02*rad).Draw();
		}
		
		// render bounding box (assume it's the same for all bindings...)
		assert(m_pBinding->NBindings()>0);
		Binding * b0 = m_pBinding->GetBinding(0);
		const InteractorPose * pose = b0->GetPose(Interactor::ACT_Object);
		double objGround = m_pBinding->GetActor(Interactor::ACT_Object)->Ground(pose);
		R3Vector objTrans = R3Vector(0, 0, -objGround);
		R3Box bbox = m_pBinding->GetActor(Interactor::ACT_Object)->RelevantVolume(pose);
		//bbox.Translate(objTrans);
		glColor3d(.3, .3, .3);
		R3Point p[8];
		for (int i=0; i<8; i++)
		{
			p[i] = R3Point(i<4 ? bbox.XMin() : bbox.XMax(),
						   (i==2 || i==3 || i==6 || i==7) ? bbox.YMin() : bbox.YMax(),
						   i%2==0 ? bbox.ZMin() : bbox.ZMax());
			b0->GetBindingXForm(Interactor::ACT_Object)->Apply(p[i]);
			xforms[2].Apply(p[i]);
			R3Sphere(p[i], 0.01).Draw();
		}
		static const int va[16] = {0, 2, 3, 1, 5, 7, 3, 2, 6, 4, 0, 2, 6, 7, 5, 4};
		glLineWidth(2);
		glBegin(GL_LINE_STRIP);
		for (int i=0; i<16; i++)
			glVertex3d(p[va[i]].X(), p[va[i]].Y(), p[va[i]].Z());
		glEnd();
	}
}

void BindingVisApp::DrawEnergy(DatabaseVisWindow * window, const R3Affine * xforms) const
{
//	// render accepted points
//	if (IsFlagSet(VIS_ActEnergy))
//	{
//		window->LoadStandardSceneProjectionCamera();
//		glMatrixMode(GL_MODELVIEW);
//		glLoadIdentity();
//				
//		Binding * binding = m_pBinding->GetBinding(m_iCurrBinding);
//		std::vector<BindingManager::PointInfo> pnts;
//		m_pBinding->ComputeBindingEnergy(binding, &pnts);
//		
//		glDisable(GL_LIGHTING);
//		window->LoadStandardSceneProjectionCamera();
//		glMatrixMode(GL_MODELVIEW);
//		glLoadIdentity();
//		
//		for (int p=0; p<(int)pnts.size(); p++)
//		{
//			//if (pnts[p].m_PntType != BindingManager::PNT_Null)
//			{
//				double r = 0.005;
//				R3Point vp = pnts[p].m_Pos;
//				xforms[Interactor::ACT_Actor].Apply(vp);
//				if (pnts[p].m_PntType == BindingManager::PNT_Touch)
//					glColor3d(.5, .5, 1.);
//				else if (pnts[p].m_PntType == BindingManager::PNT_TouchAR)
//					glColor3d(1., 1., 0.);
//				else if (pnts[p].m_PntType == BindingManager::PNT_Isect)
//					glColor3d(1., 0., 0.);
//				else
//				{
//					glColor3d(.4, 0., 0.);
//					r = 0.0025;
//				}
//				R3Sphere(vp, r).Draw();
//			}
//		}
//		
//		
//		VKString str;
//		if (binding->GetEnergy()<=0)
//			str = VKString::number(((double)((int)(binding->GetEnergy()*-10000)))/100)+"%";
//		else
//			str = "Intersecting";
//		window->GetRenderer()->RenderText(R2Point(0, .95), str, .1, .1, .6, VisWindowRenderer::TEXT_SIZE_LARGE);
//	}
}

void BindingVisApp::DrawGround(DatabaseVisWindow * window, const R3Affine * xforms) const
{
	window->LoadStandardSceneProjectionCamera();
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();

	// render ground plane
	glDisable(GL_LIGHTING);
	glColor3d(.95, .95, 1.);
	glBegin(GL_QUADS);
	double size = 10, height = 0;
	if (IsFlagSet(VIS_ScreenshotMode))
		size = 2;
	int grid = 20;
	double w = 0.5/(double)grid *size;
	for (int x=0; x<grid; x++)
		for (int y=0; y<grid; y++)
		{
			double cx = ((double)x / (double)grid - 0.5) * size;
			double cy = ((double)y / (double)grid - 0.5) * size;
			R3Point p1(cx-w, cy-w, -height);		xforms[2].Apply(p1);		glVertex3d(p1.X(), p1.Y(), p1.Z());
			R3Point p2(cx-w, cy+w, -height);		xforms[2].Apply(p2);		glVertex3d(p2.X(), p2.Y(), p2.Z());
			R3Point p3(cx+w, cy+w, -height);		xforms[2].Apply(p3);		glVertex3d(p3.X(), p3.Y(), p3.Z());
			R3Point p4(cx+w, cy-w, -height);		xforms[2].Apply(p4);		glVertex3d(p4.X(), p4.Y(), p4.Z());
		}
	glEnd();
}

void BindingVisApp::DrawPoses(DatabaseVisWindow * window, const R3Affine * xforms, std::vector<RenderableFace> & outFaces,
							  VKStringList & texts, std::vector<R3Point> & points) const
{
	window->LoadStandardSceneProjectionCamera();
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();

	int currbinding = m_iCurrBinding;
	if (IsFlagSet(VIS_ScreenshotMode) && IsFlagSet(VIS_CandidatePnts))
		currbinding = 0;

	Binding * binding = m_pBinding->GetBinding(currbinding);
	while (m_VisInteractors.size() < 2)
		((BindingVisApp*)this)->m_VisInteractors.push_back(NULL);
	
	// Render meshes
	std::vector<Interactor::InteractorID> rlist;
	rlist.push_back(Interactor::ACT_Object);
	rlist.push_back(Interactor::ACT_Actor);
	
	for (int i=0; i<(int)rlist.size(); i++)
	{
		Interactor::InteractorID interactorID = rlist[i];
		if (!IsFlagSet(VIS_ActMesh) && interactorID == Interactor::ACT_Actor)
			continue;
		if (!IsFlagSet(VIS_ObjMesh) && interactorID == Interactor::ACT_Object)
			continue;
		const InteractorPose * pose = binding->GetPose(interactorID);
		const Interactor * actor = binding->GetActor(interactorID);
		if (m_VisInteractors[interactorID]==NULL)
			((BindingVisApp*)this)->m_VisInteractors[interactorID] = new VisInteractor(actor);
		
		if (actor->GetActorType()==Interactor::TYPE_Meshes && IsFlagSet(VIS_MeshSurface))
			DrawMesh(window, xforms, outFaces, interactorID);
		else if (actor->GetActorType()==Interactor::TYPE_PntCorrs)
		{
			DrawSkeleton(window, xforms, actor, (const PntCorrsPose*)pose, texts, points);
			DrawPosesGrid(window, xforms[interactorID], outFaces, (const PntCorrsPose*)pose);
		}
	}
}

void BindingVisApp::DrawSymmPlanes(DatabaseVisWindow * window, const R3Affine & cam, std::vector<RenderableFace> & outFaces)
{
	if (!IsFlagSet(VIS_SymmPlanes))
		return;
	window->LoadStandardSceneProjectionCamera();
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	
	// point correspondences...
	glEnable(GL_LIGHTING);
	GLfloat material[4];
	material[0] = 1.;	material[1] = 0.7;	material[2] = 0.7;	material[3] = 1.;
	glMaterialfv(GL_FRONT, GL_AMBIENT_AND_DIFFUSE, material);

	for (int i=0; i<(int)m_pBinding->GetEnergy()->m_ReflectionPlanes.size(); i++)
	{
		ReflectionPlane & rp = m_pBinding->GetEnergy()->m_ReflectionPlanes[i];
		R3Point p = rp.m_Plane.Point();
		R3Vector n = rp.m_Plane.Normal();
		//double r = rp.m_fRadius*rp.m_fWeight / 250.;
		double r = rp.m_fRadius*2.;
		
		R3Vector v1 = n%R3Vector(0, 0, 1);
		assert(v1.Length()>0);
		v1.Normalize();
		R3Vector v2 = n % v1;
		v2.Normalize();
		
		cam.Apply(n);
		cam.Apply(p);
		cam.Apply(v1);
		cam.Apply(v2);
		
		R3Point v;
		double step = 2*M_PI/10.;
		for (double a=0; a<2*M_PI; a+=step)
		{
			RenderableFace f;
			f.norm = n;
			f.r = 1.;
			f.g = .8;
			f.b = .8;
			f.a = .7;
			f.m_FaceType = RenderableFace::RENDFACE_Triangle;
			f.vtx[0] = p;
			f.vtx[1] = p+r*cos(a)*v1+r*sin(a)*v2;
			f.vtx[2] = p+r*cos(a+step)*v1+r*sin(a+step)*v2;
			outFaces.push_back(f);
		}
	}
}

void BindingVisApp::DrawSkeleton(DatabaseVisWindow * window, const R3Affine * xforms, const Interactor * actor,
								 const PntCorrsPose * corrs, VKStringList & texts, std::vector<R3Point> & points) const
{
	Binding * binding = m_pBinding->GetBinding(m_iCurrBinding);
	
	// point correspondences...
	glEnable(GL_LIGHTING);
	GLfloat surfaceMaterial[4];
	surfaceMaterial[0] = 0.;	surfaceMaterial[1] = 0.;	surfaceMaterial[2] = 1.;	surfaceMaterial[3] = 1.;
	glMaterialfv(GL_FRONT, GL_AMBIENT_AND_DIFFUSE, surfaceMaterial);
//#ifdef MESA_RENDERING
//	glDisable(GL_DEPTH_TEST);
//	glDisable(GL_DEPTH_BUFFER_BIT);
//#else
	glEnable(GL_DEPTH_TEST);
	glEnable(GL_DEPTH_BUFFER_BIT);
//#endif
	
	assert(actor->GetActorType()==Interactor::TYPE_PntCorrs);
	const Interactor * object = binding->GetActor(Interactor::ACT_Object);
	const InteractorPose * opose = object->GetEnumeratedPose(0);
	
	//IKSolver * ik = ((Interactor*)actor)->GetIK();
	assert(m_pIK!=NULL);
	std::vector<R3Point> pnts;
	std::vector<int> edges;
	static bool done = false;
	//std::cout<<"Fill renderable.. "<<m_iCurrBinding<<" - "<<corrs;
	//std::cout<<" params("<<m_pIK->NJointParams()<<"): "<<corrs->IsParamsInit(m_pIK->NJointParams())<<std::endl;
	m_pIK->FillRenderableSkeleton(corrs, true, pnts, edges);
	
	if (IsFlagSet(VIS_CoMPrediction))
	{
		R3Affine xform = R3identity_affine;
		m_pIK->FindReferenceNode(corrs, xform);
		R3Point trans(0,0,0);
		R3Vector orientX(1,0,0);
		R3Vector orientY(0,1,0);
		R3Vector orientZ(0,0,1);
		xform.Apply(trans);
		xform.Apply(orientX);
		xform.Apply(orientY);
		xform.Apply(orientZ);
		orientX.Normalize();
		orientY.Normalize();
		orientZ.Normalize();
//		feature[0] = orientX.Dot(objCOM-trans);
//		feature[1] = orientY.Dot(objCOM-trans);
//		feature[2] = orientZ.Dot(objCOM-trans);
		PosePointGauss * gauss = NULL;
		int poseid = corrs->GetPoseID();
		if (poseid<0)
			gauss = m_pIK->m_pPrior->m_comDistrGlobal;
		else
			gauss = m_pIK->m_pPrior->m_comDistrPerStyle[poseid];
		
		std::vector<int> rois;
		for (int i=0; i<corrs->NMatches(); i++)
			rois.push_back(corrs->Region(i));
		double e;
		int bp = BindingEnergy::FindBestPlaneOfReflection(m_pBinding->GetEnergy()->m_ReflectionPlanes, corrs, m_pIK, rois, e);
		R3Point objCOM = m_pBinding->GetEnergy()->m_ReflectionPlanes[bp].m_Plane.Point();
		//std::cout<<"Value: "<<orientX.Dot(objCOM-trans)<<" "<<orientY.Dot(objCOM-trans)<<" "<<orientZ.Dot(objCOM-trans)<<" "<<std::endl;
		//std::cout<<"Mean: "<<gauss->m_aMeans[0]<<" "<<gauss->m_aMeans[1]<<" "<<gauss->m_aMeans[2]<<" "<<std::endl;
		//std::cout<<"Var: "<<gauss->m_aVariances[0]<<" "<<gauss->m_aVariances[1]<<" "<<gauss->m_aVariances[2]<<" "<<std::endl;
		R3Point pos = orientX*gauss->m_aMeans[0]+orientY*gauss->m_aMeans[1]+orientZ*gauss->m_aMeans[2]+trans;
		object->GetCanonical(opose).Apply(pos); xforms[Interactor::ACT_Object].Apply(pos);
		R3Sphere(pos, 0.03).Draw();
	}
	//R3Vector forward(1, 0, 0);
	//int id = (int)pnts.size();
	//forward.ZRotate(corrs->GetParam(3));
	//pnts.push_back(pnts[0]+forward);
//	edges.push_back(0);
//	edges.push_back(1);
//	edges.push_back(0);
//	edges.push_back(2);
//	edges.push_back(0);
//	edges.push_back(3);
	//std::cout<<"  done!"<<std::endl;
//	int ikIter = 0;
//	while (!done && ikIter++<1000)
//		done = m_pIK->FillRenderableSkeleton(corrs, true, true, pnts, edges);

	// calculate energy
	EnergyInfo energy;
	//energy.m_EnergyVis = m_EnergyVis;
//	m_pBinding->GetEnergy()->ComputeBindingEnergy(binding, energy);
	double minval = RN_INFINITY;
	double maxval = -RN_INFINITY;
	for (int e=0; e<(int)energy.m_PerPointEnergy.size(); e++)
	{
		minval = vkMin(energy.m_PerPointEnergy[e], minval);
		maxval = vkMax(energy.m_PerPointEnergy[e], maxval);
	}
	
	// draw joints
	glEnable(GL_LIGHTING);
	static GLfloat material[4];
	material[0] = .7;   material[1] = .7;   material[2] = 1.;   material[3] = 1.;
	bool drawEnergy = energy.m_PerPointEnergy.size()>0 && minval<maxval;
	bool linksEnergy = (energy.m_EnergyVis==ENVIS_AngleAll||energy.m_EnergyVis==ENVIS_Angle1);
	linksEnergy = (linksEnergy||energy.m_EnergyVis==ENVIS_Angle2||energy.m_EnergyVis==ENVIS_Angle3);
	bool jointEnergy = (energy.m_EnergyVis==ENVIS_EndEffectorDist||energy.m_EnergyVis==ENVIS_ContactFeatures);
	jointEnergy = (jointEnergy||energy.m_EnergyVis==ENVIS_PotentialTotalError);
	bool drawLinksText =drawEnergy && linksEnergy && IsFlagSet(VIS_ActEnergyText);
	bool drawJointsText =drawEnergy && jointEnergy && IsFlagSet(VIS_ActEnergyText);
	
	for (int i=0; i<(int)pnts.size(); i++)
	{
		R3Point pos = pnts[i];
		object->GetCanonical(opose).Apply(pos); xforms[Interactor::ACT_Object].Apply(pos);
		double r= 0.01;

		if (drawEnergy && jointEnergy)
		{
			//std::cout<<"Getting: "<<i<<" / "<<pnts.size()<<" / "<<energy.m_PerPointEnergy.size()<<std::endl;
			assert(i < (int)energy.m_PerPointEnergy.size());
			window->MapScalarToColor(energy.m_PerPointEnergy[i], minval, maxval, material);
			r = 0.02;
			if (drawJointsText && energy.m_PerPointEnergy[i]>0)
			{
				points.push_back(pos);
				texts.push_back(VKString::number(energy.m_PerPointEnergy[i]));
			}
		}
		glMaterialfv(GL_FRONT, GL_AMBIENT_AND_DIFFUSE, material);
		R3Sphere(pos, r).Draw();
	}
	
	// draw links
	glDisable(GL_LIGHTING);
	glLineWidth(10.);
	
	for (int i=0; i<(int)edges.size(); i+=2)
	{
		glColor3d(1., 0., 0.);
		int p1 = edges[i+0];
		int p2 = edges[i+1];
		assert(p1>=0 && p1 < (int)pnts.size() && p2>=0 && p2 < (int)pnts.size());
		R3Point p1pos = pnts[p1];
		R3Point p2pos = pnts[p2];
		if (drawEnergy && linksEnergy)
		{
			int lid = i / 2;
			assert(lid<(int)energy.m_PerPointEnergy.size());
			if (energy.m_PerPointEnergy[lid]>0)
			{
				glEnable(GL_LIGHTING);
				window->MapScalarToColor(energy.m_PerPointEnergy[lid], minval, maxval, material);
				glMaterialfv(GL_FRONT, GL_AMBIENT_AND_DIFFUSE, material);
				R3Point pos = 0.5*p1pos+0.5*p2pos;
				object->GetCanonical(opose).Apply(pos); xforms[Interactor::ACT_Object].Apply(pos);
				R3Sphere(pos, 0.02).Draw();
				if (drawLinksText)
				{
					points.push_back(pos);
					texts.push_back(VKString::number(energy.m_PerPointEnergy[lid]));
				}
				glDisable(GL_LIGHTING);
			}
		}
		object->GetCanonical(opose).Apply(p1pos); xforms[Interactor::ACT_Object].Apply(p1pos);
		object->GetCanonical(opose).Apply(p2pos); xforms[Interactor::ACT_Object].Apply(p2pos);
		glBegin(GL_LINES);
		glVertex3d(p1pos.X(), p1pos.Y(), p1pos.Z());
		glVertex3d(p2pos.X(), p2pos.Y(), p2pos.Z());
		glEnd();
	}
//#ifdef MESA_RENDERING
//	glEnable(GL_DEPTH_TEST);
//	glEnable(GL_DEPTH_BUFFER_BIT);
//#endif

	
	//assert(actor!=object && object->NumEnumeratedPoses()==1);
	
	// write region names
	if (!drawLinksText && !drawJointsText && !IsFlagSet(VIS_ScreenshotMode) && !m_bAnimation)
	for (int i=0; i<(int)corrs->NMatches(); i++)
	{
		//R3Point pos = object->GetCanonicalPoint(corrs->m_aFeaturePoints[i].PointID(), opose);
		glEnable(GL_LIGHTING);
		
		if (corrs->Prim(i).IsType(TargetPrimitive::PRIM_Point))
		{
			R3Point pos = corrs->Prim(i).GetPosition();
			window->MapIntToRadicalColor(corrs->Region(i), material);
			glMaterialfv(GL_FRONT, GL_AMBIENT_AND_DIFFUSE, material);
			object->GetCanonical(opose).Apply(pos);
			xforms[Interactor::ACT_Object].Apply(pos);
			R3Sphere(pos, .013).Draw();
			int rid = corrs->Region(i);
			assert(rid >=0 && rid < m_pBinding->RegionNames().count());
			VKString rName = m_pBinding->RegionNames()[rid];
			texts.push_back(rName);
			points.push_back(pos);
		}
	}
}

void BindingVisApp::DrawMesh(DatabaseVisWindow * window, const R3Affine * xforms,
							 std::vector<RenderableFace> & outFaces, Interactor::InteractorID interactorID) const
{
	int currbinding = m_iCurrBinding;
	if (IsFlagSet(VIS_ScreenshotMode) && IsFlagSet(VIS_CandidatePnts))
		currbinding = 0;

	Binding * binding = m_pBinding->GetBinding(currbinding);
	const InteractorPose * pose = binding->GetPose(interactorID);
	int currFace = (int)outFaces.size();
#ifdef INCLUDE_BULLET
	if (IsFlagSet(VIS_Physics))
		m_VisInteractors[interactorID]->FillPhysicsRenderableArray(pose, xforms[2], outFaces);
	else
#endif
		m_VisInteractors[interactorID]->FillRenderableArray(pose, xforms[interactorID], xforms[2], outFaces);
	
//#ifdef MESA_RENDERING
	if (m_VisInteractors[interactorID]->m_pInteractor->m_pObjectDensePoints!=NULL)
	{
		glPointSize(1.);
		glDisable(GL_LIGHTING);
		glEnable(GL_COLOR);
		glEnable(GL_DEPTH_TEST);
		glColor4d(.7, .7, .7, .5);
		glBegin(GL_POINTS);
		for (int p=0; p<m_VisInteractors[interactorID]->m_pInteractor->m_pObjectDensePoints->NumPoints(); p++)
		{
			R3Point pos = m_VisInteractors[interactorID]->m_pInteractor->m_pObjectDensePoints->PointPosition(p);
			m_VisInteractors[interactorID]->m_pInteractor->GetCanonical(pose).Apply(pos);
			xforms[interactorID].Apply(pos);
			glVertex3d(pos.X(), pos.Y(), pos.Z());
		}
		glEnd();
	}

//	glEnable(GL_LIGHTING);
//	glDisable(GL_COLOR);
//	glEnable(GL_DEPTH_TEST);
//	
//	glBegin(GL_TRIANGLES);
//	for (int f=currFace; f<(int)outFaces.size(); f++)
//	{
//		static GLfloat surfaceMaterial[4];
//		surfaceMaterial[0] = .5;
//		surfaceMaterial[1] = .5;
//		surfaceMaterial[2] = .5;
//		surfaceMaterial[3] = 1.;
//		glColor3d(0.5, 0.5, 0.5);
//		glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE, surfaceMaterial);
//		glNormal3d(outFaces[f].norm.X(), outFaces[f].norm.Y(), outFaces[f].norm.Z());
//		
//		for (int i=0; i<3; i++)
//			glVertex3d(outFaces[f].vtx[i].X(), outFaces[f].vtx[i].Y(), outFaces[f].vtx[i].Z());
//	}
//	glEnd();
//	outFaces.clear();
//#else
//	for (int i=currFace; i<(int)outFaces.size(); i++)
//	{
//		outFaces[i].a = .6;
//		if (interactorID==Interactor::ACT_Object)
//		{	outFaces[i].r = 0.5;	outFaces[i].g = .5;	outFaces[i].b = 0.5;	}
//		else if (interactorID==Interactor::ACT_Actor)
//		{	outFaces[i].r = 0;	outFaces[i].g = 0;	outFaces[i].b = .8;	}
//	}
//#endif
}

void BindingVisApp::DrawIsectGrid(DatabaseVisWindow * window, const R3Affine & cam, std::vector<RenderableFace> & outFaces) const
{
//	std::cout<<"Draw grid: "<<IsFlagSet(VIS_IsectGrid)<<std::endl;
	double r=0, g=0, b=0, a=0.5;
	if (IsFlagSet(VIS_IsectGrid))
	{
		IntersectionGrid * igrid = m_pBinding->GetEnergy()->GetIntersectionGrid();
		if (igrid==NULL)
			return;
		const ShapePointCloud * pnts = igrid->InsidePoints(NULL);
		double pointsize = 4.;
		glDisable(GL_LIGHTING);
		glPointSize(pointsize);
		glBegin(GL_POINTS);
		for (int i=0; i<pnts->NumPoints(); i++)
		{
			R3Point pos = pnts->PointPosition(i);
			double val = igrid->ComputePenalty(pos);
//			{// inside-out debug
//				if (vkAbs(val-1.)<1e-10)
//				{ r=0; g=0; b=0;}
//				else if (vkAbs(val-.5)<1e-10)
//				{ r=1; g=0; b=0;}
//				else
//				{ r=0; g=1.; b=0;}
//			}
			{
				double maxdist = 5.;	// 60cm
				val /= maxdist;
				if (val>=1.)
					val = 1.;
				a = val;
			}
			const InteractorPose * pose = m_VisInteractors[Interactor::ACT_Object]->m_pInteractor->GetEnumeratedPose(0);
			m_VisInteractors[Interactor::ACT_Object]->m_pInteractor->GetCanonical(pose).Apply(pos);
			cam.Apply(pos);
			//glColor3d(1.-val, 1.-val, 1.-val);
			//glVertex3d(pos.X(), pos.Y(), pos.Z());
			outFaces.push_back(RenderableFace(pos, RenderableFace::RENDFACE_Point, pointsize, r, g, b, a));
		}
		glEnd();
	}
}

void BindingVisApp::DrawCandidatePnts(DatabaseVisWindow * window, const R3Affine * xforms,
									  VKStringList & texts, std::vector<R3Point> & points) const
{
	if (m_pBinding->m_CandidateCorrs.size() == 0)
		return;
	if (!IsFlagSet(VIS_CandidatePnts))
		return;
	window->LoadStandardSceneProjectionCamera();
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	
	// point correspondences...
	glEnable(GL_LIGHTING);
	GLfloat surfaceMaterial[4];
	
	glEnable(GL_DEPTH_TEST);
	glEnable(GL_DEPTH_BUFFER_BIT);
	
	const Interactor * object = m_pBinding->GetActor(Interactor::ACT_Object);
	const InteractorPose * opose = object->GetEnumeratedPose(0);
	const R3Affine & canonical = object->GetCanonical(opose); 

	int pointID = 0;
	int c2f = 0;
	if (m_iCurrBinding < (int)m_pBinding->m_CandidateCorrs.size())
		c2f = m_iCurrBinding;

	if (IsFlagSet(VIS_HighProbContacts))
		c2f = ((int)m_pBinding->m_CandidateCorrs.size())-1;	// assume this is where 'all' are..
	
	Binding * binding = m_pBinding->GetBinding(m_iCurrBinding);
	const PntCorrsPose * corrs = (const PntCorrsPose*)binding->GetPose(Interactor::ACT_Actor);
	
	R3Point refPos(0,0,0);
	PriorAngles * prior=NULL;
	PntCorrsPose * cond = NULL;
	int refID = -1;
	if (IsFlagSet(VIS_HighProbContacts))
	{
		assert(m_pIK->m_pPrior!=NULL && m_pIK->m_pPrior->IsAngles());
		prior = (PriorAngles*)m_pIK->m_pPrior;
		cond = new PntCorrsPose();
		cond->SetPoseID(corrs->GetPoseID());
		for (int i=0; i<corrs->NMatches(); i++)
		{
			if (corrs->Prim(i).IsType(TargetPrimitive::PRIM_Point)
				&& (m_iReferencePntID==-1 || corrs->Region(i)==m_iReferencePntID))
			{
				refID = corrs->Region(i);
				cond->AddMatch(corrs->Region(i), corrs->Prim(i));
				refPos = corrs->Prim(i).GetPosition();
				break;
			}
		}
		if (refID==-1)
			return;
	}
	
	R3Affine w2wg = R3identity_affine;
	w2wg.ZRotate(-corrs->GetParam(3));
	
	for (int r=0; r<(int)m_pBinding->m_CandidateCorrs[c2f].size(); r++)
	{
		const R3Grid * grid = NULL;
		if (IsFlagSet(VIS_HighProbContacts))
		{
			if (r==refID)
				continue;
			int gid = prior->GetGridID(r, cond);
			if (gid<0 && IsFlagSet(VIS_HighProbContacts))
				continue;
			grid = prior->GetGrid(gid);
		}
		int npnts = (int)m_pBinding->m_CandidateCorrs[c2f][r].size();
		if (m_iSelectedPntID>=0 && r!=m_iSelectedPntID)
			continue;
		for (int p=0; p<(int)m_pBinding->m_CandidateCorrs[c2f][r].size(); p++)
		{
			if (m_fPntPercentile>0 && p > (int)(m_pBinding->m_CandidateCorrs[c2f][r].size() * m_fPntPercentile))
				break;
			assert(r >=0 && r < m_pBinding->RegionNames().count());
			VKString rName = m_pBinding->RegionNames()[r];
			
			int colorid = r;
			if (rName.startsWith("left") && !IsFlagSet(VIS_HighProbContacts))
			{
				colorid = -1;
				rName = rName.replace("left_", "right_");
				for (int other = 0; other < m_pBinding->RegionNames().count(); other++)
				{
					if (rName == m_pBinding->RegionNames()[other])
						colorid = other;
				}
				assert(colorid!=-1);
			}

			pointID++;
//			if (pointID!=m_iSelectedPntID && m_iSelectedPntID>0)
//				continue;
			R3Point pos = m_pBinding->m_CandidateCorrs[c2f][r][p].GetPosition();
			
			if (IsFlagSet(VIS_HighProbContacts))
			{
				R3Point gp = pos;
				gp += -refPos;
				w2wg.Apply(gp);
				
				grid->WorldToGridTransformation().Apply(gp);
				
				//std::cout<<"GP: "<<gp.X()<<" "<<gp.Y()<<" "<<gp.Z()<<":  "<<grid->GridValue(gp)<<std::endl;
				
				int gi = (int)gp.X();
				int gj = (int)gp.Y();
				int gk = (int)gp.Z();
				if (gi<0 || gi>=grid->XResolution() || gj<0 || gj>=grid->YResolution() ||
					gk<0 || gk>=grid->ZResolution() || grid->GridValue(gi, gj, gk) >= RN_INFINITY)
					continue;
			}
			
			canonical.Apply(pos);
			if (IsFlagSet(VIS_ScreenshotMode))
			{
				double r1 = ((double)(rand()%1000))/1000.;
				double r2 = ((double)(rand()%1000))/1000.;
				double r3 = ((double)(rand()%1000))/1000.;
				pos += R3Vector(r1, r2, r3) * m_fPointScale;
			}

			xforms[Interactor::ACT_Object].Apply(pos);
			window->MapIntToRadicalColor(colorid, surfaceMaterial);
			if (m_iSelectedPntID>=0 && m_iCurrBinding>=0 && m_pBinding->m_CandidateCorrValues.size()>0)
			{
				if (m_iSelectedPntID < (int)m_pBinding->m_CandidateCorrValues[c2f].size())
				{
					std::vector<double> & vals = m_pBinding->m_CandidateCorrValues[c2f][r];
					if (vals.size()>0 && p < (int)vals.size() && p>=0)
					{
						double t = vals[p] / vals[0];
						surfaceMaterial[0] = t * surfaceMaterial[0] + (1.-t);
						surfaceMaterial[1] = t * surfaceMaterial[1] + (1.-t);
						surfaceMaterial[2] = t * surfaceMaterial[2] + (1.-t);
					}
				}
			}
			glMaterialfv(GL_FRONT, GL_AMBIENT_AND_DIFFUSE, surfaceMaterial);
			R3Vector testRadius(1., 1., 1.);
			testRadius.Normalize();
			testRadius *= m_fPointScale;
			canonical.Apply(testRadius);
			xforms[Interactor::ACT_Object].Apply(testRadius);
			R3Sphere(pos, testRadius.Length()).Draw();
			
			if (IsFlagSet(VIS_CandidatePntsText))
			{
				texts.push_back(rName);
				points.push_back(pos);
			}
		}
	}
	delete cond;
}

void BindingVisApp::DrawActiveRegions(DatabaseVisWindow * window, const R3Affine * xforms) const
{
//	if (IsFlagSet(VIS_ActAR))
//	{
//		for (int i=0; i<2; i++)
//		{
//			if (m_iCurrActorAR[i]>=0)
//			{
//				const InteractorPose * pose = m_pBinding->GetBinding(m_iCurrBinding)->GetPose((Interactor::InteractorID)i);
//				const Interactor * actor = m_pBinding->GetBinding(m_iCurrBinding)->GetActor(i);
//				assert(false);
//				//DrawAR(actor->GetActiveRegion(m_iCurrActorAR[i]), pose, xforms[i], window);
//			}
//		}
//	}
//	if (m_pAR!=NULL)
//	{
//		VKString str = VKString(Interactor::ActorName(m_ARActorID)) + " (AR): "+m_pAR->GetRegionName();
//		if (m_Mode == INTER_AddAR)
//			str += "_";
//		window->GetRenderer()->RenderText(R2Point(-.9, .95), str, .5, .2, .2, VisWindowRenderer::TEXT_SIZE_LARGE);
//		if (m_Mode==INTER_SelectAR)
//		{
//			assert(false);
//			//int poseid = m_pBinding->GetBinding(m_iCurrBinding)->GetPoseID(m_ARActorID);
//			//DrawAR(m_pAR, poseid, xforms[m_ARActorID], window);
//		}
//	}
}

//void BindingVisApp::DrawAR(const ActiveRegion * ar, int poseid, const R3Affine & xform, DatabaseVisWindow * window) const
//{
//	const ShapeMesh * mesh = ar->GetActor()->GetMesh(poseid);
//	window->LoadStandardSceneProjectionCamera();
//	glMatrixMode(GL_MODELVIEW);
//	glLoadIdentity();
//	glDisable(GL_LIGHTING);
//	
//	glBegin(GL_TRIANGLES);
//	const std::vector<int> arfaces = ar->GetRegionFaces();
//	for (int i=0; i<(int)arfaces.size(); i++)
//	{
//		for (int v=0; v<3; v++)
//		{
//			int vid = mesh->VertexOnFace(arfaces[i], v);
//			R3Point p =mesh->VertexPosition(vid);
//			xform.Apply(p);
//			glColor3d(1., 0, 0);
////			std::cout<<"DrawAR: ["<<p.X()<<" "<<p.Y()<<" "<<p.Z()<<"] "<<std::endl;
//			glVertex3d(p.X(), p.Y(), p.Z());
//		}
//	}
//	glEnd();
//}


void BindingVisApp::DrawPosesGrid(DatabaseVisWindow * window, const R3Affine & camera,
								  std::vector<RenderableFace> & outFaces, const PntCorrsPose * corrs) const
{
	if (IsFlagSet(VIS_PrecompPoseGrids))
	{
		assert(m_pIK->m_pPrior!=NULL && m_pIK->m_pPrior->IsAngles());
		PriorAngles * prior = (PriorAngles*)m_pIK->m_pPrior;
		
		PntCorrsPose * cond = new PntCorrsPose();
		cond->SetPoseID(corrs->GetPoseID());
		
		R3Point refPos(0,0,0);
		int refID = -1;
		for (int i=0; i<corrs->NMatches(); i++)
		{
			if (corrs->Prim(i).IsType(TargetPrimitive::PRIM_Point)
				&& (m_iReferencePntID==-1 || corrs->Region(i)==m_iReferencePntID))
			{
				refID = corrs->Region(i);
				cond->AddMatch(corrs->Region(i), corrs->Prim(i));
				refPos = corrs->Prim(i).GetPosition();
				break;
			}
		}
		if (refID==-1)
			return;

		R3Affine w2wg = R3identity_affine;
		w2wg.ZRotate(corrs->GetParam(3));

		GLfloat surfaceMaterial[4];
		for (int i=0; i<m_pIK->m_RegionNames.count(); i++)
		{
			if (refID==i)
				continue;
			int gid = prior->GetGridID(i, cond);
			if (gid<0)
				continue;
			const R3Grid * grid = prior->GetGrid(gid);
			double maxval = 0;
			double minval = RN_INFINITY;
			for (int g=0; g<grid->NEntries(); g++)
			{
				if (grid->GridValue(g) < RN_INFINITY)
				{
					minval = vkMin(grid->GridValue(g), minval);
					maxval = vkMax(grid->GridValue(g), maxval);
				}
			}

			double gridsize;
			const ShapePointCloud * pnts = prior->GetNonZeroProb(gid, &gridsize);
			
			window->MapIntToRadicalColor(i, surfaceMaterial);
			
			R3Vector refSize(1., 1., 1.);
			refSize.Normalize();
			refSize *= gridsize;
			camera.Apply(refSize);
			gridsize = refSize.Length();

			for (int p=0; p<pnts->NumPoints(); p++)
			{
				R3Point pos = pnts->PointPosition(p);
				double wval = PriorAngles::ExactWorldValue(pos, grid);
				double value = (wval-minval) / (maxval-minval);
				surfaceMaterial[3] = .5 + .5*value;
				
				glMaterialfv(GL_FRONT, GL_AMBIENT_AND_DIFFUSE, surfaceMaterial);
				glColor3d(surfaceMaterial[0], surfaceMaterial[1], surfaceMaterial[2]);
				w2wg.Apply(pos);
				pos += refPos;
				camera.Apply(pos);
				R3Sphere(pos, gridsize).Draw();
			}
		}
		
		delete cond;
	}
}


void BindingVisApp::UpdatePhysics(DatabaseVisWindow * window)
{
#ifdef INCLUDE_BULLET
	if (m_bAnimation)
	{
		if (!m_bValidLastUpdate)
		{
			gettimeofday(&m_LastUpdate, NULL);
			m_bValidLastUpdate = true;
		}
		else
		{
			timeval endtime;
			gettimeofday(&endtime, NULL);
			if (endtime.tv_usec > m_LastUpdate.tv_usec || endtime.tv_sec > m_LastUpdate.tv_sec)
			{
				double dt = (((double)endtime.tv_usec - (double)m_LastUpdate.tv_usec)) / 1000000.0;
				dt += (double)(endtime.tv_sec - m_LastUpdate.tv_sec);
				m_pBinding->SimulatePhysics(m_iCurrBinding, dt);
				m_LastUpdate = endtime;
			}
		}
		//std::cout<<"Update... valid="<<m_bValidLastUpdate<<std::endl;
		window->GetRenderer()->UpdateScreen();
	}
#endif
}


VisModel3D* BindingVisApp::GetVisModel(DatabaseVisWindow * window, const Interactor * actor, const InteractorPose* pose)
{
	assert(pose!=NULL);
	std::map<const InteractorPose*, VisModel3D*>::iterator iter = m_Pose2VisModel.find(pose);
	if (iter==m_Pose2VisModel.end())
	{
		int flags = VisModel3D::DRAW_FACES|VisModel3D::DRAW_PDOTS|VisModel3D::DRAW_MESHES|VisModel3D::DRAW_POINTS;
		VisModel3D * newvis = new VisModel3D((Model3D*)(actor->GetModel(pose)), flags);
		newvis->AddRenderableGeometry("Mesh");
		newvis->AddRenderableGeometry("Pnts");
		int newCam = newvis->CreateNewCamera();
		assert(newCam==0);
		//newvis->Translate(newCam, 0, 0, 0, -.1 * window->Width(), window);
		newvis->Scale(newCam, -.8, window);
		m_Pose2VisModel[pose] = newvis;
		return newvis;
	}
	else
		return iter->second; 
}


void BindingVisApp::SetScreenshotType(const VKString & sName, int value)
{
	int visFlags = 0;
	if (sName=="img_poses")
	{
		visFlags = VIS_ActMesh | VIS_ObjMesh | VIS_MeshSurface | VIS_ScreenshotMode;
		m_iCurrBinding = value;
	}
	else if (sName=="img_candidates")
	{
		visFlags = VIS_ObjMesh | VIS_MeshSurface | VIS_ScreenshotMode | VIS_CandidatePnts;
		m_iCurrBinding = value;
	}
	else if (sName!="Default")
		std::cout<<"[WARNING] CreateGTApp: unknown screenshot type: "<<sName.c_str()<<std::endl;
	
	SetRenderingFlags(visFlags);
}







