#include "VKString.h"
#include "Sortable.h"
#include "FileManagement.h"
#include "DBApp.h"

#include "RendererGLUT.h"
#include "DatabaseVisWindow.h"
#include "ModelDatabase.h"
#include "ShapeTriangleSoup.h"

#include "Interactor.h"
#include "VisInteractor.h"
#include "BindingManager.h"

class BindingManager;
class AnalysisWriter;

#ifndef __BindingVisApp_H
#define __BindingVisApp_H

class BindingVisApp : public DBApp
{
	public:
		enum VisFlags
		{
			VIS_ActMesh 	      = 1<<0,
			VIS_ObjMesh  		  = 1<<1,
			VIS_ActVolume		  = 1<<2,
			VIS_ObjVolume		  = 1<<3,
			VIS_ActEnergy		  = 1<<4,
			VIS_ActEnergyText	  = 1<<5,
			VIS_MeshDetails		  = 1<<6,
			VIS_MeshSurface		  = 1<<7,
			VIS_Distribution	  = 1<<8,
			VIS_Physics			  = 1<<9,
			VIS_ScreenshotMode    = 1<<10,
			VIS_CandidatePnts	  = 1<<11,
			VIS_CandidatePntsText = 1<<12,
			VIS_IsectGrid		  = 1<<13,
			VIS_PrecompPoseGrids  = 1<<14,
			VIS_HighProbContacts  = 1<<15,
			VIS_SymmPlanes		  = 1<<16,
			VIS_CoMPrediction	  = 1<<17,
		};
	
		enum InteractionMode
		{
			INTER_None		= 0,
			INTER_RotX		= 1,
			INTER_RotY		= 2,
			INTER_RotZ		= 3,
			INTER_RotA		= 4,
			INTER_TransX	= 5,
			INTER_TransY	= 6,
			INTER_TransZ	= 7,
			INTER_AddAR		= 8,	// add active region (type name)
			INTER_SelectAR	= 9		// select active region
		};
	
		BindingVisApp(BindingManager * binding, AnalysisWriter * writer);
		virtual void SetIK(IKSolver * solver);
		
		virtual void StartDemo();
		
		virtual bool KeyPressed(DatabaseVisWindow * window, const char key, int mouseX, int mouseY);
		virtual bool MouseButtonEvent(DatabaseVisWindow * window, int x, int y,
									  VisWindow::MouseButton mouseButtonID, VisWindow::MouseButtonState newState);
		virtual bool MouseMotionEvent(DatabaseVisWindow * window, int x, int y, int dx, int dy);
		
		virtual bool Draw(DatabaseVisWindow * window);
		virtual void FillTransforms(DatabaseVisWindow * window, const R3Affine & cam, R3Affine * outxforms) const;
		virtual void DrawPhysics(DatabaseVisWindow * window, const R3Affine * xforms) const;
		virtual void DrawPoses(DatabaseVisWindow * window, const R3Affine * xforms, std::vector<RenderableFace> & outFaces,
							   VKStringList & texts, std::vector<R3Point> & points) const;
		virtual void DrawMesh(DatabaseVisWindow * window, const R3Affine * xforms,
							  std::vector<RenderableFace> & outFaces, Interactor::InteractorID id) const;
		virtual void DrawSkeleton(DatabaseVisWindow * window, const R3Affine * xforms, const Interactor * actor,
								  const PntCorrsPose * corrs, VKStringList & texts, std::vector<R3Point> & points) const;
		virtual void DrawGround(DatabaseVisWindow * window, const R3Affine * xforms) const;
		virtual void DrawCandidatePnts(DatabaseVisWindow * window, const R3Affine * xforms,
									   VKStringList & texts, std::vector<R3Point> & points) const;
		virtual void DrawDistribution(DatabaseVisWindow * window, const R3Affine * cam) const;
		virtual void DrawEnergy(DatabaseVisWindow * window, const R3Affine * xforms) const;
		virtual void DrawActiveRegions(DatabaseVisWindow * window, const R3Affine * xforms) const;
		virtual void DrawIsectGrid(DatabaseVisWindow * window, const R3Affine & cam,
								   std::vector<RenderableFace> & outFaces) const;
		virtual void DrawPosesGrid(DatabaseVisWindow * window, const R3Affine & cam, std::vector<RenderableFace> & outFaces,
								   const PntCorrsPose * corrs) const;
		virtual void DrawSymmPlanes(DatabaseVisWindow * window, const R3Affine & cam, std::vector<RenderableFace> & outFaces);
		
	
		//virtual void DrawAR(const ActiveRegion * ar, int poseid, const R3Affine & xform, DatabaseVisWindow * window) const;
		virtual void UpdatePhysics(DatabaseVisWindow * window);
		virtual VisModel3D* GetVisModel(DatabaseVisWindow * window, const Interactor * actor, const InteractorPose * pose);
		virtual void SetScreenshotType(const VKString & sName, int value);	
	
	protected:
		AnalysisWriter * m_pAnlaysisWriter;
		BindingManager * m_pBinding;
		std::map<const InteractorPose*, VisModel3D*> m_Pose2VisModel;
		int m_iCurrBinding;
		
		InteractionMode m_Mode;
		Interactor::InteractorID m_ARActorID;
		std::vector<VisInteractor*> m_VisInteractors;
		ActiveRegion * m_pAR;
		double m_BrushSize;
		int m_iCurrActorAR[2];
		bool m_bValidLastUpdate;
		timeval m_LastUpdate;
		bool m_bAnimation;
		int m_iDemoID;
		double m_fPointScale;
		int m_iSelectedPntID;
		double m_fPntPercentile;

		int m_iReferencePntID;
		EnergyVisType m_EnergyVis;
	
		IKSolver * m_pIK;
	
};

#endif
