#include "RendererGLUT.h"
#include "RendererOSMesa.h"
#include "DatabaseVisWindow.h"
#include "ModelDatabase.h"
#include "ShapeDistance.h"
#include "VKString.h"
#include "TimeProfiler.h"
#include "FileManagement.h"

#include "DBApp.h"
#include "AppScreenshots.h"

#include "BindingManager.h"
#include "Binding.h"
#include "Interactor.h"
#include "BindingVisApp.h"
#include "AnalysisWriter.h"

#include "PosePrior.h"
#include "PosePriorGauss.h"
#include "PosePriorMixGauss.h"

#include "IKSolver.h"
#include "PntCorrsPose.h"


Interactor * GetInteractor(const char * word, Interactor * actor, Interactor * object)
{
	if (strcmp(word, "actor")==0)
		return actor;
	else if (strcmp(word, "object")==0)
		return object;
	assert(false);
	return NULL;
}

double GetUpRotationFromOrigXForm(const VKString & origxform);

int main(int argc, char** argv )
{
	if (argc<2)
	{
		std::cout<<"Usage: ./"<<argv[0]<<" [params]"<<std::endl;
		std::cout<<"Params:"<<std::endl;
		std::cout<<"    -w/r type filename - write/read file of 'type' to 'filename'. Types:\n";
		std::cout<<"         bindings - bindings file\n";
		std::cout<<"         actor - actor mesh\n";
		std::cout<<"         object - object mesh\n";
		std::cout<<"         actor_meshes - actor poses in a directory\n";
		std::cout<<"         actor_doll - actor is a ragdoll (no filename)\n";
		std::cout<<"         actor_pnts - actor is just a set of points (no filename)\n";
//		std::cout<<"         actor_analyzed - actor pre-analyzed dir\n";
//		std::cout<<"         object_analyzed - object pre-analyzed dir\n";
		std::cout<<"         actor_poseprior - actor pose priors\n";
		std::cout<<"         object_points - object points with sampled values\n";
		std::cout<<"         object_predictions - object predicted positions\n";
		std::cout<<"         gtpose - read ground truth binding\n";
		std::cout<<"         regionNames - read region names\n";
		std::cout<<"         skeleton - read skeleton\n";
		std::cout<<"         screenshot - current scene\n";
		std::cout<<"         energy - energy file\n";
		std::cout<<"         regionStats - region stats\n";
		std::cout<<"         timing - timing info\n";
		std::cout<<"    -projectgt  - project ground truth to valid points\n";
		std::cout<<"    -bind  - simple (note: has to go before all arguments)\n";
		std::cout<<"    -bindnocompute  - do not execute algorithm\n";
//		std::cout<<"    -train_poseprior type - train pose prior\n";
		std::cout<<"    -reshuffle actor/object x y z - reshuffle axis (+/-) \n";
		std::cout<<"    -rescale actor/object factor - rescale by a factor \n";
		std::cout<<"    -img_poses output directory \n";
		std::cout<<"    -img_candidates output directory \n";
		std::cout<<"    -origrotate filename \n";
		std::cout<<"    -vis\n";
		std::cout<<"Algorithm params:"<<std::endl;
		std::cout<<"    -param paramtype p1 p2 ... pN - algorithm params\n";
		std::cout<<"    	wUnassigned value - unassinged points weight\n";
		std::cout<<"    	wEndEffectors value - end effector weight\n";
		std::cout<<"    	wFeatures value - features weight\n";
		std::cout<<"    	wPosePrior value - pose prior weight\n";
		std::cout<<"    	wSymmetryPrior value - symm prior weight\n";
		std::cout<<"    	apose poseID - set actor's pose\n";
		std::cout<<"    	nsamp N - number of transformations to sample\n";
		std::cout<<"    	mapAR actorAR objectAR - energy only measured in a region\n";
		std::cout<<"    	energy flag1 flag2 flag3 ... - energy flags\n";
		std::cout<<"    	opt flag1 flag2 flag3 ... - optimization flags\n";
		std::cout<<"    	gprior maxPenalty maxAngle -- gauss prior\n";
		exit(0);
	}
	
	bool demo = false;
	int demoW, demoH, gx, gy;
	
//	double wPosePrior = 0.0005;
//	double wEndEffectors = 1.;
//	double wFeatures = 0.01;
//	double wUnassigned = 0.01;
//	int nsamples = 1000;
	VKString origrotate = "none";
	
	EnergyParams eparams;
	BindingAlgParams algParams;
	
	int visflags = 0;
	//int optimize = OPT_RaTrans | OPT_RaRot;
	
	Interactor * actor = NULL;
	Interactor * object = NULL;
	BindingManager * bindings = NULL;
	AnalysisWriter * writer = new AnalysisWriter();
	
	VKString screenshot = "none";
	bool vis = false;
//	VKString train_poseprior = "none";
	VKString regionNamesFile = "none";
	
	IKSolver * skeleton = NULL;
	
	VKString img_poses = "none";
	VKString img_candidates = "none";
	bool computeBindings = false;
	VKStringList priorParams;
	VKString readSkeleton = "none";
	bool projectgt = false;
	PosePrior * prior = NULL;
	
	VKString reflplanes = "none";
	double probGround = 1.;
	// read parameters
	for (int i=1; i<argc; i++)
	{
		if (strcmp(argv[i], "-vis")==0)
			vis = true;
		else if (strcmp(argv[i], "-randseed")==0)
			srand(atoi(argv[++i]));
		else if (strcmp(argv[i], "-w")==0)
		{
			if (strcmp(argv[i+1], "img_poses")==0)
				img_poses = argv[i+2];
			else if (strcmp(argv[i+1], "img_candidates")==0)
				img_candidates = argv[i+2];
			else if (strcmp(argv[i+1], "screenshot")==0)
				screenshot = VKString(argv[i+2]);
			else
				writer->AddWriteMap(argv[i+1], argv[i+2]);
			i+=2;
		}
		else if (strcmp(argv[i], "-r")==0)
		{
			if (strcmp(argv[i+1], "bindings")==0)
				bindings = BindingManager::ReadBindings(argv[i+2], actor, object);
			else if (strcmp(argv[i+1], "regionNames")==0)
				regionNamesFile = argv[i+2];
			else if (strcmp(argv[i+1], "skeleton")==0)
				readSkeleton = argv[i+2];
			else if (strcmp(argv[i+1], "actor")==0)
				actor = new Interactor(new ShapeTriangleSoup(argv[i+2]), Interactor::ACT_Actor);
			else if (strcmp(argv[i+1], "object")==0)
				object = new Interactor(new ShapeTriangleSoup(argv[i+2]), Interactor::ACT_Object);
			else if (strcmp(argv[i+1], "actor_meshes")==0)
				actor = new Interactor(VKString(argv[i+2]), Interactor::ACT_Actor);
			else if (strcmp(argv[i+1], "actor_doll")==0)
			{	actor = new Interactor(Interactor::ACT_Actor, Interactor::TYPE_RagDoll);  i--; }
			else if (strcmp(argv[i+1], "actor_pnts")==0)
			{	actor = new Interactor(Interactor::ACT_Actor, Interactor::TYPE_PntCorrs);  i--; }
			else if (strcmp(argv[i+1], "object_points")==0)
			{ 	assert(object!=NULL);	object->ReadObjectPoints(argv[i+2]);}
			else if (strcmp(argv[i+1], "object_densepoints")==0)
			{ 	assert(object!=NULL);	object->ReadObjectDensePoints(argv[i+2]);}
			else if (strcmp(argv[i+1], "object_predictions")==0)
			{ 	assert(object!=NULL);	object->ReadObjectPredictions(argv[i+2]);}
			else if (strcmp(argv[i+1], "object_xform")==0)
			{ 	assert(object!=NULL);	object->ReadPosesTransforms(argv[i+2]); }
			else if (strcmp(argv[i+1], "actor_poseprior")==0)
			{ 	prior = PosePrior::ReadPrior(argv[i+2]);
				prior->ReadPointStats(VKString(argv[i+2])+".pntstats.txt");
			}
			else if (strcmp(argv[i+1], "regionStats")==0)
			{ 	eparams.m_sRegionStats = argv[i+2];}
			else if (strcmp(argv[i+1], "actor_gtpose")==0)
				algParams.m_sReadPoseFile.push_back(argv[i+2]);
			else
				assert(false);
			i+=2;
		}
//		else if (strcmp(argv[i], "-train_poseprior")==0)
//			train_poseprior = argv[++i];
//		else if (strcmp(argv[i], "-reshuffle")==0)
//		{
//			GetInteractor(argv[i+1], actor, object)->Reshuffle(&(argv[i+2]));
//			i+=4;
//		}
//		else if (strcmp(argv[i], "-rescale")==0)
//		{
//			GetInteractor(argv[i+1], actor, object)->Rescale(atof(argv[i+2]));
//			i+=2;
//		}
		else if (strcmp(argv[i], "-origrotate")==0)
			origrotate = argv[++i];
		else if (strcmp(argv[i], "-bindnocompute")==0)
			bindings = new BindingManager(actor, object);
		else if (strcmp(argv[i], "-refl_planes")==0)
			reflplanes = argv[++i];
		else if (strcmp(argv[i], "-bind")==0)
		{
			//assert(bindings==NULL && actor!=NULL && object!=NULL);
			bindings = new BindingManager(actor, object);
			computeBindings = true;
		}
		else if (strcmp(argv[i], "-param")==0)
		{
			assert(bindings!=NULL);
			if (strcmp(argv[i+1], "apose")==0)
			{	algParams.m_iOptPoseID = atoi(argv[i+2]);   i+=2;  }
			else if (strcmp(argv[i+1], "nsamp")==0)
			{	 algParams.m_iNSamples = atoi(argv[i+2]);	i+=2;  }
			else if (strcmp(argv[i+1], "candidateValueCutoff")==0)
			{	 algParams.m_fCandidateValueCutoff = atof(argv[i+2]);	i+=2;	}
			else if (strcmp(argv[i+1], "candidateDistThreshold")==0)
			{	 algParams.m_fCandidateDistThresh = atof(argv[i+2]);	i+=2;	}
			else if (strcmp(argv[i+1], "nbest")==0)
			{	 algParams.m_iNBest = atoi(argv[i+2]);	i+=2;	}
			else if (strcmp(argv[i+1], "ninit")==0)
			{	 algParams.m_iInitPopulation = atoi(argv[i+2]);	i+=2;	}
			else if (strcmp(argv[i+1], "wPosePrior")==0)
			{	 eparams.m_wPosePrior = atof(argv[i+2]);	i+=2;	}
			else if (strcmp(argv[i+1], "wEndEffectors")==0)
			{	 eparams.m_wEndEffectorDist = atof(argv[i+2]);	i+=2;	}
			else if (strcmp(argv[i+1], "wSymmetryPrior")==0)
			{	 eparams.m_wSymmetryPrior = atof(argv[i+2]);	i+=2;	}
			else if (strcmp(argv[i+1], "wFeatures")==0)
			{	 eparams.m_wFeatures = atof(argv[i+2]);	i+=2;	}
			else if (strcmp(argv[i+1], "wUnassigned")==0)
			{	 eparams.m_wUnassigned = atof(argv[i+2]);	i+=2;	}
			else if (strcmp(argv[i+1], "wGround")==0)
			{	 eparams.m_wGround = atof(argv[i+2]);	i+=2;	}
			else if (strcmp(argv[i+1], "wContactPairwise")==0)
			{	 eparams.m_wContactPairwise = atof(argv[i+2]);	i+=2;	}
			else if (strcmp(argv[i+1], "wSymmPlane")==0)
			{	 eparams.m_wSymmPlane = atof(argv[i+2]);	i+=2;	}
			else if (strcmp(argv[i+1], "wCoMPenalty")==0)
			{	 eparams.m_wCoMPenalty = atof(argv[i+2]);	i+=2;	}
			else if (strcmp(argv[i+1], "wIsectPenalty")==0)
			{	 eparams.m_wIsectPenalty = atof(argv[i+2]);	i+=2;	}
			else if (strcmp(argv[i+1], "minGroundProb")==0)
			{	 eparams.m_fMaxGroundPenalty = -log(atof(argv[i+2]));	i+=2;	}
			else if (strcmp(argv[i+1], "minContactProb")==0)
			{	 eparams.m_fMaxContactPenalty = -log(atof(argv[i+2]));	i+=2;	}
			else if (strcmp(argv[i+1], "energy")==0)	// energy flags
			{
				int fid = 2;
				eparams.ClearEnergyFlags();
				while (i + fid < argc)
				{
					EnergyParams::EnergyFlags flag = EnergyParams::Str2EnergyFlag(argv[i+fid]);
					if (flag != EnergyParams::E_Invalid)
					{	eparams.AddEnergyFlag(flag);	fid++;	}
					else {	fid--; 	break; }
				}
				i+=fid;
			}
			else if (strcmp(argv[i+1], "algorithm")==0)		// optimization flags
			{
				//std::cout<<"Algorithm flags: "<<std::endl;
				int fid = 2;
				algParams.ClearOptimizationFlags();
				while (i + fid < argc)
				{
					//std::cout<<" "<<argv[i+fid]<<std::endl;
					BindingAlgParams::OptFlags flag = BindingAlgParams::Str2OptFlag(argv[i+fid]);
					if (flag != BindingAlgParams::OPT_Invalid)
					{	algParams.AddOptimizationFlag(flag);	fid++;	}
					else {	fid--; 	break; }
				}
				i+=fid;
			}
			else if (strcmp(argv[i+1], "projectgtwithIK")==0)
			{	 algParams.m_iExternalFlags |= BindingAlgParams::EXT_SolveIK;  assert(strcmp(argv[i+2], "1")==0); i+=2;	}
			else if (strcmp(argv[i+1], "projectgt2candidates")==0)
			{	 algParams.m_iExternalFlags |= BindingAlgParams::EXT_Project2Corrs;  assert(strcmp(argv[i+2], "1")==0); i+=2; }
			else if (strcmp(argv[i+1], "keepgtinput")==0)
			{	 algParams.m_iExternalFlags |= BindingAlgParams::EXT_Input;  assert(strcmp(argv[i+2], "1")==0); i+=2; }
			else if (strcmp(argv[i+1], "disableIK")==0)
			{	 algParams.m_bDisableIK = true;  assert(strcmp(argv[i+2], "1")==0); i+=2;		}
			else if (strcmp(argv[i+1], "coarse2fine")==0)
			{	 algParams.m_iCoarse2Fine = atoi(argv[i+2]); i+=2;		}
			else if (strcmp(argv[i+1], "nmaxpermutations")==0)
			{	 algParams.m_iNMaxPermutations = atoll(argv[i+2]);	i+=2;	}
			else if (strcmp(argv[i+1], "pntradiusstep")==0)
			{	 algParams.m_fPntRadiusStep = atof(argv[i+2]);	i+=2;	}
			else
			{
				std::cout<<"[ERROR] Unknown argument: "<<argv[i+1]<<std::endl;
				assert(false);
			}
		}
		else if (strcmp(argv[i], "-demo")==0)
		{
			demo = true;
			demoW = atoi(argv[++i]);
			demoH = atoi(argv[++i]);
			gx = atoi(argv[++i]);
			gy = atoi(argv[++i]);
		}
		else
		{
			std::cout<<"[FnAlign.cpp] Invalid Argument: "<<argv[i]<<std::endl;
			assert(false);
		}
	}
		
	if (regionNamesFile!="none")
	{
		assert(bindings!=NULL);
		bindings->ReadRegionNames(regionNamesFile);
	}
	
	if (readSkeleton!="none")
	{
		skeleton = new IKSolver(eparams.m_wEndEffectorDist, eparams.m_wPosePrior, eparams.m_wSymmetryPrior,  prior);
		skeleton->ReadSkeleton(readSkeleton, regionNamesFile);
	}
			
	
	if (skeleton!=NULL && prior!=NULL && prior->IsAngles())
		((PriorAngles*)prior)->SetIK(skeleton);
	
	if (bindings!=NULL)
	{
		//assert(origrotate!="none");
		bindings->GetEnergy()->SetPosePrior(skeleton, prior);
		//bindings->GetEnergy()->SetParams(eparams, GetUpRotationFromOrigXForm(origrotate));
		bindings->GetEnergy()->SetParams(eparams, 0);
		bindings->GetEnergy()->ReadPlanes(reflplanes);
	}
	
	assert(priorParams.count()==0);
//	if (priorParams.count()>0)
//	{
//		bool ok;
//		assert(prior!=NULL);
//		if (priorParams[0]=="gprior")
//		{
//			assert(prior->PriorType()==PosePrior::PRIOR_GaussAngles && priorParams.count()==3);
//			double clampVal = priorParams[1].toDouble(&ok);		assert(ok);
//			double maxAngle = priorParams[2].toDouble(&ok);		assert(ok);
//			double variance = -1;
//			if (maxAngle>=0)
//				variance = pow(maxAngle, 2.) / (2. * clampVal);
//			((PosePriorGauss*)prior)->SetParams(clampVal, variance);
//		}
//		else
//			std::cout<<"[WARNING] Unknown parameter for shape prior: "<<priorParams[0].c_str()<<std::endl;
//	}
	
	if (computeBindings && bindings!=NULL)
		bindings->ComputeBindings(algParams);

	// write bindings
	if (bindings->NBindings()>1000)
		std::cout<<"[ERROR] Too many poses, skipping write..."<<std::endl;
	else
	{
		writer->SetBindingManager(bindings);
		writer->SetInteractor(Interactor::ACT_Actor, actor);
		writer->SetInteractor(Interactor::ACT_Object, object);
		writer->Write();
	}
	
#ifdef VISUALIZATION
	if (vis || screenshot!="none" || img_candidates!="none" || img_poses!="none")
	{
		int listFlags = DatabaseVisWindow::LISTGP_Models | DatabaseVisWindow::LISTGP_Geoms;
		listFlags |= DatabaseVisWindow::LISTGP_MeshFlags;
		ModelDatabase * db = new ModelDatabase();

		int w=-1, h=-1;
		if (screenshot!="none" || img_candidates != "none" || img_poses != "none")
		{  w=1600; h=1200; }
		if (demo)
		{
			w = demoW;
			h = demoH;
		}

		DatabaseVisWindow * dbVis = new DatabaseVisWindow(db, NULL, listFlags, 1, 1, 0, 0, 1., w, h);
		BindingVisApp * bindingApp = new BindingVisApp(bindings, writer);
		bindingApp->SetIK(skeleton);
		if (demo)
			bindingApp->StartDemo();
		//bindingApp->SetRenderingFlags(flags);
		
		//std::cout<<"shot,cand,poses: "<<screenshot.c_str()<<", "<<img_candidates.c_str()<<", "<<img_poses.c_str()<<std::endl;
		if (screenshot=="none" && img_candidates == "none" && img_poses == "none")
			dbVis->SetApp(bindingApp);
		else
		{ 
			if (visflags==0)
				visflags = bindingApp->GetRenderingFlags();
			AppScreenshots * appScreen = new AppScreenshots(true);
			if (screenshot!="none")
				appScreen->TakeShotOfApp(bindingApp, "Default", visflags, screenshot);
			if (img_poses!="none")
			{
				FileManagement::CreateDirsOnPathToFile((img_poses+"/energies.txt").c_str());
				std::ofstream textStreamEnergies((img_poses+"/energies.txt").c_str());
				assert(textStreamEnergies.is_open());
				for (int i=0; i<bindings->NBindings(); i++)
				{
					appScreen->TakeShotOfApp(bindingApp, "img_poses", i, img_poses+"pose"+VKString::number(i)+".jpg");
					textStreamEnergies<<i<<": "<<bindings->GetBinding(i)->GetEnergy()<<"\n";
				}
				textStreamEnergies.close();
			}
			if (img_candidates!="none")
			{
				
				for (int i=0; i<(int)bindings->m_CandidateCorrs.size(); i++)
				{
					if (i == bindings->NBindings())
						appScreen->TakeShotOfApp(bindingApp, "img_candidates", i, img_candidates+".jpg");
					else
						appScreen->TakeShotOfApp(bindingApp, "img_candidates", i, img_candidates+VKString::number(i)+".jpg");
				}
			}
			dbVis->SetApp(appScreen);
		}
		
#	ifdef MESA_RENDERING
		VisWindowRenderer * wndMesa = new RendererOSMesa(dbVis);
		wndMesa->UpdateScreen();
#	else
		VisWindowRenderer * wndGLUT = NULL;
		if (demo)
			wndGLUT = new RendererGLUT(argc, argv, dbVis, gx*(w+25), gy*(h+75));
		else
			wndGLUT = new RendererGLUT(argc, argv, dbVis);
#  endif
	}
#else
	std::cout<<"[WARNING] Visualization disabled (compiling flag)"<<std::endl;
#endif
}

double GetUpRotationFromOrigXForm(const VKString & origxform)
{
	return 0;
//	double rot = -1;
//	VKString tempStr;
//	std::ifstream textStream(origxform.c_str());
//	assert(textStream.is_open());
//	tempStr.readLine(textStream);
//	tempStr.readToken(textStream);	assert(tempStr=="RotZ");
//	textStream>>rot;
//	textStream.close();
//	return rot;
}






