#include "RendererGLUT.h"
#include "RendererOSMesa.h"
#include "DatabaseVisWindow.h"
#include "ModelDatabase.h"
#include "ShapeDistance.h"
#include "VKString.h"
#include "TimeProfiler.h"
#include "FileManagement.h"

#include "DBApp.h"
#include "AppScreenshots.h"

#include "BindingManager.h"
#include "Binding.h"
#include "Interactor.h"

#include "PosePrior.h"
#include "PosePriorGauss.h"
#include "PosePriorMixGauss.h"

#include "IKSolver.h"
#include "PntCorrsPose.h"

Interactor * GetInteractor(const char * word, Interactor * actor, Interactor * object)
{
	if (strcmp(word, "actor")==0)
		return actor;
	else if (strcmp(word, "object")==0)
		return object;
	assert(false);
	return NULL;
}

int main(int argc, char** argv )
{
	if (argc<2)
	{
		std::cout<<"Usage: ./"<<argv[0]<<" [params]"<<std::endl;
		std::cout<<"Params:"<<std::endl;
		std::cout<<"   -from deformedMesh origMesh gtfile - transfer corrs from mesh"<<std::endl;
		std::cout<<"   -to  deformedMesh origMesh outGTFile  - transfer corrs to mesh"<<std::endl;
		std::cout<<"   -transfer - transfer contact points"<<std::endl;
		std::cout<<"   -distance - compute distance "<<std::endl;
		std::cout<<"   -fuzzycorrs fromname toname modelsfile corrfile"<<std::endl;
		std::cout<<"   -vis  - visualize"<<std::endl;
		exit(0);
	}
	
	bool vis = false;
	// Read all models with ground truth
	VKString fromDeformedMesh="none";
	VKString fromOrigMesh="none";
	VKString fromGTFile="none";

	VKString toDeformedMesh="none";
	VKString toOrigMesh="none";
	VKString toOutFile="none";
	
	VKString fuzzyFromName = -1;
	VKString fuzzyToName = -1;
	VKString fuzzyModelsFile = "none";
	VKString fuzzyCorrsFile = "none";

	bool distance = false;
	bool transfer = false;
	
	for (int i=1; i<argc; i++)
	{
		if (strcmp(argv[i], "-from")==0)
		{
			fromDeformedMesh = argv[i+1];
			fromOrigMesh = argv[i+2];
			fromGTFile = argv[i+3];
			i+=3;
		}
		else if (strcmp(argv[i], "-to")==0)
		{
			toDeformedMesh = argv[i+1];
			toOrigMesh = argv[i+2];
			toOutFile = argv[i+3];
			i+=3;
		}
		else if (strcmp(argv[i], "-fuzzycorrs")==0)
		{
			fuzzyFromName = argv[i+1];
			fuzzyToName = argv[i+2];
			fuzzyModelsFile = argv[i+3];
			fuzzyCorrsFile = argv[i+4];
			i+=4;
		}
		else if (strcmp(argv[i], "-transfer")==0)
			transfer = true;
		else if (strcmp(argv[i], "-distance")==0)
			distance = true;
		else if (strcmp(argv[i], "-vis")==0)
			vis = true;
		else
		{
			std::cout<<"[ERROR] Unknown argument (FNAlignWithCorrs): "<<argv[i]<<std::endl;
			assert(false);
		}
	}
	
	Model3D * fromM = new Model3D("from", 0);
	Model3D * toM = new Model3D("to", 1);
	
	ShapeTriangleSoup *fromDeformed=NULL, *fromOriginal=NULL, *toDeformed=NULL, *toOriginal=NULL;
	PntCorrsPose * fromPose = NULL;
	if (fromDeformedMesh!="none")
		fromDeformed = new ShapeTriangleSoup(fromDeformedMesh, "deformed", fromM);
	if (fromOrigMesh!="none")
		fromOriginal = new ShapeTriangleSoup(fromOrigMesh, "mesh", fromM);
	if (fromGTFile!="none")
	{
		std::vector<PntCorrsPose*> poses;
		PntCorrsPose::CreatePosesFromGTFile(fromGTFile, fromOriginal, "none", poses);
		assert(poses.size()>0);
		fromPose = poses[0];
	}
	
	if (toDeformedMesh!="none")
		toDeformed = new ShapeTriangleSoup(toDeformedMesh, "deformed", toM);
	if (toOrigMesh!="none")
		toOriginal = new ShapeTriangleSoup(toOrigMesh, "mesh", toM);
	
	PntCorrsPose * toPose = new PntCorrsPose();

	R3Mesh *fromR3Mesh = NULL, *fromOrigR3Mesh=NULL, *toR3Mesh = NULL;
	std::vector<int> fromTriangleMap, fromVertexMap, toTriangleMap, toVertexMap;
	std::vector<int> fromOrigTriangleMap, fromOrigVertexMap;
//	R3MeshSearchTree *fromTree=NULL, *toTree=NULL;
//	R3MeshSearchTree *fromOrigTree=NULL;
//	if (fromDeformed!=NULL)
//		fromTree = fromDeformed->CreateSearchTree(&fromR3Mesh, fromTriangleMap, fromVertexMap);
//	if (toDeformed!=NULL)
//		toTree = toDeformed->CreateSearchTree(&toR3Mesh, toTriangleMap, toVertexMap);
//	if (fromOriginal!=NULL)
//		fromOrigTree = fromOriginal->CreateSearchTree(&fromOrigR3Mesh, fromOrigTriangleMap, fromOrigVertexMap);
	
	fromM->AddGeomShape("mesh", fromOriginal);
	fromM->AddGeomShape("deformed", fromDeformed);
	toM->AddGeomShape("mesh", toOriginal);
	toM->AddGeomShape("deformed", toDeformed);
	
	int nfromorig = 100000 * fromDeformed->Area()/16;
//	int nfromdeform = 100000 * fromOriginal->Area()/16;
	int ntoorig = 100000 * toDeformed->Area()/16;
//	int ntodeform = 100000 * toOriginal->Area()/16;
	ShapePointCloud * fromOriginalRandom = ShapePointCloud::CreateRandom("original_random", "mesh", fromM, nfromorig);
//	ShapePointCloud * fromDeformedRandom = ShapePointCloud::CreateRandom("deformed_random", "deformed", fromM, nfromdeform);
	ShapePointCloud * fromDeformedRandom = new ShapePointCloud(fromDeformed, "deformed_random");
	
	ShapePointCloud * toOriginalRandom = ShapePointCloud::CreateRandom("original_random", "mesh", toM, ntoorig);
//	ShapePointCloud * toDeformedRandom = ShapePointCloud::CreateRandom("deformed_random", "deformed", toM, ntodeform);
	ShapePointCloud * toDeformedRandom = new ShapePointCloud(toDeformed, "deformed_random");
	
	fromM->AddGeomShape("original_random", fromOriginalRandom);
	fromM->AddGeomShape("deformed_random", fromDeformedRandom);
	toM->AddGeomShape("original_random", toOriginalRandom);
	toM->AddGeomShape("deformed_random", toDeformedRandom);

	bool identicalFaces = false;
	std::vector<std::map<VKString, int> > fromDeformed_vtx2faceMap;
	std::vector<std::map<VKString, int> > toDeformed_vtx2faceMap;
	
	std::vector<int> subsetFrom;
	std::vector<int> subsetTo;
	
	std::vector<int> fromPntsMap_Orig2Deform;
	std::vector<int> toPntsMap_Orig2Deform;
	std::vector<int> toPntsMap_Deform2Orig;
	
	if (!identicalFaces)	// map via vertices
	{
		assert(fromOriginal->NVertices()==fromDeformed->NVertices());
		assert(toOriginal->NVertices()==toDeformed->NVertices());
		for (int v=0; v<fromDeformed->NVertices(); v++)
			fromDeformed_vtx2faceMap.push_back(std::map<VKString, int>());
			
		for (int f=0; f<fromDeformed->NFaces(); f++)
		{
			int v0 = fromDeformed->VertexOnFace(f, 0);
			int v1 = fromDeformed->VertexOnFace(f, 1);
			int v2 = fromDeformed->VertexOnFace(f, 2);
			VKString vstr = VKString::number(v1)+"_"+VKString::number(v2);
			fromDeformed_vtx2faceMap[v0][vstr] = f;
		}
		
		for (int v=0; v<toDeformed->NVertices(); v++)
			toDeformed_vtx2faceMap.push_back(std::map<VKString, int>());
		
		for (int f=0; f<toDeformed->NFaces(); f++)
		{
			int v0 = toDeformed->VertexOnFace(f, 0);
			int v1 = toDeformed->VertexOnFace(f, 1);
			int v2 = toDeformed->VertexOnFace(f, 2);
			VKString vstr = VKString::number(v1)+"_"+VKString::number(v2);
			toDeformed_vtx2faceMap[v0][vstr] = f;
		}
	}
	
	for (int i=0; i<fromOriginalRandom->NumPoints(); i++)
	{
		int fid; double b1, b2; const ShapeMesh*tempmesh;
		fromOriginalRandom->GetPoint(i).GetBarycentricCoords(&fid, &b1, &b2, &tempmesh);
		if (identicalFaces)
		{
			assert(fid < fromDeformed->NFaces());
			fromDeformedRandom->AddPoint(ShapePoint(fromDeformed, fid, b1, b2, fromDeformedRandom, fromDeformedRandom->NumPoints()));
		}
		else
		{
			int v0 = fromOriginal->VertexOnFace(fid, 0);
			int v1 = fromOriginal->VertexOnFace(fid, 1);
			int v2 = fromOriginal->VertexOnFace(fid, 2);	// assume vertex ids are the same
			VKString vstr = VKString::number(v1)+"_"+VKString::number(v2);
			assert(v0 < (int)fromDeformed_vtx2faceMap.size());
			std::map<VKString, int>::iterator iter = fromDeformed_vtx2faceMap[v0].find(vstr);
			if (iter != fromDeformed_vtx2faceMap[v0].end())
			{
				fromPntsMap_Orig2Deform.push_back(fromDeformedRandom->NumPoints());
				assert(iter->second < fromDeformed->NFaces());
				fromDeformedRandom->AddPoint(ShapePoint(fromDeformed, iter->second, b1, b2,
														fromDeformedRandom, fromDeformedRandom->NumPoints()));
				subsetFrom.push_back(i);
			}
			else
				fromPntsMap_Orig2Deform.push_back(-1);
		}
	}

	for (int i=0; i<toOriginalRandom->NumPoints(); i++)
	{
		int fid; double b1, b2; const ShapeMesh*tempmesh;
		toOriginalRandom->GetPoint(i).GetBarycentricCoords(&fid, &b1, &b2, &tempmesh);
		if (identicalFaces)
		{
			assert(fid < toDeformed->NFaces());
			toDeformedRandom->AddPoint(ShapePoint(toDeformed, fid, b1, b2, toDeformedRandom, toDeformedRandom->NumPoints()));
		}
		else
		{
			int v0 = toOriginal->VertexOnFace(fid, 0);
			int v1 = toOriginal->VertexOnFace(fid, 1);
			int v2 = toOriginal->VertexOnFace(fid, 2);	// asssume vertex ids are the same
			VKString vstr = VKString::number(v1)+"_"+VKString::number(v2);
			assert(v0 < (int)toDeformed_vtx2faceMap.size());
			std::map<VKString, int>::iterator iter = toDeformed_vtx2faceMap[v0].find(vstr);
			if (iter != toDeformed_vtx2faceMap[v0].end())
			{
				toPntsMap_Orig2Deform.push_back(toDeformedRandom->NumPoints());
				toPntsMap_Deform2Orig.push_back(i);
				assert(iter->second < toDeformed->NFaces());
				toDeformedRandom->AddPoint(ShapePoint(toDeformed, iter->second, b1, b2,
													  toDeformedRandom, toDeformedRandom->NumPoints()));
				subsetTo.push_back(i);
			}
			else
				toPntsMap_Orig2Deform.push_back(-1);
		}
	}
		
	const R3Kdtree<const PointInSet *> * fromOriginalKdTree = NULL;
	const R3Kdtree<const PointInSet *> * toOriginalKdTree = NULL;
	if (identicalFaces)
	{
		fromOriginalKdTree = fromOriginalRandom->GetKdTree();
		toOriginalKdTree = toOriginalRandom->GetKdTree();
	}
	else
	{
		fromOriginalKdTree = fromOriginalRandom->CreateSubsetKdTree(subsetFrom);
		toOriginalKdTree = toOriginalRandom->CreateSubsetKdTree(subsetTo);

	}
	const R3Kdtree<const PointInSet *> * fromDeformedKdTree = fromDeformedRandom->GetKdTree();
	const R3Kdtree<const PointInSet *> * toDeformedKdTree = toDeformedRandom->GetKdTree();
	
	//R3MeshIntersection closest;
	if (distance)
	{
		//std::cout<<"Dist: "<<fromDeformedMesh.c_str()<<" -> "<<toDeformedMesh.c_str()<<std::endl;
		//std::cout<<"NV: "<<fromDeformed->NVertices()<<" -> "<<toDeformed->NVertices()<<std::endl;
		double dist2 = 0;
//		assert(fromDeformed!=NULL && fromTree!=NULL && toDeformed!=-NULL && toOriginal!=NULL);
//		for (int i=0; i<fromDeformed->NVertices(); i++)
//		{
//			toTree->FindClosest(fromDeformed->VertexPosition(i), closest);
//			dist2 += R3SquaredDistance(closest.point, fromDeformed->VertexPosition(i))/fromDeformed->NVertices();
//		}
//		for (int i=0; i<toDeformed->NVertices(); i++)
//		{
//			fromTree->FindClosest(toDeformed->VertexPosition(i), closest);
//			dist2 += R3SquaredDistance(closest.point, toDeformed->VertexPosition(i))/toDeformed->NVertices();
//		}
		
		for (int i=0; i<fromDeformedRandom->NumPoints(); i++)
		{
			R3Point pos = toDeformedKdTree->FindClosest(fromDeformedRandom->PointPosition(i))->GetPosition();
			dist2 += R3SquaredDistance(pos, fromDeformedRandom->PointPosition(i))/fromDeformedRandom->NumPoints();
		}

		for (int i=0; i<toDeformedRandom->NumPoints(); i++)
		{
			R3Point pos = fromDeformedKdTree->FindClosest(toDeformedRandom->PointPosition(i))->GetPosition();
			dist2 += R3SquaredDistance(pos, toDeformedRandom->PointPosition(i))/toDeformedRandom->NumPoints();
		}

		std::cout<<"Distance "<<dist2<<std::endl;
	}
	
	if (transfer)
	{
		ShapePointCloud * pntsFromDeformed = new ShapePointCloud(fromDeformed, "pnts_deformed");
		ShapePointCloud * pntsFromOriginal = new ShapePointCloud(fromOriginal, "pnts_original");
		//ShapePointCloud * pntsFromOriginalPos = new ShapePointCloud(fromOriginal, "pnts_original_pos");
		ShapePointCloud * pntsToDeformed = new ShapePointCloud(toDeformed, "pnts_deformed");
		ShapePointCloud * pntsToOriginal = new ShapePointCloud(toOriginal, "pnts_original");
		//ShapePointCloud * pntsToOriginalPos = new ShapePointCloud(toOriginal, "pnts_original_pos");
		fromM->AddGeomShape("pnts_deformed", pntsFromDeformed);
		fromM->AddGeomShape("pnts_original", pntsFromOriginal);
		//fromM->AddGeomShape("pnts_original_pos", pntsFromOriginalPos);
		toM->AddGeomShape("pnts_deformed", pntsToDeformed);
		toM->AddGeomShape("pnts_original", pntsToOriginal);
		//toM->AddGeomShape("pnts_original_pos", pntsToOriginalPos);
		
		std::cout<<"FromDeformed="<<fromDeformed->NVertices()<<" "<<fromDeformed->NFaces()<<std::endl;
		std::cout<<"FromOriginal="<<fromOriginal->NVertices()<<" "<<fromOriginal->NFaces()<<std::endl;
		std::cout<<"ToDeformed="<<toDeformed->NVertices()<<" "<<toDeformed->NFaces()<<std::endl;
		std::cout<<"ToOriginal="<<toOriginal->NVertices()<<" "<<toOriginal->NFaces()<<std::endl;
		
		for (int i=0; i<fromPose->NMatches(); i++)
		{
			int rid = fromPose->Region(i);
			TargetPrimitive prim = fromPose->Prim(i);
			
			if (prim.IsType(TargetPrimitive::PRIM_Point))
			{
				int from_fid;
				double from_bary1, from_bary2;
				const ShapeMesh * testmesh;
				
				// project original point to a random sample (for robustness, sometimes actually negative barycentric coords :( )
				int frompntid = fromOriginalKdTree->FindClosest(prim.GetPosition())->id;
				ShapePoint neworig = fromOriginalRandom->GetPoint(frompntid);
				pntsFromOriginal->AddPoint(neworig);	// DEBUG: from original
				
				// find corresponding deformed 'from' point (assume triangle ids are the same)
				ShapePoint from_point_deformed;
				if (identicalFaces)
					from_point_deformed = fromDeformedRandom->GetPoint(frompntid);
				else
				{
					assert(frompntid>=0 && frompntid < (int)fromPntsMap_Orig2Deform.size()
						   && fromPntsMap_Orig2Deform[frompntid]>=0);
					from_point_deformed = fromDeformedRandom->GetPoint(fromPntsMap_Orig2Deform[frompntid]);
				}
				pntsFromDeformed->AddPoint(from_point_deformed);  // DEBUG: from deformed
				
				// find corresponding point on 'to' mesh (assumes deformed meshes are co-aligned)
				int topntid = toDeformedKdTree->FindClosest(from_point_deformed.GetPosition())->id;
				pntsToDeformed->AddPoint(toDeformedRandom->GetPoint(topntid)); // DEBUG: to deformed
				
				// find corresponding original 'to' point (assume triangle ids are the same)
				ShapePoint to_point_original;
				if (identicalFaces)
					to_point_original = toOriginalRandom->GetPoint(topntid);
				else
				{
					assert(topntid>=0 && topntid < (int)toPntsMap_Deform2Orig.size()
						   && toPntsMap_Deform2Orig[topntid]>=0);
					to_point_original = toOriginalRandom->GetPoint(toPntsMap_Deform2Orig[topntid]);
				}

				pntsToOriginal->AddPoint(to_point_original); // DEBUG: to deformed

				// add primitive (match)
				prim = TargetPrimitive(TargetPrimitive::PRIM_Point, to_point_original);
			}
			
			toPose->AddMatch(rid, prim);
		}
		std::vector<PntCorrsPose*> poses;
		poses.push_back(toPose);
		VKStringList regionNames;
		VKStringList regionTypes;
		std::vector<int> regionNPnts;
		PntCorrsPose::WritePosesGTFile(toOutFile, poses, regionNames, regionTypes, regionNPnts);
	}

#	ifdef VISUALIZATION
	if (vis)
	{
		int listFlags = DatabaseVisWindow::LISTGP_Models | DatabaseVisWindow::LISTGP_Geoms;
		listFlags |= DatabaseVisWindow::LISTGP_MeshFlags | DatabaseVisWindow::LISTGP_GeomVals;
		int w=-1, h=-1;
		ModelDatabase * db = new ModelDatabase();
		db->AddModel(fromM);
		db->AddModel(toM);
		DatabaseVisWindow * dbVis = new DatabaseVisWindow(db, NULL, listFlags, 1, 1, 0, 0, 1., w, h);
		VisModel3D * mvis = dbVis->GetDrawableModels()[0];
		
#		ifdef MESA_RENDERING
		VisWindowRenderer * wndMesa = new RendererOSMesa(dbVis);
		wndMesa->UpdateScreen();
#		else
		VisWindowRenderer * wndGLUT = new RendererGLUT(argc, argv, dbVis);
#  	 endif
	}
#	endif
}








