#include "FileManagement.h"
#include "VKString.h"
#include "TimeProfiler.h"

#include "RendererGLUT.h"
#include "DatabaseVisWindow.h"
#include "ModelDatabase.h"
#include "ShapeDistance.h"

#include "ShapeTriangleSoup.h"
#include "DBApp.h"
#include <vector>
#include "ShapePoint.h"

#include "Interactor.h"
#include "InteractorPose.h"
#include "PosePrior.h"
#include "PosePriorGauss.h"

#ifndef __CREATE_GT_APP_H
#define __CREATE_GT_APP_H


class Interactor;

class CreateGTApp : public DBApp
{
	public:
		enum PointType
		{
			PT_GroundTruth,
			PT_Prediction 			= 1,
			PT_PredictionOK			= 2,
			PT_PredictionTooFar 	= 3,
			PT_PredictionNotExists 	= 4
		};

		enum VisFlags
		{
			VIS_Model    			= 1<<0,
			VIS_Transparent    		= 1<<1,
			VIS_Reference 			= 1<<2,
			VIS_Axes    			= 1<<3,
			VIS_PosesPnts    		= 1<<4,
			VIS_PosesHuman			= 1<<5,
			VIS_PosesPredictedPnts	= 1<<6,
			VIS_GTSkeleton			= 1<<7,
			VIS_PredictedSkeleton	= 1<<8,
			VIS_ScreenshotMode		= 1<<9,
			VIS_PosesPntNames		= 1<<10
		};
		CreateGTApp(ModelDatabase * db, const VKString & writeFile,
					const VKString & writeXForm, const VKString & writeMesh, bool readPose);
		virtual void SetSkeleton(Interactor * actor, IKSolver * solver);
		virtual void SetRegionsFile(const VKString & featFile);
		virtual bool KeyPressed(DatabaseVisWindow * window, const char key, int mouseX, int mouseY);
		virtual bool Draw(DatabaseVisWindow * window);
		virtual void DrawPosePnts(DatabaseVisWindow * window, PntCorrsPose * pose,
								  const R3Affine & xform, const R3Affine & camera);
		virtual bool DrawSkeleton(DatabaseVisWindow * window, PntCorrsPose * pose, const R3Affine & xform);
		virtual void PrintPoints() const;
		virtual void SetPredictions(const std::vector<PntCorrsPose * > & predictions);
		virtual int NPredictions() const;
		virtual bool WritePosesFile() const;
		virtual void ReadPosesFile();
		virtual void ReadPriorAsPose(const VKString & priorFile);
		//static void ReadPosesFile(const VKString & posesFile, const ShapeMesh * mesh, std::vector<PntCorrsPose * > & poses  );
		virtual void SetScreenshotType(const VKString & sName, int value);
		virtual int NPoses() const;
		virtual const PntCorrsPose * GetPose(int pid) const;
		virtual ModelDatabase * GetDB();
		virtual double GetDistanceScale() const;
	
		virtual int NRegions() const;
		virtual const VKString & RegionName(int rid) const;
		virtual void UpdatePoseScales();
	

//	protected:
		
		int m_iSelectedSkeletonLink;
		int m_iSelectedGaussianMean;
	
		Interactor * m_pActor;
		IKSolver * m_pSolver;
		VKString m_sWriteMesh;
		VKString m_sWriteXFormFile;
		VKString m_sWriteFile;
		VKStringList m_sRegionNames;
		VKStringList m_sRegionTypes;
		std::vector<int> m_RegionNPoints;

		std::vector<PntCorrsPose * > m_Poses;
		std::vector<PntCorrsPose * > m_Predictions;	// optional
		std::vector<std::vector<PointType> > m_ClassifiedPredicted;
		std::map<PntCorrsPose*, int> m_Pose2ClassifiedPnts;

		int m_iCurrPredictedPose;
		int m_iCurrPose;
		int m_iCurrRegion;
		int m_iCurrPoint;
		
		double m_fScale;
		double m_fAngle;
		double m_fGroundTranslate;
		R3Point m_OrigCentroid;
		double m_fOrigRadius;
	
		R3Affine m_CameraXForm;
	
		BufferID * m_pGroundPlaneBuffer;
		bool m_bRunIK;
	
		std::vector<PntCorrsPose *> m_SavedPoses;
};


#endif

