#include "VisClassifierApp.h"

std::vector<double> VisClassifierApp::s_Thresholds;

VisClassifierApp::VisClassifierApp(Classifier * cla)
: DBApp(cla->m_pDB)
{
	m_pCla = cla;
	m_fScaleEverything = .4;
	m_sShotType = "none";
	m_iShotValue = -1;
	m_fValueThreshold = 1E-2;
	m_fReferenceDistance = .15;
	m_iTrainedLabel = -1;
}

bool VisClassifierApp::KeyPressed(DatabaseVisWindow * window, const char key, int mouseX, int mouseY)
{
	if (key=='`')
	{
		std::cout<<"VisClassifierApp hotkeys:"<<std::endl;
		std::cout<<"    c - classify current shape"<<std::endl;
		std::cout<<"    s - show best segmentation (todo)"<<std::endl;
		std::cout<<"    '/; - visualize input to weka"<<std::endl;
		std::cout<<"    </> - distance threshold"<<std::endl;
		std::cout<<"    ,/. - value threshold"<<std::endl;
		return false;
	}
	else if (key=='c')
	{
		int mid = window->ModelID(window->SelectedWindow());
		VKString objName = m_pCla->GetDB()->GetModel(mid)->ModelName();
		m_pCla->Classify(objName);
		window->UpdateLists();
		return true;
	}
	else if (key=='\'' || key==';')	// next label
	{
		m_iTrainedLabel += (key==';'?-1:1);
		if (m_iTrainedLabel<=-1)
		{
			m_iTrainedLabel = -1;
			std::cout<<"Label: "<<m_iTrainedLabel<<" - none"<<std::endl;;
		}
		else if (m_iTrainedLabel >= m_pCla->m_sRegionNames.count())
			m_iTrainedLabel = m_pCla->m_sRegionNames.count() - 1;
		else
		{
			bool mainName = false;
			while (!mainName)
			{
				int nameID = m_pCla->GetSymmEqClass(m_iTrainedLabel, &mainName);
				if (mainName)
				{
					std::cout<<"Label: "<<m_iTrainedLabel<<" - "<<m_pCla->m_SymmEqClassName[m_iTrainedLabel].c_str()<<std::endl;;
					break;
				}
				m_iTrainedLabel += (key==';'?-1:1) + m_pCla->m_sRegionNames.count();
				m_iTrainedLabel = m_iTrainedLabel % m_pCla->m_sRegionNames.count();
			}
		}

		return true;
	}
	else if (key=='.'||key==',')  // value threshold
	{
		m_fValueThreshold += (key=='.') ? .025 : -.025;
		if (m_fValueThreshold<=0)
			m_fValueThreshold = 0;
		if (m_fValueThreshold>=1)
			m_fValueThreshold = 1;
		std::cout<<"Value Threshold: "<<m_fValueThreshold<<std::endl;
		return true;
	}
	else if (key=='<'||key=='>')	// distance threshold
	{
		m_fReferenceDistance += (key=='>') ? .05 : -.05;
		if (m_fReferenceDistance<=0)
			m_fReferenceDistance = 0;
		else if (m_fReferenceDistance>=2.)
			m_fReferenceDistance = 2.;
		std::cout<<"Dist Thresh: "<<m_fReferenceDistance<<std::endl;
		return true;
	}
	else if (key=='s')
	{
		// todo...
		return true;
	}
	return false;
}

bool VisClassifierApp::Draw(DatabaseVisWindow * window)
{
	static bool updatedXForms = false;
	if (!updatedXForms)
	{
		if (m_pCla->m_Scales.size()>0)
		{
			int nm =(int)window->GetDrawableModels().size();
			assert((nm==(int)m_pCla->m_Scales.size() && nm==(int)m_pCla->m_AnglesUp.size()));
			for (int i=0; i<nm; i++)
			{
				R3Affine canonical = R3identity_affine;
				canonical.Scale(m_pCla->m_Scales[i]);
				canonical.Scale(m_fScaleEverything);
				canonical.Rotate(R3Vector(0, 0, m_pCla->m_AnglesUp[i]));
				canonical.Scale(1./m_pCla->m_AveRad[i]);
				canonical.Translate(-m_pCla->m_Translate[i]);
				VisModel3D * m = window->GetDrawableModels()[i];
				m->SetObjSpaceTransform(0, canonical, false);
			}
		}
	}
	
	int bk = 1;
	window->SetBackgroundColor(1);
	double br, bg, bb;
	window->GetBackgroundColor(&br, &bg, &bb);
	glClearColor(br, bg, bb, 1.0);
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT | GL_ACCUM_BUFFER_BIT | GL_STENCIL_BUFFER_BIT);
	
	window->SetBackgroundColor(-1);
	
	if (m_sShotType.startsWith("img_predict"))
	{
		VKString objName = m_sShotType;
		int featID = m_iShotValue;
		objName.replace("img_predict#", "");
		
		VisModel3D * obj = NULL;
		// switch to 'objName' model
		for (int i=0; i<m_pCla->GetDB()->NumModels() & obj==NULL; i++)
		{
			if (m_pCla->GetDB()->GetModel(i)->ModelName()==objName)
			{
				window->SelectModelInWindow(i);
				obj = window->GetDrawableModels()[i];
			}
		}

		// switch to "featpnts" geometry (also remove mesh (?))
		//obj->ClearRenderablgeGeometries();
		obj->AddRenderableGeometry("featpnts");
		// switch to "prediction" geometry values and change feature id
		obj->AddRenderableGeomValue("prediction", featID);
		window->SetDesignFlags(0);
	}
	else if (m_sShotType=="img_combined")
	{
		assert(m_iShotValue>=0 && m_iShotValue<(int)s_Thresholds.size());
		return DrawCombined(window, s_Thresholds[m_iShotValue]);
	}
	else
		assert(m_sShotType=="none");
	
	// models
	int mid = window->ModelID(0);
	assert(mid < (int)window->GetDrawableModels().size());
	VisModel3D * mvis = window->GetDrawableModels()[mid];

	
	// render training points
	if (m_iTrainedLabel>=0)
	{
		glEnable(GL_LIGHTING);
		window->LoadStandardSceneProjectionCamera();
		ShapePointCloud * pnts = m_pDB->GetModel(mid)->GetGeomShape("featpnts")->SafeCastToPointCloud();
		int nregions = m_pCla->m_sRegionNames.count();
		//int labelID = m_iTrainedLabel;
		for (int labelID=0; labelID<m_pCla->m_sRegionNames.count(); labelID++)
		for (int p=0; p<pnts->NumPoints(); p++)
		{
			int tid = m_pCla->m_aMap2TrainingData[mid][p * nregions + labelID];
			if (tid==-1)
				continue;
			bool mainName;
			int symmid = m_pCla->GetSymmEqClass(labelID, &mainName);
			if (!mainName)
				continue;
			
				
			double val = m_pCla->m_TrainingData[symmid]->m_MembershipValue[tid];
			int type = m_pCla->m_aMembershipTypes[mid][p * nregions + labelID];
			R3Point pos = pnts->PointPosition(p);
			R3Affine camera = mvis->GetCameraTransform(mvis->SubwindowCameraID(0, window));
			camera.Apply(pos);
			GLfloat surfaceMaterial[4];
			surfaceMaterial[3] = 1.;
			if (type & Classifier::MEMBER_Inlcuded)
			{
				double radius = .01;
				if (val>0)
				{
					double t = val;
					if (t < m_fValueThreshold)
						t = 0;
					if (m_pCla->m_sRegionNames[labelID]=="right_palm")
					{
						surfaceMaterial[0] = 0;
						surfaceMaterial[1] = 0.;
						surfaceMaterial[2] = 1;
					}
					else if (m_pCla->m_sRegionNames[labelID]=="pelvis")
					{
						surfaceMaterial[0] = 0;
						surfaceMaterial[1] = 1.;
						surfaceMaterial[2] = 0;
					}
					else if (m_pCla->m_sRegionNames[labelID]=="back")
					{
						surfaceMaterial[0] = 0;
						surfaceMaterial[1] = 0.2;
						surfaceMaterial[2] = 0;
					}
					else
						continue;
//					{
//						surfaceMaterial[0] = 0.5;
//						surfaceMaterial[1] = 0.;
//						surfaceMaterial[2] = 0.6;
//					}
					
					for (int i=0; i<3; i++)
						surfaceMaterial[i] = surfaceMaterial[i]*t + (1-t);
					radius = radius + t * radius;
				}
				else
				{
					surfaceMaterial[0] = 1.;
					surfaceMaterial[1] = 0.;
					surfaceMaterial[2] = 0.;
				}
				glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE, surfaceMaterial);
				//R3Vector norm(1, 1, 1); norm.Normalize(); camera.Apply(norm); norm *= radius;
				R3Sphere(pos, radius).Draw();
			}
		}
	}
	
	
	
	// if value selected - render name
	mvis->SetValueLimits(0, 1, m_fValueThreshold);
	mvis->SetPointRadius(m_fReferenceDistance * m_pCla->GetScale(0));
	
	int ngeom = mvis->NumRenderableGeometries();
	int ngeomvals = mvis->NumRenderableValues();
	for (int g=0; g<ngeom; g++)
	{
		GeomShape * shape = mvis->GetRenderableGeometry(g);
		for (int gv=0; gv<ngeomvals; gv++)
		{
			int labelid = -1;
			VKString gvStr = mvis->GetRenderableGeomValueName( gv, &labelid);

			ShapeValue * val = shape->GetShapeVal(gvStr);
			if (val!=NULL && val->GetShapeValueName()=="prediction")	// rendering it...
			{
				VKString label = "Unknown";
				if (labelid>=0 && labelid < m_pCla->m_sRegionNames.count())
					label = m_pCla->m_sRegionNames[labelid];
				//std::cout<<"G: "<<g<<" "<<gv<<" "<<label.c_str()<<std::endl;
				window->LoadStandardSceneProjectionCamera();
				if (m_sShotType=="none")
					window->GetRenderer()->RenderText(R2Point(0, .93), label, 1., 0, 0, VisWindowRenderer::TEXT_SIZE_LARGE);
			}
		}
	}
	
	return false;
}

bool VisClassifierApp::DrawCombined(DatabaseVisWindow * window, double threshold)
{
	VKStringList membershipNames;
	std::map<VKString, int> membership2cluster;
	std::vector<R3Point> centroids;
	std::vector<int> counts;
	membershipNames.push_back("");
	centroids.push_back(R3Point(0,0,0));
	counts.push_back(0);
	membership2cluster[""] = 0;
	
	
	// go over all points
	VisModel3D * mvis = window->GetDrawableModels()[0];
	Model3D * m = m_pCla->GetDB()->GetModel(0);
	GeomShape * shape = m->GetGeomShape("featpnts");
	assert(shape!=NULL && shape->SafeCastToPointCloud()!=NULL);
	ShapePointCloud * pnts = shape->SafeCastToPointCloud();
	ShapeValue * vals = pnts->GetShapeVal("prediction");
	assert(vals!=NULL);

	R3Affine camera = mvis->GetCameraTransform(0);
	
	// go over all points
	window->LoadStandardSceneProjectionCamera();
	for (int i=0; i<pnts->NumPoints(); i++)
	{
		// classify a point based on its memberships
		const ShapePoint & point = pnts->GetPoint(i);
		VKString groupName = "";
		for (int d=0; d<vals->NDims(); d++)
		{
			VKString label = "other";
			if (d>=0 && d < m_pCla->m_sRegionNames.count())
				label = m_pCla->m_sRegionNames[d];
			if (vals->Value(point, d) > threshold)
				groupName += label+", ";
		}
		
		// color point based on the membership...
		std::map<VKString, int>::iterator iter = membership2cluster.find(groupName);
		int groupID = -1;
		if (iter == membership2cluster.end())
		{
			groupID = membershipNames.count();
			membership2cluster[groupName] = groupID;
			membershipNames.push_back(groupName);
			centroids.push_back(R3Point(0,0,0));
			counts.push_back(0);
			assert(groupID < (int)centroids.size());
		}
		else
			groupID = iter->second;
		
		
		// draw point
		R3Point pos = pnts->PointPosition(i);
		camera.Apply(pos);
		centroids[groupID] += pos;
		counts[groupID]++;
		
		glEnable(GL_LIGHTING);
		GLfloat surfaceMaterial[4];
		if (groupID==0)
		{
			surfaceMaterial[0] = .4;
			surfaceMaterial[1] = .4;
			surfaceMaterial[2] = .4;
			surfaceMaterial[3] = 1.;
		}
		else
			window->MapIntToRadicalColor(groupID, surfaceMaterial);
		glMaterialfv(GL_FRONT, GL_AMBIENT_AND_DIFFUSE, surfaceMaterial);
		R3Sphere(pos, 0.01).Draw();;
	}
	
	// render membeships
	glDisable(GL_DEPTH_TEST);
	glDisable(GL_LIGHTING);
	assert(membershipNames.count()==(int)counts.size() && counts.size()==centroids.size());
	VKStringList renderText;
	std::vector<R2Point> textPos;
	std::vector<int> groupIDs;
	double fh = 35. / window->Height();
	double fw = 20. / window->Width();

	for (int i=0; i<(int)counts.size(); i++)
	{
		VKStringList strings = membershipNames[i].split(", ");
		GLdouble projection[16];
		GLdouble modelview[16];
		GLint viewport[4];
		glGetDoublev(GL_PROJECTION_MATRIX, projection);
		glGetDoublev(GL_MODELVIEW_MATRIX, modelview);
		glGetIntegerv(GL_VIEWPORT, viewport);

		R3Point pnt = centroids[i]/counts[i];
		GLdouble wx, wy, wz;
		gluProject(pnt.X(), pnt.Y(), pnt.Z(), modelview, projection, viewport, &wx, &wy, &wz);
		wx = (wx/window->Width())*2. - 1.;
		wy = (wy/window->Height())*2. - 1.;
				
		for (int j=0; j<strings.count(); j++)
		{
			textPos.push_back(R2Point(wx, wy + fh*j));
			renderText.push_back(strings[j]);
			groupIDs.push_back(i);
		}
	}
	
	glMatrixMode(GL_MODELVIEW);
	glPushMatrix();
	glLoadIdentity();
	glMatrixMode(GL_PROJECTION);
	glPushMatrix();
	glLoadIdentity();

	for (int i=0; i<renderText.count(); i++)
	{
		double wx = textPos[i].X();
		double wy = textPos[i].Y();
		double z = 0;
		glBegin(GL_QUADS);
		glColor4d(.95, .95, .95, .5);
		glVertex3d(wx-fw*0.1, wy-fh*0.1, z);
		glVertex3d(wx-fw*0.1, wy+fh*1, z);
		glVertex3d(wx+fw*renderText[i].length(), wy+fh*1, z);
		glVertex3d(wx+fw*renderText[i].length(), wy-fh*0.1, z);
		glEnd();
	}
	
	glMatrixMode(GL_PROJECTION);
	glPopMatrix();
	glMatrixMode(GL_MODELVIEW);
	glPopMatrix();

	for (int i=0; i<renderText.count(); i++)
	{
		double r, g, b;
		if (groupIDs[i]==0)
		{
			r = .4;
			g = .4;
			b = .4;
		}
		else
			window->MapIntToRadicalColor(groupIDs[i], r, g, b);
		window->GetRenderer()->RenderText(R2Point(textPos[i].X(), textPos[i].Y()), renderText[i],
										  r, g, b, VisWindowRenderer::TEXT_SIZE_LARGE);
	}
	glEnable(GL_DEPTH_TEST);
	glEnable(GL_LIGHTING);
	return true;
}

void VisClassifierApp::SetScreenshotType(const VKString & sName, int value)
{
	m_sShotType = sName;
	m_iShotValue = value;
}
