#include "FileManagement.h"
#include "VKString.h"
#include "TimeProfiler.h"

#include "RendererGLUT.h"
#include "DatabaseVisWindow.h"
#include "ModelDatabase.h"
#include "ShapeDistance.h"

#include "ShapeTriangleSoup.h"
#include "DBApp.h"
#include <vector>
#include "ShapePoint.h"

#include "Interactor.h"
#include "InteractorPose.h"
#include "PosePrior.h"
#include "Clustering.h"

#ifndef __VIS_PRIOR_APP_H
#define __VIS_PRIOR_APP_H


class VisPriorApp : public DBApp
{
	public:
		enum VisPoseFlags
		{
			VIS_Skeleton 			= 1<<0,
			VIS_LinkDistrPoints 	= 1<<1,
			VIS_LinkDistrRadius		= 1<<2,
			VIS_LinkDistrAngles		= 1<<3,
			VIS_AveragePose			= 1<<4,
			VIS_MeanGauss			= 1<<5,
			VIS_Clustering			= 1<<6,
			VIS_SkeletonNames		= 1<<7,
			VIS_EndEffDistr			= 1<<8,
			VIS_EndEffGridDistr		= 1<<9
		};
		VisPriorApp(const std::vector<PntCorrsPose*> & poses, IKSolver * solver);
		virtual void SetVisPriors(PosePrior * prior, std::vector<ClusteringProblem*> & angleClusters);
	
		virtual bool KeyPressed(DatabaseVisWindow * window, const char key, int mouseX, int mouseY);
		virtual bool Draw(DatabaseVisWindow * window);
		virtual void DrawAngleHistogram(DatabaseVisWindow * window);
		virtual void DrawGaussian(DatabaseVisWindow * window, const R3Affine & camera);
		
	protected:
		int m_iCurrPose;
		int m_iNextInitPose;
		std::vector<PntCorrsPose*> m_Poses;
		PntCorrsPose * m_pAveragePose;
		IKSolver * m_pSolver;
		int m_iSelectedLink;
		int m_iSelectedCOI;
		int m_iSelectedRefID;
		double m_fRetRotation;
	
		virtual void PreparePntDistr();
		virtual void PreparePntDistrPerGaussian();
		virtual void PreparePntDistrPerRegion();
		ShapePointCloud * m_pDistrPoints;
		std::vector<double> m_PerPointDistr;
		std::vector<int> m_PerPointCluster;
		double m_fDistrRadius;
	
		std::vector<std::vector<double> > m_AngleDistr;
	
		PosePrior * m_pPosePrior;
		std::vector<ClusteringProblem*> m_AngleClusters;
		double m_fEnergyThreshold;	
};

#endif

