#include "RendererGLUT.h"
#include "DatabaseVisWindow.h"
#include "ModelDatabase.h"
#include "ShapeDistance.h"
#include "VKString.h"
#include "TimeProfiler.h"
#include "FileManagement.h"
#include "Model3D.h"

#include "DBApp.h"
#include "MetricVisApp.h"
#include "DistEuclidean.h"
#include "DistGeodPnts.h"

int main(int argc, char** argv )
{
	if (argc<2)
	{
		std::cout<<"Usage: ./"<<argv[0]<<" [params]"<<std::endl;
		std::cout<<"Params:"<<std::endl;
		std::cout<<"    -mesh filename 		- read mesh"<<std::endl;;
		std::cout<<"    -pnts filename 		- read points of interest"<<std::endl;;
		std::cout<<"	-dist [eucl, geod, geodGraph] - distance metric"<<std::endl;
		std::cout<<"    -readDist filename 	- read distances"<<std::endl;;
		std::cout<<"    -writeDist filename - read distances"<<std::endl;;
		std::cout<<"    -writeTiming filename - write timing"<<std::endl;;
		std::cout<<"    -vis           		- visualize"<<std::endl;
		exit(0);
	}
	bool vis = false;
	VKString meshFile = "none";
	VKString pntsFile = "none";
	VKString distType = "eucl";
	VKString readDist = "none";
	VKString writeDist = "none";

	VKString writeTiming = "none";
	TimeProfiler timer;
	for (int i=1; i<argc; i++)
	{
		if (strcmp(argv[i], "-mesh")==0)
			meshFile = VKString(argv[++i]);
		else if (strcmp(argv[i], "-pnts")==0)
			pntsFile = VKString(argv[++i]);
		else if (strcmp(argv[i], "-dist")==0)
			distType = VKString(argv[++i]);
		else if (strcmp(argv[i], "-readDist")==0)
			readDist = VKString(argv[++i]);
		else if (strcmp(argv[i], "-writeDist")==0)
			writeDist = VKString(argv[++i]);
		else if (strcmp(argv[i], "-writeTiming")==0)
			writeTiming = VKString(argv[++i]);
		else if (strcmp(argv[i], "-vis")==0)
			vis = true;
	}
	ShapePointCloud * pnts = NULL;
	ShapeMesh * mesh = NULL;
	Model3D * model = new Model3D("model", 0);
	ShapeDistance * dist = NULL;
	
	timer.startedProcess("metric");
	if (meshFile!="none")
	{
		mesh = new ShapeTriangleSoup(meshFile, "mesh", model);
		model->AddGeomShape(mesh->GeometryName(), mesh);
	}
	
	if (pntsFile!="none")
	{
		pnts = new ShapePointCloud(mesh, "pnts");
		pnts->ReadGeometryFile(pntsFile);
		model->AddGeomShape(pnts->GeometryName(), pnts);
	}

	assert(pnts!=NULL);		// assume interested in point distances
	if (readDist!="none")
		dist = ShapeDistance::ReadMetric(readDist, pnts, "readdist", 1);
	else if (distType=="eucl")
		dist = new DistEuclidean(pnts, "euclidean", 1.);
	else if (distType=="geodGraph")
	{
		DistGeodPnts * distGeod = new DistGeodPnts(pnts, "geodesicNN", 1.);
		distGeod->InitNNGraph(0.1, 6, 10);
		dist = distGeod;
	}
	else if (distType=="geod")
		assert(false);
	else
		assert(false);

	timer.finishedProcess("metric");
	
	if (writeTiming!="none")
	{
		FileManagement::CreateDirsOnPathToFile(writeTiming.c_str());
		std::ofstream textStream(writeTiming.c_str());
		assert(textStream.is_open());
		textStream<<timer.profile().c_str();
		textStream.close();
	}
	if (writeDist!="none")
		dist->WriteMetric(writeDist.c_str());

	
#ifdef VISUALIZATION
	if (vis)
	{
		int listFlags = DatabaseVisWindow::LISTGP_Models | DatabaseVisWindow::LISTGP_Geoms;
		listFlags |= DatabaseVisWindow::LISTGP_MeshFlags;
		ModelDatabase * db = new ModelDatabase();
		db->AddModel(model);
		
		MetricVisApp * metricApp = new MetricVisApp(model, mesh, pnts, dist);
		
		int w=-1, h=-1;
		
		DatabaseVisWindow * dbVis = new DatabaseVisWindow(db, NULL, listFlags, 1, 1, 0, 0, 1., w, h);
		dbVis->SetApp(metricApp);

		VisWindowRenderer * wndGLUT = new RendererGLUT(argc, argv, dbVis);
	}
#else
	std::cout<<"[WARNING] Visualization disabled (compiling flag)"<<std::endl;
#endif
}








