#include "FileManagement.h"
#include <unistd.h>

void FileManagement::WaitForFile(const char * fileName, 
								 int checkEveySeconds,
								 int maxSeconds)
{
	if (maxSeconds==-1)
		maxSeconds = 8000;
	bool foundFile = FileExists(fileName);
	
	if (foundFile)
		return;
	
	std::cout<<"Waiting for file: "<<fileName<<std::endl;
	
	time_t currTime = time(NULL);
	time_t terminateTime = currTime + maxSeconds;
	
	int nIter=0;
	while (!foundFile && currTime < terminateTime)
	{
		nIter++;
		if (FileExists(fileName))
			foundFile = true;
		sleep(checkEveySeconds);
		currTime = time(NULL);	
		if (nIter%10==0)
			std::cout<<(currTime-terminateTime+maxSeconds)<<"s "<<std::flush;
	}
	std::cout<<std::endl;
	if (foundFile)
		std::cout<<"\tFile exists!"<<std::endl;
	else
	{
		std::cout<<"[ERROR] Could not find file: "<<fileName<<std::endl;
		assert(false);
	}
}


bool FileManagement::DirExists(const char * dirname)
{
	struct stat st;
	if(stat(dirname, &st) == 0)
		return S_ISDIR(st.st_mode);
	return false;
}

bool FileManagement::FileExists(const char * filename)
{
	struct stat st;
	if(stat(filename, &st) == 0)
		return S_ISREG(st.st_mode);
	return false;
}

bool FileManagement::CreateDirsOnPath(const char * dirnameInput)
{
	VKString dirname(dirnameInput);
	VKString currDir("");
	VKStringList dirs = dirname.split("/");
	for (int i=0; i<dirs.count(); i++)
	{
		currDir += dirs[i]+"/";
		if (!DirExists(currDir.c_str()))
		{
			bool ok = mkdir(currDir.c_str(), 07777);
			if (!ok) sleep(1);	// might be created by another thread...
		}
		//system((VKString("mkdir ") + currDir).c_str());
	}
	return true;
}

bool FileManagement::CreateDirsOnPathToFile(const char * filenameInput)
{
	VKString filename(filenameInput);
	VKString currDir("");
	VKStringList dirs = filename.split("/");
	if (filename.startsWith("/") && dirs[0]!="")
		currDir += "/";
	for (int i=0; i<dirs.count() - 1; i++)
	{
		currDir += dirs[i]+"/";
		if (!DirExists(currDir.c_str()) && currDir!="/")
		{
			bool ok = mkdir(currDir.c_str(), 07777);
			if (!ok) sleep(1);	// might be created by another thread...
		}
	}
	return true;
}

VKStringList FileManagement::ListSubdirsAndFiles(const char * dirName, 
												 bool file, bool dir)
{
	VKStringList strList;
	DIR *directory = opendir(dirName);
    if(directory == NULL)
		return strList;
	
    struct dirent *dirp = readdir(directory);
	int counter = 0;
	bool addMe;
    while ( dirp != NULL )
	{
		counter++;
		addMe = false;
		if (strcmp(dirp->d_name, ".")!=0
			&& strcmp(dirp->d_name, "..")!=0)
		{
			if (dir)
			{
				VKString fullDir = VKString(dirName)+"/"+dirp->d_name;
				
				if (dirp->d_type==DT_DIR)
					addMe = true;
				else if (dirp->d_type==DT_UNKNOWN)
					addMe = addMe || DirExists(fullDir.c_str());
			}
			
			if (file)
			{
				VKString fullDir = VKString(dirName)+"/"+dirp->d_name;
				if (dirp->d_type==DT_REG)
					addMe = true;
				else if (dirp->d_type==DT_UNKNOWN)
					addMe = addMe || FileExists(fullDir.c_str());
			}
		}
		
		if (addMe)
			strList.push_back(dirp->d_name);
//		else
//		{
//			VKString fullDir = VKString(dirName)+"/"+dirp->d_name;
//			std::cout<<"checkDir: "<<dirp->d_name<<" tp="<<(int)(dirp->d_type)<<std::endl;
//			std::cout<<"\tFullPath: "<<fullDir.c_str()<<std::endl;
//			std::cout<<"\tdir="<<DT_DIR<<" unknown="<<DT_UNKNOWN<<std::endl;
//			std::cout<<"\tdirExists="<<DirExists(fullDir.c_str())<<std::endl;
//			std::cout<<"\tdirCanOpen="<<(opendir(fullDir.c_str())!=NULL)<<std::endl;
//		}
		dirp = readdir(directory);
    }
    closedir(directory);
//	std::cout<<"Found Files: "<<strList.count()<<" Examined = "<<counter<<std::endl;
	strList.Sort();
//	if (strList.count()>0)
//		std::cout<<"\tInterval: "<<strList[0].c_str()<<" - ";
//	for (int i=0; i<strList.count()-1; i++)
//	{
//		int id1 = strList[i].right(6).toInt();
//		int id2 = strList[i+1].right(6).toInt();
//		if (id2 != id1+1)
//			std::cout<<strList[i].c_str()<<"\n\tInterval: "<<strList[i+1].c_str()<<" - ";
//	}
//	exit(0);
	return strList;
}


VKStringList FileManagement::ListAllFiles(const char * dirName)
{
	return ListSubdirsAndFiles(dirName, true, false);
}

VKStringList FileManagement::ListAllSubdirs(const char * dirName)
{
	return ListSubdirsAndFiles(dirName, false, true);
}

