/***************************************************************************
 *   Copyright (C) 2008 by Vladimir Kim									   *
 *   Author: Vladimir Kim (vk@princeton.edu)							   *
 ***************************************************************************/

#ifndef __CollectionPairwise_H
#define __CollectionPairwise_H

#include <vector>
#include <map>
#include <iostream>
#include <assert.h>

#include "VKString.h"

#include "GeomShape.h"
#include "ShapePoint.h"
#include "ShapeCorrs.h"
#include "CorrsPair.h"
#include "ModelDatabase.h"

class DBCorrs;

/** 
 * This is not a correspondence class. 
 * It contains multiple pairwise correspondences.
 */
class CollectionPairwise
{
	public:
		CollectionPairwise(const VKString & pairwiseCollName);
		virtual ~CollectionPairwise();
		virtual int AddPairCorrs(CorrsPair * pairCorr);
		virtual const VKString & CollectioName() const;
		
		virtual int NumCorrs() const;
		virtual int NumCorrsFrom(const VKString & modelFrom) const;
		virtual int NumCorrsTo(const VKString & modelTo) const;
		virtual int NumCorrsForPair(const VKString & modelFrom, const VKString & modelTo) const;
	
		virtual CorrsPair * GetCorr(int pairID);
		virtual const CorrsPair * GetCorr(int pairID) const;
		virtual const std::vector<int> & GetCorrsFrom(const VKString & modelFrom) const;
		virtual const std::vector<int> & GetCorrsTo(const VKString & modelTo) const;
		virtual const std::vector<int> & GetCorrPairs(const VKString & modelFrom, 
													  const VKString & modelTo) const;
	
		
		virtual void GetBestMapForEachPair(std::vector<int> & mapIDs) const;
		virtual void SortMapsForEachPair(const VKString & evalPnts = "none",
										 bool beVerbouse = false);
	

	// write format is one of basic pairwise formats (map, corrs, other)
		virtual void WriteConcatenated(const VKString & filename, 
									   ShapeCorrs::CoreCorrsType writeFormat);
		virtual void WriteConcatenatedHeader(FILE * file,
											 ShapeCorrs::CoreCorrsType writeFormat);
		virtual void WriteMaps(FILE * file, ShapeCorrs::CoreCorrsType writeFormat);	

		
		static CollectionPairwise * ReadConcatenated(const VKString & corrName,
													 ModelDatabase * db, 
													 const VKString & filename, DBCorrs * corrs);
	
		static FILE * ReadConcatenatedHeader(const VKString & filename, 
											 int & nGeoms,
											 VKStringList & modelNames, 
											 VKStringList & geomNames,
											 int & totalNMaps,
											 std::vector<int> & nMapsPerType,
											 VKStringList & corrNames,
											 VKStringList & classNames,
											 std::vector<int> & fromToPairs, 
							std::map<VKString, std::map<VKString, std::vector<int> > > & pairsToCorrsSorted);
		
	protected:
		std::vector<int> m_EmptyCorrs;
		VKString m_sPairwiseCollName;
		ModelDatabase * m_pDB;
	
		std::vector<CorrsPair* > m_PairwiseCorrs;
		
		std::map<VKString, std::vector<int> > m_FromModelToCorrs;
		std::map<VKString, std::vector<int> > m_ToModelToCorrs;
		std::map<VKString, std::map<VKString, std::vector<int> > > m_PairToCorrs;
};

#endif

