/***************************************************************************
 *   Copyright (C) 2008 by Vladimir Kim									   *
 *   Author: Vladimir Kim (vk@princeton.edu)							   *
 ***************************************************************************/

#ifndef __CorrsPairMap_H
#define __CorrsPairMap_H

#include <vector>
#include <map>
#include <iostream>
#include <assert.h>

#include "VKString.h"

#include "GeomShape.h"
#include "ShapePoint.h"
#include "ShapeCorrs.h"
#include "CorrsPair.h"

/** 
 * CorrsPairMap: pairwise point-to-point map. 
 */
class CorrsPairMap : public CorrsPair
{
	public:
		CorrsPairMap(const VKString & corrName, GeomShape * shapeFrom, 
					 GeomShape * shapeTo, DBCorrs * corrs=NULL);
		virtual ~CorrsPairMap();
		virtual VKString GetCorrsClassName() const;
		
		virtual int GetCorrs(const ShapePoint & pnt,
							 std::vector<ShapePoint> & corrs, 
							 const VKString & corrValName="none",
							 std::vector<double> *corrVals = NULL);
		
		virtual int PairwiseCorrs(const ShapePoint & pnt,
								  GeomShape * toShape,
								  std::vector<ShapePoint> & corrs, 
								  const VKString & corrValName = "none",
								  std::vector<double> * corrValue = NULL);	
		
		virtual ShapePoint BestCorr(const ShapePoint & pnt,
									GeomShape * toShape,
									const VKString & corrValName = "default",
									double * corrValue = NULL);
	
		virtual ShapePoint MapPoint(const ShapePoint & pnt, 
									const VKString & corrValName = "default",
									double * corrValue = NULL);
	
		virtual void WriteShapeCorrs(FILE * file);
		virtual void ReadShapeCorrs(FILE * file);
		void WriteShapeCorrsAsCorrsPairMap(FILE * file);
		void ReadShapeCorrsAsCorrsPairMap(FILE * file);
		
		virtual void WriteVtx2VtxMap(const VKString & outFile);
		virtual void WriteVtx2PntMap(const VKString & outFile);
	
		void AutoFillMapCache(const VKString & corrValueName = "default");
		void InitMapCache(int nFromPnts,
						  const VKString & corrValueName = "default");
		bool SetCachedMapCorrs(int fromPntID, const ShapePoint & corr);
		bool SetCachedMapVals(int fromPntID, double val);
		ShapePoint GetCachedMapCorrs(int fromPntID);
		double GetCachedMapVals(int fromPntID);
		void SetMapCacheFilled(bool filled);	
		bool MapCacheFilled() const;
	
		virtual double DefaultIntegratedValue(const VKString & evalPnts = "none") const;


	// optional functionality: if produced by some generating corrs
		virtual void RememberGenerators(const ShapePointCloud * pnts1, 
										const ShapePointCloud * pnts2,
										const std::vector<int> &corrs1,
										const std::vector<int> &corrs2);
		virtual bool SameGenPnts(const ShapePointCloud * pnts1, 
								 const ShapePointCloud * pnts2) const;
		virtual void GetGenerators(std::vector<int> & corrs1, 
								   std::vector<int> & corrs2) const;
		virtual void WriteGenerators(FILE * file);
		virtual void ReadGenerators(FILE * file);

	protected:
		// optional params: if produced by some generating corrs
		const ShapePointCloud * m_GenPnts1;
		const ShapePointCloud * m_GenPnts2;
		std::vector<int> m_Corrs1;
		std::vector<int> m_Corrs2;

	private:
		bool m_bMapCacheFilled;
		int m_iNMapCachedFromPnts;
		std::vector<ShapePoint> m_aMapCachedCorrs; 
		std::vector<double> m_aMapCachedCorrVals;

};

#endif

