/***************************************************************************
 *   Copyright (C) 2008 by Vladimir Kim									   *
 *   Author: Vladimir Kim (vk@princeton.edu)							   *
 ***************************************************************************/

#include "AppScreenshots.h"

#include "FileManagement.h"
#include "TimeProfiler.h"

ShotModelInput::ShotModelInput()
{
	m_sModelName = "none";
	m_sModelFile = "none";
	m_sGeometryName = "none";

}

ShotDescription::ShotDescription()
{
	m_ShotType = SHOTTP_Invalid;
	m_sLabelsName = "none";
	m_iSegmentID = -1;
	m_pApp = NULL;
	m_iAppVisFlags = -1;
	
	m_Image = NULL;
	x0 = -1;
	y0 = -1;
	x1 = -1;
	y1 = -1;
}

AppScreenshots::AppScreenshots(bool terminateOnceDone, ModelDatabase * db, DBCorrs * dbCorrs)
: DBApp(db, dbCorrs)
{
	m_bTerminateOnceDone = terminateOnceDone;
	m_CropType = CROP_PerImage;
}

AppScreenshots::~AppScreenshots()
{
	
}

void AppScreenshots::SetCrop(CropType crop)
{
	m_CropType = crop;
}
void AppScreenshots::TakeShotOfModels(const VKStringList & modelFiles, const VKStringList & modelNames,
									  const std::vector<VKStringList> & outFiles)
{	
	for (int i=0; i<modelFiles.count(); i++)
	{
		int offset = (int)m_ShotsQueue.size();
		m_ShotsQueue.push_back(ShotDescription());
		m_ShotsQueue[offset].m_ShotType = ShotDescription::SHOTTP_WholeModel;
		m_ShotsQueue[offset].m_InputModel.m_ModelInputType = ShotModelInput::MODELINPUT_File;
		m_ShotsQueue[offset].m_InputModel.m_sModelName = modelNames[i];
		m_ShotsQueue[offset].m_InputModel.m_sModelFile = modelFiles[i];
		m_ShotsQueue[offset].m_InputModel.m_sGeometryName = Model3D::m_sMainGeomName;
		m_ShotsQueue[offset].m_OutputFiles = outFiles[i];
	}
}

void AppScreenshots::TakeShotOfModel(const VKString & geomName, const VKString & outFile,
									 const R3Affine & camera)
{
	assert(m_pDB!=NULL);
	for (int i=0; i<m_pDB->NumModels(); i++)
	{
		Model3D * m = m_pDB->GetModel(i);
		assert(m!=NULL);
		int offset = (int)m_ShotsQueue.size();
		m_ShotsQueue.push_back(ShotDescription());
		m_ShotsQueue[offset].m_ShotType = ShotDescription::SHOTTP_WholeModel;
		m_ShotsQueue[offset].m_InputModel.m_ModelInputType = ShotModelInput::MODELINPUT_NameInDB;
		m_ShotsQueue[offset].m_InputModel.m_sModelName = m->ModelName();
		m_ShotsQueue[offset].m_InputModel.m_sGeometryName = geomName;
		m_ShotsQueue[offset].m_OutputFiles.push_back(outFile);
		m_ShotsQueue[offset].m_ObjectXForm = camera;
	}
}

void AppScreenshots::TakeShotOfSeg(const VKString & geomName, const VKString & outFile, 
								   const VKString & labelsName, int level, const R3Affine & camera)
{	assert(m_pDB!=NULL);
	for (int i=0; i<m_pDB->NumModels(); i++)
	{
		Model3D * m = m_pDB->GetModel(i);
		assert(m!=NULL);

		int offset = (int)m_ShotsQueue.size();
		m_ShotsQueue.push_back(ShotDescription());
		m_ShotsQueue[offset].m_ShotType = ShotDescription::SHOTTP_Segmentation;
		m_ShotsQueue[offset].m_InputModel.m_ModelInputType = ShotModelInput::MODELINPUT_NameInDB;
		m_ShotsQueue[offset].m_InputModel.m_sModelName = m->ModelName();
		m_ShotsQueue[offset].m_InputModel.m_sGeometryName = geomName;
		m_ShotsQueue[offset].m_OutputFiles.push_back(outFile);
		m_ShotsQueue[offset].m_sLabelsName = labelsName;
		m_ShotsQueue[offset].m_iSegmentID = level;
		m_ShotsQueue[offset].m_ObjectXForm = camera;
	}
}

void AppScreenshots::TakeShotOfTextured(const VKString & geomName, const VKString & outFile,
										const R3Affine & camera)
{
	assert(m_pDB!=NULL);
	for (int i=0; i<m_pDB->NumModels(); i++)
	{
		Model3D * m = m_pDB->GetModel(i);
		assert(m!=NULL);
		int offset = (int)m_ShotsQueue.size();
		m_ShotsQueue.push_back(ShotDescription());
		m_ShotsQueue[offset].m_ShotType = ShotDescription::SHOTTP_Textured;
		m_ShotsQueue[offset].m_InputModel.m_ModelInputType = ShotModelInput::MODELINPUT_NameInDB;
		m_ShotsQueue[offset].m_InputModel.m_sModelName = m->ModelName();
		m_ShotsQueue[offset].m_InputModel.m_sGeometryName = geomName;
		m_ShotsQueue[offset].m_OutputFiles.push_back(outFile);
		m_ShotsQueue[offset].m_ObjectXForm = camera;
	}
}

void AppScreenshots::TakeShotOfVolume(const VKString & gridFile, const VKString & outFile,
									  const VKString & mName, const R3Affine & camera)
{
	int offset = (int)m_ShotsQueue.size();
	m_ShotsQueue.push_back(ShotDescription());
	m_ShotsQueue[offset].m_ShotType = ShotDescription::SHOTTP_VolumeDensityGrid;
	m_ShotsQueue[offset].m_InputModel.m_ModelInputType = ShotModelInput::MODELINPUT_NoModel;
	m_ShotsQueue[offset].m_InputModel.m_sModelName = mName;
	m_ShotsQueue[offset].m_InputModel.m_sGeometryName = Model3D::m_sMainGeomName;
	m_ShotsQueue[offset].m_sVolumeFile = gridFile;
	m_ShotsQueue[offset].m_OutputFiles.push_back(outFile);
	m_ShotsQueue[offset].m_ObjectXForm = camera;
}

void AppScreenshots::TakeShotOfPart(const VKString & geomName, const VKString & labelsName, 
									const VKString & keyword, const VKString & matchingType, 
									const VKString & outFile, const R3Affine & camera)
{	
	assert(m_pDB!=NULL);
	
	VKString lowCaseKeyword = keyword;
	lowCaseKeyword.toLowerCase();
	
	for (int i=0; i<m_pDB->NumModels(); i++)
	{
		// check all nodes
		Model3D * m = m_pDB->GetModel(i);
		
		GeomShape * shape = m->GetGeomShape(geomName);
		assert(shape!=NULL);
		ShapeValue * geomVal = shape->GetShapeVal(labelsName);
		assert(geomVal!=NULL);
		ShapeLabels * labels = geomVal->SafeCastToLabels();
		assert(labels!=NULL);
		
		for (int j=0; j<labels->NNodes(); j++)
		{
			VKString nodeLabel = labels->NodeLabel(j);
			nodeLabel.toLowerCase();
			bool include = false;
			if (matchingType=="substring")
				include = (nodeLabel.lastIndexOf(lowCaseKeyword)!=-1);
			else if (matchingType=="exact")
				include = (nodeLabel == lowCaseKeyword);
			else
				assert(false);
			
			// found a part (render multiple copies of the same model)
			if (include)
			{
				int offset = (int)m_ShotsQueue.size();
				m_ShotsQueue.push_back(ShotDescription());
				m_ShotsQueue[offset].m_ShotType = ShotDescription::SHOTTP_ModelPart;
				m_ShotsQueue[offset].m_InputModel.m_ModelInputType = ShotModelInput::MODELINPUT_NameInDB;
				m_ShotsQueue[offset].m_InputModel.m_sModelName = m->ModelName();
				m_ShotsQueue[offset].m_InputModel.m_sModelFile = "none";
				m_ShotsQueue[offset].m_InputModel.m_sGeometryName = geomName;
				m_ShotsQueue[offset].m_OutputFiles.push_back(outFile);
				m_ShotsQueue[offset].m_sLabelsName = labelsName;
				m_ShotsQueue[offset].m_iSegmentID = j;
				m_ShotsQueue[offset].m_ObjectXForm = camera;
			}
		}
	}
}

void AppScreenshots::TakeShotOfApp(DBApp * app, const VKString & shotType, int visFlags, const VKString & outFile)
{
	int offset = (int)m_ShotsQueue.size();
	m_ShotsQueue.push_back(ShotDescription());
	m_ShotsQueue[offset].m_InputModel.m_ModelInputType = ShotModelInput::MODELINPUT_NoModel;
	m_ShotsQueue[offset].m_ShotType = ShotDescription::SHOTTP_AnyApp;
	m_ShotsQueue[offset].m_pApp = app;
	m_ShotsQueue[offset].m_iAppVisFlags = visFlags;
	m_ShotsQueue[offset].m_OutputFiles.push_back(outFile);
	m_ShotsQueue[offset].m_sLabelsName = shotType;
}

bool AppScreenshots::Draw(DatabaseVisWindow * window)
{
	if (m_ShotsQueue.size() > 1)
		std::cout<<"Writing Screenshots: "<<m_ShotsQueue.size()<<std::endl;
	int nmaxshots = 1000;
	if (m_ShotsQueue.size() > nmaxshots)
	{
		std::cout<<"[WARNING] Num shots exceeds maximal ("<<nmaxshots<<"). Removing last..."<<std::endl;
		m_ShotsQueue.erase(m_ShotsQueue.begin() + nmaxshots, m_ShotsQueue.end());
	}
	
	TimeProfiler timer;
	std::map<DBApp*, int> app2InitFlags;
	
	int x0 = window->Width(), y0 = window->Height(), x1 = 0, y1 = 0;

	for (int i=0; i<m_ShotsQueue.size(); i++)
	{
		if (m_ShotsQueue.size() > 1)
			timer.WriteProgress("Screenshots", i, (int)m_ShotsQueue.size());
		window->StartDrawToBuffer();
		
		glClearColor(0, 0, 0, 1.0);
		//glClearColor(1., 1., 1., 1.0);
		//glClearColor(.5, .5, .5, 1.0);
		glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT | GL_ACCUM_BUFFER_BIT | GL_STENCIL_BUFFER_BIT);

		ShotDescription & shot = m_ShotsQueue[i];
		
		// render several models / representation / etc...		
		// models
		Model3D * model = NULL;
		VisModel3D * modelVis = NULL;

		if (shot.m_InputModel.m_ModelInputType == ShotModelInput::MODELINPUT_NoModel)
		{
			switch(shot.m_ShotType)
			{
				case ShotDescription::SHOTTP_VolumeDensityGrid:
				{
					assert(false);
//					CorrsVolume * volCorrs = new CorrsVolume(shot.m_sVolumeFile);
//					Vis3DVolume * vol = new Vis3DVolume(volCorrs, NULL);
//					glClearColor(1., 1., 1., 1.0);
//					glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT 
//							| GL_ACCUM_BUFFER_BIT | GL_STENCIL_BUFFER_BIT);
//					
//					// get model 0 in cluster...
//					VKString mName = shot.m_InputModel.m_sModelName;
//					assert(mName!="none");
//					
//					assert(m_pDB!=NULL);
//					model = m_pDB->GetModel(mName);
//					assert(model!=NULL);
//					assert((int)window->GetDrawableModels().size()==m_pDB->NumModels());
//					int mID = model->ModelID();
//					assert(mID < (int)window->GetDrawableModels().size());
//					modelVis = window->GetDrawableModels()[mID];
//					assert (model!=NULL && modelVis!=NULL);	
//					
//					// prepare model for rendering
//					modelVis->AddRenderableGeometry(shot.m_InputModel.m_sGeometryName);
//					int cam0 = modelVis->CreateNewCamera();
//					modelVis->Scale(cam0, -.2, window);
//
//					//int cam0 = modelVis->SubwindowCameraID(0, window);
//					
//					R3Affine modelCamera = modelVis->GetCameraTransform(cam0);	// centroid model camera
//					assert(volCorrs->ModelNames().count()>0);
//					int locID = -1;
//					for (locID=0; locID < volCorrs->ModelNames().count(); locID++)
//					{
//						if (volCorrs->ModelNames()[locID] == mName)
//							break;
//					}
//					assert(locID < volCorrs->ModelNames().count());
//					R3Affine modelToVolume = volCorrs->GetLocalTransform(locID);
//					
//					modelVis->Draw(window, cam0);
//					
//					Vis3DVolume::DrawGrid(window, volCorrs, volCorrs->GetDensityGrid(),
//										  modelToVolume, modelCamera);
//					delete vol;
//					delete volCorrs;
//					break;
				}
				case ShotDescription::SHOTTP_AnyApp:
				{
					if (app2InitFlags.find(shot.m_pApp)==app2InitFlags.end())
						app2InitFlags[shot.m_pApp] = shot.m_pApp->GetRenderingFlags();
					
					if (shot.m_iAppVisFlags>=0)
						shot.m_pApp->SetScreenshotType(shot.m_sLabelsName, shot.m_iAppVisFlags);
					bool dmlist = window->IsDesignFlagOn(DatabaseVisWindow::RENDER_MODEL_LIST);
					if (dmlist)
						window->ToggleDesignFlag(DatabaseVisWindow::RENDER_MODEL_LIST);
					if (!shot.m_pApp->Draw(window))
					{
						window->SetApp(shot.m_pApp);
						window->Draw();
						window->SetApp(this);
					}
					if (dmlist)
						window->ToggleDesignFlag(DatabaseVisWindow::RENDER_MODEL_LIST);
					break;
				}
				default:
					std::cout<<"[ERROR] Unknown multi-model shot: "<<shot.m_ShotType<<std::endl;
					assert(false);
			}
		}
		else	// rendering single model
		{
			// read model / get model from database
			bool deleteModel = false;

			if (shot.m_InputModel.m_ModelInputType == ShotModelInput::MODELINPUT_File)
			{
				deleteModel = true;
				model = Model3D::CreateFromFile(shot.m_InputModel.m_sModelFile, 0xFFFFFFFF, 0, 
												shot.m_InputModel.m_sModelName);
				int flags = (VisModel3D::DRAW_FACES | VisModel3D::DRAW_MESHES);
				modelVis = new VisModel3D(model, flags);
			}
			else if (shot.m_InputModel.m_ModelInputType == ShotModelInput::MODELINPUT_NameInDB)
			{
				assert(m_pDB!=NULL);
				model = m_pDB->GetModel(shot.m_InputModel.m_sModelName);
				assert((int)window->GetDrawableModels().size()==m_pDB->NumModels());
				int mID = model->ModelID();
				assert(mID < (int)window->GetDrawableModels().size());
				modelVis = window->GetDrawableModels()[mID];
			}

			assert (model!=NULL && modelVis!=NULL);	
			// prepare model for rendering
			modelVis->AddRenderableGeometry(shot.m_InputModel.m_sGeometryName);
			int cam0 = modelVis->CreateNewCamera();
			modelVis->Scale(cam0, -.2, window);
	 
			// todo: use shot's camera...
			
			// prepare shot-specific things...
			int interpAsNodeID = VisModel3D::LABELS_OPTVAL_IS_NODEID;
			window->SetAlpha(1.);
			switch(shot.m_ShotType)
			{
				case ShotDescription::SHOTTP_ModelPart: 
					window->SetAlpha(.2);
					modelVis->AddRenderableGeomValue(shot.m_sLabelsName, interpAsNodeID+shot.m_iSegmentID);
					break;
				case ShotDescription::SHOTTP_Segmentation:
					modelVis->AddRenderableGeomValue(shot.m_sLabelsName, shot.m_iSegmentID);
					break;
				case ShotDescription::SHOTTP_Textured:
					modelVis->SetFlag(VisModel3D::DRAW_MATERIALS, true);
					break;
				case ShotDescription::SHOTTP_WholeModel:
					break;
				default:
					std::cout<<"[ERROR] Unknown single-model shot: "<<shot.m_ShotType<<std::endl;
					assert(false);
			}
			
			// draw model
			modelVis->Draw(window, cam0);
			
//			// cleanup
//			if (deleteModel)
//			{
//				delete model;
//				delete modelVis;
//			}
		}
		
		// write screenshots
		BufferID & bufferID = window->EndDrawToBuffer();
		
		for (int j=0; j<(int)shot.m_OutputFiles.count(); j++)
		{
			VKString filename = shot.m_OutputFiles[j];
			//WriteCurrentImage(bufferID, filename, window, m_bCrop);
			shot.m_Image = GetCurrentImage(bufferID, window, &(shot.x0), &(shot.y0), &(shot.x1), &(shot.y1));
			x0 = vkMin(shot.x0, x0);
			x1 = vkMax(shot.x1, x1);
			y0 = vkMin(shot.y0, y0);
			y1 = vkMax(shot.y1, y1);
		}
	}
	
	std::vector<int> x0grp, x1grp, y0grp, y1grp;
	if (m_CropType==CROP_AskAppGroups || m_CropType==CROP_AskAppCopyFirst)
	{
		for (int i=0; i<m_ShotsQueue.size(); i++)
		{
			ShotDescription & shot = m_ShotsQueue[i];
			assert(shot.m_pApp!=NULL);
			int gid = shot.m_pApp->GroupID(i);
			while (gid >= x0grp.size())
			{
				x0grp.push_back(window->Width());
				x1grp.push_back(0);
				y0grp.push_back(window->Height());
				y1grp.push_back(0);
			}
			if (m_CropType==CROP_AskAppGroups || (m_CropType==CROP_AskAppCopyFirst && x0grp[gid]==window->Width()))
			{
				x0grp[gid] = vkMin(shot.x0, x0grp[gid]);
				x1grp[gid] = vkMax(shot.x1, x1grp[gid]);
				y0grp[gid] = vkMin(shot.y0, y0grp[gid]);
				y1grp[gid] = vkMax(shot.y1, y1grp[gid]);
			}
		}
	}
	
	for (int i=0; i<m_ShotsQueue.size(); i++)
	{
		ShotDescription & shot = m_ShotsQueue[i];

		if (shot.m_Image!=NULL)
		{
			for (int j=0; j<(int)shot.m_OutputFiles.count(); j++)
			{
				if (m_CropType==CROP_None)
					WriteCroppedImage(shot.m_Image, shot.m_OutputFiles[j], window, 0, 0, window->Width()-1, window->Height()-1);
				else if (m_CropType==CROP_PerImage)
					WriteCroppedImage(shot.m_Image, shot.m_OutputFiles[j], window, shot.x0, shot.y0, shot.x1, shot.y1);
				else if (m_CropType==CROP_Joint)
					WriteCroppedImage(shot.m_Image, shot.m_OutputFiles[j], window, x0, y0, x1, y1);
				else if (m_CropType==CROP_AskAppGroups || m_CropType==CROP_AskAppCopyFirst)
				{
					int gid = shot.m_pApp->GroupID(i);
					assert(gid>=0 && gid < (int)x0grp.size());
					//std::cout<<"\nWrite Image: ["<<x0grp[gid]<<" "<<y0grp[gid]<<" "<<x1grp[gid]<<" "<<y1grp[gid]<<"]";
					//std::cout<<" - "<<shot.m_OutputFiles[j].c_str()<<std::endl;
					
					WriteCroppedImage(shot.m_Image, shot.m_OutputFiles[j], window, x0grp[gid], y0grp[gid], x1grp[gid], y1grp[gid]);
				}
				else
					assert(false);
			}
		}
		else if (shot.m_OutputFiles.count()>0)
		{
			std::cout<<"[ERROR] Null image object (perhaps no content in image): "<<shot.m_OutputFiles[0].c_str()<<std::endl;
		}
	}
	
	
	if (m_ShotsQueue.size()>1)
	{
		std::cout<<std::endl;
		m_ShotsQueue[0].m_pApp->FinishedScreenshots(this);
	}
	if (m_bTerminateOnceDone)
	{
		window->GetRenderer()->TerminateWindow();
		//exit(0);
	}
	else if (m_ShotsQueue.size()>0 && m_ShotsQueue[0].m_pApp!=NULL)
	{
		// undo rendering flags
		m_ShotsQueue[0].m_pApp->SetRenderingFlags(app2InitFlags[m_ShotsQueue[0].m_pApp]); 
		window->SetApp(m_ShotsQueue[0].m_pApp);
	}
	m_ShotsQueue.clear();
	window->GetRenderer()->UpdateScreen();
	return true;
}

//void AppScreenshots::WriteCurrentImage(BufferID & bufferID, const VKString & filename, DatabaseVisWindow * window,
//									   bool cropToWhite, int x0, int y0, int x1, int y1)
//{
//	if (x0<0)	x0 = 0;
//	if (x1<0)	x1 = window->Width()-1;
//	if (y0<0)	y0 = 0;
//	if (y1<0)	y1 = window->Height()-1;
//		
//	assert(x0>=0 && x0 < window->Width());
//	assert(x1>=0 && x1 < window->Width());
//	assert(y0>=0 && y0 < window->Height());
//	assert(y1>=0 && y1 < window->Height());
//	assert(window->Width()==bufferID.m_iWidth && window->Height()==bufferID.m_iHeight);
//	double minContX = x1;
//	double maxContX = x0;
//	double minContY = y1;
//	double maxContY = y0;
//	
//	double br, bg, bb;
//	window->GetBackgroundColor(&br, &bg, &bb);
//	unsigned char cbr, cbg, cbb;
//	cbr = (unsigned char)(255 * br);
//	cbg = (unsigned char)(255 * bg);
//	cbb = (unsigned char)(255 * bb);
//	//std::cout<<"Background: ["<<(int)cbr<<" "<<(int)cbg<<" "<<(int)cbb<<"]"<<std::endl;
//	
//	for (int i=x0; i<=x1; i++)
//	for (int j=y0; j<=y1; j++)	
//	{
//		int id = (j * bufferID.m_iWidth + i) * VisWindow::BUFFER_COLOR_COMPONENTS;
//		unsigned char r = bufferID.m_aColorBuffer[id+0];
//		unsigned char g = bufferID.m_aColorBuffer[id+1];
//		unsigned char b = bufferID.m_aColorBuffer[id+2];
//		//bool zero = (r==255 && g==255 && b==255);
//		bool zero = (r==cbr && g==cbg && b==cbb);
//		//bool zero = false;
//		if (!zero || !cropToWhite)
//		{
//			minContX = vkMin(minContX, i);
//			maxContX = vkMax(maxContX, i);
//			minContY = vkMin(minContY, j);
//			maxContY = vkMax(maxContY, j);
//		}
//	}
//	
//	// add some padding around the object
//	x0 = vkMax(x0, minContX-10);
//	x1 = vkMin(x1, maxContX+10);
//	y0 = vkMax(y0, minContY-10);
//	y1 = vkMin(y1, maxContY+10);
//	if (!(x1 > x0 && y1 > y0))
//	{
//		std::cout<<"[WARNING] No content in image: "<<filename.c_str()<<std::endl;
//		return;
//	}
//	R2Image img(x1-x0+1, y1-y0+1);
//	for (int i=x0; i<=x1; i++)
//	for (int j=y0; j<=y1; j++)
//	{
//		int id = (j * bufferID.m_iWidth + i) * VisWindow::BUFFER_COLOR_COMPONENTS;
//		unsigned char r = bufferID.m_aColorBuffer[id+0];
//		unsigned char g = bufferID.m_aColorBuffer[id+1];
//		unsigned char b = bufferID.m_aColorBuffer[id+2];
//		RNRgb rgb((double)r / 255., (double)g / 255., (double)b / 255.);
//		img.SetPixelRGB(i-x0, j-y0, rgb);
//	}
//	
//	FileManagement::CreateDirsOnPathToFile(filename.c_str());
//		
//	//std::cout<<"Writing image: "<<filename.c_str()<<std::endl;
//	img.Write(filename.c_str());
//	//std::cout<<" -- done!"<<std::endl;
//}

R2Image * AppScreenshots::GetCurrentImage(BufferID & bufferID, DatabaseVisWindow * window,
										  int * x0out, int * y0out, int * x1out, int * y1out)
{
	*x0out = 0;
	*y0out = 0;
	*x1out = 0;
	*y1out = 0;
	
	int x0=0;
	int x1=window->Width()-1;
	int y0=0;
	int y1=window->Height()-1;
	
	assert(x0>=0 && x0 < window->Width());
	assert(x1>=0 && x1 < window->Width());
	assert(y0>=0 && y0 < window->Height());
	assert(y1>=0 && y1 < window->Height());
	assert(window->Width()==bufferID.m_iWidth && window->Height()==bufferID.m_iHeight);
	double minContX = x1;
	double maxContX = x0;
	double minContY = y1;
	double maxContY = y0;
	
	double br, bg, bb;
	window->GetBackgroundColor(&br, &bg, &bb);
	unsigned char cbr, cbg, cbb;
	cbr = (unsigned char)(255 * br);
	cbg = (unsigned char)(255 * bg);
	cbb = (unsigned char)(255 * bb);
	//std::cout<<"Background: ["<<(int)cbr<<" "<<(int)cbg<<" "<<(int)cbb<<"]"<<std::endl;
	
	for (int i=x0; i<=x1; i++)
	for (int j=y0; j<=y1; j++)
	{
		int id = (j * bufferID.m_iWidth + i) * VisWindow::BUFFER_COLOR_COMPONENTS;
		unsigned char r = bufferID.m_aColorBuffer[id+0];
		unsigned char g = bufferID.m_aColorBuffer[id+1];
		unsigned char b = bufferID.m_aColorBuffer[id+2];
		//bool zero = (r==255 && g==255 && b==255);
		bool zero = (r==cbr && g==cbg && b==cbb);
		//bool zero = false;
		if (!zero)
		{
			minContX = vkMin(minContX, i);
			maxContX = vkMax(maxContX, i);
			minContY = vkMin(minContY, j);
			maxContY = vkMax(maxContY, j);
		}
	}
	
	// add some padding around the object
	x0 = vkMax(x0, minContX-10);
	x1 = vkMin(x1, maxContX+10);
	y0 = vkMax(y0, minContY-10);
	y1 = vkMin(y1, maxContY+10);
	if (!(x1 > x0 && y1 > y0))
	{
		//std::cout<<"[WARNING] No content in image: "<<filename.c_str()<<std::endl;
		return NULL;
	}
	
	R2Image * fullImg = new R2Image(window->Width(), window->Height());
	for (int i=0; i<window->Width(); i++)
	for (int j=0; j<window->Height(); j++)
	{
		int id = (j * bufferID.m_iWidth + i) * VisWindow::BUFFER_COLOR_COMPONENTS;
		unsigned char r = bufferID.m_aColorBuffer[id+0];
		unsigned char g = bufferID.m_aColorBuffer[id+1];
		unsigned char b = bufferID.m_aColorBuffer[id+2];
		RNRgb rgb((double)r / 255., (double)g / 255., (double)b / 255.);
		fullImg->SetPixelRGB(i, j, rgb);
	}
		
	*x0out = x0;
	*x1out = x1;
	*y0out = y0;
	*y1out = y1;
	return fullImg;
}

void AppScreenshots::WriteCroppedImage(R2Image * img, const VKString & filename, DatabaseVisWindow * window,
									   int x0, int y0, int x1, int y1)
{
	//std::cout<<"Writing Screenshot: ["<<x0<<" "<<y0<<"] - ["<<x1<<" "<<y1<<"]"<<std::endl;
	R2Image * cropped = new R2Image(x1-x0+1, y1-y0+1);
	for (int i=x0; i<=x1; i++)
	for (int j=y0; j<=y1; j++)
		cropped->SetPixelRGB(i-x0, j-y0, img->PixelRGB(i, j));
	
	FileManagement::CreateDirsOnPathToFile(filename.c_str());
	cropped->Write(filename.c_str());
	delete cropped;
}

ShotDescription & AppScreenshots::GetShot(int shotID)
{
	assert(shotID>=0 && shotID < (int)m_ShotsQueue.size());
	return m_ShotsQueue[shotID];
}

void AppScreenshots::WriteHTMLHeader(std::ofstream & textStream,
									 const VKString & title)
{
	textStream<<"<html><head><title>"<<title.c_str()<<"</title></head><body>\n";
}

void AppScreenshots::WriteHTMLFooter(std::ofstream & textStream)
{
	textStream<<"</body></html>\n";
}

void AppScreenshots::WriteHTMLTableBegin(std::ofstream & textStream, int border)
{
	textStream<<"<table border="<<border<<">";
}

void AppScreenshots::WriteHTMLTableEnd(std::ofstream & textStream)
{
	textStream<<"</table>\n";
}

void AppScreenshots::WriteHTMLTableRow(std::ofstream & textStream, 
									   const VKStringList & entries, 
									   const std::vector<int> & width)
{
	textStream<<"<tr>\n";
	for (int i=0; i<entries.count(); i++)
	{
		if (i < (int)width.size() && width[i]>0)
			textStream<<"\t<td width="<<width[i]<<">"<<entries[i].c_str()<<"</td>\n";
		else
			textStream<<"\t<td>"<<entries[i].c_str()<<"</td>\n";
	}
	textStream<<"</tr>\n";
}

void AppScreenshots::WriteHTMLListTable(std::ofstream & textStream, 
										const VKStringList & images,
										const VKStringList & text,
										int imgWidth, int gridCols)
{
	WriteHTMLTableBegin(textStream, 1);
	int globalID = 0;
	VKString widthStr = "";
	if (imgWidth>0)
		widthStr = VKString(" width=")+VKString::number(imgWidth);
	while (globalID < images.count())
	{
		VKStringList entries;
		for (int i=0; i<gridCols; i++)
		{
			int gID = globalID+i;
			if (gID >= images.count())
				break;
			
			VKString imgStr=VKString("<a href=")+images[gID]+">";
			imgStr = imgStr + "<img src=\""+images[gID]+"\""+widthStr+"></a>";
			if (gID < text.count())
				imgStr += VKString("<br>") + text[gID];
			entries.push_back(imgStr);
		}
		WriteHTMLTableRow(textStream, entries);
		globalID += gridCols;
	}
	WriteHTMLTableEnd(textStream);
}

void AppScreenshots::WriteHTMLListTables(const VKString & fileprefix, 
										 const VKStringList & images, 
										 const VKStringList & text, 
										 int nMaxEntriesPerFile,
										 const VKString & title,
										 int imgWidth, int gridCols)
{
	std::ofstream textStream((fileprefix+".html").c_str());
	assert(textStream.is_open());
	WriteHTMLHeader(textStream, title);
	if (images.count() <= nMaxEntriesPerFile)
	{
		WriteHTMLListTable(textStream, images, text, imgWidth, gridCols);
		return;
	}
	
	int globalID = 0;
	int fileCounter=0;
	while (globalID < images.count())
	{
		VKString filename = fileprefix + VKString::number(fileCounter)+".html";
		VKString relFilename = VKString("./")+filename.FileName()+filename.FileExtension();
		int toID = vkMin(images.count(), globalID+nMaxEntriesPerFile);
		textStream<<"<a href=\""<<relFilename.c_str();
		textStream<<"\">Images "<<globalID<<"-"<<toID<<"</a><br>\n";
		
		VKStringList currImages;
		VKStringList currText;
		
		for (int i=globalID; i<toID; i++)
		{
			currImages.push_back(images[i]);
			currText.push_back(text[i]);
		}
		std::ofstream currTextStream(filename.c_str());
		assert(currTextStream.is_open());
		WriteHTMLListTable(currTextStream, currImages, currText, imgWidth, gridCols);
		currTextStream.close();
		globalID += nMaxEntriesPerFile;
		fileCounter++;
	}
	WriteHTMLFooter(textStream);
	textStream.close();
}



