/***************************************************************************
 *   Copyright (C) 2008 by Vladimir Kim									   *
 *   Author: Vladimir Kim (vk@princeton.edu)							   *
 ***************************************************************************/

#ifndef __AppScreenshots_H
#define __AppScreenshots_H

#include <vector>
#include <map>
#include <iostream>
#include <assert.h>

#include "VKString.h"
#include "DBApp.h"
#include "ModelDatabase.h"
#include "DBCorrs.h"
#include "ShapeCorrs.h"
//#include "AppSandbox.h"

struct ShotModelInput
{
	ShotModelInput();
	enum ShotModelInputType
	{
		MODELINPUT_NoModel = 0,
		MODELINPUT_File = 1,
		MODELINPUT_NameInDB = 2
	};
	
	ShotModelInputType m_ModelInputType;
	VKString m_sModelName;
	VKString m_sModelFile;
	VKString m_sGeometryName;
};

struct ShotDescription
{
	enum ShotType
	{
		SHOTTP_Invalid,
		SHOTTP_WholeModel,
		SHOTTP_Segmentation,
		SHOTTP_Textured,
		SHOTTP_ModelPart,
		SHOTTP_VolumeDensityGrid,
		SHOTTP_AnyApp
	};
	
	ShotDescription();
		
	ShotType m_ShotType;
	ShotModelInput m_InputModel;
	
	VKStringList m_OutputFiles;	// can print multiple copies of the same screenshot
	
	// todo: camera
	R3Affine m_ObjectXForm;
	
	// some optional parameters
		// (single) part visualization
	VKString m_sLabelsName;
	int m_iSegmentID;
	
		// volume file
	VKString m_sVolumeFile;
	
	DBApp * m_pApp;
	int m_iAppVisFlags;
	
	R2Image * m_Image;
	int x0, y0, x1, y1;
};

class AppScreenshots : public DBApp
{
	public:
		enum CropType
		{
			CROP_None,
			CROP_PerImage,
			CROP_Joint,
			CROP_AskAppGroups,		// cropping is grouped for images (groups defined by the app)
			CROP_AskAppCopyFirst	// copy cropping boundaries from another image (in group)
		};
		
		AppScreenshots(bool terminateAfterDone, ModelDatabase * db = NULL, DBCorrs * dbCorrs = NULL);
		virtual ~AppScreenshots();
		virtual void TakeShotOfModels(const VKStringList & modelFiles, const VKStringList & modelNames,
									  const std::vector<VKStringList> & outFiles);
		virtual void TakeShotOfModel(const VKString & geomName, const VKString & outFile,
									 const R3Affine & camera = R3identity_affine);
		virtual void TakeShotOfSeg(const VKString & geomName, const VKString & outFile,
								   const VKString & labelsName = ShapeLabels::m_sDefaultSceneGraphLabels,
								   int level = -1, const R3Affine & camera = R3identity_affine);
		virtual void TakeShotOfTextured(const VKString & geomName, const VKString & outFile,
										const R3Affine & camera = R3identity_affine);
		virtual void TakeShotOfPart(const VKString & geomName, const VKString & labelsName, 
									const VKString & keyword, const VKString & matchingType, 
									const VKString & outFile, const R3Affine & camera = R3identity_affine);
		virtual void TakeShotOfVolume(const VKString & gridFile, const VKString & outFile,
									  const VKString & mName, const R3Affine & camera = R3identity_affine);
		virtual void TakeShotOfApp(DBApp * app, const VKString & shotType, int visFlags, const VKString & outFile);
		virtual bool Draw(DatabaseVisWindow * window);
		virtual void SetCrop(CropType crop);
	
//		static void WriteCurrentImage(BufferID & bufferID, const VKString & filename, 
//									  DatabaseVisWindow * window, bool cropToWhite = false,
//									  int x0 = -1, int y0 = -1, int x1 = -1, int y1 = -1);
		static R2Image * GetCurrentImage(BufferID & bufferID, DatabaseVisWindow * window,
										 int * x0, int * y0, int * x1, int * y1);
		static void WriteCroppedImage(R2Image * img, const VKString & filename, DatabaseVisWindow * window,
									  int x0 = -1, int y0 = -1, int x1 = -1, int y1 = -1);
		virtual ShotDescription & GetShot(int shotID);

	// html stuff
		static void WriteHTMLHeader(std::ofstream & textStream, const VKString & title);
		static void WriteHTMLFooter(std::ofstream & textStream);
		static void WriteHTMLTableBegin(std::ofstream & textStream, int border);
		static void WriteHTMLTableEnd(std::ofstream & textStream);
		static void WriteHTMLTableRow(std::ofstream & textStream, const VKStringList & entries,
									  const std::vector<int> & width = std::vector<int>());
	
		static void WriteHTMLListTable(std::ofstream & textStream, const VKStringList & images,
									   const VKStringList & text, int imgWidth = -1, int gridCols = 6);
	
		static void WriteHTMLListTables(const VKString & fileprefix, const VKStringList & images, 
										const VKStringList & text, int nMaxEntriesPerFile,
										const VKString & title="", int imgWidth=-1, int gridCols = 6);
	protected:
		bool m_bTerminateOnceDone;
		CropType m_CropType;

		std::vector<ShotDescription> m_ShotsQueue;
	
	
	// rendering instructions (directly from files)
//		VKStringList m_ModelFiles;
//		VKStringList m_ModelNames;
//		std::vector<VKStringList> m_OutputFiles;
		
};


#endif

