/***************************************************************************
 *   Copyright (C) 2008 by Vladimir Kim									   *
 *   Author: Vladimir Kim (vk@princeton.edu)							   *
 ***************************************************************************/

#include "DistEuclidean.h"
#include "ShapeMesh.h"
#include "ShapePointCloud.h"

DistEuclidean::DistEuclidean(GeomShape * shape, const VKString & metricName, double norm)
: ShapeDistance(shape, metricName)
{
	m_fNorm = norm;
}

DistEuclidean::~DistEuclidean()
{
	
}

DistEuclidean * DistEuclidean::SafeCastToEuclidean()
{
	return this;
}

double DistEuclidean::NormalizedDistance(double distance) const
{
	if (m_fNorm<0)
	{
		ShapeMesh * mesh = m_pShape->SafeCastToMesh();
		ShapePointCloud * pnts = m_pShape->SafeCastToPointCloud();
		if (mesh!=NULL)
			((double&)m_fNorm) = mesh->AverageRadius(mesh->Centroid());
		else if (pnts!=NULL)
			((double&)m_fNorm) = pnts->AverageRadius(pnts->Centroid());
		else
			assert(false);
	}
	
	return distance / m_fNorm;
}

double DistEuclidean::NormalizedDistance(const ShapePoint & p1, 
											 const ShapePoint & p2) const
{
	return NormalizedDistance(R3Distance(p1.GetPosition(), p2.GetPosition()));
}

void DistEuclidean::NearestNeighborhood(const ShapePoint & p,  double distanceThreshold,
										std::vector<ShapePoint> * neighborhood, std::vector<double> * normDists) const
{
	ShapePointCloud * pnts = m_pShape->SafeCastToPointCloud();
	assert(pnts!=NULL && neighborhood!=NULL);
	const R3Kdtree<const PointInSet *> * kdtree = pnts->GetKdTree();
	RNArray<const PointInSet*> nhd;
	kdtree->FindAll(p.GetPosition(), 0, distanceThreshold, nhd);
	for (int i=0; i<nhd.NEntries(); i++)
	{
		neighborhood->push_back(pnts->GetPoint(nhd[i]->id));
		if (normDists!=NULL)
			normDists->push_back(R3Distance(nhd[i]->GetPosition(), pnts->PointPosition(nhd[i]->id)));
	}
}

ShapePoint DistEuclidean::NearestNeighbor(const ShapePoint & p, double* normDists) const
{
	ShapePointCloud * pnts = m_pShape->SafeCastToPointCloud();
	assert(pnts!=NULL);
	const R3Kdtree<const PointInSet *> * kdtree = pnts->GetKdTree();
	const PointInSet* closest = kdtree->FindClosest(p.GetPosition());
	assert(closest!=NULL);
	if (normDists!=NULL)
		*normDists = R3Distance(p.GetPosition(), closest->GetPosition());
	return pnts->GetPoint(closest->id);
}



