/***************************************************************************
 *   Copyright (C) 2008 by Vladimir Kim									   *
 *   Author: Vladimir Kim (vk@princeton.edu)							   *
 ***************************************************************************/

#ifndef __DistGeodPnts_H
#define __DistGeodPnts_H

#include <vector>
#include <map>
#include <iostream>
#include <assert.h>

#include "VKGraph.h"
#include "GeomShape.h"
#include "ShapeDistance.h"

class DistGeodPnts : public ShapeDistance
{
	public:
		DistGeodPnts(ShapePointCloud * densepnts, const VKString & metricName, double norm = 1);
		virtual ~DistGeodPnts();
		virtual DistGeodPnts * SafeCastToGraphNN();
	
		virtual void InitNNGraph(double distThreshold, int minNN, int maxNN);	// needs to be called
	
		virtual double NormalizedDistance(const ShapePoint & p1, const ShapePoint & p2) const;
		virtual void NearestNeighborhood(const ShapePoint & p,  double distanceThreshold,
										 std::vector<ShapePoint> * neighborhood, std::vector<double> * normDists = NULL) const;
		virtual ShapePoint NearestNeighbor(const ShapePoint & p, double* normDists = NULL) const;
		virtual const VKGraph * GetGraph() const;

		virtual void WriteMetric(const VKString & filename) const;
		virtual void ReadMetric(FILE * file);


	protected:
		ShapePointCloud * m_pPnts;
		VKGraph * m_pGraph;
};

#endif

