/***************************************************************************
 *   Copyright (C) 2008 by Vladimir Kim									   *
 *   Author: Vladimir Kim (vk@princeton.edu)							   *
 ***************************************************************************/

#ifndef __DistGeodesicDijkstra_H
#define __DistGeodesicDijkstra_H

#include <vector>
#include <map>
#include <iostream>
#include <assert.h>

#include "GeomShape.h"
#include "ShapeDistance.h"

class DistGeodesicDijkstra : public ShapeDistance
{
	public:
		struct GeoVertexData 
		{
			R3MeshVertex *vertex;
			double distance;
			RNBoolean selected;
			GeoVertexData **heappointer;
		};	

		DistGeodesicDijkstra(GeomShape * shape, const VKString & metricName, int cacheSize = 5000, double norm = -1);
		virtual ~DistGeodesicDijkstra();
		virtual DistGeodesicDijkstra * SafeCastToGeodesic();
	
	
		virtual double NormalizedDistance(double distance) const;
		virtual double NormalizedDistance(const ShapePoint & p1, const ShapePoint & p2) const;
	
	// cached distances
		virtual bool IsPrecomputed(int vertexID) const;
		virtual double VertexDistance(int v1, int v2) const;
		virtual void ComputeVertexPaths(int v1, std::vector<int> & prevVtx) const;
		virtual void PrecomputeRow(int vertexID);
		virtual const double * GetPrecomputedRow(int vID, int * rowSize) const;
	
	protected:
		R3Mesh * m_pManifoldMesh;
	
		
		int m_iMaxNumCached;		
		std::vector<int> m_AddedVertices;
		int * m_aVertexIDToRow;
		double ** m_PrecomputedRows;
	
		GeoVertexData * m_aPrecompData;
		
		// Static functions: Dijkstra (Tom's implementation)
		static GeoVertexData * InitPrecomputation(R3Mesh * mesh, GeoVertexData * exists=NULL);
	
		static void PrecomputeFill(GeoVertexData * vertex_data, R3Mesh * mesh, int fromVertex,
								   double * toDistanceArray, const std::vector<int> & toVerticesRemember=std::vector<int>());
	
		static void FillThresholded(GeoVertexData * vertex_data, R3Mesh * mesh, int fromVertex,
									std::map<int, double> & verticesWithValues, double maxDistance);
	
		static double GetThresholded(int toVertex, R3Mesh * mesh, std::map<int, double> & verticesWithValues, double maxDistance);
};

#endif

