/***************************************************************************
 *   Copyright (C) 2008 by Vladimir Kim									   *
 *   Author: Vladimir Kim (vk@princeton.edu)							   *
 ***************************************************************************/
#include "ShapeDistance.h"
#include "DistEuclidean.h"
#include "DistGeodesicDijkstra.h"
#include "DistGeodPnts.h"

VKString ShapeDistance::m_sDefaultEuclidean = "eucl";
VKString ShapeDistance::m_sDefaultGeodesic = "geod";

ShapeDistance::ShapeDistance(GeomShape * shape, const VKString & distMetric)
{
	m_pShape = shape;
	m_sDistMetricName = distMetric;
	m_fMetersPerUnit = -1;
	m_fNorm = -1;
}

ShapeDistance::~ShapeDistance()
{
	
}

const VKString & ShapeDistance::MetricName()
{
	return m_sDistMetricName;
}

const GeomShape * ShapeDistance::Geometry()
{
	return m_pShape;
}
bool ShapeDistance::SameGeometry(GeomShape * geom)
{
	return Geometry()==geom;
}

DistEuclidean * ShapeDistance::SafeCastToEuclidean()
{
	return NULL;
}

DistGeodesicDijkstra * ShapeDistance::SafeCastToGeodesic()
{
	return NULL;
}

DistGeodPnts * ShapeDistance::SafeCastToGraphNN()
{
	return NULL;
}

void ShapeDistance::NearestNeighborhood(const ShapePoint & p,  double distanceThreshold,
								 std::vector<ShapePoint> * neighborhood, std::vector<double> * normDists) const
{
	assert(false);
}

ShapePoint ShapeDistance::NearestNeighbor(const ShapePoint & p, double* normDists) const
{
	assert(false);
}

double ShapeDistance::NormalizedDistance(double distance) const
{
	assert(m_fNorm>0);
	return distance / m_fNorm;
}

void ShapeDistance::SetUnitToMeters(double metersPerUnit)
{
	m_fMetersPerUnit = metersPerUnit;
}

double ShapeDistance::ConvertToMeters(double distance) const
{
	if (m_fMetersPerUnit<0)
		return -1;	// invalid -- conversion not provided

	return distance * m_fMetersPerUnit;
}

double ShapeDistance::ConvertFromMeters(double ciDistance) const
{
	return ciDistance / m_fMetersPerUnit;
}

void ShapeDistance::WriteMetric(const VKString & filename) const
{
	FILE * f = fopen(filename.c_str(), "wb");
	assert(f!=NULL);
	WriteBasicMetricData(f);
	fclose(f);
}

void ShapeDistance::WriteBasicMetricData(FILE * file) const
{
	char tp='x';
	if (((ShapeDistance*)this)->SafeCastToEuclidean()!=NULL)
		tp = 'e';
	else if (((ShapeDistance*)this)->SafeCastToGeodesic()!=NULL)
		tp = 'g';
	else if (((ShapeDistance*)this)->SafeCastToGraphNN()!=NULL)
		tp = 'n';
	fwrite(&tp, sizeof(char), 1, file);
//	fwrite(&m_fNorm, sizeof(double), 1, file);
//	fwrite(&m_fMetersPerUnit, sizeof(double), 1, file);
}

void ShapeDistance::ReadMetric(FILE * file)
{
}

ShapeDistance * ShapeDistance::ReadMetric(const VKString & filename, GeomShape * shape,
										  const VKString & metricName, double norm)
{
	FILE * file = fopen(filename.c_str(), "rb");
	assert(file!=NULL);
	char tp='x';
	fread(&tp, sizeof(char), 1, file);
	ShapeDistance * dist = NULL;
	if (tp=='e')
		dist = new DistEuclidean(shape, metricName, norm);
	else if (tp=='g')
		dist = new DistGeodesicDijkstra(shape, metricName, 5000, norm);
	else if (tp=='n')
	{
		ShapePointCloud * pnts = shape->SafeCastToPointCloud();
		assert(pnts!=NULL);
		dist = new DistGeodPnts(pnts, metricName, norm);
	}
	
	assert(dist!=NULL);
	dist->ReadMetric(file);
	fclose(file);
	return dist;
}




