/***************************************************************************
 *   Copyright (C) 2008 by Vladimir Kim									   *
 *   Author: Vladimir Kim (vk@princeton.edu)							   *
 ***************************************************************************/

#ifndef __ShapeDistance_H
#define __ShapeDistance_H

#include <vector>
#include <map>
#include <iostream>
#include <assert.h>

#include "GeomShape.h"
#include "ShapePoint.h"

class DistEuclidean;
class DistGeodesicDijkstra;
class DistGeodPnts;

class ShapeDistance
{
	public:
		static VKString m_sDefaultEuclidean;
		static VKString m_sDefaultGeodesic;
	
		ShapeDistance(GeomShape * shape, const VKString & metricName = "none");
		virtual ~ShapeDistance();
	
		virtual DistEuclidean * SafeCastToEuclidean();
		virtual DistGeodesicDijkstra * SafeCastToGeodesic();
		virtual DistGeodPnts * SafeCastToGraphNN();
	
		const VKString & MetricName();
		virtual const GeomShape * Geometry();
		virtual bool SameGeometry(GeomShape * geom);
	
		virtual double NormalizedDistance(double distance) const;
	
		virtual double NormalizedDistance(const ShapePoint & p1, const ShapePoint & p2) const = 0;
		virtual void NearestNeighborhood(const ShapePoint & p,  double distanceThreshold,
										 std::vector<ShapePoint> * neighborhood, std::vector<double> * normDists = NULL) const;
		virtual ShapePoint NearestNeighbor(const ShapePoint & p, double* normDists = NULL) const;
	
		virtual void SetUnitToMeters(double metersPerUnit);
		virtual double ConvertToMeters(double distance) const;
		virtual double ConvertFromMeters(double ciDistance) const;
	
		virtual void WriteMetric(const VKString & filename) const;
		virtual void WriteBasicMetricData(FILE * file) const;
		virtual void ReadMetric(FILE * file);
		static ShapeDistance * ReadMetric(const VKString & filename, GeomShape * shape,
										  const VKString & metricName, double norm) ;
	
	protected:
		double m_fNorm;	
		GeomShape * m_pShape;
		VKString m_sDistMetricName;
	
		double m_fMetersPerUnit;
};

#endif

