/***************************************************************************
 *   Copyright (C) 2008 by Vladimir Kim									   *
 *   Author: Vladimir Kim (vk@princeton.edu)							   *
 ***************************************************************************/
#include "ShapeImg.h"

ShapeImg::ShapeImg(const VKString & geomName,
				   Model3D * model)
: GeomShape(GeomShape::SHAPE_TYPE_IMAGE, geomName, model)
{
	m_pImage = NULL; 
}

ShapeImg * ShapeImg::SafeCastToImg()
{
	return this;
}

void ShapeImg::FreeMemory()
{
	if (m_MemoryStatus == GeomShape::SHAPEMEM_IgnoreMemoryCommands)
		return;
	assert((m_pImage!=NULL) == (m_MemoryStatus == SHAPEMEM_Loaded));
	
	if (m_pImage!=NULL)
	{
		m_MemoryStatus = SHAPEMEM_NotLoaded;
		delete m_pImage;
		m_pImage = NULL;
	}
}

void ShapeImg::SetSrcFile(const VKString & filename)
{
	assert(m_pImage == NULL);
	m_MemoryStatus = SHAPEMEM_NotLoaded;
	m_sSrcFile = filename;
}

void ShapeImg::SetImg(R2Image * img)
{
	if (m_pImage!=NULL)
		delete m_pImage;
	m_pImage = img;
}

R2Image * ShapeImg::GetImage()
{
	ReadIntoMemory();
	assert(m_pImage!=NULL);
	return m_pImage;
}

void ShapeImg::WriteGeometryFile(const VKString & geomFile) const
{
	assert(m_pImage!=NULL);
	m_pImage->Write(geomFile.c_str());
}

void ShapeImg::ReadGeometryFile(const VKString & geomFile)
{
	if (m_pImage!=NULL)
		delete m_pImage;
	m_pImage = new R2Image(geomFile.c_str());
}


