/***************************************************************************
 *   Copyright (C) 2008 by Vladimir Kim									   *
 *   Author: Vladimir Kim (vk@princeton.edu)							   *
 ***************************************************************************/

#ifndef __ShapeInterpolator_H
#define __ShapeInterpolator_H

#include <vector>
#include <map>
#include <iostream>
#include <assert.h>

#include "GeomShape.h"
#include "ShapeDistance.h"

class ShapeInterpolator
{
	public:
		enum InterpolatorType
		{
			INTERP_Unknown,
			INTERP_P2PGaussian,
			INTERP_P2PNN
		};

		ShapeInterpolator(GeomShape * shapeFrom, GeomShape * shapeTo, InterpolatorType tp);
		virtual ~ShapeInterpolator();
	
		virtual void GetInterpWeights(const ShapePoint & pntOnToShape,
									  const std::vector<ShapePoint> ** savePntsOnFromShape,
									  const std::vector<double> ** saveWeights) const = 0;
	
		GeomShape * ShapeFrom();
		GeomShape * ShapeTo();
	
	protected:
		InterpolatorType m_Type;
		GeomShape * m_pShapeFrom;
		GeomShape * m_pShapeTo;
};

class ShapeInterpP2PGaussian : public ShapeInterpolator
{
	public:
		ShapeInterpP2PGaussian(ShapePointCloud * pntsFrom, ShapePointCloud * pntsTo, 
							   const VKString & distMetric = "auto", double sigma = -1);
		virtual ~ShapeInterpP2PGaussian();
	
		virtual void GetInterpWeights(const ShapePoint & pntOnToShape,
									  const std::vector<ShapePoint> ** savePntsOnFromShape,
									  const std::vector<double> ** saveWeights) const;
		virtual void PrecomputeDistances();

	protected:
		std::vector<double> * m_aWeights;
		std::vector<ShapePoint> * m_aNeighbors;
	
		bool m_bDistancesPrecomputed;
		double m_fSigma;
		ShapePointCloud * m_pPntsFrom;
		ShapePointCloud * m_pPntsTo;
		ShapeDistance * m_pDistanceMetric;
};

#endif

