/***************************************************************************
 *   Copyright (C) 2008 by Vladimir Kim									   *
 *   Author: Vladimir Kim (vk@princeton.edu)							   *
 ***************************************************************************/

#ifndef __ShapeMesh_H
#define __ShapeMesh_H

#include <vector>
#include <map>
#include <iostream>
#include <assert.h>
#include "Sortable.h"

#include "GeomShape.h"

class ShapeLabels;

class ShapeMesh : public GeomShape
{
	public:	
		ShapeMesh(GeomShape::GeomShapeTypes shapeTp, const VKString & geomName, Model3D * model);
		virtual ~ShapeMesh();
	
		virtual ShapeMesh * SafeCastToMesh();
	
	// mesh global
		virtual R3Point Centroid() const;
		virtual double AverageRadius(const R3Point & centroid) const;
		virtual R3Triad PCAFromFaces(const std::vector<int> & faces, 
									 double * variances = NULL, double * center = NULL) const;
		virtual R3Triad PCAFromFaces(double * variances = NULL) const;
	
		virtual R3Box BBox() const;
	
		virtual int AddVertex(double x, double y, double z,
							  double nx = 0, double ny = 0, double nz = 0) = 0;
		virtual int AddFace(int v1, int v2, int v3) = 0;
		virtual void RemoveFace(int f) = 0;
		virtual void RemoveFaces(int f1, int f2);
		virtual void RemoveVertex(int v) = 0;

	// vertices
		virtual int NVertices() const = 0;
		virtual void SetVertexPosition(int vID, const R3Point & newPos ) = 0;
		virtual R3Point VertexPosition(int vID) const = 0;
		virtual R3Vector VertexNormal(int vID) const = 0;
		virtual int FaceOnVertex(int vID) const = 0;
		virtual int NAdjacentFaces(int vID) const = 0;
		virtual int FaceOnVertex(int vID, int nFace) const = 0;
		virtual int VertexValence(int vID) const;
		virtual std::vector<int> AdjacentVertices(int vID) const = 0;
	
	// edges
		virtual int NEdges() const = 0;
	
	// faces
		virtual int NFaces() const = 0;
		virtual int VertexOnFace(int fID, int vID) const = 0;
		virtual R3Point FaceCentroid(int fID) const;
		virtual R3Point FacePoint(int fID, double b1, double b2) const = 0;
		virtual R3Point FaceBarycentric(int fID, const R3Point & p) const = 0;
		virtual R3Vector FaceNormal(int fID) const = 0;
		virtual double FaceArea(int fID) const = 0;
		virtual void AdjacentFaces(int fID, std::vector<int> & fids) const;
		virtual void EdgeBetweenFaces(int f1, int f2, int & v1, int & v2) const;
		virtual double Area() const;
		
	// nearest point
		virtual ShapePoint GetShapePointOnMesh(const R3Point & pnt) const;
		virtual R3Point GetPointPositionOnMesh(const R3Point & pnt) const;
	
	// connected components
//		virtual int GetConnectedComponents(std::vector<int> * vtx2cc = NULL, 
//										   std::vector<std::vector<int> > * cc2vtx = NULL) const;
		virtual const ShapeLabels * GetConnectedComponents() const;
	
//	protected:
		virtual R3MeshSearchTree * CreateSearchTree(R3Mesh ** mesh,
													std::vector<int> & triangleMap,
													std::vector<int> & vertexMap) const = 0;
		R3MeshSearchTree * m_pSearchTree;
		R3Mesh * m_pSearchTreeMesh;
		std::vector<int> m_SearchTreeToTriangleMap;
		std::vector<int> m_SearchTreeToVertexMap;
};

#endif

