#include "Binding.h"
#include "Interactor.h"

/////////////////
/// Binding
/////////////////
Binding::Binding(Interactor * actor, Interactor *object)
{
	m_Interactors[Interactor::ACT_Actor] = actor;
	m_Interactors[Interactor::ACT_Object] = object;
	for (int i=0; i<2; i++)
	{
		//SetPoseID((Interactor::InteractorID) i, 0);
		m_aPoses[i] = NULL;
		m_apXFormInteractor[i] = NULL;
		SetXForm((Interactor::InteractorID) i, new RigidAlign());
	}
	m_fEnergy = -1;
}

void Binding::SetPose(Interactor::InteractorID id, const InteractorPose * pose)
{
	m_aPoses[id] = pose;
}

 void Binding::SetXForm(Interactor::InteractorID id, BindingXForm * xformActor)
{
//	if (m_apXFormInteractor[id]!=NULL)
//		delete m_apXFormInteractor[id];
	m_apXFormInteractor[id] = xformActor;
}

void Binding::SetEnergy(double energy)
{
	m_fEnergy = energy;
}

double Binding::GetEnergy() const
{
	return m_fEnergy;
}

const Interactor * Binding::GetActor(int id) const
{
	return m_Interactors[id];
}

const InteractorPose * Binding::GetPose(Interactor::InteractorID id) const
{
	return m_aPoses[id];
}

const BindingXForm * Binding::GetBindingXForm(Interactor::InteractorID id) const
{
	return m_apXFormInteractor[id];	
}

BindingXForm * Binding::GetBindingXForm(Interactor::InteractorID id)
{
	return m_apXFormInteractor[id];	
}

void Binding::WriteBinding(FILE * file) const
{
	fwrite(&m_fEnergy, sizeof(double), 1, file);
	
	for (int i=0; i<2; i++)
		m_apXFormInteractor[i]->WriteXForm(file);
	
	m_Interactors[Interactor::ACT_Actor]->WritePoseToFile(file, m_aPoses[Interactor::ACT_Actor]);
	m_Interactors[Interactor::ACT_Object]->WritePoseToFile(file, m_aPoses[Interactor::ACT_Object]);
}

Binding * Binding::ReadBinding(FILE * file, Interactor * actor, Interactor* object)
{
	Binding * b = new Binding(actor, object);
	fread(&(b->m_fEnergy), sizeof(double), 1, file);
	for (int i=0; i<2; i++)
		b->m_apXFormInteractor[i] = BindingXForm::CreateXForm(file);
	b->m_aPoses[Interactor::ACT_Actor] = actor->CreatePoseFromFile(file, object);
	b->m_aPoses[Interactor::ACT_Object] = object->CreatePoseFromFile(file, NULL);
	return b;
}

/////////////////
/// BindingXForm
/////////////////
BindingXForm * BindingXForm::CreateXForm(FILE * file)
{
	BindingXForm * binding = NULL;
	VKString className = VKString::ReadString(file);
	if (className=="RigidAlign")
		binding = new RigidAlign();
	else
		std::cout<<"[ERROR] Invalid class: "<<className.c_str()<<std::endl;
	assert(binding!=NULL);
	binding->ReadXForm(file);
	return binding;
}

BindingXForm::~BindingXForm()
{
	
}

/////////////////
/// RigidAlign
/////////////////
RigidAlign::RigidAlign()
{
	SetTranslation(R3Vector(0,0,0));
	SetRotation(R3Vector(0,0,1), 0);
}

RigidAlign::RigidAlign(const R3Vector & translate, const R3Vector & axis, double angle)
{
	//SetXForm(translate, rotate);
	SetTranslation(translate);
	SetRotation(axis, angle);
}

RigidAlign * RigidAlign::SafeCastToRigidAlign()
{
	return this;
}

const RigidAlign * RigidAlign::SafeCastToRigidAlign() const
{
	return this;
}

void RigidAlign::WriteXForm(FILE * file) const
{
	VKString::WriteString("RigidAlign", file);
	double v[6];
	v[0] = m_Trans[0];		v[1] = m_Trans[1];		v[2] = m_Trans[2];
	v[3] = m_Axis[0];		v[4] = m_Axis[1];		v[5] = m_Axis[2];
	fwrite(v, sizeof(double), 6, file);
	fwrite(&m_fAngle, sizeof(double), 1, file);
}

void RigidAlign::ReadXForm(FILE * file) 
{
	m_bComputedTransform = false;
	double v[6];
	fread(v, sizeof(double), 6, file);
	m_Trans[0] = v[0];		m_Trans[1] = v[1];		m_Trans[2] = v[2];
	m_Axis[0]  = v[3];		m_Axis[1]  = v[4];		m_Axis[2]  = v[5];
	fread(&m_fAngle, sizeof(double), 1, file);
}


const R3Vector & RigidAlign::GetTranslation() const
{
	return m_Trans;
}

const R3Vector & RigidAlign::GetRotationAxis() const
{
	return m_Axis;
}

double RigidAlign::GetRotationAngle() const
{
	return m_fAngle;
}

void RigidAlign::SetTranslation(const R3Vector & translate)
{
	m_Trans = translate;
	m_bComputedTransform = false;
}

void RigidAlign::SetRotation(const R3Vector & axis, double angle)
{
	m_Axis = axis;
	if (m_Axis.Length()>0)
		m_Axis.Normalize();
	m_fAngle = angle;
	m_bComputedTransform = false;
}


//void RigidAlign::SetXForm(const R3Vector & translate, const R3Vector & rotate)
//{
//	m_Trans = translate;
//	m_Rotate = rotate;
//	m_bComputedTransform = false;
//}

const R3Affine & RigidAlign::GetXForm() const
{
	if (!m_bComputedTransform)
	{
		((RigidAlign*)this)->m_AffineXForm = R3identity_affine;
		((RigidAlign*)this)->m_AffineXForm.Translate(m_Trans);
		//((RigidAlign*)this)->m_AffineXForm.Rotate(m_Rotate);
		((RigidAlign*)this)->m_AffineXForm.Rotate(m_Axis, m_fAngle);
		((RigidAlign*)this)->m_bComputedTransform = true;
	}
	return m_AffineXForm;
}

void RigidAlign::Apply(R3Point & p) const
{
	GetXForm().Apply(p);
}

void RigidAlign::ApplyInverse(R3Point & p) const
{
	GetXForm().ApplyInverse(p);
}


