#include "VKString.h"
#include "Sortable.h"
#include "FileManagement.h"

#include "ModelDatabase.h"
#include "ShapeTriangleSoup.h"


#ifndef __Binding_H
#define __Binding_H

#include "Interactor.h"
#include "InteractorPose.h"

class BindingXForm;

/**
 * This class describes an actor or an object
 */
class Binding
{
	public:
		Binding(Interactor * actor, Interactor* object);
			
		virtual void SetPose(Interactor::InteractorID actor, const InteractorPose * pose);
		virtual void SetXForm(Interactor::InteractorID actor, BindingXForm * xformActor);
		virtual void SetEnergy(double energy);
	
		virtual const Interactor * GetActor(int id) const;
		virtual const InteractorPose * GetPose(Interactor::InteractorID id) const;
		virtual const BindingXForm * GetBindingXForm(Interactor::InteractorID id) const;
		virtual BindingXForm * GetBindingXForm(Interactor::InteractorID id);
		virtual double GetEnergy() const;
	
		virtual void WriteBinding(FILE * file) const;
		static Binding * ReadBinding(FILE * file, Interactor * actor, Interactor* object);
	
		
	protected:
		double m_fEnergy;
		BindingXForm * m_apXFormInteractor[2];
		//int m_PoseIDs[2];
		const InteractorPose * m_aPoses[2];
		Interactor * m_Interactors[2];
};

class RigidAlign;
/**
 * Binding transformation
 */
class BindingXForm
{
	public:
		virtual ~BindingXForm();
		virtual void Apply(R3Point & p) const = 0;
		virtual void ApplyInverse(R3Point & p) const = 0;
		virtual RigidAlign * SafeCastToRigidAlign(){return NULL;}
		virtual const RigidAlign * SafeCastToRigidAlign() const{return NULL;}
	
		virtual void WriteXForm(FILE * file) const = 0;
		virtual void ReadXForm(FILE * file) = 0;
		static BindingXForm * CreateXForm(FILE * file);
};

/**
 * Rigid alignment version of the binding transformation
 */
class RigidAlign : public BindingXForm
{
	public:
		RigidAlign();
		RigidAlign(const R3Vector & translate, const R3Vector & rotate, double angle);
		virtual RigidAlign * SafeCastToRigidAlign();
		virtual const RigidAlign * SafeCastToRigidAlign() const;
		virtual const R3Vector & GetTranslation() const;
		virtual const R3Vector & GetRotationAxis() const;
		virtual double GetRotationAngle() const;
		//virtual void SetXForm(const R3Vector & translate, const R3Vector & rotate);
		virtual void SetTranslation(const R3Vector & translate);
		virtual void SetRotation(const R3Vector & axis, double angle);
		virtual const R3Affine & GetXForm() const;
	 	virtual void Apply(R3Point & p) const;
		virtual void ApplyInverse(R3Point & p) const;
	
		virtual void WriteXForm(FILE * file) const;
		virtual void ReadXForm(FILE * file) ;

	protected:
		bool m_bComputedTransform;
		R3Affine m_AffineXForm;
		R3Vector m_Trans;
		//R3Vector m_Rotate;
		// represent as quarternions
		R3Vector m_Axis;
		double m_fAngle;
};


#endif



