#include "TimeProfiler.h"

#include "Binding.h"
#include "Interactor.h"
#include "InteractorPose.h"
#include "BindingManager.h"
#include "PntCorrsPose.h"

//////////////////////////////
////  InteractorPose
//////////////////////////////
InteractorPose::InteractorPose()
{
	m_pPnts = NULL;
	m_pPosedPnts = NULL;
	m_MeshArea = 0;
	
	m_pModel = NULL;
	m_pRelevantVolume = NULL;
	m_sPoseName = "none";
	m_NormalizingXForm = R3identity_affine;
	m_CanonicalPoseComputed = false;
	m_CanonicalPose = R3identity_affine;

	m_pSearchTree = NULL;
	m_pR3Mesh = NULL;
	
#ifdef INCLUDE_BULLET
	m_pPhysicsShape = NULL;
#endif
	m_Type = TYPE_Invalid;
}

InteractorPose::~InteractorPose()
{
	
}

InteractorPose * InteractorPose::CreatePose(FILE * file, Interactor * object)
{
	PoseType tp;
	fread(&tp, sizeof(PoseType), 1, file);
	InteractorPose * newPose = NULL;
	if (tp==TYPE_IndexedPose)
		newPose = new IndexedPose(-1);
	else if (tp==TYPE_FKPose)
		newPose = new FKPose(-1);
	else if (tp==TYPE_PntCorrsPose)
		newPose = new PntCorrsPose(object);
	else
		assert(false);
	
	assert(newPose!=NULL);
	newPose->ReadPose(file);
	return newPose;
}

InteractorPose::PoseType InteractorPose::GetPoseType() const
{
	return m_Type;
}

double InteractorPose::PoseDistanceScale() const
{
	return 1;
}

//////////////////////////////
////  IndexedPose
//////////////////////////////
IndexedPose::IndexedPose(int poseid)
: InteractorPose()
{
	m_Type = TYPE_IndexedPose;
	m_iPoseID = poseid;
}

void IndexedPose::WritePose(FILE * file) const
{
	fwrite(&m_Type, sizeof(PoseType), 1, file);
	fwrite(&m_iPoseID, sizeof(int), 1, file);
}

void IndexedPose::ReadPose(FILE * file) 
{
	fread(&m_iPoseID, sizeof(int), 1, file);
}


//////////////////////////////
////  FKPose
//////////////////////////////
FKPose::FKPose(int nfeat, const double * featVals)
: InteractorPose()
{
	m_Type = TYPE_FKPose;
	m_iNFeatures = nfeat;
	m_afFeatures = new double[m_iNFeatures];
	for (int i=0; i<m_iNFeatures; i++)
		m_afFeatures[i] = 0;
	
	if (featVals!=NULL)
	{
		for (int i=0; i<m_iNFeatures; i++)
			m_afFeatures[i] = featVals[i];
	}
}

FKPose::~FKPose()
{
	delete [] m_afFeatures;
}


void FKPose::WritePose(FILE * file) const
{
	fwrite(&m_Type, sizeof(PoseType), 1, file);
	fwrite(&m_iNFeatures, sizeof(int), 1, file);;
	fwrite(m_afFeatures, sizeof(double), m_iNFeatures, file);

}

void FKPose::ReadPose(FILE * file) 
{
	fread(&m_iNFeatures, sizeof(int), 1, file);;
	assert(m_iNFeatures>0 && m_iNFeatures < 10000);	// assume low-dimensional, just in case
	m_afFeatures = new double[m_iNFeatures];
	fread(m_afFeatures, sizeof(double), m_iNFeatures, file);
}





