#include "VKStringList.h"
#include "VKString.h"
#include "Sortable.h"
#include "FileManagement.h"

#include "ModelDatabase.h"
#include "ShapeTriangleSoup.h"

#ifdef INCLUDE_BULLET
	#include "LinearMath/btAlignedObjectArray.h"
	#include "btBulletDynamicsCommon.h"
	#include "BulletCollision/Gimpact/btGImpactShape.h"
	#include "BulletCollision/Gimpact/btGImpactCollisionAlgorithm.h"
#endif


#ifndef __InteractorPose_H
#define __InteractorPose_H

class Binding;
class ActiveRegion;
class BindingXForm;
class PhysicsConstants;
class Interactor;


/**
 * This interface is a feature vector describing a pose.
 */
class InteractorPose
{
	friend class Interactor;
	friend class BindingManager;
	
	public:
		enum PoseType
		{
			TYPE_Invalid,
			TYPE_IndexedPose,
			TYPE_FKPose,
			TYPE_PntCorrsPose
		};

		virtual PoseType GetPoseType() const;
		virtual double PoseDistanceScale() const;
	
	//protected:
		virtual void WritePose(FILE * file) const = 0;
		virtual void ReadPose(FILE * file)  = 0;
		static InteractorPose * CreatePose(FILE * file, Interactor * object);

		InteractorPose();
		virtual ~InteractorPose();
	
		PoseType m_Type;
	
	// Input geometry
		Model3D * m_pModel;
		R3Box * m_pRelevantVolume;
		R3Affine m_NormalizingXForm;
		bool m_CanonicalPoseComputed;
		R3Affine m_CanonicalPose;
		VKString m_sPoseName;
		double m_MeshArea;
		ShapePointCloud * m_pPnts;
		std::vector<int> m_BodyPartID;
		ShapePointCloud * m_pPosedPnts;
	
	// Search tree for a mesh
		R3MeshSearchTree * m_pSearchTree;
		R3Mesh * m_pR3Mesh;
		std::vector<int> m_Tree2MeshMapVertices;
		std::vector<int> m_Tree2MeshMapFaces;
		
	// Physics
#ifdef INCLUDE_BULLET
		std::vector<btRigidBody *> m_pPhysicsRigidBody;
		std::vector<btTypedConstraint *> m_Constraints;
		btCollisionShape * m_pPhysicsShape;
#endif
};

/**
 * Pose 1: indexed pose
 */
class IndexedPose : public InteractorPose
{
	friend class Interactor;
	friend class InteractorPose;
	
	protected:
		IndexedPose(int poseid);
	
		virtual void WritePose(FILE * file) const;
		virtual void ReadPose(FILE * file) ;

		int m_iPoseID;
};

/**
 * Pose 2: use forward kinematics to compute the pose -- DEPRICATED!!!!
 */
class FKPose : public InteractorPose
{
	friend class Interactor;
	friend class InteractorPose;
	
	protected:
		FKPose(int nfeat, const double * featVals = NULL);
		virtual ~FKPose();
		virtual void WritePose(FILE * file) const;
		virtual void ReadPose(FILE * file) ;

		int m_iNFeatures;
		double * m_afFeatures;
	
	// feature-to-joint mapping
};




#endif



