////////////////////////
////  This class describes the rigid body constraints
////////////////////////
#include "Interactor.h"
#include "ShapePointCloud.h"
#include "LinAlg.h"

//#define CONSTRAINT_DEBUG_SIZE 0.2f	// what's that?

#ifdef INCLUDE_BULLET
void Interactor::CreateRagDoll(InteractorPose * pose) const
{
	std::vector<btRigidBody *> & m_bodies = pose->m_pPhysicsRigidBody;
	std::vector<btTypedConstraint *> & m_joints = pose->m_Constraints;

	btTransform identity;
	identity.setIdentity();

//	btTransform offset;
//	offset.setIdentity();
	
	assert(m_bodies.size()==0);
	btCollisionShape* m_shapes[BODYPART_COUNT];
	assert(m_bodies.size()==0 && m_joints.size()==0);
	for (int i=0; i<BODYPART_COUNT; i++)
		m_bodies.push_back(NULL);

	for (int i=0; i<JOINT_COUNT; i++)
		m_joints.push_back(NULL);
	
	// Setup the geometry
//	m_shapes[BODYPART_PELVIS] = new btCapsuleShape(btScalar(0.15), btScalar(0.20));
//	m_shapes[BODYPART_SPINE] = new btCapsuleShape(btScalar(0.15), btScalar(0.28));
//	m_shapes[BODYPART_HEAD] = new btCapsuleShape(btScalar(0.10), btScalar(0.05));
//	m_shapes[BODYPART_LEFT_UPPER_LEG] = new btCapsuleShape(btScalar(0.07), btScalar(0.45));
//	m_shapes[BODYPART_LEFT_LOWER_LEG] = new btCapsuleShape(btScalar(0.05), btScalar(0.37));
//	m_shapes[BODYPART_RIGHT_UPPER_LEG] = new btCapsuleShape(btScalar(0.07), btScalar(0.45));
//	m_shapes[BODYPART_RIGHT_LOWER_LEG] = new btCapsuleShape(btScalar(0.05), btScalar(0.37));
//	m_shapes[BODYPART_LEFT_UPPER_ARM] = new btCapsuleShape(btScalar(0.05), btScalar(0.33));
//	m_shapes[BODYPART_LEFT_LOWER_ARM] = new btCapsuleShape(btScalar(0.04), btScalar(0.25));
//	m_shapes[BODYPART_RIGHT_UPPER_ARM] = new btCapsuleShape(btScalar(0.05), btScalar(0.33));
//	m_shapes[BODYPART_RIGHT_LOWER_ARM] = new btCapsuleShape(btScalar(0.04), btScalar(0.25));

	m_shapes[BODYPART_PELVIS] 			= new btCylinderShape (btVector3(0.15, 0.20/2., 0)) ;
	m_shapes[BODYPART_SPINE] 			= new btCylinderShape (btVector3(0.15, 0.28/2., 0));
	m_shapes[BODYPART_HEAD] 			= new btCylinderShape (btVector3(0.10, 0.05/2., 0));
	m_shapes[BODYPART_LEFT_UPPER_LEG] 	= new btCylinderShape (btVector3(0.07, 0.45/2., 0));
	m_shapes[BODYPART_LEFT_LOWER_LEG] 	= new btCylinderShape (btVector3(0.05, 0.37/2., 0));
	m_shapes[BODYPART_RIGHT_UPPER_LEG] 	= new btCylinderShape (btVector3(0.07, 0.45/2., 0));
	m_shapes[BODYPART_RIGHT_LOWER_LEG] 	= new btCylinderShape (btVector3(0.05, 0.37/2., 0));
	m_shapes[BODYPART_LEFT_UPPER_ARM] 	= new btCylinderShape (btVector3(0.05, 0.33/2., 0));
	m_shapes[BODYPART_LEFT_LOWER_ARM] 	= new btCylinderShape (btVector3(0.04, 0.25/2., 0));
	m_shapes[BODYPART_RIGHT_UPPER_ARM] 	= new btCylinderShape (btVector3(0.05, 0.33/2., 0));
	m_shapes[BODYPART_RIGHT_LOWER_ARM] 	= new btCylinderShape (btVector3(0.04, 0.25/2., 0));

	// Setup all the rigid bodies
	double mass = 1./BODYPART_COUNT;
	m_bodies[BODYPART_PELVIS] = CreateRigidBody(btScalar(mass), identity, m_shapes[BODYPART_PELVIS]);
	m_bodies[BODYPART_SPINE] = CreateRigidBody(btScalar(mass), identity, m_shapes[BODYPART_SPINE]);
	m_bodies[BODYPART_HEAD] = CreateRigidBody(btScalar(mass), identity, m_shapes[BODYPART_HEAD]);
	m_bodies[BODYPART_LEFT_UPPER_LEG] = CreateRigidBody(btScalar(mass), identity, m_shapes[BODYPART_LEFT_UPPER_LEG]);
	m_bodies[BODYPART_LEFT_LOWER_LEG] = CreateRigidBody(btScalar(mass), identity, m_shapes[BODYPART_LEFT_LOWER_LEG]);
	m_bodies[BODYPART_RIGHT_UPPER_LEG] = CreateRigidBody(btScalar(mass), identity, m_shapes[BODYPART_RIGHT_UPPER_LEG]);
	m_bodies[BODYPART_RIGHT_LOWER_LEG] = CreateRigidBody(btScalar(mass), identity, m_shapes[BODYPART_RIGHT_LOWER_LEG]);
	m_bodies[BODYPART_LEFT_UPPER_ARM] = CreateRigidBody(btScalar(mass), identity, m_shapes[BODYPART_LEFT_UPPER_ARM]);
	m_bodies[BODYPART_LEFT_LOWER_ARM] = CreateRigidBody(btScalar(mass), identity, m_shapes[BODYPART_LEFT_LOWER_ARM]);
	m_bodies[BODYPART_RIGHT_UPPER_ARM] = CreateRigidBody(btScalar(mass), identity, m_shapes[BODYPART_RIGHT_UPPER_ARM]);
	m_bodies[BODYPART_RIGHT_LOWER_ARM] = CreateRigidBody(btScalar(mass), identity, m_shapes[BODYPART_RIGHT_LOWER_ARM]);
	
	// Setup some damping on the m_bodies
	for (int i = 0; i < BODYPART_COUNT; ++i)
	{
		m_bodies[i]->setDamping(0.05, 0.85);
		m_bodies[i]->setDeactivationTime(0.8);
		m_bodies[i]->setSleepingThresholds(1.6, 2.5);
	}
	
	//m_joints.clear();
	
	// Now setup the constraints
	btHingeConstraint* hingeC;
	btConeTwistConstraint* coneC;
	
	btTransform localA, localB;
	
	
	localA.setIdentity(); localB.setIdentity();
	localA.getBasis().setEulerZYX(0,M_PI_2,0); localA.setOrigin(btVector3(btScalar(0.), btScalar(0.15), btScalar(0.)));
	localB.getBasis().setEulerZYX(0,M_PI_2,0); localB.setOrigin(btVector3(btScalar(0.), btScalar(-0.15), btScalar(0.)));
	hingeC =  new btHingeConstraint(*m_bodies[BODYPART_PELVIS], *m_bodies[BODYPART_SPINE], localA, localB);
	hingeC->setLimit(btScalar(-M_PI_4), btScalar(M_PI_2));
	//hingeC->setLimit(0,0);
	m_joints[JOINT_PELVIS_SPINE] = hingeC;

	localA.setIdentity(); localB.setIdentity();
	localA.getBasis().setEulerZYX(0,0,M_PI_2); localA.setOrigin(btVector3(btScalar(0.), btScalar(0.30), btScalar(0.)));
	localB.getBasis().setEulerZYX(0,0,M_PI_2); localB.setOrigin(btVector3(btScalar(0.), btScalar(-0.14), btScalar(0.)));
	coneC = new btConeTwistConstraint(*m_bodies[BODYPART_SPINE], *m_bodies[BODYPART_HEAD], localA, localB);
	coneC->setLimit(M_PI_4, M_PI_4, M_PI_2);
	//coneC->setLimit(0, 0, 0);
	m_joints[JOINT_SPINE_HEAD] = coneC;	
	
	localA.setIdentity(); localB.setIdentity();
	localA.getBasis().setEulerZYX(0,0,-M_PI_4*5); localA.setOrigin(btVector3(btScalar(-0.18), btScalar(-0.10), btScalar(0.)));
	localB.getBasis().setEulerZYX(0,0,-M_PI_4*5); localB.setOrigin(btVector3(btScalar(0.), btScalar(0.225), btScalar(0.)));
	coneC = new btConeTwistConstraint(*m_bodies[BODYPART_PELVIS], *m_bodies[BODYPART_LEFT_UPPER_LEG], localA, localB);
	coneC->setLimit(M_PI_4, M_PI_4, 0);
	//coneC->setLimit(0, 0, 0);
	m_joints[JOINT_LEFT_HIP] = coneC;

	localA.setIdentity(); localB.setIdentity();
	localA.getBasis().setEulerZYX(0,M_PI_2,0); localA.setOrigin(btVector3(btScalar(0.), btScalar(-0.225), btScalar(0.)));
	localB.getBasis().setEulerZYX(0,M_PI_2,0); localB.setOrigin(btVector3(btScalar(0.), btScalar(0.185), btScalar(0.)));
	hingeC =  new btHingeConstraint(*m_bodies[BODYPART_LEFT_UPPER_LEG], *m_bodies[BODYPART_LEFT_LOWER_LEG], localA, localB);
	hingeC->setLimit(btScalar(0), btScalar(M_PI_2));
	//hingeC->setLimit(0, 0);
	m_joints[JOINT_LEFT_KNEE] = hingeC;
	
	localA.setIdentity(); localB.setIdentity();
	localA.getBasis().setEulerZYX(0,0,M_PI_4); localA.setOrigin(btVector3(btScalar(0.18), btScalar(-0.10), btScalar(0.)));
	localB.getBasis().setEulerZYX(0,0,M_PI_4); localB.setOrigin(btVector3(btScalar(0.), btScalar(0.225), btScalar(0.)));
	coneC = new btConeTwistConstraint(*m_bodies[BODYPART_PELVIS], *m_bodies[BODYPART_RIGHT_UPPER_LEG], localA, localB);
	coneC->setLimit(M_PI_4, M_PI_4, 0);
	//coneC->setLimit(0, 0, 0);
	m_joints[JOINT_RIGHT_HIP] = coneC;
	
	localA.setIdentity(); localB.setIdentity();
	localA.getBasis().setEulerZYX(0,M_PI_2,0); localA.setOrigin(btVector3(btScalar(0.), btScalar(-0.225), btScalar(0.)));
	localB.getBasis().setEulerZYX(0,M_PI_2,0); localB.setOrigin(btVector3(btScalar(0.), btScalar(0.185), btScalar(0.)));
	hingeC =  new btHingeConstraint(*m_bodies[BODYPART_RIGHT_UPPER_LEG], *m_bodies[BODYPART_RIGHT_LOWER_LEG], localA, localB);
	hingeC->setLimit(btScalar(0), btScalar(M_PI_2));
	//hingeC->setLimit(0,0);
	m_joints[JOINT_RIGHT_KNEE] = hingeC;
		
	localA.setIdentity(); localB.setIdentity();
	localA.getBasis().setEulerZYX(0,0,M_PI); localA.setOrigin(btVector3(btScalar(-0.2), btScalar(0.15), btScalar(0.)));
	localB.getBasis().setEulerZYX(0,0,M_PI_2); localB.setOrigin(btVector3(btScalar(0.), btScalar(-0.18), btScalar(0.)));
	coneC = new btConeTwistConstraint(*m_bodies[BODYPART_SPINE], *m_bodies[BODYPART_LEFT_UPPER_ARM], localA, localB);
	coneC->setLimit(M_PI_2, M_PI_2, 0);
	//coneC->setLimit(0,0,0);
	m_joints[JOINT_LEFT_SHOULDER] = coneC;
	
	localA.setIdentity(); localB.setIdentity();
	localA.getBasis().setEulerZYX(0,M_PI_2,0); localA.setOrigin(btVector3(btScalar(0.), btScalar(0.18), btScalar(0.)));
	localB.getBasis().setEulerZYX(0,M_PI_2,0); localB.setOrigin(btVector3(btScalar(0.), btScalar(-0.14), btScalar(0.)));
	hingeC =  new btHingeConstraint(*m_bodies[BODYPART_LEFT_UPPER_ARM], *m_bodies[BODYPART_LEFT_LOWER_ARM], localA, localB);
	hingeC->setLimit(btScalar(0), btScalar(M_PI_2));
	//hingeC->setLimit(0,0);
	m_joints[JOINT_LEFT_ELBOW] = hingeC;
	
	localA.setIdentity(); localB.setIdentity();
	localA.getBasis().setEulerZYX(0,0,0); localA.setOrigin(btVector3(btScalar(0.2), btScalar(0.15), btScalar(0.)));
	localB.getBasis().setEulerZYX(0,0,M_PI_2); localB.setOrigin(btVector3(btScalar(0.), btScalar(-0.18), btScalar(0.)));
	coneC = new btConeTwistConstraint(*m_bodies[BODYPART_SPINE], *m_bodies[BODYPART_RIGHT_UPPER_ARM], localA, localB);
	coneC->setLimit(M_PI_2, M_PI_2, 0);
	//coneC->setLimit(0,0,0);
	m_joints[JOINT_RIGHT_SHOULDER] = coneC;
	
	localA.setIdentity(); localB.setIdentity();
	localA.getBasis().setEulerZYX(0,M_PI_2,0); localA.setOrigin(btVector3(btScalar(0.), btScalar(0.18), btScalar(0.)));
	localB.getBasis().setEulerZYX(0,M_PI_2,0); localB.setOrigin(btVector3(btScalar(0.), btScalar(-0.14), btScalar(0.)));
	hingeC =  new btHingeConstraint(*m_bodies[BODYPART_RIGHT_UPPER_ARM], *m_bodies[BODYPART_RIGHT_LOWER_ARM], localA, localB);
	hingeC->setLimit(btScalar(0), btScalar(M_PI_2));
	//hingeC->setLimit(0,0);
	m_joints[JOINT_RIGHT_ELBOW] = hingeC;

	UpdateRagDollPhysicsFromPose(pose);
}

void Interactor::UpdateRagDollPoseFromPhysics(InteractorPose * pose) const
{
	assert(pose->GetPoseType()==InteractorPose::TYPE_FKPose);
	FKPose * fk = static_cast<FKPose*>(pose);

	std::vector<btRigidBody *> & m_bodies = pose->m_pPhysicsRigidBody;
	//std::vector<btTypedConstraint *> & m_joints = pose->m_Constraints;

	assert(m_bodies.size()>=BODYPART_COUNT);
	
	btTransform offset;
	m_bodies[0]->getMotionState()->getWorldTransform(offset);
	
	btTransform transform;
	int stride = 6;
	for (int i=1; i<BODYPART_COUNT; i++)
	{
		m_bodies[i]->getMotionState()->getWorldTransform(transform);
		
		// get fk pose
		transform = offset.inverseTimes (transform);
		
		btScalar yaw, pitch, roll;
		transform.getBasis().getEulerZYX(yaw, pitch, roll);
		
		int f = (i-1)*stride;
		assert(f>=0 && f+stride <= fk->m_iNFeatures);
		fk->m_afFeatures[f+0] = transform.getOrigin().x();
		fk->m_afFeatures[f+1] = transform.getOrigin().y();
		fk->m_afFeatures[f+2] = transform.getOrigin().z();
		fk->m_afFeatures[f+3] = roll;
		fk->m_afFeatures[f+4] = pitch;
		fk->m_afFeatures[f+5] = yaw;
	}
	
	// update points
	std::vector<R3Affine> bodypart2affine;
	for (int i=0; i<BODYPART_COUNT; i++)
	{
		m_bodies[i]->getMotionState()->getWorldTransform(transform);
		//transform.setIdentity();
		transform = offset.inverseTimes (transform);
		
		// get r3affine matrix - transform points
		const btVector3 & t = transform.getOrigin ();
		btQuaternion r = transform.getRotation ();
		const btVector3 & a = r.getAxis();
		R3Affine newXForm = R3identity_affine;
		newXForm.Translate(R3Vector(t.x(), t.y(), t.z()));
		newXForm.Rotate(R3Vector(a.x(), a.y(), a.z()), r.getAngle());
		bodypart2affine.push_back(newXForm);
	}
	
	int nsamp = RagDollNSamples(pose);
	assert(fk->m_pPnts!=NULL && fk->m_pPosedPnts!=NULL);
	for (int i=0; i<nsamp; i++)
	{
		R3Point p = fk->m_pPnts->PointPosition(i);
		int bid = fk->m_BodyPartID[i];
		//std::cout<<"pnt->samp: "<<i<<" -> "<<bid<<std::endl;
		assert(bid >=0 && bid < (int)bodypart2affine.size());
		bodypart2affine[bid].Apply(p);
		fk->m_pPosedPnts->ReplacePoint(i, ShapePoint(p));
	}
}


void Interactor::UpdateRagDollPhysicsFromPose(InteractorPose * pose) const
{
	assert(pose->GetPoseType()==InteractorPose::TYPE_FKPose);
	FKPose * fk = static_cast<FKPose*>(pose);
	std::vector<btRigidBody *> & m_bodies = pose->m_pPhysicsRigidBody;
	//std::vector<btTypedConstraint *> & m_joints = pose->m_Constraints;

	btTransform transform;
	transform.setIdentity();
	
	transform.setOrigin(btVector3(btScalar(0.), btScalar(0.), btScalar(0.)));
	m_bodies[BODYPART_PELVIS]->getMotionState()->setWorldTransform(transform);
	
	int stride = 6;
	for (int i=1; i<BODYPART_COUNT; i++)
	{
		int f = (i-1)*stride;
		assert(f>=0 && f+stride <= fk->m_iNFeatures);
		transform.setIdentity();
		transform.setOrigin(btVector3(fk->m_afFeatures[f+0], fk->m_afFeatures[f+1], fk->m_afFeatures[f+2]));
		transform.getBasis().setEulerZYX(fk->m_afFeatures[f+3], fk->m_afFeatures[f+4], fk->m_afFeatures[f+5]);
		m_bodies[i]->getMotionState()->setWorldTransform(transform);
	}
}


FKPose * Interactor::GenerateRandomFKPose() const
{
	// parameterize by part positions (pelvis is at the center of the coordinate system)
	double * partPtr;
	int stride = 6;
	double * arrangement = new double[stride * (BODYPART_COUNT-1)];
	
	partPtr = &(arrangement[(BODYPART_SPINE-1)*stride + 0]);
	partPtr[0] = 0.0;	partPtr[1] = 0.2;	partPtr[2] = 0.0;	// translation
	partPtr[3] = 0.0;	partPtr[4] = 0.0;	partPtr[5] = 0.0;	// euler rotation
	
	partPtr = &(arrangement[(BODYPART_HEAD-1)*stride + 0]);
	partPtr[0] = 0.0;	partPtr[1] = 0.6;	partPtr[0] = 0.0;	// translation
	partPtr[3] = 0.0;	partPtr[4] = 0.0;	partPtr[5] = 0.0;	// euler rotation
	
	partPtr = &(arrangement[(BODYPART_LEFT_UPPER_LEG-1)*stride + 0]);
	partPtr[0] =-0.18;	partPtr[1] =-0.35;	partPtr[2] = 0.0;	// translation
	partPtr[3] = 0.0;	partPtr[4] = 0.0;	partPtr[5] = 0.0;	// euler rotation
	
	partPtr = &(arrangement[(BODYPART_LEFT_LOWER_LEG-1)*stride + 0]);
	partPtr[0] =-0.18;	partPtr[1] =-0.8;	partPtr[2] = 0.0;	// translation
	partPtr[3] = 0.0;	partPtr[4] = 0.0;	partPtr[5] = 0.0;	// euler rotation
	
	partPtr = &(arrangement[(BODYPART_RIGHT_UPPER_LEG-1)*stride + 0]);
	partPtr[0] =0.18;	partPtr[1] =-0.35;	partPtr[2] = 0.0;	// translation
	partPtr[3] = 0.0;	partPtr[4] = 0.0;	partPtr[5] = 0.0;	// euler rotation
	
	partPtr = &(arrangement[(BODYPART_RIGHT_LOWER_LEG-1)*stride + 0]);
	partPtr[0] =0.18;	partPtr[1] =-0.8;	partPtr[2] = 0.0;	// translation
	partPtr[3] = 0.0;	partPtr[4] = 0.0;	partPtr[5] = 0.0;	// euler rotation
	
	partPtr = &(arrangement[(BODYPART_LEFT_UPPER_ARM-1)*stride + 0]);
	partPtr[0] =-0.35;	partPtr[1] =0.45;	partPtr[2] = 0.0;	// translation
	partPtr[3] = 0.0;	partPtr[4] = 0.0;	partPtr[5] = M_PI_2;// euler rotation
	
	partPtr = &(arrangement[(BODYPART_LEFT_LOWER_ARM-1)*stride + 0]);
	partPtr[0] =-0.7;	partPtr[1] =0.45;	partPtr[2] = 0.0;	// translation
	partPtr[3] = 0.0;	partPtr[4] = 0.0;	partPtr[5] = M_PI_2;// euler rotation
	
	partPtr = &(arrangement[(BODYPART_RIGHT_UPPER_ARM-1)*stride + 0]);
	partPtr[0] = 0.35;	partPtr[1] = 0.45;	partPtr[2] = 0.0;	// translation
	partPtr[3] = 0.0;	partPtr[4] = 0.0;	partPtr[5] =-M_PI_2;// euler rotation
	
	partPtr = &(arrangement[(BODYPART_RIGHT_LOWER_ARM-1)*stride + 0]);
	partPtr[0] = 0.7;	partPtr[1] = 0.45;	partPtr[2] = 0.0;	// translation
	partPtr[3] = 0.0;	partPtr[4] = 0.0;	partPtr[5] =-M_PI_2;// euler rotation
	
	FKPose * pose = new FKPose(stride * (BODYPART_COUNT-1), arrangement);
	delete [] arrangement;
	return pose;
}

void Interactor::FillHeightAndRadius(const btCollisionShape * shape, double & r, double & h, int & upAxis) const
{
	if (shape->getShapeType()==CAPSULE_SHAPE_PROXYTYPE)
	{
		const btCapsuleShape * cShape = static_cast<const btCapsuleShape*>(shape);
		r = cShape->getRadius();
		h = 2*cShape->getHalfHeight();
		upAxis = cShape->getUpAxis();
	}
	else if (shape->getShapeType()==CYLINDER_SHAPE_PROXYTYPE)
	{
		const btCylinderShape * cShape = static_cast<const btCylinderShape*>(shape);
		btVector3 cylinder = cShape->getHalfExtentsWithoutMargin ();
		r = cylinder.x();
		h = 2*cylinder.y();
		upAxis = cShape->getUpAxis();
	}
	else
		assert(false);
}

int Interactor::RagDollNSamples(const InteractorPose * pose) const
{
	if (pose->m_pPnts==NULL)
	{
		InteractorPose * ncp = (InteractorPose*)pose;

		ncp->m_pPnts = new ShapePointCloud(NULL, "none");
		ncp->m_pPosedPnts = new ShapePointCloud(NULL, "none");
		double area = 0;
		assert((int)ncp->m_pPhysicsRigidBody.size()>=BODYPART_COUNT);
		double r, h;
		int upAxis;
		for (int i=0; i<BODYPART_COUNT; i++)
		{
			FillHeightAndRadius(ncp->m_pPhysicsRigidBody[i]->getCollisionShape(), r, h, upAxis);
			area += 2*r*r + 2*h*r;
		}
		
		int npoints = 1000;		
		for (int i=0; i<BODYPART_COUNT; i++)
		{
			FillHeightAndRadius(ncp->m_pPhysicsRigidBody[i]->getCollisionShape(), r, h, upAxis);
			double diskArea = r*r;
			double sidesArea = 2*h*r;
			int currPntID = ncp->m_pPnts->NumPoints();
			FillEvenDisk(ncp->m_pPnts, r, h/2, upAxis, npoints * diskArea / area);
			FillEvenDisk(ncp->m_pPnts, r, -h/2, upAxis, npoints * diskArea / area);
			FillCylinderSides(ncp->m_pPnts, r, h, upAxis, npoints * sidesArea / area);

			// fill body ids
			for (int p=currPntID; p<ncp->m_pPnts->NumPoints(); p++)
				ncp->m_BodyPartID.push_back(i);
		}
		for (int i=0; i<ncp->m_pPnts->NumPoints(); i++)
			ncp->m_pPosedPnts->AddPoint(ncp->m_pPnts->GetPoint(i));
		
		UpdateRagDollPoseFromPhysics(ncp);
		//std::cout<<"Created: "<<pose->m_pPnts->NumPoints()<<std::endl;
	}
	assert(pose->m_pPnts!=NULL);
	return pose->m_pPnts->NumPoints();
}

int Interactor::FillEvenDisk(ShapePointCloud * pnts, double radius, double height, int axis, int npoints) const
{
	int a2 = 1, a3 = 2;
	if (axis==1)
		a2 = 0;
	else if (axis==2)
		a3 = 0;

	int ngrid = (int)(sqrt((double)npoints * 4. / vkPI)+0.5);
	if (ngrid==0)
		ngrid = 1;
	
	int nadded = 0;
	double pos[3];
	for (int x=0; x<ngrid; x++)
	for (int y=0; y<ngrid; y++)
	{
		double xscaled = radius * 2*((double)x / (double)ngrid - 0.5);
		double yscaled = radius * 2*((double)y / (double)ngrid - 0.5);
		
		if (xscaled * xscaled + yscaled*yscaled < radius*radius)
		{
			pos[axis] = height;
			pos[a2] = xscaled;
			pos[a3] = yscaled;
		
			pnts->AddPoint(ShapePoint(R3Point(pos[0], pos[1], pos[2])));
			nadded++;
		}
	}
	return nadded;
}
int Interactor::FillCylinderSides(ShapePointCloud * pnts, double radius, double height, int axis, int npoints) const
{
	int a2 = 1, a3 = 2;
	if (axis==1)
		a2 = 0;
	else if (axis==2)
		a3 = 0;
	
	double circ = 2 * vkPI * radius;
	
	double side = pow(circ * height, .5);
	
	int cgrid = (int)(circ / side * sqrt(npoints) + 0.5);
	int hgrid = (int)(height / side * sqrt(npoints) + 0.5);
	
	if (cgrid==0)
		cgrid = 1;
	if (hgrid==0)
		hgrid = 1;
	
	double pos[3];
	for (int c=0; c<cgrid; c++)
	for (int h=0; h<hgrid; h++)
	{
		double angle = (double)c / (double)cgrid * 2. * vkPI;
		double xscaled = radius * cos(angle);
		double yscaled = radius * sin(angle);
		double zscaled = ((double)h/((double)hgrid-1.) - 0.5) * height;
		pos[axis] = zscaled;
		pos[a2] = xscaled;
		pos[a3] = yscaled;
		pnts->AddPoint(ShapePoint(R3Point(pos[0], pos[1], pos[2])));
	}
	return cgrid * hgrid;
}


R3Point Interactor::RagDollSample(const InteractorPose * pose, int pid) const
{
	int nsamp = RagDollNSamples(pose);
	assert(pid >=0 && pid < nsamp);
	assert(pose->m_pPnts!=NULL);
	return pose->m_pPosedPnts->PointPosition(pid);
}
#endif




