#include "VKString.h"
#include "Sortable.h"
#include "FileManagement.h"

#include "ModelDatabase.h"
#include "ShapeTriangleSoup.h"
#include "PntCorrsPose.h"


#ifndef __IntersectionGrid_H
#define __IntersectionGrid_H

#include "Interactor.h"
#include "InteractorPose.h"

/**
 * Actor-object intersection module
 */
class IntersectionGrid
{
	public:
		enum IsectGridFlags
		{
			ISECT_DepthPenalty		 = 1<<0,
			ISECT_WeightByContacts	 = 1<<1,
			ISECT_WeightByNormals	 = 1<<2,
			ISECT_PenetrationPenalty = 1<<3
		};
	
		IntersectionGrid(Interactor * actor, Interactor* object, double rotation );
		virtual void ComputeEnergy(const PntCorrsPose * posepts, IKSolver * solver, EnergyInfo & energy) const;
		virtual const ShapePointCloud * InsidePoints(double * radius) const;
		virtual double ComputePenalty(const R3Point & point) const;
	
	protected:
		virtual void InitGrid(double step, int maxcells, double rotation);
		
		Interactor * m_Interactors[2];
		R3Grid * m_pObjectDepthGrid;
		R3Grid * m_pObjectVoronoiGrid;
		R3Grid * m_pDist2SurfaceGrid;
		ShapePointCloud * m_pInsidePoints;
		double m_fStep;
		R3Affine m_XForm;
	
	// params
		double m_fGridCellSize;
		int m_iNumDilations;
		double m_fDist2ContactSigma;
	
		int m_iGridFlags;
};


#endif



