#include "VKStringList.h"
#include "VKString.h"
#include "Sortable.h"
#include "FileManagement.h"

#include "ModelDatabase.h"
#include "ShapeTriangleSoup.h"
#include "InteractorPose.h"

#ifndef __PntCorrsPose_H
#define __PntCorrsPose_H

class Binding;
class ActiveRegion;
class BindingXForm;
class PhysicsConstants;
class Interactor;
class IKSolver;

/**
 * Pose 3: Pose as human-to-object correspondences (ACTUALLY USES FORWARD KINEMATICS)
 * 		   Each feature point
 */
class TargetPrimitive
{
	public:
		friend class PntCorrsPose;
	
		enum PrimType
		{
			PRIM_None,
			PRIM_Point,
			PRIM_Ground,
			PRIM_PointMeshless,
			PRIM_PartialUnassigned	// unassigned due to partial assignment (no penalty)
		};
	
		TargetPrimitive();
		TargetPrimitive(PrimType tp, const ShapePoint & pnt = ShapePoint());
	
		virtual bool IsType(PrimType type) const;
		virtual R3Point GetPosition() const;
		virtual const ShapePoint & GetPoint() const;
		virtual int PointID(ShapePointCloud * pointset) const;
	
	protected:
		PrimType m_Type;
		ShapePoint m_Point;
};

class PntCorrsPose : public InteractorPose
{
	friend class Interactor;
	friend class InteractorPose;
	friend class BindingManager;	// correspondence algorithm fills this pose...
	
	public:
		PntCorrsPose(const PntCorrsPose * match);
		PntCorrsPose(Interactor * object);
		PntCorrsPose();
		virtual ~PntCorrsPose();
	
		virtual void WritePose(FILE * file) const;
		virtual void ReadPose(FILE * file) ;
		static void CreatePosesFromGTFile(const VKString & gtFile, const VKString & meshFile, const VKString & xform,
										  std::vector<PntCorrsPose*> & poses);
		static void CreatePosesFromGTFile(const VKString & gtFile, const ShapeMesh * mesh, const VKString & xform,
										  std::vector<PntCorrsPose*> & poses);
		static bool WritePosesGTFile(const VKString & gtFile, const std::vector<PntCorrsPose*> & poses,
									 const VKStringList & regionNames, const VKStringList & regionTypes,
									 const std::vector<int> & regionNPnts);
		virtual double PoseDistanceScale() const;
	
		virtual void Clear(int matchID);
		virtual int NMatches() const;
		virtual int Region(int matchID) const;
		virtual int & Region(int matchID);
		virtual TargetPrimitive & Prim(int matchID);
		virtual const TargetPrimitive & Prim(int matchID) const;
		virtual int AddMatch(int regionID, const TargetPrimitive & trg);
	
		virtual void InitParams(int nparams, int paramFlags);
		virtual void InitJointPositions(int njoints);
		virtual bool IsParamsInit(int nparams) const;
		virtual bool IsPositionsInit(int npositions) const;
		virtual double & Param(int skeletonParam);
		virtual double GetParam(int skeletonParam) const;
		virtual R3Point & Position(int jointID);
		virtual R3Point GetPosition(int jointID) const;
	
		virtual VKString GetStr(const IKSolver * ik) const;
	
		virtual int GetPoseParamsFlags() const;
	
		virtual int GetPoseID() const;
		virtual void SetPoseID(int poseID);
		
	protected:
		std::vector<TargetPrimitive> m_aTargets;		// point on the object -- feature points.  m_aFeaturePoints
		std::vector<int> m_RegionID;					// point on a human body
		
		std::vector<double> m_SkeletonParams;
		std::vector<R3Point> m_JointPositions;
		
		Interactor * m_pObject;
	
		int m_iPoseParamsFlags;
		int m_iPoseID;
};



#endif



