#include "PosePrior.h"
#include "InteractorPose.h"
#include "VkFunctions.h"
#include "IKSolver.h"
#include "BindingManager.h"
#include "PosePriorGauss.h"
#include "PosePriorMixGauss.h"
#include "PntCorrsPose.h"
#include "ShapePointCloud.h"


JacobianInfo::JacobianInfo()
{
}

////////
// PosePrior
PosePrior::PosePrior()
{
	m_comDistrGlobal = NULL;
}
PosePrior * PosePrior::ReadPrior(const VKString & filename)
{
	FILE * file = fopen(filename.c_str(), "rb");
	if (file==NULL)
		std::cout<<"[ERROR] Cannot find "<<filename.c_str()<<std::endl;
	assert(file!=NULL);
	
	PosePriorType type;
	
	PosePrior * prior = NULL;
	fread(&type, sizeof(PosePriorType), 1, file);
	if (type==PRIOR_GaussAngles)
		prior = new PosePriorGauss();
	else if (type==PRIOR_MultiGauss)
		prior = new PosePriorMixGauss();
	else
		assert(false);
	prior->ReadPrior(file);
	fclose(file);
	return prior;
}


void PosePrior::Train(const std::vector<const InteractorPose *> & pose){ assert(false); }
void PosePrior::WritePrior(const VKString & filename) const{ assert(false); }
void PosePrior::ReadPrior(FILE * file){ assert(false); }
PosePrior::PosePriorType PosePrior::PriorType() const{ assert(false); }
bool PosePrior::IsAngles() const{return false;}

int PosePrior::NTerms() const{ assert(false); }
bool PosePrior::IsSymmetryTerm(int termID) const 	{	return false;	}
double PosePrior::EnergyTerm(const InteractorPose * pose, int termID, EnergyInfo & info) const{assert(false);}
double PosePrior::Jacobian(const InteractorPose * pose, JacobianInfo & info, int termID, int  paramID) const
{	assert(false);	}
void PosePrior::NonZeroJacobians(std::vector<int> & termIDs, std::vector<int> & paramIDs) const {assert(false);}

void PosePrior::RandomSample(InteractorPose * pose) const {assert(false);}
int PosePrior::NSuggestedInitPoses() const { assert(false); }
void PosePrior::InitPose(int poseID, InteractorPose * pose) const {assert(false);}
bool PosePrior::QuickValidityTest(const InteractorPose * pose) const {assert(false);}

void PosePrior::RecordPointStats(const std::vector<PntCorrsPose*> & poses, int nregions, const VKStringList & comFiles)
{
	// prepare datastructures (empty)
	int nstyles = 0;
	for (int i=0; i<(int)poses.size(); i++)
		nstyles = vkMax(poses[i]->GetPoseID()+1, nstyles);
	
//	std::cout<<"NStyles: "<<nstyles<<" nregions: "<<nregions<<std::endl;
	
	for (int i=0; i<nstyles; i++)
	{
		m_iNPoses.push_back(0);
		m_RegionNGround.push_back(std::vector<int>());
		m_RegionNSurface.push_back(std::vector<int>());
		
		for (int j=0; j<nregions; j++)
		{
			m_RegionNGround[i].push_back(0);
			m_RegionNSurface[i].push_back(0);
		}
	}
	
//	std::cout<<"Fill Datstructure: "<<poses.size()<<std::endl;
	// fill datastructures
	std::map<int, bool> matchedRegions;
	for (int i=0; i<(int)poses.size(); i++)
	{
		const PntCorrsPose * pose = poses[i];
		int poseid = pose->GetPoseID();
		assert(poseid>=0 && poseid<(int)m_iNPoses.size());
		assert(poseid<(int)m_RegionNGround.size() && poseid<(int)m_RegionNSurface.size());
		m_iNPoses[poseid]++;
		for (int m=0; m<pose->NMatches(); m++)
		{
			int rid = pose->Region(m);
			matchedRegions[rid] = true;
			assert(rid>=0 && rid < (int)m_RegionNGround[poseid].size() && rid<(int)m_RegionNSurface[poseid].size());
//			std::cout<<"Pointflags: "<<pose->Prim(m).IsType(TargetPrimitive::PRIM_Ground);
//			std::cout<<" "<<pose->Prim(m).IsType(TargetPrimitive::PRIM_Point)<<std::endl;
			if (pose->Prim(m).IsType(TargetPrimitive::PRIM_Ground))
			{
				m_RegionNGround[poseid][rid]++;
//				std::cout<<"("<<poseid<<", "<<rid<<", "<<m<<"): ground - "<<m_RegionNGround[poseid][rid]<<std::endl;
			}
			else if (pose->Prim(m).IsType(TargetPrimitive::PRIM_Point))
			{
				m_RegionNSurface[poseid][rid]++;
//				std::cout<<"("<<poseid<<", "<<rid<<", "<<m<<"): surface - "<<m_RegionNSurface[poseid][rid]<<std::endl;
			}
			else
				assert(false);
		}
	}
	
	m_comDistrGlobal = new PosePointGauss(3, false);
	for (int i=0; i<(int)poses.size(); i++)
	{
		while ((int)poses[i]->GetPoseID()>=(int)m_comDistrPerStyle.size())
			m_comDistrPerStyle.push_back(new PosePointGauss(3, false));
	}

	if ((int)poses.size()!=comFiles.count())
		std::cout<<"[ERROR] # poses != # center-of-mass files: "<<comFiles.count()<<"/"<<poses.size()<<std::endl;
	assert((int)poses.size()==comFiles.count());
	double * feature = new double[3];
	for (int i=0; i<comFiles.count(); i++)
	{
#if 0	// OBJECT's CENTER OF MASS
		std::ifstream textStream(comFiles[i].c_str());
		if (!textStream.is_open())
			std::cout<<"[ERROR] Cannot find: "<<comFiles[i].c_str()<<std::endl;
		assert(textStream.is_open());
		double x, y, z;
		textStream>>x>>y>>z;
		textStream.close();
		R3Point objCOM(x, y, z);
#endif
		// OBJECT's CENTER OF SYMMETRY
		std::vector<ReflectionPlane> planes;
		std::vector<int> rois;
		for (std::map<int, bool>::iterator iter = matchedRegions.begin(); iter != matchedRegions.end(); iter++)
			rois.push_back(iter->first);
		std::sort(rois.begin(), rois.end());
		BindingEnergy::ReadPlanes(comFiles[i], planes);
		if (planes.size()==0)
			continue;
		double energy;
		assert(IsAngles());
		IKSolver * ik = ((PriorAngles*)this)->GetIK();
		int bp = BindingEnergy::FindBestPlaneOfReflection(planes, poses[i], ik, rois, energy);
		assert(bp>=0 && bp<(int)planes.size());
		R3Point objCOM = planes[bp].m_Plane.Point();
		
		// NOW GET STATS
		R3Affine xform = R3identity_affine;
		ik->FindReferenceNode(poses[i], xform);
		R3Point trans(0,0,0);
		R3Vector orientX(1,0,0);
		R3Vector orientY(0,1,0);
		R3Vector orientZ(0,0,1);
		xform.Apply(trans);
		xform.Apply(orientX);
		xform.Apply(orientY);
		xform.Apply(orientZ);
		orientX.Normalize();
		orientY.Normalize();
		orientZ.Normalize();
		feature[0] = orientX.Dot(objCOM-trans);
		feature[1] = orientY.Dot(objCOM-trans);
		feature[2] = orientZ.Dot(objCOM-trans);
		m_comDistrGlobal->Update(feature);
		m_comDistrPerStyle[poses[i]->GetPoseID()]->Update(feature);
	}
	delete [] feature;
}

double PosePrior::COMDeviation(const PntCorrsPose * pose, const R3Point & objCOM) const
{
	double * feature = new double[3];
	assert(IsAngles());
	R3Affine xform = R3identity_affine;
	((PriorAngles*)this)->GetIK()->FindReferenceNode(pose, xform);
	R3Point trans(0,0,0);
	R3Vector orientX(1,0,0);
	R3Vector orientY(0,1,0);
	R3Vector orientZ(0,0,1);
	xform.Apply(trans);
	xform.Apply(orientX);
	xform.Apply(orientY);
	xform.Apply(orientZ);
	orientX.Normalize();
	orientY.Normalize();
	orientZ.Normalize();
	feature[0] = orientX.Dot(objCOM-trans);
	feature[1] = orientY.Dot(objCOM-trans);
	feature[2] = orientZ.Dot(objCOM-trans);
		
	PosePointGauss * gauss = NULL;
	int poseid = pose->GetPoseID();
	if (poseid<0)
		gauss = m_comDistrGlobal;
	else
		gauss = m_comDistrPerStyle[poseid];
	assert(gauss!=NULL && gauss->m_aVariances[0]>0 && gauss->m_aVariances[1]>0 && gauss->m_aVariances[2]>0);
	double penalty=0;
	for (int i=0; i<2; i++)
		penalty += pow(feature[i]-gauss->m_aMeans[i], 2) / gauss->m_aVariances[i];
	delete [] feature;
	return penalty;
}

double PosePrior::SurfaceProbability(int poseid, int regionid) const
{
	int nposes = (int)m_iNPoses.size();	assert(poseid>=0 && poseid<nposes);
	assert(nposes==(int)m_RegionNSurface.size() && nposes==(int)m_RegionNGround.size());
	int nregions = (int)m_RegionNSurface[poseid].size(); assert(regionid>=0 && regionid<nregions);
	assert(nregions==(int)m_RegionNSurface[poseid].size() && nregions==(int)m_RegionNGround[poseid].size());
	return (double)m_RegionNSurface[poseid][regionid]/(double)m_iNPoses[poseid];
}

double PosePrior::GroundProbability(int poseid, int regionid) const
{
	int nposes = (int)m_iNPoses.size();	assert(poseid>=0 && poseid<nposes);
	assert(nposes==(int)m_RegionNSurface.size() && nposes==(int)m_RegionNGround.size());
	int nregions = (int)m_RegionNSurface[poseid].size(); assert(regionid>=0 && regionid<nregions);
	assert(nregions==(int)m_RegionNSurface[poseid].size() && nregions==(int)m_RegionNGround[poseid].size());
	return (double)m_RegionNGround[poseid][regionid]/(double)m_iNPoses[poseid];
}

double PosePrior::UnassignedProbability(int poseid, int regionid) const
{
	int nposes = (int)m_iNPoses.size();	assert(poseid>=0 && poseid<nposes);
	assert(nposes==(int)m_RegionNSurface.size() && nposes==(int)m_RegionNGround.size());
	int nregions = (int)m_RegionNSurface[poseid].size(); assert(regionid>=0 && regionid<nregions);
	assert(nregions==(int)m_RegionNSurface[poseid].size() && nregions==(int)m_RegionNGround[poseid].size());
	int nunassigned = m_iNPoses[poseid] - m_RegionNGround[poseid][regionid] - m_RegionNSurface[poseid][regionid];
	return (double)nunassigned/(double)m_iNPoses[poseid];
}

void PosePrior::WritePointStats(const VKString & filename) const
{
	assert(m_iNPoses.size()==m_RegionNSurface.size() && m_iNPoses.size()==m_RegionNGround.size());
	int nposeStyles = (int)m_iNPoses.size();
	std::ofstream textStream(filename.c_str());
	assert(textStream.is_open());
	textStream<<"NPoseStyles "<<nposeStyles<<"\n";
	
	for (int i=0; i<nposeStyles; i++)
	{
		int nposes = m_iNPoses[i];
		int nregions = (int)m_RegionNSurface[i].size();
		assert(nregions == (int)m_RegionNGround[i].size());
		textStream<<"NPoses "<<nposes<<" NRegions "<<nregions<<"\n";
		textStream<<"Surface ";
		for (int j=0; j<nregions; j++)
			textStream<<m_RegionNSurface[i][j]<<" ";
		textStream<<"\n";
		textStream<<"Ground ";
		for (int j=0; j<nregions; j++)
			textStream<<m_RegionNGround[i][j]<<" ";
		textStream<<"\n";
	}
	
	for (int i=0; i<nposeStyles+1; i++)
	{
		PosePointGauss * gauss = NULL;
		if (i>=nposeStyles)
			gauss = m_comDistrGlobal;
		else
			gauss = m_comDistrPerStyle[i];
			
		assert(gauss!=NULL && gauss->m_iNDims==3 && gauss->m_bAngles==false);
		textStream<<"comDistribution "<<gauss->m_iNSamples<<" ";
		textStream<<gauss->m_aMeans[0]<<" "<<gauss->m_aMeans[1]<<" "<<gauss->m_aMeans[2]<<" ";
		textStream<<gauss->m_aVariances[0]<<" "<<gauss->m_aVariances[1]<<" "<<gauss->m_aVariances[2]<<"\n";
	}
	textStream.close();
}

void PosePrior::ReadPointStats(const VKString & filename)
{
	int nposeStyles = 0;
	std::ifstream textStream(filename.c_str());
	if(textStream.is_open())
	{
		VKString tempStr;
		tempStr.readToken(textStream);	assert(tempStr=="NPoseStyles");
		textStream>>nposeStyles;
		
		for (int i=0; i<nposeStyles; i++)
		{
			int nposes = 0;
			int nregions = 0;
			
			tempStr.readToken(textStream);	assert(tempStr=="NPoses");
			textStream>>nposes;
			tempStr.readToken(textStream);	assert(tempStr=="NRegions");
			textStream>>nregions;

			m_iNPoses.push_back(nposes);
			m_RegionNSurface.push_back(std::vector<int>());
			m_RegionNGround.push_back(std::vector<int>());
			
			for (int j=0; j<nregions; j++)
			{
				m_RegionNSurface[i].push_back(0);
				m_RegionNGround[i].push_back(0);
			}
			
			tempStr.readToken(textStream);	assert(tempStr=="Surface");
			for (int j=0; j<nregions; j++)
				textStream>>m_RegionNSurface[i][j];
			
			tempStr.readToken(textStream);	assert(tempStr=="Ground");
			for (int j=0; j<nregions; j++)
				textStream>>m_RegionNGround[i][j];
		}
		
		for (int i=0; i<nposeStyles+1; i++)
		{
			PosePointGauss * gauss = NULL;
			if (i>=nposeStyles)
			{
				m_comDistrGlobal = new PosePointGauss(3, false);
				gauss = m_comDistrGlobal;
			}
			else
			{
				m_comDistrPerStyle.push_back(new PosePointGauss(3, false));
				gauss = m_comDistrPerStyle[i];
			}
			
			assert(gauss!=NULL && gauss->m_iNDims==3 && gauss->m_bAngles==false);
			tempStr.readToken(textStream);	assert(tempStr=="comDistribution");
			textStream>>gauss->m_iNSamples>>gauss->m_aMeans[0]>>gauss->m_aMeans[1]>>gauss->m_aMeans[2];
			textStream>>gauss->m_aVariances[0]>>gauss->m_aVariances[1]>>gauss->m_aVariances[2];
		}
		textStream.close();
	}
	else
		std::cout<<"[WARNING] No per pose point stats: "<<filename.c_str()<<std::endl;
}


//////////
// PriorAngles
PriorAngles::PriorAngles()
{
	m_pIK = NULL;
	m_iGridStyleFlags = GRID_None;
	m_fGridCellSize = .2;
}

void PriorAngles::SetIK(IKSolver * solver) { m_pIK = solver; }

IKSolver * PriorAngles::GetIK() { return m_pIK; }

bool PriorAngles::IsAngles() const{return true;}

double PriorAngles::Jacobian(const InteractorPose * poseIn, JacobianInfo & info, int termID, int  paramID) const
{
	// because we know parameterization -- can approximate compute the jacobian w.r.t each term
	
//	currParamPnt = currParamPnt * 3;
//	for (int i=0; i<nTheta-m_iNInitSkeletonParams; i++)
//		J.AddVal(currParamPnt + i, i+m_iNInitSkeletonParams, w);	// dtheta / dtheta = 1.
	
	PntCorrsPose * pose = (PntCorrsPose *)CastPose(poseIn);
	
	double h = M_PI / 100.;
	EnergyInfo noinfo;
	// find param

	double origval = pose->Param(paramID);
	
	pose->Param(paramID) = origval;
	double e0 = EnergyTerm(pose, termID, noinfo);
	pose->Param(paramID) = origval + h;
	double e1 = EnergyTerm(pose, termID, noinfo);
	pose->Param(paramID) = origval;
	
	return (e1 - e0) / h;
}

const PntCorrsPose * PriorAngles::CastPose(const InteractorPose * pose) 
{
	assert(pose->GetPoseType()==InteractorPose::TYPE_PntCorrsPose);
	return (const PntCorrsPose *)pose;
}

bool PriorAngles::QuickValidityTest(const InteractorPose * pose) const
{
	return ValidPairwiseDistances(CastPose(pose));
//	return true;
}

bool PriorAngles::ValidPairwiseDistances(const PntCorrsPose * pose1) const
{
	double dist_threshold = 0.1;
	PosePriorGauss * ncthis =(PosePriorGauss*)this;
	bool valid = true;
	// for all pairs of feature points
	for (int i=0; i<(int)pose1->NMatches(); i++)
	for (int j=i+1; j<(int)pose1->NMatches(); j++)
	{
		// check skeleton
		int skeleton1 = m_pIK->m_Pnts2Skeleton[pose1->Region(i)];
		int skeleton2 = m_pIK->m_Pnts2Skeleton[pose1->Region(j)];
		
		//VKString str1 = m_pIK->m_Joints[skeleton1]->m_sJointName;
		//VKString str2 = m_pIK->m_Joints[skeleton2]->m_sJointName;
		//std::cout<<"Measuing distance: "<<str1.c_str()<<" - "<<str2.c_str()<<std::endl;
		
		// 	 try to lookup maximal pairwise distance
		while (m_CachedPairwiseDists.size()<=vkMax(skeleton1, skeleton2))
			ncthis->m_CachedPairwiseDists.push_back(std::map<int, double>());
		
		double max_distance = -1;
		std::map<int, double>::iterator iter = ncthis->m_CachedPairwiseDists[skeleton1].find(skeleton2);
		if (iter != m_CachedPairwiseDists[skeleton1].end())	// already computed
			max_distance = iter->second;
		else	// compute maximal pairwise distance
		{
			std::vector<double> parentsof1;
			for (int p=0; p<(int)m_pIK->m_Joints.size(); p++)
				parentsof1.push_back(-1);
			int joint = skeleton1;
			double dist = 0;
			while (joint>=0)
			{
				parentsof1[joint] = dist;
				if (m_pIK->m_Joints[joint]->m_pParent==NULL)
					joint = -1;
				else
				{
					SkeletonLink * link = m_pIK->m_Joints[joint]->m_pParent;
					dist += link->m_ChildOffset.Length();
					joint = link->m_pParent->m_iJointID;
				}
			}
			
			joint = skeleton2;
			dist = 0;
			while (joint>=0)
			{
				if (parentsof1[joint]>=0)
				{
					max_distance = dist + parentsof1[joint];
					break;
				}
				assert(m_pIK->m_Joints[joint]->m_pParent!=NULL);
				SkeletonLink * link = m_pIK->m_Joints[joint]->m_pParent;
				dist += link->m_ChildOffset.Length();
				joint = link->m_pParent->m_iJointID;
			}
			
			
			ncthis->m_CachedPairwiseDists[skeleton1][skeleton2] = max_distance;
			ncthis->m_CachedPairwiseDists[skeleton2][skeleton1] = max_distance;
		}
		
		//	 if pairwise distance not precomputed - traverse graph (assume tree structure -- common parent)
		assert(max_distance>=0);
		double dist = 0;
		
		R3Point p1;
		R3Point p2;
		
		bool isp1pnt = pose1->Prim(i).IsType(TargetPrimitive::PRIM_Point);
		isp1pnt = isp1pnt || pose1->Prim(i).IsType(TargetPrimitive::PRIM_PointMeshless);
		
		bool isp2pnt = pose1->Prim(j).IsType(TargetPrimitive::PRIM_Point);
		isp2pnt = isp2pnt || pose1->Prim(i).IsType(TargetPrimitive::PRIM_PointMeshless);
		
		if (isp1pnt)
		{
			p1 = pose1->Prim(i).GetPosition();
			pose1->m_NormalizingXForm.Apply(p1);
		}
		if (isp2pnt)
		{
			p2 = pose1->Prim(j).GetPosition();
			pose1->m_NormalizingXForm.Apply(p2);
		}
		
		if (isp1pnt && isp2pnt)
			dist = R3Distance(p1, p2);
		else if (isp1pnt && pose1->Prim(j).IsType(TargetPrimitive::PRIM_Ground))
			dist = vkAbs(p1.Z());
		else if (isp2pnt && pose1->Prim(i).IsType(TargetPrimitive::PRIM_Ground))
			dist = vkAbs(p2.Z());

		//std::cout<<"Dist="<<dist<<" / "<<max_distance<<" diff="<<vkAbs(dist-max_distance)<<std::endl;
		if (dist>(max_distance+dist_threshold))
		{
			valid = false;
			break;
		}
		else if (dist<0.2 && isp1pnt && isp2pnt)	// no coincidental points
		{
			valid = false;
			break;
		}
	}
	
	//   check if euclidean distance below pairwise...
	return valid;
}

////////// END EFFECTORS DISTRIBUTION GRIDS /////////
void PriorAngles::ReadGridData(FILE * file)
{
	int version = 1;
	fread(&version, sizeof(int), 1, file);
	fread(&m_iGridStyleFlags, sizeof(int), 1, file);
	fread(&m_iNCondContactPnts, sizeof(int), 1, file);
	fread(&m_iNCondPoses, sizeof(int), 1, file);
	fread(&m_iNCondSymm, sizeof(int), 1, file);
	fread(&m_fGridCellSize, sizeof(double), 1, file);

	int nposes=0;
	fread(&nposes, sizeof(int), 1, file);
	for (int i=0; i<nposes; i++)
	{
		m_CandidateRegions.push_back(std::vector<int>());
		int nregions=0;
		fread(&nregions, sizeof(int), 1, file);
		for (int j=0; j<nregions; j++)
			m_CandidateRegions[i].push_back(0);
		for (int j=0; j<nregions; j++)
			fread(&(m_CandidateRegions[i][j]), sizeof(int), 1, file);
	}
	
	int c2gpairs = 0;
	int p1, p2;
	fread(&c2gpairs, sizeof(int), 1, file);
	for (int c2gp=0; c2gp < c2gpairs; c2gp++)
	{
		fread(&p1, sizeof(int), 1, file);
		fread(&p2, sizeof(int), 1, file);
		m_CondID2GridID[p1] = p2;
	}
	
	// grid stuff:
	int ngrids = 0;
	fread(&ngrids, sizeof(int), 1, file);
	
	double x, y, z;
	double val;
	double * xform = new double[4];
	int * res = new int[3];
	for (int i=0; i<ngrids; i++)
	{

//		res[0] = m_ContactProbGrids[i]->XResolution();
//		res[1] = m_ContactProbGrids[i]->YResolution();
//		res[2] = m_ContactProbGrids[i]->ZResolution();
//		
//		const R3Affine & w2g = m_ContactProbGrids[i]->WorldToGridTransformation();
//		double s = w2g.Matrix()[0][0];
//		assert(s==w2g.Matrix()[1][1] && s==w2g.Matrix()[2][2] && w2g.Matrix()[3][3]==1);
//		assert(w2g.Matrix()[0][3]==0 && w2g.Matrix()[1][3]==0 && w2g.Matrix()[2][3]==0);
//		xform[0] = w2g.Matrix()[3][0];
//		xform[1] = w2g.Matrix()[3][1];
//		xform[2] = w2g.Matrix()[3][2];
//		xform[3] = s;
		
		fread(xform, sizeof(double), 4, file);
		int npoints = 0;
		fread(res, sizeof(int), 3, file);
		fread(&npoints, sizeof(int), 1, file);
		
		R3Grid * grid = new R3Grid(res[0], res[1], res[2]);
		grid->Clear(RN_INFINITY);
		m_ContactProbGrids.push_back(grid);
		double s=xform[3];
		R3Affine w2g(R4Matrix(s, 0, 0, xform[0],
							  0, s, 0, xform[1],
							  0, 0, s, xform[2],
							  0, 0, 0, 1));
		grid->SetWorldToGridTransformation(w2g);
		ShapePointCloud * pnts = new ShapePointCloud(NULL, "gridpnts");
		m_NonZeroGridPnts.push_back(pnts);
				
		for (int nzp=0; nzp<npoints; nzp++)
		{
			fread(&x, sizeof(double), 1, file);
			fread(&y, sizeof(double), 1, file);
			fread(&z, sizeof(double), 1, file);
			fread(&val, sizeof(double), 1, file);
			pnts->AddPoint(ShapePoint(R3Point(x, y, z)));
			R3Point gp = grid->GridPosition(R3Point(x,y,z));
			grid->SetGridValue((int)gp.X(), (int)gp.Y(), (int)gp.Z(), val);
			assert(val>0);
		}
	}
	
	delete [] res;
	delete [] xform;
}

void PriorAngles::WriteGridData(FILE * file) const
{
	int version = 1;
	fwrite(&version, sizeof(int), 1, file);
	fwrite(&m_iGridStyleFlags, sizeof(int), 1, file);
	fwrite(&m_iNCondContactPnts, sizeof(int), 1, file);
	fwrite(&m_iNCondPoses, sizeof(int), 1, file);
	fwrite(&m_iNCondSymm, sizeof(int), 1, file);
	fwrite(&m_fGridCellSize, sizeof(double), 1, file);
	
	int nposes = (int)m_CandidateRegions.size();
	fwrite(&nposes, sizeof(int), 1, file);
	for (int i=0; i<nposes; i++)
	{
		int nregions = (int)m_CandidateRegions[i].size();
		fwrite(&nregions, sizeof(int), 1, file);
		for (int j=0; j<nregions; j++)
			fwrite(&(m_CandidateRegions[i][j]), sizeof(int), 1, file);
	}
	
	int c2gpairs = (int)m_CondID2GridID.size();
	fwrite(&c2gpairs, sizeof(int), 1, file);
	for (std::map<int, int>::iterator iter = ((PriorAngles*)this)->m_CondID2GridID.begin(); iter != m_CondID2GridID.end(); iter++)
	{
		fwrite(&(iter->first), sizeof(int), 1, file);
		fwrite(&(iter->second), sizeof(int), 1, file);
	}
	
	// grid stuff:
	int ngrids = (int)m_ContactProbGrids.size();
	fwrite(&ngrids, sizeof(int), 1, file);
	
	double * xform = new double[4];
	int * res = new int[3];
	for (int i=0; i<(int)m_ContactProbGrids.size(); i++)
	{
		
		res[0] = m_ContactProbGrids[i]->XResolution();
		res[1] = m_ContactProbGrids[i]->YResolution();
		res[2] = m_ContactProbGrids[i]->ZResolution();
		
		const R3Affine & w2g = m_ContactProbGrids[i]->WorldToGridTransformation();
		double s = w2g.Matrix()[0][0];
		
		assert(s==w2g.Matrix()[1][1] && s==w2g.Matrix()[2][2] && w2g.Matrix()[3][3]==1);
		assert(w2g.Matrix()[3][0]==0 && w2g.Matrix()[3][1]==0 && w2g.Matrix()[3][2]==0);
		xform[0] = w2g.Matrix()[0][3];
		xform[1] = w2g.Matrix()[1][3];
		xform[2] = w2g.Matrix()[2][3];
		xform[3] = s;
		
		fwrite(xform, sizeof(double), 4, file);
		
		int npoints = m_NonZeroGridPnts[i]->NumPoints();
		
		fwrite(res, sizeof(int), 3, file);
		fwrite(&npoints, sizeof(int), 1, file);
		
		for (int nzp=0; nzp<npoints; nzp++)
		{
			R3Point p = m_NonZeroGridPnts[i]->PointPosition(nzp);
			double val = ExactWorldValue(p, m_ContactProbGrids[i]);
			double x=p.X(), y=p.Y(), z=p.Z();
			fwrite(&x, sizeof(double), 1, file);
			fwrite(&y, sizeof(double), 1, file);
			fwrite(&z, sizeof(double), 1, file);
			fwrite(&val, sizeof(double), 1, file);
			assert(val>0);
		}
	}
	
	delete [] res;
	delete [] xform;
}

void PriorAngles::SamplePosesAndFillGrid(int gridFlags, int nsamplesPerGaussian, std::vector<PntCorrsPose*> * saveposes)
{
	std::vector<PntCorrsPose *> poses;
//	std::vector<double> energies;
	
	EnergyInfo einfo;
	TimeProfiler timer;
	std::cout<<"Sample poses"<<std::endl;
	int nposes = NSuggestedInitPoses();
	for (int i=0; i<nposes; i++)
	{
		for (int s=0; s<nsamplesPerGaussian; s++)
		{
			if (s%2000==0)
				timer.WriteProgress("sample", i*nsamplesPerGaussian+s, nsamplesPerGaussian*nposes);
			PntCorrsPose * pose = new PntCorrsPose();
				
			pose->SetPoseID(i);
			
			bool validPose = false;
			int nmaxiter = 2000;
			while (!validPose && nmaxiter-->0)
			{
				RandomSample(pose);
				m_pIK->ForwardKinematics(pose);
				validPose = true;
				for (int r=0; r<NCandidateContacts(i); r++)
				{
					int rid = RegionID(i, r);
					int jid = m_pIK->m_Pnts2Skeleton[rid];;
					R3Point pos = pose->GetPosition(jid);
					
					bool rightSide = pos.Y()<m_fGridCellSize/2.;
					bool leftSide = pos.Y()>-m_fGridCellSize/2.;
					bool rightLimb = m_pIK->m_RegionNames[rid].startsWith("right_");
					bool leftLimb = m_pIK->m_RegionNames[rid].startsWith("left_");
					validPose = ((rightSide && rightLimb) || (leftSide && leftLimb) || (!leftLimb && !rightLimb));
//					if (!validPose)
//					{
//						std::cout<<"\nInvalid: pid="<<i<<" s="<<s<<". Pos: "<<pos.Y();
//						std::cout<<" Limb: "<< m_pIK->m_RegionNames[rid].c_str()<<std::endl;
//					}
					if (!validPose)
						break;
				}
			}
			assert(nmaxiter>0);
			
			//Energy(pose, einfo);
			m_pIK->GetEnergy(pose, einfo);
			poses.push_back(pose);
			//energies.push_back(einfo.m_fPosePriorEnergy/*+einfo.m_fSymmetryEnergy*/);
			//std::cout<<"E = "<<(einfo.m_fPosePriorEnergy/*+einfo.m_fSymmetryEnergy*/)<<std::endl;

			if (saveposes!=NULL)
			{
				for (int r=0; r<NCandidateContacts(i); r++)
				{
					int rid = RegionID(i, r);
					int jid = m_pIK->m_Pnts2Skeleton[rid];;
					R3Point pos = pose->GetPosition(jid);
					pose->AddMatch(rid, TargetPrimitive(TargetPrimitive::PRIM_Point, ShapePoint(pos)));
				}
			}
			
			//std::cout<<"Energy: "<<einfo.m_fPosePriorEnergy<<std::endl;
		}
	}
	std::cout<<"\nFill Grid ("<<poses.size()<<")"<<std::endl;
	FillGridFromClusteredPoses(gridFlags, poses);
	if (saveposes!=NULL)
		*saveposes = poses;
	else
	{
		for (int i=0; i<(int)poses.size(); i++)
			delete poses[i];
		poses.clear();
	}
}

void PriorAngles::FillGridFromClusteredPoses(int gridFlags, const std::vector<PntCorrsPose *> & poses)
{
	assert(m_CandidateRegions.size()>0);
	
	m_iGridStyleFlags = gridFlags;
	// check if current implementation supports flags
	assert((m_iGridStyleFlags & GRID_CondOnRefPnt)!=0
		   && (m_iGridStyleFlags & GRID_CondOnSymm)==0);	// for now just works for one config
	

	// find conditional 'parameterization'
	if ((m_iGridStyleFlags & GRID_CondOnRefPnt)==0)
		m_iNCondContactPnts = 1;
	else
		m_iNCondContactPnts = m_pIK->m_RegionNames.count();
	
	m_iNCondPoses = 1;
	if ((m_iGridStyleFlags & GRID_CondOnSymm)==0)
		m_iNCondSymm = 1;
	else
		m_iNCondSymm = 2;
	
	int standardxform = 4;
	double * xform = new double[standardxform];

	// cancel transformations
	TimeProfiler timer;
	for (int poseID=0; poseID<(int)poses.size(); poseID++)	// for each contact - check bounding box
	{
		if (poseID%100000==0)
			timer.WriteProgress("fk", poseID, (int)(poses.size()));

		PntCorrsPose * pose = (PntCorrsPose*)poses[poseID];
		for (int xf=0; xf<standardxform; xf++)
		{
			xform[xf] = pose->Param(xf);
			pose->Param(xf) = 0;
		}
		m_pIK->ForwardKinematics(pose);
	}
	std::cout<<" - fk done"<<std::endl;
	
	// fill bounding boxes
	std::vector<R3Box> boxes;
	for (int p=0; p<(int)poses.size(); p++)	// for each contact - check bounding box
	{
		int pid = poses[p]->GetPoseID();
		int ncontacts = NCandidateContacts(pid);
		for (int ref=0; ref<ncontacts; ref++)
		{
			if (p%100000==0)
				timer.WriteProgress("bboxes", ref*(int)poses.size()+p, (int)(ncontacts*poses.size()));

			int refreg = RegionID(pid, ref);
			
			if (!IsCompatible(pid, refreg))
				continue;
			
			int refjid = m_pIK->m_Pnts2Skeleton[refreg];
			R3Point refpos = poses[p]->GetPosition(refjid);
			for (int r2=0; r2<ncontacts; r2++)
			{
				int corr = RegionID(pid, r2);
				int jid = m_pIK->m_Pnts2Skeleton[corr];

				if (refreg==corr)
					continue;
				int condID = UniqueCondID(corr, refreg, pid);
				int gridID = -1;
				R3Point pos = (poses[p]->GetPosition(jid)) +(-refpos);

				if (m_CondID2GridID.find(condID)==m_CondID2GridID.end())
				{
					gridID = (int)boxes.size();
					m_CondID2GridID[condID] = gridID;
					boxes.push_back(R3Box(pos, pos));
				}
				else
					gridID = m_CondID2GridID[condID];
				boxes[gridID].Union(pos);
			}
		}
	}
	std::cout<<" - boxes filled"<<std::endl;
	
	// create empty grids
	for (int i=0; i<(int)boxes.size(); i++)
	{
		boxes[i] = R3Box(boxes[i].XMin()-m_fGridCellSize, boxes[i].YMin()-m_fGridCellSize, boxes[i].ZMin()-m_fGridCellSize,
						 boxes[i].XMax()+m_fGridCellSize, boxes[i].YMax()+m_fGridCellSize, boxes[i].ZMax()+m_fGridCellSize);

		double s = 1./m_fGridCellSize;		// world -> grid scale
		double tx = -s * boxes[i].XMin();	// world -> grid translation (s.t. boxes.min is a grid [0, 0, 0]
		double ty = -s * boxes[i].YMin();
		double tz = -s * boxes[i].ZMin();
		
		int xres = s * boxes[i].XMax() + tx;	// find resolution to accomodate necessary box size
		int yres = s * boxes[i].YMax() + ty;
		int zres = s * boxes[i].ZMax() + tz;
		
		assert(xres*yres*zres < pow(1024, 3));	// check if grid is too big to allocate

		R3Grid * grid = new R3Grid(xres, yres, zres);	// create grid
		R3Affine w2g(R4Matrix(s, 0, 0, tx,
							  0, s, 0, ty,
							  0, 0, s, tz,
							  0, 0, 0, 1));
		grid->SetWorldToGridTransformation(w2g);	// set map
		grid->Clear(RN_INFINITY);
		assert(i==(int)m_ContactProbGrids.size());
		m_ContactProbGrids.push_back(grid);
		m_NonZeroGridPnts.push_back(new ShapePointCloud(NULL, "nzpnts"));
	}
	
	std::vector<double> nhdX;
	std::vector<double> nhdY;
	std::vector<double> nhdZ;
	for (int i=-1; i<=1; i++)
	for (int j=-1; j<=1; j++)
	for (int k=-1; k<=1; k++)
	{
		nhdX.push_back(((double)i)*0.5);
		nhdY.push_back(((double)j)*0.5);
		nhdZ.push_back(((double)k)*0.5);
	}
	
	// fill grids
	for (int p=0; p<(int)poses.size(); p++)	// for each contact - check bounding box
	{
		int pid = poses[p]->GetPoseID();
		int ncontacts = NCandidateContacts(pid);
		for (int ref=0; ref<ncontacts; ref++)	// fewer references are needed...
		{
			if (p%100000==0)
				timer.WriteProgress("fillgrid", ref*(int)poses.size()+p, (int)(ncontacts*poses.size()));
			int refreg = RegionID(pid, ref);
			if (!IsCompatible(pid, refreg))
				continue;
			
			int refjid = m_pIK->m_Pnts2Skeleton[refreg];
			R3Point refpos = poses[p]->GetPosition(refjid);
			//double energy = energies[p];
			std::vector<double> cntEnergies;
			m_pIK->GetPerContactEnergy(poses[p], refreg, m_CandidateRegions[pid], cntEnergies);
			for (int r2=0; r2<(int)ncontacts; r2++)
			{
				int corr = RegionID(pid, r2);
				int jid = m_pIK->m_Pnts2Skeleton[corr];
				
				if (refreg==corr)
					continue;

				int condID = UniqueCondID(corr, refreg, pid);
				R3Point pos = (poses[p]->GetPosition(jid)) + (-refpos);
								
				assert(m_CondID2GridID.find(condID)!=m_CondID2GridID.end());
				int gridID = m_CondID2GridID[condID];
				R3Grid * grid = m_ContactProbGrids[gridID];
				R3Point gp = grid->GridPosition(pos);
				for (int n=0; n<(int)nhdX.size(); n++)
				{
					int x = (int)(gp.X()+nhdX[n]);
					int y = (int)(gp.Y()+nhdY[n]);
					int z = (int)(gp.Z()+nhdZ[n]);
					if (x<0 || y<0 || z<0 || x>=grid->XResolution() || y>=grid->YResolution() || z>=grid->ZResolution())
					{
						assert(nhdX[n]!=0 || nhdY[n]!=0 || nhdZ[n]!=0);
						continue;
					}
					R3Point wp = R3Point((double)x+0.5, (double)y+0.5, (double)z+0.5);
					grid->GridToWorldTransformation().Apply(wp);
					
					// Do not allow: left & right limbs on opposite sides
//					bool rightSide = wp.Y()<-m_fGridCellSize;
//					bool leftSide = wp.Y()>m_fGridCellSize;
//					bool rightLimb = m_pIK->m_RegionNames[corr].startsWith("right_");
//					bool leftLimb = m_pIK->m_RegionNames[corr].startsWith("left_");
//					bool refCenterLimb = !m_pIK->m_RegionNames[refReg].startsWith("right_");
//					refCenterLimb = refCenterLimb && !m_pIK->m_RegionNames[refReg].startsWith("left_")
//					if (refCenterLimb && ((rightSide&&rightLimb) || (leftSide && leftLimb) || (!rightLimb && !leftLimb)))
//					{
						double val = grid->GridValue(x, y, z);

						if (val==RN_INFINITY)
							m_NonZeroGridPnts[gridID]->AddPoint(ShapePoint(wp));
						grid->SetGridValue(x, y, z, vkMax(vkMin(cntEnergies[r2], val), 1e-40));
//					}
				}
			}
		}
	}
	std::cout<<" - filled grids"<<std::endl;
	
//	// DEBUG
//	int uniqueID = UniqueCondID(1, 0, 1);
//	assert(m_CondID2GridID.find(uniqueID)!=m_CondID2GridID.end());
//	// END DEBUG
		
	// revert to transformations
	for (int poseID=0; poseID<(int)poses.size(); poseID++)	// for each contact - check bounding box
	{
		if (poseID%100000==0)
			timer.WriteProgress("fk2", poseID, (int)(poses.size()));

		PntCorrsPose * pose = (PntCorrsPose*)poses[poseID];
		for (int xf=0; xf<standardxform; xf++)
			pose->Param(xf) = xform[xf];
		m_pIK->ForwardKinematics(pose);
	}
	
	std::cout<<" - reverted fk"<<std::endl;
	
	delete [] xform;
}

double PriorAngles::ExactWorldValue(const R3Point & wp, const R3Grid * grid)
{
	R3Point gp = wp;
	grid->WorldToGridTransformation().Apply(gp);
	int x = (int)gp.X();
	int y = (int)gp.Y();
	int z = (int)gp.Z();
	assert(x>=0 && x < grid->XResolution() && y>=0 && y < grid->YResolution() && z>=0 && z < grid->ZResolution());
	return grid->GridValue(x, y, z);
}

void PriorAngles::FillCandidateRegions(const std::vector<PntCorrsPose *> & poses)
{
	std::vector<std::map<int, bool> > visited;
	for (int p=0; p<(int)poses.size(); p++)
	{
		int pid = poses[p]->GetPoseID();
		assert(pid>=0);
		while (pid>=(int)m_CandidateRegions.size())
		{
			m_CandidateRegions.push_back(std::vector<int>());
			visited.push_back(std::map<int, bool>());
		}
		assert(visited.size()==m_CandidateRegions.size());
		for (int m=0; m<poses[p]->NMatches(); m++)
		{
			int rid = poses[p]->Region(m);
			if (visited[pid].find(rid)==visited[pid].end())
			{
				m_CandidateRegions[pid].push_back(rid);
				visited[pid][rid] = true;
			}
		}
	}
}

int PriorAngles::GetGridID(int regionID, const PntCorrsPose * cond) const
{
	assert(cond!=NULL);
	assert((m_iGridStyleFlags & GRID_CondOnRefPnt)!=0
		   && (m_iGridStyleFlags & GRID_CondOnSymm)==0);	// for now just works for one config
	assert(cond->GetPoseID()>=0);
	
	// single reference point should be defined...
	int reference = -1;
	for (int i=0; i<cond->NMatches(); i++)
	{
		if (cond->Prim(i).IsType(TargetPrimitive::PRIM_Point))
		{
			assert(reference==-1);
			reference = cond->Region(i);
		}
		// NOTE 1: also might have more contacts if symmetry pruning is enabled
		// NOTE 2: also might be parameterized by poseid
	}
	assert(reference!=-1);
	
	assert(regionID!=reference);	// this probability always == 1
	int condID = UniqueCondID(regionID, reference, cond->GetPoseID());
	//	std::cout<<"UniqueID: ["<<regionID<<", "<<reference<<"] -> ["<<condID<<"]"<<std::endl;
	std::map<int, int>::iterator iter = ((PriorAngles*)this)->m_CondID2GridID.find(condID);
	if (iter!=m_CondID2GridID.end())
	{
		assert(iter->second>=0 && iter->second < (int)m_ContactProbGrids.size());
		return iter->second;
	}

	return -1;
}

int PriorAngles::NCandidateContacts(int poseid) const
{
	assert(poseid>=0 && poseid<(int)m_CandidateRegions.size());
	return (int)m_CandidateRegions[poseid].size();
}

bool PriorAngles::IsCompatible(int poseID, int regionID) const
{
	int ncand = NCandidateContacts(poseID);
	for (int i=0; i<ncand; i++)
	{
		if (RegionID(poseID, i)==regionID)
			return true;
	}
	return false;
}

int PriorAngles::RegionID(int poseid, int candidateID) const
{
	assert(poseid>=0 && poseid<(int)m_CandidateRegions.size());
	assert(candidateID>=0 && candidateID<(int)m_CandidateRegions[poseid].size());
	return m_CandidateRegions[poseid][candidateID];
}

const R3Grid * PriorAngles::GetGrid(int gridID) const
{
	assert(gridID>=0 && gridID < (int)m_ContactProbGrids.size());
	return m_ContactProbGrids[gridID];
}

const ShapePointCloud * PriorAngles::GetNonZeroProb(int gridID, double * gridSize) const
{
	if (gridSize!=NULL)
		*gridSize = m_fGridCellSize;
	if (gridID!=-1)
	{
		assert(gridID>=0 && gridID < (int)m_NonZeroGridPnts.size());
		return m_NonZeroGridPnts[gridID];
	}
	return NULL;
}

int PriorAngles::UniqueCondID(int targetRegion, int referenceRegion, int poseID, int symmetryRegion) const
{
	assert((m_iGridStyleFlags & GRID_CondOnSymm)==0 && symmetryRegion==0);
	assert((m_iGridStyleFlags & GRID_CondOnRefPnt)!=0 && m_iNCondContactPnts>1);
	assert(poseID>=0);
	
	int condID = referenceRegion * m_iNCondContactPnts * m_iNCondContactPnts * m_iNCondSymm * m_iNCondPoses;
	condID += poseID * m_iNCondContactPnts * m_iNCondSymm;
	condID += symmetryRegion * m_iNCondContactPnts;
	condID += targetRegion;
	return condID;
}


///////////
//  PosePointGauss
PosePointGauss::PosePointGauss(int ndims, bool angles)
{
	m_fMagnitude = 1.;
	m_bAngles = angles;
	if (ndims>0)
	{
		m_aMeans = new double[ndims];
		m_aVariances = new double[ndims];
		for (int i=0; i<ndims; i++)
		{
			m_aMeans[i] = 0;
			m_aVariances[i] = 0;
		}
		m_iNSamples = 0;
		m_iNDims = ndims;
	}
}

double PosePointGauss::Sample(int dimid, double u1, double u2) const
{
	assert(dimid < m_iNDims);
	assert(u1>=0 && u1<1 && u2>=0 && u2<1);
	if (m_aVariances[dimid]<0)
		return m_aMeans[dimid];
	
	double angle = m_aMeans[dimid] + sqrt(m_aVariances[dimid])*sqrt(-2*log(1-u2)) * cos(2*M_PI*u1);
	angle = GetNearestAngle(m_aMeans[dimid], angle);
	
	if (!(angle>=0 || angle<=0))
	{
		std::cout<<"Val="<<m_aMeans[dimid]<<" + sqrt("<<m_aVariances[dimid]<<")*";
		std::cout<<"sqrt(-2*"<<log(1-u2)<<") * cos(2pi*"<<u1<<")"<<std::endl;
		std::cout<<"[ERROR] Invalid: "<<angle<<std::endl;
		assert(false);
	}
	return angle;
	//return m_aMeans[dimid];
	
	//	double theta = 2 * M_PI * u1;
	//	double rho = sqrt(-2 * log(1 - u2));
	//	double scale = sqrt(m_aVariances[dimid]) * rho;
	//	return m_aMeans[dimid] + scale * cos(theta);
	//y = m_aMeans[dimid] + scale * sin(theta)	// another number
}

void PosePointGauss::Update(const double * sample)
{
	for (int i=0; i<m_iNDims; i++)
	{
		int N = m_iNSamples;
		double oldMean = m_aMeans[i];
		
		double value = sample[i];
		if (m_bAngles)
			value = GetNearestAngle(m_aMeans[i], value);
		
		double newMean = (oldMean * N + value) / (N+1);
		double oldVariance = m_aVariances[i];
		
		m_aVariances[i] = (N*oldVariance + N*pow(oldMean, 2.) - 2 * N * oldMean * newMean + N * pow(newMean, 2.));
		m_aVariances[i] = (m_aVariances[i] + pow(value - newMean, 2.)) / (N+1);
		m_aMeans[i] = newMean;
		
		if (m_aVariances[i]<0)
			m_aVariances[i] = 0;
		assert(m_aVariances[i]>=0);
	}
	m_iNSamples++;
}

void PosePointGauss::UpdateVarianceFromZero(const double * sample)
{
	for (int i=0; i<m_iNDims; i++)
	{
		m_aMeans[i] = 0;
		int N = m_iNSamples;
		
		double value = sample[i];
		if (m_bAngles)
			value = GetNearestAngle(m_aMeans[i], value);
		
		m_aVariances[i] = (m_aVariances[i] * N + value*value)/(N+1);
	}
	m_iNSamples++;
}

void PosePointGauss::WriteDistribution(FILE * file) const
{
	fwrite(&m_iNSamples, sizeof(int), 1, file);
	fwrite(&m_iNDims, sizeof(int), 1, file);
	fwrite(&m_bAngles, sizeof(int), 1, file);
	fwrite(m_aMeans, sizeof(double), m_iNDims, file);
	fwrite(m_aVariances, sizeof(double), m_iNDims, file);
	fwrite(&m_fMagnitude, sizeof(double), 1, file);
}

void PosePointGauss::ReadDistribution(FILE * file)
{
	fread(&m_iNSamples, sizeof(int), 1, file);
	fread(&m_iNDims, sizeof(int), 1, file);
	fread(&m_bAngles, sizeof(int), 1, file);
	m_aMeans = new double[m_iNDims];
	m_aVariances = new double[m_iNDims];
	fread(m_aMeans, sizeof(double), m_iNDims, file);
	fread(m_aVariances, sizeof(double), m_iNDims, file);
	double epsilon = 1E-5;	// avoid small variances for numerical stability
	for (int i=0; i<m_iNDims; i++)
	{
		if (m_aVariances[i] < 0)
			m_aVariances[i] = RN_INFINITY;
		else if (m_aVariances[i] < epsilon)
			m_aVariances[i] = epsilon;
	}
	fread(&m_fMagnitude, sizeof(double), 1, file);
	
	//	std::cout<<"Read Distribution: "<<m_iNDims<<" "<<m_iNSamples<<": m=[ ";
	//	for (int i=0; i<m_iNDims; i++)
	//		std::cout<<m_aMeans[i]<<" ";
	//	std::cout<<"] v=[ ";
	//	for (int i=0; i<m_iNDims; i++)
	//		std::cout<<m_aVariances[i]<<" ";
	//	std::cout<<"]"<<std::endl;
}

PosePointGauss * PosePointGauss::CreateDistribution(FILE * file)
{
	PosePointGauss * gauss = new PosePointGauss(0);
	gauss->ReadDistribution(file);
	return gauss;
}


double PosePointGauss::GetNearestAngle(double mean, double angle)
{
	while (angle<-M_PI)
		angle += 2*M_PI;
	while (angle>M_PI)
		angle -= 2*M_PI;
	
	double angle1 = angle+2*M_PI;
	double angle2 = angle-2*M_PI;
	double dist = vkAbs(mean-angle);
	double dist1 = vkAbs(mean-angle1);
	double dist2 = vkAbs(mean-angle2);
	
	if (dist<=dist1 && dist<=dist2)
		return angle;
	else if (dist1<=dist2)
		return angle1;
	return angle2;
}



