#include "PosePrior.h"
#include "PosePriorGauss.h"

#ifndef __POSE_PRIOR_MIX_GAUSS_H
#define __POSE_PRIOR_MIX_GAUSS_H

// some forward declarations for gausssian
class PosePointGauss;
class MultiGauss;

/**
 * mixture of gaussians
 */
class PosePriorMixGauss : public PriorAngles
{
	public:
		enum TermTypes
		{
			PRIORTERM_Gaussians 		= 1<<0,
			PRIORTERM_Symmetry  		= 1<<1,
		};
	
		PosePriorMixGauss();
		virtual void AddGaussian(int linkID, PosePointGauss * gauss);
		virtual void WritePrior(const VKString & filename) const;
		virtual void ReadPrior(FILE * file);

		virtual PosePriorType PriorType() const;
		virtual MultiGauss * GetDistr(int linkid);
	
		virtual int NTerms() const;
		virtual bool IsSymmetryTerm(int termID) const;
		virtual double EnergyTerm(const InteractorPose * pose, int termID, EnergyInfo & info) const;
		virtual void NonZeroJacobians(std::vector<int> & termIDs, std::vector<int> & paramIDs) const;
		virtual int NSuggestedInitPoses() const;
		virtual void InitPose(int poseID, InteractorPose * pose) const;
		virtual void RandomSample(InteractorPose * pose) const;
	
		virtual void InitDistributions();

		virtual void SetTerms(int terms);
		virtual void UpdateSymmetry(const InteractorPose * trainingPose, bool noToesPalms);
		virtual void UpdatePoseDNAs(std::vector<int> & dnaStr);
		virtual void SortDNAs();

		virtual void InitSymmetryCoupling();
		virtual int NSymmetryCouplings() const;
		virtual int NParamsForSymmJoint(int symmID) const;
		virtual int SymmParamID(int symmID, int paramID) const;
		virtual double SignedDistToSymmetry(int symmID, int paramID, const InteractorPose * pose) const;	// always for left
		virtual void SymmCoupling(int symmID, int * left, int * right) const;
	
		virtual VKString GetStr() const;
	
	
	protected:
		std::vector<MultiGauss*> m_LinkDistributions;
		std::vector<int> m_NonZeroLinkDistr;
		std::vector<PosePointGauss*> m_LinkSymmetry;	// variance-only (deviation from 'mean')
	
		enum SymmType
		{
			SYM_None,
			SYM_Equal,
			SYM_DiffSign,
			SYM_Zero
		};
		std::vector<int> m_SymmLinkLeft;	// left (or center parameters)
		std::vector<int> m_SymmLinkRight;	// right
		std::vector<std::vector<SymmType> > m_PerParamRelationships;
	
		std::map<VKString, int> m_Str2DNA;
		std::vector<std::vector<int> > m_GaussianPoseDNAs;	// plausible poses (in training data)
		std::vector<int> m_PerDNAPoseCounter;				// # models in pose
		
		double m_fMaxPenalty;

		bool IsTermSet(TermTypes term);
		int m_iTermTypes;
};

/**
 * Mixture of gaussians
 */
class MultiGauss
{
	public:
		MultiGauss();
		virtual void AddGaussian(PosePointGauss * gauss);
		virtual int NGaussians() const;
		virtual PosePointGauss * GetGaussian(int g);
		virtual const PosePointGauss * GetGaussian(int g) const;
		
	protected:
		std::vector<PosePointGauss*> m_Gaussians;
};

#endif


