/***************************************************************************
 *   Copyright (C) 2008 by Vladimir Kim									   *
 *   Author: Vladimir Kim (vk@princeton.edu)							   *
 ***************************************************************************/

#ifndef __ShapeLocalPCA_H
#define __ShapeLocalPCA_H

#include <vector>
#include <map>
#include <iostream>
#include <assert.h>

#include "ShapeValue.h"

#include "ShapePoint.h"
#include "ShapePointCloud.h"


/**
 * 
 */
class ShapeLocalPCA : public ShapeValue
{
	public:
		ShapeLocalPCA(GeomShape * shape, const VKString & valueName, double radius, int kNN, bool normRad,
					  const std::vector<int> & subset = std::vector<int>());
		ShapeLocalPCA(ShapePointCloud * pnts, const VKString & valueName, double radius,
					  ShapePointCloud * densepnts, ShapeDistance * metric);
		virtual ~ShapeLocalPCA();
		virtual VKString GetClassName() const;
		
		virtual void ComputePntsPCA(bool verbouse = false);
		virtual R3Triad GetPCA(const ShapePoint & pnt, double * stdDev) const;
	
		virtual double GetPntsRadius() const;
		virtual int GetKNN() const;
		virtual bool IsNormalized() const;
		virtual double GetRadiusScale() const;
	
	
		static void GetNhd(const R3Point & pos, const R3Kdtree<const PointInSet *> * kdTree, 
						   double rad, int kNN, std::vector<int> & nhdIDs);
		
		
	
	
	protected:
		double m_fRadius;
		int m_ikNN;
	
		bool m_bNormRad;
		double m_fRadScale;
	
		// if subset
		const R3Kdtree<const PointInSet *> * m_pSubsetKdTree;
		std::vector<int> m_Subset;
		ShapeDistance * m_pMetric;
		ShapePointCloud * m_pDensePoints;
		
		
};

#endif

