/***************************************************************************
 *   Copyright (C) 2008 by Vladimir Kim									   *
 *   Author: Vladimir Kim (vk@princeton.edu)							   *
 ***************************************************************************/

#ifndef __DatabaseVisWindow_H
#define __DatabaseVisWindow_H

#include "OpenGLHeaders.h"

#include "VisWindow.h"
#include "ModelDatabase.h"
#include "VisModel3D.h"
#include "VisButtonsMenu.h"
#include "DBCorrs.h"
#include "VisCorrs.h"

class DBApp;

class DatabaseVisWindow : public VisWindow
{
	public:
		enum ListGroups
		{
			LISTGP_Models		= 1,
			LISTGP_Geoms		= 2,
			LISTGP_GeomVals		= 4,
			LISTGP_GeomDists	= 8,
			LISTGP_MeshFlags	= 16,
			LISTGP_Corrs		= 32,
			LISTGP_CorrVals		= 64,
			LISTGP_CorrFlags	= 128
		};
	
		static int LIST_GROUPS_MODELS;
		static int LIST_GROUPS_CORRS;
	
		enum RenderDesignFlags
		{
			RENDER_DESIGN_BOXES		= 1,
			RENDER_MODEL_LIST		= 2,
			RENDER_BOXID			= 4
		};
	
		// todo: optional array of apps
		DatabaseVisWindow(ModelDatabase * db, DBCorrs * corrs = NULL, int listGPFlags = 0xFFFFFFFF,
						  int nMainWindowsGridX = 1, int nMainWindowsGridY = 1,
						  int nAdditionalGridX = 1, int nAdditionalGridY = 3, double xSeparator = .6,
						  int width = -1, int height = -1);
		virtual ~DatabaseVisWindow();
		virtual VKString GetClassName() const;
		virtual void SetApp(DBApp* app);
		virtual void KeyPressed(const char key, int mouseX, int mouseY);
		virtual void MouseButtonEvent(int x, int y, MouseButton mouseButtonID, 
									  MouseButtonState newState);
		virtual void MouseMotionEvent(int x, int y, int dx, int dy);
		virtual void Resize(int w, int h);
		
		virtual void SetDesignFlags(int flags);
		virtual void ToggleDesignFlag(RenderDesignFlags flag);
		virtual bool IsDesignFlagOn(RenderDesignFlags flag);
		virtual void Draw();
		virtual void InitializeGL();	
		
	// subwindows routines
		virtual int ClickInWindow(int x, int y);
		virtual void GetWindowParams(int wndID, double * cX, double * cY, double * scale, 
									 double * dx=NULL, double * dy=NULL);
		virtual void DrawSubwindow(int wndID);
	
		virtual int NumMainWindows() const;
		virtual int NumAdditionalWindows() const;
		virtual int NumWindows();
		virtual void NumMainWindows(int & x, int & y) const;
		virtual void NumAdditionalWindows(int & x, int & y) const;
	
		virtual void SetBackgroundColor(int bk);
		virtual void GetBackgroundColor(double * r, double * g, double * b) const;
	
	// some getters for DBApps
		virtual int SelectedWindow() const;
		virtual R3Affine GetEmptySubwindowCamera(int subwindow);
		virtual VisModel3D * GetEmptyWindowXForm(int subwindow);
		
		virtual R3Affine GetSubwindowCamera(int subwindow );
		virtual std::vector<VisModel3D *> & GetDrawableModels();
		virtual std::vector<int> & WindowToModel();
		virtual int ModelID(int subwindow );
		virtual int & GetOrderedPage();
		virtual std::vector<int> & GetModelOrdering();
		virtual void UpdatePageAndOrdering();
		virtual void ChangePage(int change);
	
	// text lists
		virtual void DrawModelListSubwindow();
		virtual void ToggleGeometryValueSelection(int geomID, int dir);
		virtual void ToggleGeometrySelection(int geomID);
		virtual void ProcessClickInModelListSubwindow(double wndX, double wndY);
		virtual void SelectModelWindow(int wndID);
		virtual void SelectModelInWindow(int modelID);
	
	// read / write current transform (to have consistent viewpoint, e.g. for a paper)
		virtual void WriteAllCameras();
		virtual void ReadAllCameras();
	
		virtual void UpdateLists();
	protected:
		virtual int GroupID(ListGroups gp);
		virtual ListGroups GroupType(int groupID);
		virtual void AddGroup(ListGroups gp, const VKString & gpName,
							  VisButtonsGroup::ButtonsGroupType tp);
		std::map<ListGroups, int> m_ListToGroupID;
		std::vector<ListGroups> m_GroupToType;
	
		DBApp * m_pApp;
		ModelDatabase * m_pDB;
		int m_iSelectedWindow;
			
		std::vector<VisModel3D *> m_DrawableModels;
		std::vector<int> m_WindowToModel;
		std::vector<VisModel3D*> m_EmptyWindowXForms;
	
	// window descriptor (# subwindows, etc..)
		int m_iMainWndGridX;
		int m_iMainWndGridY;
		int m_iAdditionalWndGridX;
		int m_iAdditionalWndGridY;
		double m_fXSeparator;

	// text lists
		double m_WindowListDims[2];
		
		VisButtonsMenu * m_pLists;
		int m_iDesignFlags;
	
	// todo (quick browsing):
		int m_iOrderedModelPage;
		std::vector<int> m_ModelOrdering;
	
	// corrs
		static VKString m_sPairwisePrefix;
		static VKString m_sCorrPrefix;
		DBCorrs * m_pDBCorrs;
		VisCorrs * m_pCorrsVis;
			
		int m_iFromWindowID;
		int m_iToWindowID;
	
		int m_iBackgroundColorID;
		int m_iListGroupsFlags;
};

#endif

