/***************************************************************************
 *   Copyright (C) 2008 by Vladimir Kim									   *
 *   Author: Vladimir Kim (vk@princeton.edu)							   *
 ***************************************************************************/

#include "VisCorrs.h"

void VisCorrs::DrawErrorMsg(VisWindow * window, const VKString & errMsg)
{
	window->GetRenderer()->RenderText(R2Point(-.75, 0), errMsg, 
									  1., 0., 0., VisWindowRenderer::TEXT_SIZE_LARGE);
}

void VisCorrs::DrawCorrsPnt2Pnt(VisWindow * window, 
								const R3Affine & xForm1, double r1, 
								const std::vector<R3Point> & pnts1,
								const R3Affine & xForm2, double r2,
								const std::vector<R3Point> & pnts2,
								bool drawLines, bool drawSpheres)
{
	glEnable(GL_DEPTH_TEST);
	window->LoadStandardSceneProjectionCamera();
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	
	glDisable(GL_CULL_FACE);
	
	static GLfloat material[4];	
	assert(pnts1.size()==pnts2.size());

		// draw point-to-point corrs
	if (drawSpheres)
	{
		glLoadIdentity();
		xForm1.Load();
		for (int i=0; i<(int)pnts1.size(); i++)
		{
			if (pnts1.size()<=18)
				window->MapIntToRadicalColor(i, material);
			else
				window->MapIntToColor(i, material);
			
			glEnable(GL_LIGHTING);
			glMaterialfv(GL_FRONT, GL_AMBIENT_AND_DIFFUSE, material);
			R3Sphere(pnts1[i], r1).Draw();
		}
		
		glLoadIdentity();
		xForm2.Load();
		for (int i=0; i<(int)pnts2.size(); i++)
		{
			if (pnts2.size()<=18)
				window->MapIntToRadicalColor(i, material);
			else
				window->MapIntToColor(i, material);
			
			glEnable(GL_LIGHTING);
			glMaterialfv(GL_FRONT, GL_AMBIENT_AND_DIFFUSE, material);
			R3Sphere(pnts2[i], r2).Draw();
		}
	}
	
		// draw line-to-line corrs
	if (drawLines)
	{
		glLoadIdentity();
		for (int i=0; i<(int)pnts2.size(); i++)
		{
			glDisable(GL_LIGHTING);
			glColor3d(0, 1., 0.);
			glBegin(GL_LINES);
			R3Point p1 = pnts1[i];
			R3Point p2 = pnts2[i];
			xForm1.Apply(p1);
			xForm2.Apply(p2);
			glVertex3d(p1.X(), p1.Y(), p1.Z());
			glVertex3d(p2.X(), p2.Y(), p2.Z());
			glEnd();

		}	
	}
}

//void VisCorrs::DrawAffineCorrs(VisWindow * window, 
//							   VisModel3D * vis1, int camera1, 
//							   VisModel3D * vis2, int camera2,
//							   CorrsAffineAlign * affineCorrs)
//{
//	Model3D * m1 = vis1->GetModel();
//	Model3D * m2 = vis2->GetModel();
//	assert(m1!=NULL && m2!=NULL);
//	
//	GeomShape * g1 = m1->GetGeomShape(Model3D::m_sMainGeomName);
//	GeomShape * g2 = m2->GetGeomShape(Model3D::m_sMainGeomName);
//	
//	if (g1==NULL || g2==NULL)
//	{
//		DrawErrorMsg(window, "Affine Corrs: no main shape");
//		return;
//	}
//	
//	ShapeMesh * mesh1 = g1->SafeCastToMesh();
//	ShapeMesh * mesh2 = g2->SafeCastToMesh();
//
//	if (mesh1==NULL || mesh2==NULL)
//	{
//		DrawErrorMsg(window, "Affine Corrs: main shape is not a mesh");
//		return;
//	}
//	
//	const R3Affine & aff12 = affineCorrs->GetTransformation();
//	R3Affine aff21 = CorrsAffineAlign::InverseAffine(aff12);
//	
//	R3Affine aff1 = vis1->GetCameraTransform(camera1);
//	
////	R3Affine aff2 = vis1->GetCameraTransform(aff21, camera1);
//	vis1->SetObjSpaceTransform(camera1, aff12, true);// incorrect - need to normalize here after alinging..
//	R3Affine aff2 = vis1->GetCameraTransform(camera1);				// get transform with inserted transformation
//	vis1->SetObjSpaceTransform(camera1, R3identity_affine, true);	// return back
//	assert(false);
//	
//	VisModel3D::DrawMeshFaces(mesh1, false, window, aff1, NULL, 0, true, .8, .1, .1, .5);
//	VisModel3D::DrawMeshFaces(mesh2, false, window, aff2, NULL, 0, true, .1, .1, .8, .5);	
//}

void VisCorrs::DrawFuzzyPnts(VisWindow * window, 
							 const R3Affine & xForm2, double r2,
							 std::vector<R3Point> & pnts2,
							 std::vector<double> & vals2,
							 double minValue, double maxValue)
{
	glEnable(GL_DEPTH_TEST);
	window->LoadStandardSceneProjectionCamera();
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	xForm2.Load();
	
	static GLfloat material[4];	
	glDisable(GL_CULL_FACE);

	glEnable(GL_LIGHTING);

	double maxRad = r2 * 2.;
	double minRad = r2 * 1.01;
	assert(pnts2.size()==vals2.size());
	//std::cout<<"Drawing: "<<pnts2.size()<<" pnts"<<std::endl;
	for (int i=0; i<(int)pnts2.size(); i++)
	{
		double v = vals2[i];
		R3Point p2 = pnts2[i];
//		xForm2.Apply(p2);

		window->MapScalarToColor(v, minValue, maxValue, material);
		glMaterialfv(GL_FRONT, GL_AMBIENT_AND_DIFFUSE, material);
		double r = maxRad;
		if (minValue!=maxValue)
			r = minRad + (maxRad-minRad) * (v - minValue) / (maxValue - minValue);

		R3Sphere(p2, r).Draw();
	}
}


VisCorrs::VisCorrs(DBCorrs * corrs, int drawCorrFlags)
{
	m_fPointsRelativeRadius = .05;
	m_pSelectedModelFrom = NULL;
	m_pSelectedModelTo = NULL;
	m_sSelectedModelFromName = "none";
	m_sSelectedModelToName = "none";
	m_SelectedCorrType = DBCorrs::DBCORRS_Unknown;
	m_sSelectedCorrName = "none";
	m_iSelectedCorrID = -1;
	m_iCameraFrom = -1;
	m_iCameraTo = -1;
	m_pDBCorrs = corrs;
	m_iDrawFlags = drawCorrFlags;
	
	m_pCachedFuzzyValues = NULL;
}

void VisCorrs::Draw(VisWindow * window)
{
	if (m_iSelectedCorrID==-1 || m_SelectedCorrType==DBCorrs::DBCORRS_Unknown
		 || m_sSelectedCorrName=="none" || m_pDBCorrs==NULL)
		return;

	if (m_iCameraFrom<0 || m_iCameraTo < 0
		|| m_pSelectedModelFrom == NULL || m_pSelectedModelTo==NULL
		|| m_sSelectedModelFromName=="none" || m_sSelectedModelToName=="none")
		return;

	if (m_iDrawFlags==0)
		return;

	ShapeCorrs * corrs = NULL;
	if (m_SelectedCorrType==DBCorrs::DBCORRS_PairwiseColl)
	{
		CollectionPairwise * pairwise = m_pDBCorrs->GetPairColl(m_sSelectedCorrName);
		assert(pairwise!=NULL);
		const std::vector<int> & allCorrs = pairwise->GetCorrPairs(m_sSelectedModelFromName, 
																   m_sSelectedModelToName);
		assert(m_iSelectedCorrID < (int)allCorrs.size());
		
		corrs = pairwise->GetCorr(allCorrs[m_iSelectedCorrID]);
	}
	else if (m_SelectedCorrType==DBCorrs::DBCORRS_Corrs)
		corrs = m_pDBCorrs->GetCorrs(m_sSelectedCorrName);
	else
		std::cout<<"[WARNING] Unknown type of corrs (DBCorrs::DBCORRS_*)"<<std::endl;
	assert(corrs!=NULL);

	std::vector<GeomShape*> supportedDomain;
	std::vector<GeomShape*> supportedRange;
	// go through renderable 'from' geometries 
	for (int i=0; i<m_pSelectedModelFrom->NumRenderableGeometries(); i++)
	{
		GeomShape * shape = m_pSelectedModelFrom->GetRenderableGeometry(i);
		if (corrs->IsShapeSupportedDomain(shape))
			supportedDomain.push_back(shape);
	}

	// go through renderable 'to' geometries
	for (int i=0; i<m_pSelectedModelTo->NumRenderableGeometries(); i++)
	{
		GeomShape * shape = m_pSelectedModelTo->GetRenderableGeometry(i);
		if (corrs->IsShapeSupportedRange(shape))
			supportedRange.push_back(shape);
	}
	
	ShapePointCloud * pnts1 = NULL;
	for (int i=0; i<(int)supportedDomain.size(); i++)
	{
		ShapePointCloud * pnts = supportedDomain[i]->SafeCastToPointCloud();
		if (pnts!=NULL)	// might not be the desired choice...
		{
			pnts1 = pnts;
			break;
		}
	}	
	
	VKString errMsg  = "none";
	if (supportedDomain.size()==0 || supportedRange.size()==0)
		errMsg = "No geometries in supported range/domain";
	if ((IsFlagOn(DRAW_COLORED_PNTS) || IsFlagOn(DRAW_LINES)) && errMsg=="none")
	{
		GeomShape * shape2 = supportedRange[0];
		
		if (pnts1==NULL)
			errMsg = "Corrs Err: Could not find 'from' pnts";
		else if (shape2==NULL)
			errMsg = "Corrs Err: Could not find 'to' pnts";
		else
		{
			double s1 = 1.;
			double s2 = 1.;
			R3Affine aff1 = m_pSelectedModelFrom->GetCameraTransform(m_iCameraFrom);
			R3Affine aff2 = m_pSelectedModelTo->GetCameraTransform(m_iCameraTo);
			s1 = m_pSelectedModelFrom->GetMeshRadius() * m_fPointsRelativeRadius;
			s2 = m_pSelectedModelTo->GetMeshRadius() * m_fPointsRelativeRadius;
			DrawCorrsPnt2Pnt(window, s1, s2, pnts1, aff1, shape2, aff2, corrs);
		}
	}
	
	if (IsFlagOn(DRAW_AFFINE_ALIGN))
	{
		assert(false);
//		if (corrs->GetCorrsClassName()=="CorrsAffineAlign")
//			DrawAffineCorrs(window, m_pSelectedModelFrom, m_iCameraFrom,
//							m_pSelectedModelTo, m_iCameraTo, (CorrsAffineAlign *) corrs);
//		else
//			errMsg = "Corrs is not affine";
	}
	
	if (IsFlagOn(DRAW_FUZZY_VALS))
	{
		const ShapePoint & sel = m_pSelectedModelFrom->SelectedPoint();
		if (!sel.Invalid() && corrs->PointInDomain(sel))
			DrawFuzzyPnts(window, corrs, sel);
		else
			errMsg = "Fuzzy pnt not selected (or cannot be mapped)";
		
		// draw points on 'from' even if disabled
		if (!m_pSelectedModelFrom->IsFlagOn(VisModel3D::DRAW_POINTS))
		{
			R3Affine aff1 = m_pSelectedModelFrom->GetCameraTransform(m_iCameraFrom);
			VisModel3D::DrawPntsWithScalar(pnts1, false, false, false, false,
										   window, aff1, m_pSelectedModelFrom->GetMeshRadius(),
										   m_pSelectedModelFrom->GetRelativePntRadius());
		}
	}
	
	if (errMsg!="none")
		DrawErrorMsg(window, errMsg);
}

void VisCorrs::DrawCorrsPnt2Pnt(VisWindow * window, double r1, double r2,
								ShapePointCloud * pnts1, const R3Affine & xForm1,
								GeomShape * shape2, const R3Affine & xForm2, 
								ShapeCorrs * corr)
{
	// todo: can cache points...
	std::vector<R3Point> positions1;
	std::vector<R3Point> positions2;
	
	int nPnts = pnts1->NumPoints();
	for (int i=0; i<nPnts; i++)
	{
		ShapePoint sp1 = pnts1->GetPoint(i);
		ShapePoint sp2 = corr->BestCorr(sp1, shape2);

		if (sp1.Invalid() || sp2.Invalid())
			continue;
		
		positions1.push_back(sp1.GetPosition());
		positions2.push_back(sp2.GetPosition());
	}
	
	DrawCorrsPnt2Pnt(window, xForm1, r1, positions1, xForm2, r2, positions2, 
					 IsFlagOn(DRAW_LINES), IsFlagOn(DRAW_COLORED_PNTS));
}

void VisCorrs::DrawFuzzyPnts(VisWindow * window, ShapeCorrs * corrs, const ShapePoint & sel)
{
	assert(corrs!=NULL);
		// init cache
	if (m_pCachedFuzzyValues==NULL)
	{
		m_pCachedFuzzyValues = new CachedFuzzyValues();
		m_pCachedFuzzyValues->m_Selected = ShapePoint();
	}
	
		// recompute correspondences
	if (m_pCachedFuzzyValues->m_Selected!=sel)
	{
		m_pCachedFuzzyValues->m_Selected = sel;
		
		m_pCachedFuzzyValues->m_RenderModelNames.clear();
		m_pCachedFuzzyValues->m_CurrCorrs.clear();
		m_pCachedFuzzyValues->m_CorrVals.clear();
		m_pCachedFuzzyValues->m_MinVals.clear();
		m_pCachedFuzzyValues->m_MaxVals.clear();
		
		m_pCachedFuzzyValues->m_RenderModelNames.push_back("none");
		m_pCachedFuzzyValues->m_CurrCorrs.push_back(std::vector<R3Point>());
		m_pCachedFuzzyValues->m_CorrVals.push_back(std::vector<double>());
		m_pCachedFuzzyValues->m_MinVals.push_back(-1);
		m_pCachedFuzzyValues->m_MaxVals.push_back(-1);
		
		m_pCachedFuzzyValues->m_AllCorrs.clear();
		m_pCachedFuzzyValues->m_AllVals.clear();
		
		corrs->GetCorrs(sel, m_pCachedFuzzyValues->m_AllCorrs, 
						"default", &(m_pCachedFuzzyValues->m_AllVals));
	}
	
		// recompute cached models (todo: loop over many models)
	if (m_pCachedFuzzyValues->m_RenderModelNames[0]!=m_sSelectedModelToName)
	{
		m_pCachedFuzzyValues->m_RenderModelNames[0] = m_sSelectedModelToName;
		std::vector<R3Point> & pntCorrs = m_pCachedFuzzyValues->m_CurrCorrs[0];
		std::vector<double> & cvals = m_pCachedFuzzyValues->m_CorrVals[0];
		pntCorrs.clear();
		cvals.clear();
		double & minVal = m_pCachedFuzzyValues->m_MinVals[0];
		double & maxVal = m_pCachedFuzzyValues->m_MaxVals[0];
		
		minVal = FLT_MAX;
		maxVal = -FLT_MAX;
		
		for (int i=0; i<(int)m_pCachedFuzzyValues->m_AllCorrs.size(); i++)
		{
			const ShapePoint & p = m_pCachedFuzzyValues->m_AllCorrs[i];
			if (p.GetModel() == m_pSelectedModelTo->GetModel())
			{
				double v = m_pCachedFuzzyValues->m_AllVals[i];
				
				minVal = vkMin(minVal, v);
				maxVal = vkMax(maxVal, v);
				cvals.push_back(v);
				pntCorrs.push_back(p.GetPosition());
			}
		}
//		std::cout<<"For 'to' model: "<<minVal<<" - "<<maxVal<<std::endl;
	}
	
		// render
	assert(m_pSelectedModelTo!=NULL);
	double r2 = 1.;
	R3Affine xForm2 = m_pSelectedModelTo->GetCameraTransform(m_iCameraTo);
	
	r2 *= m_pSelectedModelTo->GetMeshRadius() * m_fPointsRelativeRadius;
	
	assert(m_pCachedFuzzyValues!=NULL);
	assert(m_pCachedFuzzyValues->m_CurrCorrs.size()>0 
		   && m_pCachedFuzzyValues->m_CorrVals.size()>0);
	
	std::vector<R3Point> & pntCorrs = m_pCachedFuzzyValues->m_CurrCorrs[0];
	std::vector<double> & cvals = m_pCachedFuzzyValues->m_CorrVals[0];
	double minVal = m_pCachedFuzzyValues->m_MinVals[0];
	double maxVal = m_pCachedFuzzyValues->m_MaxVals[0];

	DrawFuzzyPnts(window, xForm2, r2, 
				  pntCorrs, cvals, minVal, maxVal);
}

void VisCorrs::SelectPair(const VKString & modelFrom, 
						  VisModel3D * visModelFrom, int cameraFrom,
						  const VKString & modelTo, 
						  VisModel3D * visModelTo, int cameraTo)
{
	m_iCameraFrom = cameraFrom;
	m_iCameraTo = cameraTo;
	m_pSelectedModelFrom = visModelFrom;
	m_pSelectedModelTo = visModelTo;
	m_sSelectedModelFromName = modelFrom;
	m_sSelectedModelToName = modelTo;
	m_iSelectedCorrID = -1;
}

void VisCorrs::SelectCorr(DBCorrs::EntryType entryType, 
						  const VKString & corrName, int selectedCorrID)
{
	m_SelectedCorrType = entryType;
	m_sSelectedCorrName = corrName;
	m_iSelectedCorrID = selectedCorrID;
}

const VKString & VisCorrs::ModelFromName()
{
	return m_sSelectedModelFromName;
}

const VKString & VisCorrs::ModelToName()
{
	return m_sSelectedModelToName;	
}

bool VisCorrs::IsFlagOn(DrawFlags flag)
{
	return (m_iDrawFlags & flag)!=0;
}

void VisCorrs::ToggleFlag(DrawFlags drawFlag)
{
	SetFlag(drawFlag, !IsFlagOn(drawFlag));
}

void VisCorrs::SetFlag(DrawFlags drawFlag, bool enable)
{
	if (enable)
		m_iDrawFlags |= drawFlag;
	else
		m_iDrawFlags &= (~drawFlag);
}



