/***************************************************************************
 *   Copyright (C) 2008 by Vladimir Kim									   *
 *   Author: Vladimir Kim (vk@princeton.edu)							   *
 ***************************************************************************/

#ifndef __VisCorrs_H
#define __VisCorrs_H


#include "OpenGLHeaders.h"
#include "Model3D.h"
#include "GeomShape.h"
#include "ShapePointCloud.h"
#include "ShapeManifoldMesh.h"

#include "ShapeCorrs.h"
#include "VisWindow.h"
#include "DBCorrs.h"
#include "VisModel3D.h"

struct CachedFuzzyValues
{
	ShapePoint m_Selected;
	
	// cached models
	std::vector<VKString> m_RenderModelNames;
	std::vector<std::vector<R3Point> > m_CurrCorrs;
	std::vector<std::vector<double> > m_CorrVals;
	std::vector<double> m_MinVals;
	std::vector<double> m_MaxVals;

	// cached corrs (models might change)
	std::vector<ShapePoint> m_AllCorrs;
	std::vector<double> m_AllVals;
};

class VisCorrs
{
	public:
		enum DrawFlags
		{
			DRAW_NONE			= 0,
			DRAW_COLORED_PNTS	= 1,
			DRAW_LINES			= 2,
			DRAW_VALUE_TRANSFER	= 4,
			DRAW_AFFINE_ALIGN	= 8,
			DRAW_FUZZY_VALS		= 1024
		};
	
		// lines / points
		static void DrawCorrsPnt2Pnt(VisWindow * window, 
									 const R3Affine & xForm1, double r1, 
									 const std::vector<R3Point> & pnts1,
									 const R3Affine & xForm2, double r2,
									 const std::vector<R3Point> & pnts2,
									 bool drawLines, bool drawSpheres);
	
//		static void DrawAffineCorrs(VisWindow * window, 
//									VisModel3D * vis1, int camera1, 
//									VisModel3D * vis2, int camera2,
//									CorrsAffineAlign * affineCorrs);
	
		static void DrawFuzzyPnts(VisWindow * window, 
								  const R3Affine & xForm2, double r2,
								  std::vector<R3Point> & pnts2,
								  std::vector<double> & vals2,
								  double minValue, double maxValue);
	
	
		static void DrawErrorMsg(VisWindow * window, const VKString & errMsg);
	
		VisCorrs(DBCorrs * corrs, int drawCorrFlags);
		
		virtual void Draw(VisWindow * window);
	
		virtual void DrawCorrsPnt2Pnt(VisWindow * window, double r1, double r2,
									  ShapePointCloud * pnts1, const R3Affine & xForm1,
									  GeomShape * shape2, const R3Affine & xForm2,
									  ShapeCorrs * corr);
		virtual void DrawFuzzyPnts(VisWindow *window, ShapeCorrs * corrs, const ShapePoint & sel);

	
		virtual void SelectPair(const VKString & modelFrom, 
								VisModel3D * visModelFrom, int cameraFrom,
								const VKString & modelTo, 
								VisModel3D * visModelTo, int cameraTo);
	
		virtual void SelectCorr(DBCorrs::EntryType entryType, 
								const VKString & corrName, int selectedCorrID);

		virtual const VKString & ModelFromName();
		virtual const VKString & ModelToName();
	
	
		virtual bool IsFlagOn(DrawFlags flag);
		virtual void ToggleFlag(DrawFlags drawFlag);
		virtual void SetFlag(DrawFlags drawFlag, bool enable);
		
	protected:
		int m_iCameraFrom;
		int m_iCameraTo;
		VisModel3D * m_pSelectedModelFrom;
		VisModel3D * m_pSelectedModelTo;
		VKString m_sSelectedModelFromName;
		VKString m_sSelectedModelToName;
	
		DBCorrs::EntryType m_SelectedCorrType;
		VKString m_sSelectedCorrName;
		int m_iSelectedCorrID;

		DBCorrs * m_pDBCorrs;
	
		int m_iDrawFlags;
	
		CachedFuzzyValues * m_pCachedFuzzyValues;
		double m_fPointsRelativeRadius;
};

#endif

