#!/usr/bin/python

##########
#
# Execute leave one out experiment. Run the script with no arguments to see the argument list.
#
##########
import os, string, decimal, glob, re, subprocess, shlex, sys, datetime, random
sys.path.append(os.path.realpath("./"+os.path.dirname(sys.argv[0])+"/scriptlibs"))
import fas, exp, figs

if len(sys.argv)<3:
	print("Usage: "+sys.argv[0]+" dataset expname ");
	print("       $dataset$ uses directories: ../data/1_input/$dataset$/{off, gt}");
	print("       $expname$ outputs results to:");
	print("                ../data/4_experiments/$expname$        - all results");
	print("                ../data/5_images/experiments/$expname$ - images and evaluation results");
	print("Example use: "+sys.argv[0]+" small loo_test_small");
	exit();
else:
	loo = [sys.argv[1]];
	expname = sys.argv[2];


expArray = [];
# basicExpType expname   dataset   traindata   testdata   priorname   claname

# LEAVE-ONE-OUT EXPERIMENT
frac = 100;
fracStr = ""; # optional: XX

loo_prepare = True;
loo_predictRegions = True;
loo_predictPoses = True;
loo_render = True;
loo_final = True;
loo_forceAnlaysis = True;

overwriteMList = [];


#addflags = "-force train -force predict -force render_predict -force eval_predict -force prior -force poses -force render_poses -force eval_poses ";
addflags = " ";

# Run leave-one-out experiments
for dset in loo:
#	expname = "loo"+fracStr+"_"+pfix+"_"+dset;
	mListsDir = exp.ExpInfo(expname);
	looListsDir = mListsDir+dset+"_loo"+fracStr+"/";
	exp.PrepareLeaveOneOut(dset, looListsDir, frac);
	
	# prepare data
	if loo_prepare:
		e = exp.ExpConfigs();
		e.m_BasicType = "loo";
		e.m_ExpName = expname;
		e.m_Dataset = dset;
		#e.m_ClusterWorkspace = "./TempDir/"+e.m_BasicType+fracStr+"_"+pfix+"_"+dset+"/";
		e.m_ClusterWorkspace = "./TempDir/"+expname+"/";
		e.m_Flags = "-pipeline render -pipeline points -pipeline features ";
		expArray.append(e);
	
	expInfoDir = exp.ExpInfo(expname);
	if not os.path.isdir(expInfoDir):
		os.makedirs(expInfoDir);
	featNamesFile = expInfoDir+"/features.txt";
	paramFileCopy = expInfoDir+"/params.txt";
	params = exp.ReadParams(exp.ExpConfigs().m_ParamFile);

	params.WriteFeatureNames(featNamesFile);
	params.WriteParams(paramFileCopy);
	
	# train / test for single model
	mList = fas.ReadModelNames(exp.Models(dset));
	if len(overwriteMList)>0:
		mList = overwriteMList;
	for mName in mList:
		e = exp.ExpConfigs();
		e.m_ParamFile = paramFileCopy;
		e.m_BasicType = "loo";
		e.m_ExpName = expname;
		e.m_Dataset = dset;
		e.m_TestFile = looListsDir+"/"+mName+"_test.txt";
		# region classifier
		e.m_ClaDataset = dset;
		e.m_ClaTrainFile = looListsDir+"/"+mName+"_train.txt";
		e.m_ClaName = "/"+expname+"/"+mName;
		e.m_ClaType = "Weka";
		
		# pose prior
		e.m_PriorDataset = dset;
		e.m_PriorTrainFile = looListsDir+"/"+mName+"_train.txt";
		e.m_PriorName = "/"+expname+"/"+mName;
		
		# flags
		e.m_ClusterWorkspace = "./TempDir/"+expname+"/";
		e.m_WorkName = e.m_BasicType+"_"+mName;
		e.m_Flags = " ";
		if loo_predictRegions:
			e.m_Flags += " -pipeline predict -pipeline train ";
			if loo_render:
				e.m_Flags += "-pipeline render_predict ";
		
		if loo_predictPoses:
			e.m_Flags += "-pipeline prior -pipeline poses ";
			if loo_render:
				e.m_Flags += "-pipeline render_poses ";
		
		e.m_bRunInParallel = True;
		
		if (not os.path.isdir(exp.Poses(e.m_ExpName)+"/"+mName) or loo_forceAnlaysis) and e.m_Flags!=" ":
			expArray.append(e);


# Wait for results
e = exp.ExpConfigs();
e.m_bWaitForResults = True;
expArray.append(e);

# Evaluate leave-one-out experiments
if loo_final:
	for dset in loo:
		#expname = "loo"+fracStr+"_"+pfix+"_"+dset;
		mListsDir = exp.ExpInfo(expname);
		allModelsFile = mListsDir+dset+"_all.txt";
		looListsDir = mListsDir+dset+"_loo"+fracStr+"/";
		exp.PrepareAllModelsList(dset, allModelsFile);
		
		# Copy results
		e = exp.ExpConfigs();
		e.m_BasicType = "loo";
		e.m_ExpName = expname;
		e.m_Dataset = dset;
		e.m_TestFile = allModelsFile;
		
		expFilesDir = exp.ExpInfo(expname);
		if not os.path.isdir(expFilesDir):
			os.makedirs(expFilesDir);
		if not os.path.isfile(expFilesDir+"/test.txt"):
			os.system("cp "+allModelsFile +" "+expFilesDir+"/test.txt");
		
		# doesn't matter (just to avoid errors)
		mList = fas.ReadModelNames(exp.Models(dset));
		mName = mList[0];	# random name
		e.m_ClaDataset = dset;
		e.m_ClaTrainFile = looListsDir+"/"+mName+"_train.txt";
		e.m_ClaName = "/"+expname+"/"+mName;
		e.m_ClaType = "Weka";
		e.m_PriorDataset = dset;
		e.m_PriorName = "/"+expname+"/"+mName;
		e.m_PriorTrainFile = looListsDir+"/"+mName+"_train.txt";  # random - does not matter
		e.m_ClusterWorkspace = "./TempDir/"+expname+"/";
		e.m_WorkName = dset+"_final";
		
		# schedule
		if loo_predictRegions:
			e.m_Flags += "-pipeline eval_predict ";
		if loo_predictPoses:
			e.m_Flags += "-pipeline eval_poses ";
		e.m_bRunInParallel = True;
		expArray.append(e);

# Wait for evaluations to finish
e = exp.ExpConfigs();
e.m_bWaitForResults = True;
expArray.append(e);

exp.RunExperiments(expArray, addflags);


