#!/usr/bin/python

##########
#
# Create features
#
##########
import os, string, decimal, glob, re, subprocess, shlex, sys, datetime
sys.path.append(os.path.realpath("./"+os.path.dirname(sys.argv[0])))
import fas, exp

features = [];					nhds = [];
modelsDir = "none";
pntsDir = "none";
outDir = "none";
primitivesDir = "none";
nnMetricGraphsDir = "none";
densePntsDir = "none";
forceRecompute = False;
clusterWorkspace = "./TempDir";
randseed = -1;
disallowParallel = False;
dataset = "unknown";
origrotate = "none";

if len(sys.argv)<2:
	print("Usage: "+sys.argv[0]+" [options]");
	print("Options:");
	print("   -dataset dName");
	print("   -modelsDir dir");
	print("   -pntsDir dir");
	print("   -densePntsDir dir");
	print("   -nnMetricGraphsDir dir");
	print("   -outDir dir ");
	print("   -primitivesDir dir");
	print("   -origrotate xformdir");
	print("   -feature fMethod nhdSize  ");
	print("   -clusterWorkspace directory ");
	print("   -randseed seed - (seed can be -1) ");
	print("   -disallowParallel");
	exit();
else:
	i = 1;
	while i<len(sys.argv):
		if (sys.argv[i]=="-modelsDir"):
			i=i+1;
			modelsDir = sys.argv[i];
		elif (sys.argv[i]=="-dataset"):
			i=i+1;
			dataset = sys.argv[i];
		elif (sys.argv[i]=="-pntsDir"):
			i=i+1;
			pntsDir = sys.argv[i];
		elif (sys.argv[i]=="-outDir"):
			i=i+1;
			outDir = sys.argv[i];
		elif (sys.argv[i]=="-origrotate"):
			i=i+1;
			origrotate = sys.argv[i];
		elif (sys.argv[i]=="-primitivesDir"):
			i=i+1;
			primitivesDir = sys.argv[i];
		elif (sys.argv[i]=="-densePntsDir"):
			i=i+1;
			densePntsDir = sys.argv[i];
		elif (sys.argv[i]=="-nnMetricGraphsDir"):
			i=i+1;
			nnMetricGraphsDir = sys.argv[i];
		elif (sys.argv[i]=="-feature"):
			features += [sys.argv[i+1]];
			nhds.append(float(sys.argv[i+2]));
			i=i+2;
		elif (sys.argv[i]=="-forceRecompute"):
			forceRecompute = True;
		elif (sys.argv[i]=="-clusterWorkspace"):
			i=i+1;
			clusterWorkspace = sys.argv[i];
		elif (sys.argv[i]=="-disallowParallel"):
			disallowParallel = True;
		elif (sys.argv[i]=="-randseed"):
			i=i+1;
			randseed = sys.argv[i];
		else:
			print("[ERROR] Unknown argument "+sys.argv[i]+" in createfeats.py ");
			exit();
		i=i+1;


mList = fas.ReadModelNames(modelsDir);
jobIDs = [];
removeFiles = [];


###### PREPARE FOR FEATURE DETECTION #####
needNNGraph = False;
for f in range(0, len(features)):
	if "Geodesic" in features[f]:
		needNNGraph = True;

#if not "gapsNewPRST" in features:
#	features +=["gapsNewPRST"]
#	nhds += ["0"];   # change back

for mName in mList:
	offFile = modelsDir+"/"+mName+".off";
	pntsFile = pntsDir+"/"+mName+".pts";
	densePntsFile = densePntsDir + "/"+mName+".pts";
	metricFile = nnMetricGraphsDir+"/"+mName+".metric";
	metricTiming = nnMetricGraphsDir+"/"+mName+".timing.txt";
	comFile = densePntsDir+"/com_ja16/"+mName+""; # center of mass
#	csgFile = primitivesDir+"/"+mName+".csg";
#	primFeatFile = primitivesDir+"/"+mName+".arff";
#	primitivesMesh = primitivesDir+"/"+mName+"_primitives.off";
#	r3mshFile = primitivesDir+"/"+mName+"_r3mesh.off";

	######  GLOBAL FEATURES ###### -- TODO: merge Tom's code for finding planar reflective symmetry
	if not os.path.isfile(comFile+".txt"):
		print("Computing center of mass for: "+mName);
		cmd = exp.exedir+"ComputeGlobalFeatures "+offFile+" -densePnts "+densePntsFile+" -outfile "+comFile+".txt -feat com ; ";
		cmd += exp.gapsdir+"/msh2pln "+offFile+" "+comFile+"_symmplanes.txt -input_points "+pntsFile+" ";
		cmd += "-in_plane_vector 0 0 1 -min_value 0.9 -min_weight 128; ";

		if not disallowParallel:
			scriptFile = clusterWorkspace+"/globfeats/"+mName;
			jobIDs += fas.ScheduleJob(cmd, "symmaxis_"+mName, scriptFile)
			removeFiles += fas.TmpFilesNames(scriptFile);
		else:
			os.system(cmd);

	######  PRIMITIVE DETECTION ######
	if "Primitives" in features:
		if not os.path.isfile(primFeatFile) or not os.path.isfile(r3mshFile) or forceRecompute:
			print("Computing primitives for: "+mName);
			cmd = exp.gapsdir+ "/msh2csg "+offFile+" "+csgFile+" -output_properties "+primFeatFile+" ";
			cmd += "-output_csg_mesh "+primitivesMesh+" ";
			cmd += ";"
			cmd += exp.gapsdir+"/msh2msh "+offFile+" "+r3mshFile+" ";
	
			if not disallowParallel:
				scriptFile = clusterWorkspace+"/primitives/"+mName;
				jobIDs += fas.ScheduleJob(cmd, "prim_"+mName, scriptFile)
				removeFiles += fas.TmpFilesNames(scriptFile);
			else:
				os.system(cmd);
		#if visualize:
		#	cmd = exp.gapsdir+"/prpview "+offFile+" "+featFile+" "
		#	cmd = exp.gapsdir+"/mshview "+primitivesMesh+" ";
		#	os.system(cmd);
		
	######  NEAREST-NEIGHBOR GRAPHS ######
	if needNNGraph:
		if not os.path.isfile(metricFile) or forceRecompute:
			print("    NNGraph - "+mName);
			cmd = exp.exedir+"Metric -mesh "+offFile+" -pnts "+densePntsFile+" ";
			cmd += "-dist geodGraph -writeDist "+metricFile+" -writeTiming "+metricTiming+" ";
			if not disallowParallel:
				scriptFile = clusterWorkspace+"/nngraphs/"+mName;
				jobIDs += fas.ScheduleJob(cmd, "nng_"+mName, scriptFile)
				removeFiles += fas.TmpFilesNames(scriptFile);
			else:
				os.system(cmd);

fas.WaitForJobsInArray(jobIDs);
#for file in removeFiles:	# change back
#	if os.path.isfile(file):
#		os.remove(file);

# Record metric timings
concatTimeFile = nnMetricGraphsDir+"/timing.txt";
if not os.path.isfile(concatTimeFile) or forceRecompute:
	timingStr = "";
	for mName in mList:
		timeFile = nnMetricGraphsDir+"/"+mName+".timing.txt";
		with open(timeFile, 'r') as f:
			timingStr += f.read();
		os.system("rm "+timeFile);
	with open(concatTimeFile, 'w') as f:
		f.write(timingStr+"\n");

jobIDs = [];
removeFiles = [];

name2rotation = dict();
for mName in mList:
#	with open(origrotate+"/"+mName+".xform") as f:
#		f.readline();
#		x = f.readline().strip();
#		xlist = x.split(" ");
#		name2rotation[mName] = float(xlist[1]);
	name2rotation[mName] = 0;

#mList = [mList[0]]
#mList = ["bicycle000122"]
for mName in mList:
	offFile = modelsDir+"/"+mName+".off";
	pntsFile = pntsDir+"/"+mName+".pts";
	densePntsFile = densePntsDir + "/"+mName+".pts";
	metricFile = nnMetricGraphsDir+"/"+mName+".metric";
#	csgFile = primitivesDir+"/"+mName+".csg";
#	primFeatFile = primitivesDir+"/"+mName+".arff";
#	primitivesMesh = primitivesDir+"/"+mName+"_primitives.off";
#	r3mshFile = primitivesDir+"/"+mName+"_r3mesh.off";

	for f in range(0, len(features)):
		fName = features[f];
		nhdSize = nhds[f];
		nhdSizeStr = "";
		if nhdSize>=0:
			nhdSizeStr = "_"+str(nhdSize);
				
		featFile = outDir+"/"+fName+nhdSizeStr+"/"+mName+".arff";
		timeFile = outDir+"/"+fName+nhdSizeStr+"/"+mName+".timing.txt";
		params = "";
					
		######  PRIMITIVE DETECTION ######
		if fName=="Primitives":
			params += "-primitiveCSG "+csgFile+" -primitiveARFF "+primFeatFile+" ";
			params += "-r3mesh "+r3mshFile+" ";			
	
		######  PRST FEATURE DETECTION ######
		tempPRSTDir = outDir+"/gapsPRST_work/";
		tempNewPRSTDir = outDir+"/gapsNewPRST_work/";
		cmd = "";
		if fName[0:8]=="gapsPRST" and (not os.path.isfile(featFile) or forceRecompute):
			if not os.path.isdir(tempPRSTDir):
				os.makedirs(tempPRSTDir);
			if forceRecompute or not os.path.isfile(tempPRSTDir+"/"+mName+".arff"):
				cmd += exp.gapsdir+"/pts2prp "+offFile+" "+pntsFile+" "+tempPRSTDir+"/"+mName+".arff -v ";
				cmd += "-output_planes "+tempPRSTDir+"/"+mName+".planes.txt ";
				cmd += "-rotationphase "+str(name2rotation[mName])+" ";
				cmd += ";";
		elif fName[0:11]=="gapsNewPRST" and (not os.path.isfile(featFile) or forceRecompute):
			if not os.path.isdir(tempNewPRSTDir):
				os.makedirs(tempNewPRSTDir);
			if forceRecompute or not os.path.isfile(tempNewPRSTDir+"/"+mName+".arff"):
				cmd += exp.gapsdir+"/msh2pln "+offFile+" "+tempNewPRSTDir+"/"+mName+".planes.txt ";
				cmd += "-v -input_points "+pntsFile+" -output_point_properties "+tempNewPRSTDir+"/"+mName+".arff ";
				cmd += "-in_plane_vector 0 0 1 -min_value 0.9 -min_weight 128 ";
#				cmd += "-rotationphase "+str(name2rotation[mName])+" ";
				cmd += ";";

		######  NEAREST-NEIGHBOR GRAPHS ######
		if "Geodesic" in fName:
			params += "-metricFile "+metricFile+" ";

		######  Externally (or pre-computed) features (Sid's, Tom's) ######
		needcomp = not os.path.isfile(featFile) or forceRecompute;
		if needcomp and (fName[0:3]=="sid" or fName[0:8]=="gapsPRST"  or fName[0:11]=="gapsNewPRST"):
			amplifyStr = "";
			if fName[0:3]=="sid":
				sidDataDir = "./data/1_input_fromsid/"+dataset+"/points/even1000/sid/"
			elif fName[0:8]=="gapsPRST":
				sidDataDir = tempPRSTDir;
			elif fName[0:11]=="gapsNewPRST":
				sidDataDir = tempNewPRSTDir;
				#amplifyStr = "-amplify 5000 ";
				amplifyStr = "-amplify 0 ";

			cmd += exp.exedir+"ComputeLocalFeatures "+offFile+" -points "+pntsFile+"  ";
			cmd += "-arffFeatFile "+sidDataDir+"/"+mName+".arff -feat ExternalArff "+amplifyStr+" ";
			allexist = True;
			nspecial = 0;
			for fs in range(0, len(features)):
				fNameSid = features[fs];
				isSid = fNameSid[0:3]=="sid" and fName[0:3]=="sid";
				isGaps = fNameSid[0:8]=="gapsPRST" and fName[0:8]=="gapsPRST";
				isGaps2 = fNameSid[0:11]=="gapsNewPRST" and fName[0:11]=="gapsNewPRST";
				if not isSid and not isGaps and not isGaps2:
					continue;
				
				if (nhds[fs]>0):
					nhdSid = "_"+str(nhds[fs]);
				elif (nhds[fs]==0):
					nhdSid = "_0";
				else:
					nhdSid = "";
				featFileSid = outDir+"/"+fNameSid+nhdSid+"/"+mName+".arff";
				if not os.path.isfile(featFileSid) or forceRecompute:
					allexist = False;
					#offsetStr = str(int(nhds[fs]));
					#cmd += "-addOutFile "+featFileSid+" "+offsetStr+" 1 ";
					cmd += "-outfile "+featFileSid+" ";
					nspecial += 1;
			if exp.DEBUG_VIS:
				cmd += "-vis ";
			if not allexist:
				assert(nspecial==1);				
				if not disallowParallel:
					scriptFile = clusterWorkspace+"/features/"+mName+"_"+fName+"_"+str(nhdSize);
					jobIDs += fas.ScheduleJob(cmd, "feat_"+mName, scriptFile)
					removeFiles += fas.TmpFilesNames(scriptFile);
				else:
					os.system(cmd+" ");

		######  COMPUTE FEATURES (except Sid's features) ######
		elif needcomp:
			print("Computing Features: "+mName+" - "+fName+" "+str(nhdSize));
			nhdSizeStr = "";
			if nhdSize>=0:
				nhdSizeStr = "-radius "+str(nhdSize);

			###### CONVERT OR COMPUTE FEATURES ######
			addParams = "";
			if fName=="PeterXYZ":
				addParams += "-alignedMesh ./data/1_input_frompeter/"+dataset+"/"+mName+".off";
			
			cmd = exp.exedir+"ComputeLocalFeatures "+offFile+" -points "+pntsFile+" "+nhdSizeStr+" ";
			cmd += "-outfile "+featFile+" -feat "+fName+" -densePoints "+densePntsFile+" "+params+" ";
			cmd += "-writeTiming "+timeFile+" -randseed "+randseed+" "+addParams+" "
			if exp.DEBUG_VIS:
				cmd +=" -vis ";
			if not disallowParallel:
				scriptFile = clusterWorkspace+"/features/"+mName+"_"+fName+"_"+str(nhdSize);
				jobIDs += fas.ScheduleJob(cmd, "feat_"+mName, scriptFile)
				removeFiles += fas.TmpFilesNames(scriptFile);
			else:
				os.system(cmd);


fas.WaitForJobsInArray(jobIDs);
#for file in removeFiles:
#	if os.path.isfile(file):
#		os.remove(file);

# Concatenate timing
for f in range(0, len(features)):
	fName = features[f];
	nhdSize = nhds[f];
	nhdSizeStr = "";
	if fName[0:3]=="sid" or fName[0:4]=="gaps":
		nhdSizeStr = "_"+str(int(nhdSize));
	elif nhdSize>=0:
		nhdSizeStr = "_"+str(nhdSize);
	
	concatTimeFile = outDir+"/"+fName+nhdSizeStr+"/timing.txt";
	if not os.path.isfile(concatTimeFile) or forceRecompute:
		timingStr = "";
		for mName in mList:
			timeFile = outDir+"/"+fName+nhdSizeStr+"/"+mName+".timing.txt";
			if os.path.isfile(timeFile):
				with open(timeFile, 'r') as f:
					timingStr += f.read();
				os.system("rm "+timeFile);
		if os.path.isfile(concatTimeFile):
			with open(concatTimeFile, 'w') as f:
				f.write(timingStr+"\n");


