#!/usr/bin/python

##########
#
# Create points on meshes
#
##########
import os, string, decimal, glob, re, subprocess, shlex, sys, datetime
sys.path.append(os.path.realpath("./"+os.path.dirname(sys.argv[0])))
import fas, exp

modelsDir = "none";
outDir = "none";
napproxpnts = 2000;
method = "none";
forceRecompute = False;
clusterWorkspace = "./TempDir";
randseed = 0;
disallowParallel = False;

if len(sys.argv)<2:
	print("Usage: "+sys.argv[0]+" [options]");
	print("Options:");
	print("   -modelsDir dir");
	print("   -outDir dir ");
	print("   -napproxpnts int ");
	print("   -method [even, random] ");
	print("   -clusterWorkspace directory ");
	print("   -randseed seed - (seed can be -1) ");
	print("   -disallowParallel");
	exit();
else:
	i = 1;
	while i<len(sys.argv):
		if (sys.argv[i]=="-modelsDir"):
			i=i+1;
			modelsDir = sys.argv[i];
		elif (sys.argv[i]=="-outDir"):
			i=i+1;
			outDir = sys.argv[i];
		elif (sys.argv[i]=="-napproxpnts"):
			i=i+1;
			napproxpnts = int(sys.argv[i]);
		elif (sys.argv[i]=="-method"):
			i=i+1;
			if sys.argv[i]=="even":
				method = "-type IterFar eucl -areaAdjusted 16. ";
			elif sys.argv[i]=="random":
				method = "-type Random -areaAdjusted 16. ";
			elif sys.argv[i]=="evenHollow":
				method = "-type IterFar eucl -areaAdjusted 16. -hollow ";
			elif sys.argv[i]=="randomHollow":
				method = "-type Random -areaAdjusted 16. -hollow ";
		elif (sys.argv[i]=="-forceRecompute"):
			forceRecompute = True;
		elif (sys.argv[i]=="-forceRecompute"):
			forceRecompute = True;
		elif (sys.argv[i]=="-clusterWorkspace"):
			i=i+1;
			clusterWorkspace = sys.argv[i];
		elif (sys.argv[i]=="-disallowParallel"):
			disallowParallel = True;
		elif (sys.argv[i]=="-randseed"):
			i=i+1;
			randseed = sys.argv[i];
		else:
			print("[ERROR] Unknown argument "+sys.argv[i]+" in createpts.py ");
			exit();
		i = i + 1;


# Check if pnts files exists - create if not
mList = fas.ReadModelNames(modelsDir);
jobIDs = [];
removeFiles = [];

#mList = ["motorbike2"];

for mName in mList:
	ptsFile = outDir+"/"+mName+".pts";
	if not os.path.isfile(ptsFile) or forceRecompute:
		cmd = exp.exedir+"FindPoints "+modelsDir+" -dirType MeshFiles -model "+mName+".off -writeGeometry "+method+" ";
		cmd += "-npoints "+str(napproxpnts)+" -pntsName "+ptsFile+" -outGeomIsFile -outGeomIsReallyFile ";
		cmd += "-writeTiming "+outDir+"/"+mName+".timing.txt -randseed "+randseed+" ";
		if exp.DEBUG_VIS:
			cmd += "-vis ";
		if not disallowParallel:
			scriptFile = clusterWorkspace+"/"+mName;
			jobIDs += fas.ScheduleJob(cmd, "pts_"+mName, scriptFile)
			removeFiles += fas.TmpFilesNames(scriptFile);
		else:
			os.system(cmd+" ");

fas.WaitForJobsInArray(jobIDs);

# Concatenate timing
concatTimeFile = outDir+"/timing.txt";
if not os.path.isfile(concatTimeFile) or forceRecompute:
	timingStr = "";
	for mName in mList:
		timeFile = outDir+"/"+mName+".timing.txt";
		with open(timeFile, 'r') as f:
			timingStr += f.read();
		os.system("rm "+timeFile);
	
	with open(concatTimeFile, 'w') as f:
		f.write(timingStr+"\n");










