#!/usr/bin/python

##########
#
#  Predict pose
#
##########
import os, string, glob, sys
sys.path.append(os.path.realpath("./"+os.path.dirname(sys.argv[0])))
import fas, figs, exp

modelsDir = "none";
pntsDir = "none";
outDir = "none";
testFile = "none";
gtDir = "none"
forceRecompute = False;			# re-do results
predictDir = "none";
clusterWorkspace = "./TempDir";
readRegionNames = "none";
readSkeleton = "none";
disallowParallel = False;

if len(sys.argv)<2:
	print("Usage: "+sys.argv[0]+" [options]");
	print("Options:");
	print("   -modelsDir dir");
	print("   -pntsDir dir");
	print("   -predictedPoseDir dir ");
	print("   -outDir dir ");
	print("   -gtDir dir");
	print("   -outImages dir ");
	print("   -forceRecompute ");
	print("   -testFile file ");
	print("   -readRegionNames regionNamesFile ");
	print("   -readSkeleton skeletonFile ");
	print("   -clusterWorkspace directory ");
	print("   -disallowParallel");
	exit();
else:
	i = 1;
	while i<len(sys.argv):
		if (sys.argv[i]=="-modelsDir"):
			i=i+1;
			modelsDir = sys.argv[i];
		elif (sys.argv[i]=="-pntsDir"):
			i=i+1;
			pntsDir = sys.argv[i];
		elif (sys.argv[i]=="-predictedPoseDir"):
			i=i+1;
			predictedPoseDir = sys.argv[i];
		elif (sys.argv[i]=="-outDir"):
			i=i+1;
			outDir = sys.argv[i];
		elif (sys.argv[i]=="-outImages"):
			i=i+1;
			outImages = sys.argv[i];
		elif (sys.argv[i]=="-gtDir"):
			i=i+1;
			gtDir = sys.argv[i];
		elif (sys.argv[i]=="-forceRecompute"):
			forceRecompute = True;
		elif (sys.argv[i]=="-testFile"):
			i=i+1;
			testFile = sys.argv[i];
		elif (sys.argv[i]=="-readRegionNames"):
			i=i+1;
			readRegionNames = sys.argv[i];
		elif (sys.argv[i]=="-readSkeleton"):
			i=i+1;
			readSkeleton = sys.argv[i];
		elif (sys.argv[i]=="-clusterWorkspace"):
			i=i+1;
			clusterWorkspace = sys.argv[i];
		elif (sys.argv[i]=="-disallowParallel"):
			disallowParallel = True;
		else:
			print("[ERROR] Unknown argument "+sys.argv[i]+" in evalPoses.py ");
			exit();
		i=i+1;


symmInv = "";

# Prepare files
mListTest = fas.ReadModelNames(testFile)

## Prepare region lists
rList = fas.ReadRegionNames(readRegionNames);
jobIDs = [];
removeFiles = [];

##
# Evaluate discrete poses I
for mName in mListTest:
	evalOutFile = outDir+"/"+mName+"/"+mName+".pose.eval";
	if not os.path.isfile(evalOutFile) or forceRecompute:
		poseDir = predictedPoseDir+"/"+mName+"";
		offFile = modelsDir+"/"+mName+".off";
		gtFile = gtDir+"/"+mName+".gt";

		cmd = exp.exedir+"FnBenchmark "+offFile+" "+gtFile+" -evalpose "+poseDir+" "+evalOutFile+" ";
		cmd += "-readRegionNames "+readRegionNames+" -readSkeleton "+readSkeleton+" ";
		if not disallowParallel:
			scriptFile = clusterWorkspace+"/"+mName;
			jobIDs += fas.ScheduleJob(cmd, "epos_"+mName, scriptFile);
			removeFiles += fas.TmpFilesNames(scriptFile);
		else:
			os.system(cmd);
fas.WaitForJobsInArray(jobIDs);
#for file in removeFiles:
#	if os.path.isfile(file):
#		os.remove(file);

##
#  Render summary images (plots)
accuracyThreshold = 0.2;
plotFile = outImages+"/pnterrs";
imgExists = glob.glob(plotFile+"*");
if len(imgExists)==0 or forceRecompute:
	if not os.path.isdir(outImages):
		os.makedirs(outImages);
	allPntErr = [];
	gtContactErr = [];
	angErrs = [];

	type2num = dict();
	totalpnts = 0;
	totalgtpnts = 0;
	ave_errs_allpts = [];
	ave_errs_gtcontacts = [];
	ave_errs_angles = [];
	max_errs_allpts = [];
	max_errs_gtcontacts = [];
	max_errs_angles = [];
 
	accuracyAll = 0;
	accuracyContacts = 0;
	totalTime = 0;

	npermeshgtcorrect = [];
	npermeshcorrect = [];
	acceptedPntNames = [];
	
	# go over all meshes
	for mName in mListTest:
		npermeshgtcorrect += [0];
		npermeshcorrect += [0];
		evalFile = outDir+"/"+mName+"/"+mName+".pose.eval";
		timeFile = predictedPoseDir+"/"+mName+"/timing.txt";
		if os.path.isfile(timeFile):
			with open(timeFile, 'r') as f:
				tmLines = f.readlines();
			if len(tmLines)>1:
				timelist = tmLines[1].replace("\t", " ").split(" ");
				totalTime += float(timelist[-1][0:-2]);
		
		ave_all = 0;
		ave_gtcontacts = 0;
		ave_angles = 0;
		max_all = -1;
		max_gtcontacts = -1;
		max_angles = -1;
		ngtcontacts = 0;
		
		# read first pose (for now)
		if not os.path.isfile(evalFile):
			print("[ERROR] Pose file is missing: "+evalFile);

		with open(evalFile, 'r') as f:
			pair = f.readline().split(" ");
			if (pair[0]!="NPoses"):
				print("[ERROR] Invalid eval file: "+evalFile)
			assert(pair[0]=="NPoses");
			nposes = int(pair[1]);
			pair = f.readline().split(" ");   assert(pair[0]=="NJoints");
			njoints = int(pair[1]);
			pair = f.readline();   assert(pair=="PredictedPose 0\n");
		
			currRegionNames = [];

			#   read per joint-node errors
			for j in range(0,njoints):
				errsLine = f.readline().split(" ");
				rid = int(errsLine[0]);
				gt = errsLine[1];
				predicted = errsLine[2];
				err = float(errsLine[3]);
				ave_all = ave_all + err;
				allPntErr = allPntErr + [err];
				if err < accuracyThreshold:
					accuracyAll += 1;
					npermeshcorrect[-1] += 1;
				if err > max_all:
					max_all = err;
				if gt=='g' or gt=='s':
					gtContactErr = gtContactErr + [err];
					ave_gtcontacts = ave_gtcontacts + err;
					if err < accuracyThreshold:
						accuracyContacts += 1;
						npermeshgtcorrect[-1] += 1;
						currRegionNames += [rList[rid]];
					
					if err > max_gtcontacts:
						max_gtcontacts = err;
					ngtcontacts = ngtcontacts + 1;
					totalgtpnts = totalgtpnts + 1;
				
				if not (gt+predicted) in type2num:
					type2num[gt+predicted] = 1;
				else:
					type2num[gt+predicted] = type2num[gt+predicted] + 1;
				totalpnts = totalpnts + 1;

			acceptedPntNames += [currRegionNames];
			
			# read angle errors
			pair = f.readline().split(" ");   assert(pair[0]=="NSkeletonParams");
			nparams = int(pair[1]);
			ninitparams = int(pair[2]);
			angErrsStr = f.readline().split(" ");
			angErrsStr = angErrsStr[ninitparams:];
			nangles = 0;
			for err in angErrsStr:
				if err!="" and err!="\n":
					angErrs = angErrs+[float(err)];
					ave_angles = ave_angles + float(err);
					nangles = nangles + 1;
					if float(err)>max_angles:
						max_angles = float(err);
			ave_errs_allpts = ave_errs_allpts + [ave_all/njoints];
			ave_errs_gtcontacts = ave_errs_gtcontacts + [ave_gtcontacts/ngtcontacts];
			ave_errs_angles = ave_errs_angles + [ave_angles / nangles];
			max_errs_allpts = max_errs_allpts + [max_all];
			max_errs_gtcontacts = max_errs_gtcontacts + [max_gtcontacts];
			max_errs_angles = max_errs_angles + [max_angles];
	
	# write errors
	for j in range(0,len(mListTest)):
		mName = mListTest[j];
		errInfoDir = outDir+"/"+mName;
		if not os.path.isdir(errInfoDir):
			os.makedirs(errInfoDir);
		with open(errInfoDir+"/errinfo.txt", 'w') as f:
			f.write("AllPointsError "+str(ave_errs_allpts[j])+" "+str(max_errs_allpts[j])+"\n");
			f.write("GTContactsError "+str(ave_errs_gtcontacts[j])+" "+str(max_errs_gtcontacts[j])+"\n");
			f.write("AngleError "+str(ave_errs_angles[j])+" "+str(max_errs_angles[j])+"\n");
			f.write("AcceptedContacts "+str(npermeshgtcorrect[j])+" "+str(ngtcontacts)+"\n");
			f.write("AcceptedPnts "+str(npermeshcorrect[j])+" "+str(njoints)+"\n");
			f.write("AcceptedContactNames ");
			for currName in acceptedPntNames[j]:
				f.write(currName+" ");
			f.write("\n");

	# render errors
	figs.Errs2CDFPlot([allPntErr]+[gtContactErr], ["allpnt", "gtpnt"], plotFile, 0, 0.5) # 0.6 == hip to knee
	plotFile = outImages+"/angerrs";
	figs.Errs2CDFPlot([angErrs], ["angles"], plotFile, 0, 3.14/2.);

	with open(outImages+"/summary.txt", 'w') as f:
		f.write("AccContacts: "+str(float(accuracyContacts)/float(totalgtpnts))+"\n");
		f.write("AccAll: "+str(float(accuracyAll)/float(totalpnts))+"\n");
		f.write("AveTime: "+str(totalTime / len(mListTest))+"\n");

	with open(outImages+"/allpnterr.txt", 'w') as f:
		for e in allPntErr:
			f.write(str(e)+" ");
	with open(outImages+"/gtcontacterr.txt", 'w') as f:
		for e in gtContactErr:
			f.write(str(e)+" ");

if os.path.isfile(plotFile+".py") and not os.path.isfile(plotFile+".pdf"):
	os.system(plotFile+".py");

