#!/usr/bin/python

##########
#
#  Evaluate quality of regions
#
##########
import os, string, glob, sys
sys.path.append(os.path.realpath("./"+os.path.dirname(sys.argv[0])))
import fas, figs, exp

modelsDir = "none";
pntsDir = "none";
gtDir = "none";
outDir = "none";
testFile = "none";
featureNameFile = "none";
forceRecompute = False;			# re-train classifier even if exists
predictDir = "none";
outImages = "none";
symmInvariant = False;
clusterWorkspace = "./TempDir";
distThreshold = .2;
positiveValThreshold = .2;
disallowParallel = False;


if len(sys.argv)<2:
	print("Usage: "+sys.argv[0]+" [options]");
	print("Options:");
	print("   -modelsDir dir");
	print("   -gtDir dir");
	print("   -pntsDir dir");
	print("   -predictDir dir ");
	print("   -outImages dir ");
	print("   -outDir dir ");
	print("   -forceRecompute ");
	print("   -symmInvariant ");
	print("   -testFile file ");
	print("   -distThreshold threshold");
	print("   -positiveValThreshold posValThreshold ");
	print("   -clusterWorkspace directory ");
	print("   -disallowParallel");
	exit();
else:
	i = 1;
	while i<len(sys.argv):
		if (sys.argv[i]=="-modelsDir"):
			i=i+1;
			modelsDir = sys.argv[i];
		elif (sys.argv[i]=="-pntsDir"):
			i=i+1;
			pntsDir = sys.argv[i];
		elif (sys.argv[i]=="-predictDir"):
			i=i+1;
			predictDir = sys.argv[i];
		elif (sys.argv[i]=="-gtDir"):
			i=i+1;
			gtDir = sys.argv[i];
		elif (sys.argv[i]=="-outDir"):
			i=i+1;
			outDir = sys.argv[i];
		elif (sys.argv[i]=="-outImages"):
			i=i+1;
			outImages = sys.argv[i];
		elif (sys.argv[i]=="-forceRecompute"):
			forceRecompute = True;
		elif (sys.argv[i]=="-testFile"):
			i=i+1;
			testFile = sys.argv[i];
		elif (sys.argv[i]=="-symmInvariant"):
			symmInvariant = True;
		elif (sys.argv[i]=="-clusterWorkspace"):
			i=i+1;
			clusterWorkspace = sys.argv[i];
		elif (sys.argv[i]=="-disallowParallel"):
			disallowParallel = True;
		elif (sys.argv[i]=="-readRegionNames"):
			i=i+1;
			readRegionNames = sys.argv[i];
		elif (sys.argv[i]=="-readRegionSymmetry"):
			i=i+1;
			readRegionSymmetry = sys.argv[i];
		elif (sys.argv[i]=="-distThreshold"):
			i=i+1;
			distThreshold = float(sys.argv[i]);
		elif (sys.argv[i]=="-positiveValThreshold"):
			i=i+1;
			positiveValThreshold = float(sys.argv[i]);
		else:
			print("[ERROR] Unknown argument "+sys.argv[i]+" in evalRegions.py ");
			exit();
		i=i+1;


# Prepare files
mListTest = fas.ReadModelNames(testFile);

# Prepare files
regionsFile = readRegionNames;
rList = fas.ReadRegionNames(regionsFile);
jobIDs = [];
removeFiles = [];

##
# Evaluate prediction values
counter = 0;
if symmInvariant:
	symmInv = "-symm_invariant "+readRegionSymmetry+" ";
for mName in mListTest:
	evalOutDir = outDir+"/"+mName;
	if not os.path.isdir(evalOutDir) or forceRecompute:
		pntsFile = pntsDir+"/"+mName+".pts";
		predictionFile = predictDir+"/"+mName+".arff";
		offFile = modelsDir+"/"+mName+".off";
		gtFile = gtDir+"/"+mName+".gt";
		cmd = exp.exedir+"FnBenchmark "+offFile+" "+gtFile+" -readRegionNames "+regionsFile+" ";
		cmd += "-eval "+pntsFile+" "+predictionFile+" "+evalOutDir+" ";
		if not disallowParallel:
			scriptFile = clusterWorkspace+"/"+mName;
			jobIDs += fas.ScheduleJob(cmd+" "+symmInv+" ", "ereg_"+mName, scriptFile)
			removeFiles += fas.TmpFilesNames(scriptFile);
		else:
			os.system(cmd);
fas.WaitForJobsInArray(jobIDs);
#for file in removeFiles:
#	if os.path.isfile(file):
#		os.remove(file);

outEvalImages = outImages+"/";
if forceRecompute:
	if len(glob.glob(outEvalImages+"/*pdf"))>0:
		os.system("rm "+outEvalImages+"/*pdf");
	if len(glob.glob(outEvalImages+"/*png"))>0:
		os.system("rm "+outEvalImages+"/*png");
imgOutExist = glob.glob(outEvalImages+"/*pdf");
if len(imgOutExist)==0 or forceRecompute:
	if not os.path.isdir(outEvalImages):
		os.makedirs(outEvalImages)
	figs.EvalPairs2PRCurves(exp.data, mListTest, [distThreshold], [positiveValThreshold], outDir, outEvalImages, rList);

plotName = figs.GetPlotName(outImages.replace("./data//4_experiments/", "").replace("/3_prediction_eval/", ""));

cmbFile =outEvalImages+"/plotCombined.py";
if not os.path.isfile(cmbFile):
	figs.EvalPairs2PRCurvesSinglePlot(exp.data, mListTest, distThreshold, positiveValThreshold, outDir, outEvalImages, rList, plotName);

rNames = fas.ReadRegionNames(regionsFile);
for rName in rNames:
	file = outEvalImages+"/plot"+rName+".py";
	imgFile = outEvalImages+"/PR_"+rName+".pdf";
	if os.path.isfile(file) and not os.path.isfile(imgFile):
		os.system(file);

if not os.path.isfile(outImages+"/models.txt"):
	fas.WriteModelNames(mListTest, outImages+"/models.txt");



 