#!/usr/bin/python

##########
#
# Create ground truth for functionality analysis.
#
##########
import os, string, decimal, glob, re, subprocess, shlex, sys, datetime, time, getpass, random, platform
sys.path.append(os.path.realpath("./"+os.path.dirname(sys.argv[0])))
import fas, figs


scriptsdir = "./scriptlibs/"
exedir = "./"+platform.system()+"/";
data = "../data/";
#gapsdir = "../code/external/gaps/bin/x86_64/";
gapsdir = exedir;

DEBUG_VIS = False;

######   Directory Structure and Parameters ######
class ExpParams:
	# pipeline
	m_PipelineParams = dict();
	
	# point set parameters just 2 elements: regular, dense
	m_PointsMethods = ["none", "none"];
	m_PointsNSamps = [0, 0];

	# feature params
	m_FeatNames = [];
	m_FeatParams = [];
	m_FeatFullNames = [];
	m_FeatNDims = [];

	# prediction-related params
	m_PredictionParams = dict();
	
	# pose prior-related params
	m_PosePriorParams = dict();
	
	# pose-related params
	m_PoseParams = dict();
	
	# evaluation params
	m_EvalParams = dict();
	
	def WriteParams(this, paramsFile):
		with open(paramsFile, "w") as f:
			f.write("# pipeline-related\n");
			for (key, value) in this.m_PipelineParams.items():
				f.write("pipeline "+key+" "+value+"\n");
			f.write("# points-related\n");
			assert(len(this.m_PointsMethods)==2);
			f.write("pnts cand "+this.m_PointsMethods[0]+" "+str(this.m_PointsNSamps[0])+"\n");
			f.write("pnts dense "+this.m_PointsMethods[1]+" "+str(this.m_PointsNSamps[1])+"\n");
			f.write("\n");
			f.write("# feature-related\n");
			for fid in range(0, len(this.m_FeatNames)):
				if this.m_FeatParams[fid]<0:
					f.write("feat "+this.m_FeatNames[fid]+" "+str(this.m_FeatNDims[fid])+"\n");
				else:
					f.write("feat "+this.m_FeatNames[fid]+"_"+str(this.m_FeatParams[fid])+" "+str(this.m_FeatNDims[fid])+"\n");
			f.write("\n");
			f.write("# prediction-related\n");
			for (key, value) in this.m_PredictionParams.items():
				f.write("pred "+key+" "+value+"\n");
			f.write("\n");
			f.write("# prior-related\n");
			for (key, value) in this.m_PosePriorParams.items():
				f.write("prior "+key+" "+value+"\n");
			f.write("# pose-related\n");
			for (key, value) in this.m_PoseParams.items():
				f.write("pose "+key+" "+value+"\n");
			f.write("# eval-related\n");
			for (key, value) in this.m_EvalParams.items():
				f.write("eval "+key+" "+value+"\n");

	def WriteFeatureNames(this, featFile):
		with open(featFile, "w") as f:
			for fid in range(0, len(this.m_FeatNames)):
				f.write(this.m_FeatFullNames[fid]+" "+str(this.m_FeatNDims[fid])+"\n");

def ReadParams(paramsFile):
	params = ExpParams();
	with open(paramsFile, "r") as f:
		paramLines = f.readlines();
	for paramLine in paramLines:
		if len(paramLine)<=1 or paramLine[0]=="#":	# comments
			continue;
		paramLineSplit = paramLine.strip().split(" ");
		if (len(paramLineSplit)>1):
			if paramLineSplit[0]=="pnts":   # points
				assert(len(paramLineSplit)==4);
				pntset = -1;
				if paramLineSplit[1]=="dense":
					pntset = 1;
				elif paramLineSplit[1]=="cand":
					pntset = 0;
				else:
					print("[WARNING] Unknown points: "+paramLineSplit[1]);
				assert(pntset>=0);
				params.m_PointsMethods[pntset] = paramLineSplit[2];
				params.m_PointsNSamps[pntset] = paramLineSplit[3];
			elif paramLineSplit[0]=="feat":   # features
				assert(len(paramLineSplit)==3);
				params.m_FeatFullNames += [paramLineSplit[1]];
				f1 = paramLineSplit[1].strip().split("_");
				params.m_FeatNames += [f1[0]];
				if len(f1)==2:
					params.m_FeatParams += [float(f1[1])];
				else:
					params.m_FeatParams += [-1];
				params.m_FeatNDims += [paramLineSplit[2]];
			elif paramLineSplit[0]=="pred":   # prediction parameter
				if len(paramLineSplit)>=3:
					params.m_PredictionParams[paramLineSplit[1]] = " ".join(paramLineSplit[2:]);
				else:
					params.m_PredictionParams[paramLineSplit[1]] = "True";
			elif paramLineSplit[0]=="prior":   # prediction parameter
				if len(paramLineSplit)>=3:
					params.m_PosePriorParams[paramLineSplit[1]] = " ".join(paramLineSplit[2:]);
				else:
					params.m_PosePriorParams[paramLineSplit[1]] = "True";
			elif paramLineSplit[0]=="pose":   # pose parameter
				if len(paramLineSplit)>=3:
					params.m_PoseParams[paramLineSplit[1]] = " ".join(paramLineSplit[2:]);
				else:
					params.m_PoseParams[paramLineSplit[1]] = "True";
			elif paramLineSplit[0]=="eval":   # evaluation parameter
				if len(paramLineSplit)>=3:
					params.m_EvalParams[paramLineSplit[1]] = " ".join(paramLineSplit[2:]);
				else:
					params.m_EvalParams[paramLineSplit[1]] = "True";
			elif paramLineSplit[0]=="pipeline":   # pipeline parameter
				if len(paramLineSplit)>=3:
					params.m_PipelineParams[paramLineSplit[1]] = " ".join(paramLineSplit[2:]);
				else:
					params.m_PipelineParams[paramLineSplit[1]] = "True";
		else:
			print("[WARNING] Unknown param line: "+str(paramLine));
	return params;


	#### Universal Input ####
def HumanBodyParts():
	return data+"/0_body/regions.txt";

def HumanSymmetry():
	return data+"/0_body/regions_symmetry.txt";

def HumanSkeleton():
	return data+"/0_body/regions_skeleton.txt";

def HumanBodySkeletonParts():
	return data+"/0_body/regions_all.txt";

	#### Input ####
def Models(dataset):
	return data+"/1_input/"+dataset+"/off/";

def ModelsRendering(dataset):
	return data+"/1_input/"+dataset+"/models_render/";

def GT(dataset):
	return data+"/1_input/"+dataset+"/gt/";

def InXForm(dataset):
	return data+"/1_input/"+dataset+"/canonical/";

def GTRendering(dataset):
	return data+"/1_input/"+dataset+"/gt_render/";


	#### Analysis (per model) ####
def Points(dataset, method, npoints):
	return data+"/2_analysis/"+dataset+"/points/"+method+str(npoints)+"/";

def AllFeatures(dataset):
	return data+"/2_analysis/"+dataset+"/features/";

#def Features(dataset, method, param):
#	if param<0:
#		return data+"/2_analysis/"+dataset+"/features/"+method+"/";
#	return data+"/2_analysis/"+dataset+"/features/"+metho=d+"_"+str(param)+"/";

def Metric(dataset):
	return data+"/2_analysis/"+dataset+"/metrics/NNGraph/";

	#### Training data (classifiers, pose priors) ####
def Classifier(claName):
	return data+"/3_trained/classifier/"+claName+"/";

def PosePriorFile(priorName):
	return data+"/3_trained/poseprior/"+priorName+".bin";

	#### Experiments (results) ####
def ExpInfo(expName):
	return data+"/4_experiments/"+expName+"/0_setup/";

def Prediction(expName):
	return data+"/4_experiments/"+expName+"/1_prediction/";

def PredictionWorkDir(expName):
	return data+"/4_experiments/"+expName+"/1_prediction_workdir/";

def PredictionRendering(expName):
	return data+"/4_experiments/"+expName+"/2_prediction_render/";

def PredictionEval(expName):
	return data+"/4_experiments/"+expName+"/3_prediction_eval/";

def PredictionSummary(expName):
	return PredictionEval(expName);

def Poses(expName):
	return data+"/4_experiments/"+expName+"/4_poses/";

def PosesRendering(expName):
	return data+"/4_experiments/"+expName+"/5_poses_render/";

def PosesEval(expName):
	return data+"/4_experiments/"+expName+"/6_poses_eval/";

def PosesSummary(expName):
	return PosesEval(expName);

######   Experiments ######
def PrepareSplitData(dset, Ntrain, Ntest, trainFile, testFile):
	trainPercent = True;
	if not os.path.isfile(trainFile) or not os.path.isfile(testFile):
		gtList = [];
		mList = fas.ReadModelNames(Models(dset));
		for mName in mList:
			mName = fas.NameOnly(mName);
			if os.path.isfile(gtDir+mName+".gt"):
				gtList = gtList + [mName];
		if trainPercent:
			Ntrain = int(Ntrain * len(mList) / 100);
		if testPercent:
			if Ntrain + Ntest == 100:
				Ntest = len(mList)-Ntrain;
			else:
				Ntest = int(Ntest * len(mList) / 100);
		assert(Ntrain>0 and Ntrain<=len(gtList));
		random.shuffle(gtList);
		for i in range(0, Ntrain):
			trainList = trainList + [gtList[i]];
			mList.remove(gtList[i]);
		if Ntest<0:
			Ntest = len(mList); # NOTE: mList only incldues remaining models
		random.shuffle(mList);
		assert(Ntest <= len(mList));
		for i in range(0, Ntest):
			testList =testList + [mList[i]];
		
		f = file(trainFile, "w");
		f.write("\n".join(trainList));
		f.close();
		f = file(testFile, "w");
		f.write("\n".join(testList));
		f.close();

def PrepareLeaveOneOut(dset, outListsDir, fraction):
	if not os.path.isdir(outListsDir):
		os.makedirs(outListsDir);
		mList = fas.ReadModelNames(Models(dset));
		for mName in mList:
			mCandidates = mList[:];
			mCandidates.remove(mName);
			mCandidates = random.sample(mCandidates, int(float(fraction * len(mCandidates)) / 100.));
			fas.WriteModelNames([mName], outListsDir+"/"+mName+"_test.txt");
			fas.WriteModelNames(mCandidates, outListsDir+"/"+mName+"_train.txt");

def PrepareAllModelsList(dset, outFile):
	if not os.path.isfile(outFile):
		mList = fas.ReadModelNames(Models(dset));
		fas.WriteModelNames(mList, outFile);

class ExpConfigs:
	m_BasicType = "none";
	m_ExpName = "none";
	m_Dataset = "none";
	m_TestFile = "none";
	m_ParamFile = data+"/0_body/params.txt";
	
	# region classifier
	m_ClaDataset = "none";
	m_ClaTrainFile = "none";
	m_ClaName = "none";
	m_ClaType = "Weka";
	
	# pose prior
	m_PriorDataset = "none";
	m_PriorTrainFile = "none";
	m_PriorName = "none";
	
	m_ClusterWorkspace = "TempDir";
	m_WorkName = "myexp";
	
	# additional pipeline flags
	m_Flags = "";
	m_bRunInParallel = False;
	m_bWaitForResults = False;

def RunExperiments(expArray, commonFlags):
	jobIDs = [];
	removeFiles = [];
	for e in expArray:
		if e.m_bWaitForResults:
			fas.WaitForJobsInArray(jobIDs);
			jobIDs = [];
		else:
			print("Running experiment: "+e.m_ExpName+" "+e.m_WorkName);
			cmd = scriptsdir+"experiment.py -dataset "+e.m_Dataset+" -expName "+e.m_ExpName+" -test "+e.m_TestFile+" ";
			cmd += "-claData "+e.m_ClaDataset+" -claTrainFile "+e.m_ClaTrainFile+" ";
			cmd += "-claName "+e.m_ClaName+" -claType "+e.m_ClaType+" ";
			cmd += "-priorData "+e.m_PriorDataset+" -priorTrainFile "+e.m_PriorTrainFile+" -priorName "+e.m_PriorName+" ";
			cmd += "-clusterWorkspace TempDir/"+e.m_ExpName+" "+commonFlags+" ";
			cmd += "-paramFile "+e.m_ParamFile+" "+e.m_Flags+" ";
			cmd += "-readRegionNames "+HumanBodyParts()+" "
			cmd += "-readSkeleton "+HumanSkeleton()+" "
			cmd += "-readRegionSymmetry "+HumanSymmetry()+" "
			if e.m_bRunInParallel:
				cmd += "-disallowParallel ";
				scriptFile = e.m_ClusterWorkspace+"/"+e.m_WorkName;
				jobIDs += fas.ScheduleJob(cmd, e.m_WorkName, scriptFile)
				removeFiles += fas.TmpFilesNames(scriptFile);
			else:
				os.system(cmd);
	fas.WaitForJobsInArray(jobIDs);



######### WEBSITE RESULTS

def CopyWebInit(dataset, forceRecompute):
	if figs.IsWebsite():
		mList = fas.ReadModelNames(Models(dataset));
		outDir = figs.website+"/plain/"+dataset+"/";
		inDir = ModelsRendering(dataset)+"/";
		if not os.path.isdir(outDir):
			os.makedirs(outDir);
		cmd = "";
		for mName in mList:
			inFile = inDir+"/"+mName+".jpg";
			outFile = outDir+"/"+mName+".jpg";
			if (not os.path.isfile(outFile) or forceRecompute) and os.path.isfile(inFile):
				cmd += "cp " + inFile+" " + outFile+"; ";
		
		outDir = figs.website+"/gt_poses/"+dataset+"/";
		inDir = GTRendering(dataset)+"/";
		for mName in mList:
			inFile = inDir+"/"+mName;
			outFile = outDir+"/"+mName;
			if (not os.path.isdir(outFile) or forceRecompute) and os.path.isdir(inFile):
				if not os.path.isdir(outFile):
					os.makedirs(outFile);
				else:
					cmd += "if [ -d "+outFile+" ]; then rm -r -f "+outFile+"; mkdir "+outFile+"; fi; ";
				cmd += "cp " + inFile+"/* " + outFile+"/; ";
		
		if (cmd!=""):
			os.system(cmd);

def CopyWebPredict(expName, forceRecompute):
	forceRecompute = True;
	if figs.IsWebsite():
		mList = fas.ReadModelNames(ExpInfo(expName)+"/test.txt");
		outDirRoot = figs.website+"/experiments/"+expName+"/";
		outDir = outDirRoot+"predict/";
		inDir = PredictionRendering(expName);
		cmd = "";
		for mName in mList:
			inFile = inDir+"/"+mName;
			outFile = outDir+"/"+mName;
			if (not os.path.isdir(outFile) or forceRecompute) and os.path.isdir(inFile):
				if not os.path.isdir(outFile):
					os.makedirs(outFile);
				else:
					cmd += "if [ -d "+outFile+" ]; then rm -r -f "+outFile+"; mkdir "+outFile+"; fi; ";
				cmd += "cp " + inFile+"/* " + outFile+"/; ";
		
		if (cmd!=""):
			os.system(cmd);
		cmd = "";
		
		inDir = PredictionSummary(expName);
		if (not os.path.isfile(outDirRoot+"test.txt") or forceRecompute) and os.path.isfile(ExpInfo(expName)+"/test.txt"):
			cmd +="cp "+ExpInfo(expName)+"test.txt "+outDirRoot+"; ";
		if (not os.path.isfile(outDirRoot+"train.txt") or forceRecompute) and os.path.isfile(ExpInfo(expName)+"/train.txt"):
			cmd +="cp "+ExpInfo(expName)+"/train.txt "+outDirRoot+"; ";
		
		outDir = outDirRoot+"/predictsumm/";
		if forceRecompute and os.path.isdir(outDir):
			cmd += "if [ -d "+outDir+" ]; then rm -r -f "+outDir+"; mkdir "+outDir+"; fi; ";
		if not os.path.isdir(outDir):
			os.makedirs(outDir);
		if (not os.path.isfile(outDir+"avePR.txt") or forceRecompute) and os.path.isfile(inDir+"avePR.txt"):
			cmd +="cp "+inDir+"avePR.txt "+outDir+"; ";
		
		pngs = glob.glob(inDir+"PR_*.png");
		for png in pngs:
			fOnly = os.path.basename(png);
			if not os.path.isfile(outDir+fOnly) or forceRecompute:
				cmd += "cp "+png+" "+outDir+fOnly+"; ";
		if (cmd!=""):
			os.system(cmd);


def CopyWebPose(expName, forceRecompute):
	if figs.IsWebsite():
		mList = fas.ReadModelNames(ExpInfo(expName)+"/test.txt");
		outDirRoot = figs.website+"/experiments/"+expName+"/";
		outDir = outDirRoot+"poses/";
		inDir = PosesRendering(expName);
		cmd = "";
		if (not os.path.isfile(outDirRoot+"test.txt") or forceRecompute) and os.path.isfile(ExpInfo(expName)+"/test.txt"):
			cmd +="cp "+ExpInfo(expName)+"test.txt "+outDirRoot+"; ";
		if (not os.path.isfile(outDirRoot+"train.txt") or forceRecompute) and os.path.isfile(ExpInfo(expName)+"/train.txt"):
			cmd +="cp "+ExpInfo(expName)+"/train.txt "+outDirRoot+"; ";
		for mName in mList:
			inFile = inDir+"/"+mName;
			outFile = outDir+"/"+mName;
			if (not os.path.isdir(outFile) or forceRecompute) and os.path.isdir(inFile):
				if not os.path.isdir(outFile):
					os.makedirs(outFile);
				else:
					cmd += "if [ -d "+outFile+" ]; then rm -r -f "+outFile+"; mkdir "+outFile+"; fi; ";
				cmd += "cp " + inFile+"/* " + outFile+"/; ";
		if (cmd!=""):
			os.system(cmd);
		cmd = "";
		
		inDir = Poses(expName)+"/";
		for mName in mList:
			srcFiles = ["energy.txt", "timing.txt", "info.txt"];
			for f in srcFiles:
				inFile = inDir+mName+"/"+f;
				outFile = outDir+mName+"/"+f;
				if (not os.path.isfile(outFile) or forceRecompute) and os.path.isfile(inFile):
					cmd += "cp " + inFile+" " + outFile+"; ";
		
		# copy per model errors
		inDir = PosesEval(expName)+"/";
		for mName in mList:
			inFile = inDir+mName+"/errinfo.txt";
			outFile = outDir+mName+"/errinfo.txt";
			if (not os.path.isfile(outFile) or forceRecompute) and os.path.isfile(inFile):
				cmd += "cp " + inFile+" " + outFile+"; ";
		
		# copy summary
		outDir = outDirRoot+"/posessummary/";
		inDir = PosesSummary(expName)+"/";
		if forceRecompute and os.path.isdir(outDir):
			cmd += "if [ -d "+outDir+" ]; then rm -r -f "+outDir+"; mkdir "+outDir+"; fi; ";
		if not os.path.isdir(outDir):
			os.makedirs(outDir);
		if (not os.path.isfile(outDir+"summary.txt") or forceRecompute) and os.path.isfile(inDir+"summary.txt"):
			cmd +="cp "+inDir+"summary.txt "+outDir+"; ";
		
		pngs = ["angerrs.png", "pnterrs.png"];
		for png in pngs:
			if (not os.path.isfile(outDir+png) or forceRecompute) and os.path.isfile(inDir+png):
				cmd += "cp "+inDir+png+" "+outDir+png+"; ";
		if (cmd!=""):
			os.system(cmd);


def CopyWebTiming(dataset, claName, predictExpName, expName, expParams, priorFile, forceRecompute):
	forceRecompute = True;
	if figs.IsWebsite():
		mList = fas.ReadModelNames(ExpInfo(expName)+"/test.txt");
		outDirRoot = figs.website+"/experiments/"+expName+"/";
		
		if not os.path.isfile(outDirRoot+"dataset.txt"):
			f = open(outDirRoot+"dataset.txt", "w");
			f.write(dataset+"\n");
			f.write(dataset+"\n");
			f.close();
		
		if not os.path.isdir(outDirRoot):
			os.makedirs(outDirRoot);
		outFile = figs.website+"/experiments/"+expName+"/timings.txt";
		timeStr = "";
		if not os.path.isfile(outFile) or forceRecompute:
			# predictions
			aveTime = 0;
			count = 0;
			for mName in mList:
				f2read = PredictionWorkDir(predictExpName)+"/"+mName+".timing.txt";
				if os.path.isfile(f2read):
					f = open(f2read);
					fLine = f.read().strip().replace("s", "");
					f.close();
					aveTime += float(fLine);
					count += 1;
			if count>0:
				aveTime /= count;
				timeStr += "Prediction "+str(aveTime)+"\n";
			else:
				timeStr += "Prediction XX\n";
			
			# classification
			aveTime = 0;
			infoStr = open(Classifier(claName)+"/info.txt").readlines();
			infoStrSplit = infoStr[1].strip().split(" ");
			if infoStrSplit[1][-1]=="s":
				aveTime += float(infoStrSplit[1].replace("s", ""));
			elif infoStrSplit[1][-1]=="m":
				aveTime += float(infoStrSplit[1].replace("m", ""))*60;
			else:
				print("[ERROR] Invalid timing");
				assert(False);
			claFiles2 = glob.glob(Classifier(claName)+"/weka_*.stats.txt");
			for claf in claFiles2:
				fLines = open(claf, "r").readlines();
				for f in fLines:
					strMatch = "Time taken to build model:";
					if f[0:len(strMatch)]==strMatch:
						fSplit = f.split(" ");
						aveTime += float(fSplit[5]);
						assert(fSplit[6].strip()=="seconds");
			timeStr += "TrainCla "+str(aveTime)+"\n";
			
			# pose prior
			aveTime = 0;
			f = open(priorFile.replace(".bin", ".timing.txt"));
			lines = f.readlines();
			lnSplit = lines[1].split();
			assert(len(lnSplit)==2);
			f.close();
			fval = float(int(10*float(lnSplit[1].replace("s", ""))+0.5))/10;
			timeStr += "PosePrior "+str(fval)+"\n";
			
			# points
			aveTime = 0;
			nmodels = 0;
			for p in range(0, len(expParams.m_PointsMethods)):
				pntsDir = Points(dataset, expParams.m_PointsMethods[p], expParams.m_PointsNSamps[p]);
				f = open(pntsDir+"/timing.txt");
				for fLine in f.readlines():
					lnStr = fLine.strip();
					if lnStr!="Time Profile:" and lnStr!="":
						lnSplit = lnStr.replace("\t", " ").split();
						aveTime += float(lnSplit[1].strip().replace("s", ""));
						nmodels = nmodels + 1;
				f.close();
			aveTime = float(int(10*(aveTime / nmodels * len(expParams.m_PointsMethods))+0.5))/10;
			timeStr += "Points "+str(aveTime)+"\n";
			
			# features
			aveTime = 0;
			for f in range(0, len(expParams.m_FeatNames)):
				fName = expParams.m_FeatFullNames[f];
				timeFile = AllFeatures(dataset)+"/"+fName+"/timing.txt";
				if os.path.isfile(timeFile):
					f = open(timeFile);
					for fLine in f.readlines():
						lnStr = fLine.strip();
						if lnStr!="Time Profile:" and lnStr!="":
							lnSplit = lnStr.replace("\t", " ").split();
							aveTime += float(lnSplit[1].strip().replace("s", ""));
							nmodels = nmodels + 1;
					f.close();
			aveTime = float(int(10*(aveTime / nmodels * len(expParams.m_FeatNames))+0.5))/10;
			timeStr += "Features "+str(aveTime)+"\n";
			
			# metric
			aveTime = 0;
			f = open(Metric(dataset)+"/timing.txt");
			nmodels = 0;
			for fLine in f.readlines():
				lnStr = fLine.strip();
				if lnStr!="Time Profile:" and lnStr!="":
					lnSplit = lnStr.replace("\t", " ").split();
					aveTime += float(lnSplit[1].strip().replace("s", ""));
					nmodels = nmodels + 1;
			f.close();
			aveTime = float(int(10*(aveTime / nmodels * len(expParams.m_PointsMethods))+0.5))/10;
			timeStr += "Metric "+str(aveTime)+"\n";
			
			f = open(outFile, "w");
			f.write(timeStr);
			f.close();

