#!/usr/bin/python

##########
#
#  Experiment
#
##########
import os, string, glob, sys, random
sys.path.append(os.path.realpath("./"+os.path.dirname(sys.argv[0])))
import fas, figs, exp

# Assume off file is given and ground truth is created

## steps to execute
dataset = "none";
trainFile = "none";
testFile = "none";
Ntrain = -1;
Ntest = -1;
trainPercent = False;
testPercent = False;

# point sampling params
napproxpnts = 1000;
pntsmethod = "even";
featpnts = 100000;
featpntsmethod = "random";
pipeline = [];
disallowParallel = "";

# workspace
clusterWorkspace = "./TempDir/";

# feature calculation...
forceList = [];

randseed = -1;


paramFile = "none";
predictExpName = "default";
overwriteParam = [];
overwriteField = [];
overwriteValue = [];

claType = "Weka";
readExternalPoseDir = "none";
readExternalPoseFile = "none";

if len(sys.argv)<2:
	print("Usage: "+sys.argv[0]+" [options]");
	print("Options:");
	print("   -dataset datasetName");
	print("   -test filename ");
	print("   -claData claDataset ");
	print("   -claTrainFile filename ");
	print("   -claName claName ");
	print("   -priorData priorData ");
	print("   -priorName priorName ");
	print("   -predictExpName predictName (can leave as 'default') ");
	print("   -expName expName ");
	print("   -clusterWorkspace directory ");
	print("   -readRegionNames regionNamesFile ");
	print("   -readSkeleton skeletonFile ");
	print("   -readExternalPose dir file ");
	print("   -readRegionSymmetry symmetryFile ");
	print("   -force [render points features prior train predict render_predict eval_predict...");
	print("     	  ...  poses render_poses eval_poses] ");
	print("   -pipeline [same as -force, above]  ");
	print("   -disallowParallel  - note fas.IsParallel() - also checked");
	print("   -paramFile paramFile  ");
	print("   -overwrite param field value - overwrite parameter");
	exit();
else:
	i = 1;
	while i<len(sys.argv):
		if (sys.argv[i]=="-force"):
			i=i+1;
			forceList += [sys.argv[i]];
		elif (sys.argv[i]=="-pipeline"):
			i=i+1;
			pipeline += [sys.argv[i]];
		elif (sys.argv[i]=="-dataset"):
			i=i+1;
			dataset = sys.argv[i];
		elif (sys.argv[i]=="-test"):
			i=i+1;
			testFile = sys.argv[i];
		elif (sys.argv[i]=="-debugPoses"):
			debugPoses = True;
		elif (sys.argv[i]=="-setupExperiment"):
			if sys.argv[i+1][-1:]=="%":
				trainPercent = True;
				Ntrain = int(sys.argv[i+1][:-1]);
			else:
				Ntrain = int(sys.argv[i+1]);
			if sys.argv[i+2][-1:]=="%":
				testPercent = True;
				Ntest = int(sys.argv[i+2][:-1]);
			else:
				Ntest = int(sys.argv[i+2]);
			i=i+2;
		elif (sys.argv[i]=="-expName"):
			i=i+1;
			expName = sys.argv[i];
		elif (sys.argv[i]=="-claData"):
			i=i+1;
			claData = sys.argv[i];
		elif (sys.argv[i]=="-claTrainFile"):
			i=i+1;
			claTrainFile = sys.argv[i];
		elif (sys.argv[i]=="-claName"):
			i=i+1;
			claName = sys.argv[i];
		elif (sys.argv[i]=="-claType"):
			i=i+1;
			claType = sys.argv[i];
		elif (sys.argv[i]=="-priorName"):
			i=i+1;
			priorName = sys.argv[i];
		elif (sys.argv[i]=="-clusterWorkspace"):
			i=i+1;
			clusterWorkspace = sys.argv[i];
		elif (sys.argv[i]=="-readRegionNames"):
			i=i+1;
			readRegionNames = sys.argv[i];
		elif (sys.argv[i]=="-readSkeleton"):
			i=i+1;
			readSkeleton = sys.argv[i];
		elif (sys.argv[i]=="-readRegionSymmetry"):
			i=i+1;
			readRegionSymmetry = sys.argv[i];
		elif (sys.argv[i]=="-paramFile"):
			i=i+1;
			paramFile = sys.argv[i];
		elif (sys.argv[i]=="-priorData"):
			i=i+1;
			priorData = sys.argv[i];
		elif (sys.argv[i]=="-priorName"):
			i=i+1;
			priorName = sys.argv[i];
		elif (sys.argv[i]=="-priorName"):
			i=i+1;
			priorName = sys.argv[i];
		elif (sys.argv[i]=="-predictExpName"):
			i=i+1;
			predictExpName = sys.argv[i];
		elif (sys.argv[i]=="-disallowParallel"):
			disallowParallel = "-disallowParallel";
		elif (sys.argv[i]=="-priorTrainFile"):
			i=i+1;
			priorTrainFile = sys.argv[i];
		elif (sys.argv[i]=="-readExternalPose"):
			readExternalPoseDir = sys.argv[i+1];
			readExternalPoseFile = sys.argv[i+2];
			i=i+2;
		elif (sys.argv[i]=="-overwrite"):
			overwriteParam += [sys.argv[i+1]];
			overwriteField += [sys.argv[i+2]];
			overwriteValue += [sys.argv[i+3]];
			i=i+3;
		else:
			print("[ERROR] Cannot understand command: "+sys.argv[i]+" in experiment.py");
			assert(False);
		i=i+1;

# init directory names
modelsDir = exp.Models(dataset);
gtDir = exp.GT(dataset);
xformDir = exp.InXForm(dataset);
if predictExpName=="default":
	predictExpName = expName;


# copy training and test priors
if expName!="none":
	expFilesDir = exp.ExpInfo(expName);
	if not os.path.isdir(expFilesDir):
		os.makedirs(expFilesDir);
	if os.path.isfile(testFile) and expName[0:3]!="loo":
		os.system("cp "+testFile +" "+expFilesDir+"/test.txt");
	if os.path.isfile(claTrainFile) and expName[0:3]!="loo":
		os.system("cp "+claTrainFile+" "+expFilesDir+"/cla_train.txt");
	if os.path.isfile(priorTrainFile) and expName[0:3]!="loo":
		os.system("cp "+priorTrainFile +" "+expFilesDir+"/prior_train.txt");


# render if necessary
if "render" in pipeline or len(pipeline)==0:
	if figs.IsRenderable():
		print("Render meshes and ground truth poses");
		cmd = exp.scriptsdir+"render.py -modelsDir "+modelsDir+" -gtDir "+gtDir+" ";
		cmd += "-renderPoses "+exp.GTRendering(dataset)+" ";
		cmd += "-renderMeshes "+exp.ModelsRendering(dataset)+" ";
		cmd += "-clusterWorkspace "+clusterWorkspace+"/renderMeshes/ "+disallowParallel+" ";
		cmd += "-readRegionNames "+readRegionNames+" -readSkeleton "+readSkeleton+" "
		if "render" in forceList:
			cmd += "-forceRecompute ";
		os.system(cmd+" ");
	else:
		print("[WARNING] Failed to render meshes and ground truth poses");

	exp.CopyWebInit(dataset, "render" in forceList);


# parse parameters
params = exp.ReadParams(paramFile);

for o in range(0, len(overwriteParam)):
	if overwriteParam[o]=="pose":
		params.m_PoseParams[overwriteField[o]] = overwriteValue[o];
	else:
		print("[ERROR] Unknown parameter overwrite: "+overwriteParam[o]);
		assert(False);

randseed = params.m_PipelineParams["randseed"];
expInfoDir = exp.ExpInfo(expName);
if not os.path.isdir(expInfoDir):
	os.makedirs(expInfoDir);
featNamesFile = expInfoDir+"/features.txt";
paramFile = expInfoDir+"/params.txt";
if not os.path.isfile(featNamesFile):
	params.WriteFeatureNames(featNamesFile);
if not os.path.isfile(paramFile):
	params.WriteParams(paramFile);

# prepare points
pntsDir = exp.Points(dataset, params.m_PointsMethods[0], params.m_PointsNSamps[0]);
densePntsDir = exp.Points(dataset, params.m_PointsMethods[1], params.m_PointsNSamps[1]);
allptsdirs = [pntsDir, densePntsDir];
if "points" in pipeline or len(pipeline)==0:
	print("Prepare points");
	for p in range(0, len(params.m_PointsMethods)):
		cmd = exp.scriptsdir+"createpts.py -modelsDir "+modelsDir+" ";
		cmd += "-outDir "+allptsdirs[p]+" ";
		cmd += "-napproxpnts "+str(params.m_PointsNSamps[p])+" -method "+params.m_PointsMethods[p]+" ";
		wrkspacedir = clusterWorkspace+"/points_"+params.m_PointsMethods[p]+"_"+str(params.m_PointsNSamps[p])+" ";
		cmd += "-clusterWorkspace "+wrkspacedir+" -randseed "+randseed+" "+disallowParallel+" ";
		if "points" in forceList:
			cmd += "-forceRecompute ";
		os.system(cmd);

########### FEATURE-RELATED COMPUTATIONS  ###########
# which features to compute
featDir = exp.AllFeatures(dataset);
metricDir = exp.Metric(dataset);
if "features" in pipeline or len(pipeline)==0:
	print("Computing local geometric features");
	# prepare features
	cmd = exp.scriptsdir+"/createfeats.py -dataset "+dataset+" -modelsDir "+modelsDir+" -pntsDir "+pntsDir+" ";
	cmd += "-outDir "+featDir+" -densePntsDir "+densePntsDir+" -nnMetricGraphsDir "+metricDir+" ";
	cmd += "-randseed "+randseed+" "+disallowParallel+" ";
	for i in range(0, len(params.m_FeatNames)):
		cmd += "-feature "+params.m_FeatNames[i]+" "+str(params.m_FeatParams[i])+" ";
	cmd += "-origrotate "+xformDir+" "
	cmd += "-clusterWorkspace "+clusterWorkspace+"/ ";
	if "features" in forceList:
		cmd += "-forceRecompute ";
	os.system(cmd);


################## TRAINING STEP #################
# train region classifier
claDir = exp.Classifier(claName);
if ("train" in pipeline or len(pipeline)==0):
	gtTrainDir = exp.GT(claData);
	trainModelsDir = exp.Models(claData);
	trainEvenPtsDir = exp.Points(claData, params.m_PointsMethods[0], params.m_PointsNSamps[0]);
	trainDensePtsDir = exp.Points(claData, params.m_PointsMethods[1], params.m_PointsNSamps[1]);
	trainFeatsDir = exp.AllFeatures(claData);
	trainMetricDir = exp.Metric(claData);
	print("Train region classifier");
	sys.stdout.flush()
	cmd = exp.scriptsdir+"/trainRegions.py -modelsDir "+trainModelsDir+" -gtDir "+gtTrainDir+" -pntsDir "+trainEvenPtsDir+" ";
	cmd += "-featDir "+trainFeatsDir+" -outDir "+claDir+" -trainFile "+claTrainFile+" -claType "+claType+" ";
	cmd += "-features "+featNamesFile+" -symmInvariant -densePntsDir "+trainDensePtsDir+" -metricDir "+trainMetricDir+" ";
	cmd += "-clusterWorkspace "+clusterWorkspace+"/train_regions/"+claName+" ";
	cmd += "-readRegionNames "+readRegionNames+" -readRegionSymmetry "+readRegionSymmetry+" "
	cmd += "-readSkeleton "+readSkeleton+" "
	cmd += "-randseed "+randseed+" "+disallowParallel+" ";
	for (key, value) in params.m_PredictionParams.items():
		if value!="True":
			cmd += "-param "+str(len(value.split(" ")))+" "+key+" "+value+" ";
		else:
			cmd += "-param 0 "+key+" ";
	if "train" in forceList:
		cmd += "-forceRecompute ";
	os.system(cmd+" ");

# train pose priors
if priorName!="none":
	posePriorFile = exp.PosePriorFile(priorName);
	posePriorTiming = exp.PosePriorFile(priorName).replace(".bin", ".timing.txt");
else:
	posePriorFile = "none";
	posePriorTiming = "none";

if "prior" in pipeline or len(pipeline)==0:
	print("Train pose priors");
	# priorDataset priorName priorTrainFile
	if priorData!="cornell":
		priorGtDir = exp.GT(priorData);
	else:
		priorGtDir = priorData;
	comDir = densePntsDir+"/com_ja16/";
	cmd = exp.scriptsdir+"/trainPosepriors.py -gtDir "+priorGtDir+" -comDir "+comDir+" -trainFile "+priorTrainFile+" ";
	cmd += "-outFile "+posePriorFile+" -outTiming "+posePriorTiming+" ";
	cmd += "-clusterWorkspace "+clusterWorkspace+"/train_poses ";
	cmd += "-readRegionNames "+readRegionNames+" -readSkeleton "+readSkeleton+" ";
	cmd += "-randseed "+randseed+" "+disallowParallel+" ";
	for (key, value) in params.m_PosePriorParams.items():
		if value!="True":
			cmd += "-param "+str(len(value.split(" ")))+" "+key+" "+value+" ";
		else:
			cmd += "-param 0 "+key+" ";

	cmd += "-param2 wEndEffectors "+params.m_PoseParams["wEndEffectors"]+" ";
	cmd += "-param2 wSymmetryPrior "+params.m_PoseParams["wSymmetryPrior"]+" ";
	cmd += "-param2 wPosePrior "+params.m_PoseParams["wPosePrior"]+" ";
	cmd += "-readRegionStats "+claDir+"/regionstats.txt ";

	if "prior" in forceList or not os.path.isfile(posePriorFile):
		cmd += "-forceRecompute ";
		os.system(cmd);

if testFile=="none":
	exit();

############### CLASSIFICATION STEP ###############
# predict regions
predictRegionsDir = exp.Prediction(predictExpName);
wekaWorkDir = exp.PredictionWorkDir(predictExpName);
if "predict" in pipeline or len(pipeline)==0:
	print("Predict regions");
	claParams = "";
	if claType=="Weka":
		claParams += "-wekaWorkDir "+wekaWorkDir+" ";
	elif claType[0:8]=="External":
		claParams += "-wekaWorkDir "+claType[9:]+" ";

	cmd = exp.scriptsdir+"/predictRegions.py -modelsDir "+modelsDir+" -pntsDir "+pntsDir+" -claDir "+claDir+" -featDir "+featDir+" ";
	cmd += " -outDir "+predictRegionsDir+" -testFile "+testFile+" -claType "+claType+" "+claParams+" ";
	cmd += "-features "+featNamesFile+" -symmInvariant ";
	cmd += "-clusterWorkspace "+clusterWorkspace+"/predict ";
	cmd += "-readRegionNames "+readRegionNames+" -readRegionSymmetry "+readRegionSymmetry+" ";
	cmd += "-randseed "+randseed+" "+disallowParallel+" ";
	if "predict" in forceList:
		cmd += "-forceRecompute ";
	os.system(cmd+" ");


# render predictions
predictImgDir = exp.PredictionRendering(predictExpName);
if "render_predict" in pipeline or len(pipeline)==0:
	if figs.IsRenderable():
		print("Render region predictions");
		cmd = exp.scriptsdir+"/renderRegions.py -modelsDir "+modelsDir+" -pntsDir "+pntsDir+" ";
		cmd += "-outDir "+predictImgDir+" -testFile "+testFile+" -predictDir "+predictRegionsDir+" ";
		cmd += "-clusterWorkspace "+clusterWorkspace+"/render_predict "+disallowParallel+" ";
		cmd += "-readRegionNames "+readRegionNames+" ";
		if "render_predict" in forceList:
			cmd += "-forceRecompute ";
		os.system(cmd+" ");
	else:
		print("[WARNING] Failed to render region predictions");



# evaluate predictions
evalOutDir = exp.PredictionEval(predictExpName);
outEvalImgDir = exp.PredictionSummary(predictExpName);
if "eval_predict" in pipeline or len(pipeline)==0:
	print("Evaluate predictions");
	cmd = exp.scriptsdir+"/evalRegions.py -modelsDir "+modelsDir+" -pntsDir "+pntsDir+" -gtDir "+gtDir+" -symmInvariant ";
	cmd += "-outDir "+evalOutDir+" -testFile "+testFile+" -predictDir "+predictRegionsDir+" -outImages "+outEvalImgDir+" ";
	cmd += "-clusterWorkspace "+clusterWorkspace+"/eval_predict "+disallowParallel+" ";
	cmd += "-readRegionNames "+readRegionNames+" -readRegionSymmetry "+readRegionSymmetry+" ";
	if "eval_predict" in forceList:
		cmd += "-forceRecompute ";
	for (key, value) in params.m_EvalParams.items():
		cmd += "-"+key+" "+value+" ";
	os.system(cmd+"  ");

	exp.CopyWebPredict(predictExpName, ("render_predict" in forceList) or ("eval_predict" in forceList));


# predict poses
predictPoseDir = exp.Poses(expName);
if "poses" in pipeline or len(pipeline)==0:
	print("Predict poses");
	#algStatsDir = figs.website+"/experiments/"+expName+"/stats/";
	planesDir = "../data/2_analysis/"+dataset+"/features/gapsNewPRST_work/";
	cmd = exp.scriptsdir+"/predictPose.py -modelsDir "+modelsDir+" -pntsDir "+pntsDir+" -posePriorFile "+posePriorFile+" ";
	cmd += "-outDir "+predictPoseDir+" -testFile "+testFile+" -predictDir "+predictRegionsDir+" ";
	cmd += "-clusterWorkspace "+clusterWorkspace+"/poses "+disallowParallel+" ";
	cmd += "-readRegionNames "+readRegionNames+" -readSkeleton "+readSkeleton+" ";
	cmd += "-randseed "+randseed+" -readRegionStats "+claDir+"/regionstats.txt ";
	cmd += "-origrotate "+xformDir+" -densePntsDir "+densePntsDir+" ";
	cmd += "-planesDir "+planesDir+" "
	if readExternalPoseDir=="gt":
		readExternalPoseDir = gtDir;
	cmd += "-readExternalPose "+readExternalPoseDir+" "+readExternalPoseFile+" "

	for (key, value) in params.m_PoseParams.items():
		if value!="True":
			cmd += "-param "+str(len(value.split(" ")))+" "+key+" "+value+" ";
		else:
			cmd += "-param 0 "+key+" ";
	if "poses" in forceList:
		cmd += "-forceRecompute ";
	os.system(cmd+"  ");


# render poses
poseImgDir = exp.PosesRendering(expName);
if "render_poses" in pipeline or len(pipeline)==0:
	if figs.IsRenderable():
		print("Render poses");
		#poseImgDir = figs.website+"/experiments/"+expName+"/poses/";
		cmd = exp.scriptsdir+"/renderPoses.py -modelsDir "+modelsDir+" -pntsDir "+pntsDir+" -densePntsDir "+densePntsDir+" ";
		cmd += "-posePriorFile "+posePriorFile+" -outDir "+poseImgDir+" -testFile "+testFile+" ";
		cmd += "-predictDir "+predictRegionsDir+" -predictedPoseDir "+predictPoseDir+" ";
		cmd += "-readRegionStats "+claDir+"/regionstats.txt ";
		cmd += "-clusterWorkspace "+clusterWorkspace+"/render_poses "+disallowParallel+" ";
		cmd += "-readRegionNames "+readRegionNames+" -readSkeleton "+readSkeleton+" "
		cmd += "-origrotate "+xformDir+" "
		if "render_poses" in forceList:
			cmd += "-forceRecompute ";
		os.system(cmd+" ");
	else:
		print("[WARNING] Failed to render poses");


# evaluate poses
poseEvalOutDir = exp.PosesEval(expName);
poseEvalImgDir = exp.PosesSummary(expName);
if "eval_poses" in pipeline or len(pipeline)==0:
	print("Eval poses");
	#poseEvalImgDir = figs.website+"/experiments/"+expName+"/posessummary";
	cmd = exp.scriptsdir+"/evalPoses.py -modelsDir "+modelsDir+" -pntsDir "+pntsDir+" ";
	cmd += "-outDir "+poseEvalOutDir+" -testFile "+testFile+" -gtDir "+gtDir+" ";
	cmd += "-outImages " +poseEvalImgDir+ " -predictedPoseDir "+predictPoseDir+" ";
	cmd += "-clusterWorkspace "+clusterWorkspace+"/eval_poses "+disallowParallel+" ";
	cmd += "-readRegionNames "+readRegionNames+" -readSkeleton "+readSkeleton+" "

	if "eval_poses" in forceList:
		cmd += "-forceRecompute ";
	os.system(cmd+" ");

	exp.CopyWebPose(expName, ("render_poses" in forceList) or ("eval_poses" in forceList));
	exp.CopyWebTiming(dataset, claName, predictExpName, expName, params, posePriorFile, len(forceList)>0);







