#!/usr/bin/python

##########
#
# Create ground truth for functionality analysis.
#
##########
import os, string, decimal, glob, re, subprocess, shlex, sys, datetime, time
sys.path.append(os.path.realpath("./"+os.path.dirname(sys.argv[0])))
import fas
import numpy as np

######### UNIT CONVERSION #######
units2meters = .6;

website = "../data/5_images/";

##### RENDERABLE
def IsWebsite():
	return True;

def IsRenderable():
	return True;

def InitDrawStr():
	str = "";
	str += "#!/usr/bin/python\n";
	str += "import matplotlib as mpl;\n";
	str += "mpl.use('Agg');\n";
	str += "from matplotlib import pyplot as plt\n";
	str += "import numpy as np\n";
	str += "import matplotlib\n";
	str += "stdFont = 22;\n";
	str += "mpl.rc('lines', linewidth=4);\n";
	str += "mpl.rc('xtick', labelsize=stdFont);\n";
	str += "mpl.rc('ytick', labelsize=stdFont);\n";
	str += "mpl.rc('axes', labelsize=stdFont);\n";
	str += "mpl.rc('legend', fontsize=stdFont);\n";
	str += "mpl.rc('legend', labelspacing=0);\n";
	str += "mpl.rc('legend', borderpad=0.15);\n";
	str += "mpl.rc('legend', handletextpad=.0);\n";
	str += "titlefont = 33;\n";
	return str;

######   Plotting (move to eval scripts?)     ######
# see http://matplotlib.org
def EvalPairs2PRCurves(datadir, mList, distThresholds, valThresholds, evalDir, outDir, rList):
	nsteps = 20;
	counter = 0;
	Rc = [];
	Pr = [];
	nregionstrue = 0;
	valBins = [];
	for vt in valThresholds:
		valBins += [int(vt * nsteps)];
		Rc += [0];
		Pr += [0];
	assert(len(distThresholds)==1);	# note: would need to change valThresholds if more than one threhsold
	for rName in rList:
		counter = counter + 1;
		dists = [];
		vals = [];
		recallTotal = 0;
		objBegin = [];
		objEnd = [];
		objRecallTotal = [];
		for mName in mList:
			evalFile = evalDir+"/"+mName+"/"+rName+".eval_pairs";
			ndists = 0;
			objBegin.append(len(dists));
			if os.path.isfile(evalFile):
				f = open(evalFile);
				for pair in f.readlines():
					pairSplit = pair.strip().split(" ");
					vals.append(float(pairSplit[0].strip()));
					#dists.append(float(pairSplit[1].strip()));
					dlist = [];
					for dStr in pairSplit[1:]:
						dlist.append(float(dStr.strip()));
					if ndists==0:
						ndists = len(dlist);
					else:
						assert(ndists==len(dlist));
					dists.append(dlist);
					if float(pairSplit[0].strip())>1:
						print("Invalid Value: "+pairSplit[0].strip()+" -- "+mName+" "+rName)
				f.close();
			if ndists>0:
				recallTotal += ndists;
			objEnd.append(len(dists));
			objRecallTotal.append(ndists);
		
		#print(dists);
		if (len(dists)>0):
			drawStr = InitDrawStr();
			drawStr += "plt.clf();\n";
			drawStr += "fig = plt.figure();\n";
			drawStr += "nsteps="+str(nsteps)+";\n";
			legendNames = [];
			for distThresh in distThresholds:
				# compute TP, FP, FN
				precisionTruePos = np.zeros(nsteps+1);
				precisionTotal = np.zeros(nsteps+1);
				recallTruePositive = np.zeros(nsteps+1);
				for i in range(0, nsteps+1):    	# for each threshold
					valThresh = float(i) / float(nsteps);
					for j in range(0, len(dists)):	# compute precision
						if vals[j] >= valThresh: 	# passes value threshold
							precisionTotal[i] += 1;
							if len(dists[j])>0 and min(dists[j]) < distThresh:   # passes dist threshold
								precisionTruePos[i] += 1;
					for o in range(0, len(objBegin)):
						d1 = objBegin[o];
						d2 = objEnd[o];
						for d in range(0, objRecallTotal[o]):
							mindist = -1;
							for j in range(d1, d2):
								if vals[j] >= valThresh and (mindist==-1 or dists[j][d]<mindist):
									mindist = dists[j][d]; # passed value threshold -- find closest
							if (mindist < distThresh and mindist>=0): # closest point is within threshold -- accept
								recallTruePositive[i] += 1;
				
				
				
				# create PR curve data
				precision = np.zeros(nsteps+1);
				recall = np.zeros(nsteps+1);
				for i in range(0, nsteps+1):
					if precisionTruePos[i]>0:
						precision[i] = precisionTruePos[i] / precisionTotal[i];
					if recallTruePositive[i]>0:
						recall[i] = recallTruePositive[i] / recallTotal;
				
				# render matrix
				drawStr+= "recall=[";
				for x in range(0, len(recall)):
					drawStr += str(recall[x]);
					if x<len(recall)-1:
						drawStr += ", ";
				drawStr+="];\n";
				drawStr+= "precision=[";
				for x in range(0, len(precision)):
					drawStr += str(precision[x]);
					if x<len(precision)-1:
						drawStr += ", ";
				drawStr+="];\n";

				drawStr+= "res = plt.plot(recall[:-1], precision[:-1]);\n";
				drawStr+= "plt.scatter(recall[:-1], precision[:-1]);\n";
				
				if distThresh==distThresholds[-1]:
					it = 0;
					nregionstrue = nregionstrue + 1;
					for vb in valBins:
						Rc[it] += recall[vb];
						Pr[it] += precision[vb];
						it = it + 1;
						drawStr+="vb="+str(vb)+";\n";
						drawStr+="plt.scatter(recall[vb], precision[vb], s=100, c='red');\n";
						
				
				drawStr += "plt.scatter(recall[1], precision[1], s=100, c='green');\n";
				drawStr += "plt.scatter(recall[nsteps-1], precision[nsteps-1], s=100, c='yellow');\n";
				legendNames.append(str(distThresh));
			
			#plt.xticks([0, nsteps-1], [0, 1]);
			#plt.yticks([0, nbins-1], [0, 1]);
			drawStr+="plt.xlim([0, 1]);\n";
			drawStr+="plt.ylim([0, 1]);\n";
			drawStr+="legendNames="+str(legendNames)+"\n";
			drawStr+="plt.legend(legendNames);\n";
			drawStr+="plt.xlabel('Recall');\n";
			drawStr+="plt.ylabel('Precision');\n";
			drawStr+="plt.savefig(\""+outDir+"/PR_"+rName+".pdf\");\n";
			drawStr+="plt.savefig(\""+outDir+"/PR_"+rName+".png\");\n";
			f = open(outDir+"/plot"+rName+".py", "w");
			f.write(drawStr);
			f.close();
			file = outDir+"/plot"+rName+".py";
			os.system("chmod 777 "+file+"; "+file+";");


	Rcnew = [];
	Prnew = [];
	for rc in Rc:
		Rcnew += [rc/nregionstrue];
	for pr in Pr:
		Prnew += [pr/nregionstrue];
	f = open(outDir+"/avePR.txt", "w");
	f.write("AveRc: "+str(Rcnew[0])+"\n"+"AvePr: "+str(Prnew[0])+"\n");
	f.close();
	return [Rcnew, Prnew];

def EvalPairs2PRCurvesSinglePlot(datadir, mList, distThreshold, valThreshold, evalDir, outDir, rList, plotName):
	nsteps = 20;
	counter = 0;
	Rc = [];
	Pr = [];
	nregionstrue = 0;
	valBins = [];
	valBins += [int(valThreshold * nsteps)];
	Rc += [0];
	Pr += [0];

	drawStr = InitDrawStr();
	drawStr += "plt.clf();\n";
	drawStr += "fig = plt.figure();\n";
	legendNames = [];
	for rName in rList:
		legendNames
		if rName[0:4]=="left":
			continue;
		counter = counter + 1;
		dists = [];
		vals = [];
		recallTotal = 0;
		objBegin = [];
		objEnd = [];
		objRecallTotal = [];
		for mName in mList:
			evalFile = evalDir+"/"+mName+"/"+rName+".eval_pairs";
			ndists = -1;
			objBegin.append(len(dists));
			if os.path.isfile(evalFile):
				f = open(evalFile);
				for pair in f.readlines():
					pairSplit = pair.strip().split(" ");
					vals.append(float(pairSplit[0].strip()));
					#dists.append(float(pairSplit[1].strip()));
					dlist = [];
					for dStr in pairSplit[1:]:
						dlist.append(float(dStr.strip()));
					if ndists<0:
						ndists = len(dlist);
					else:
						assert(ndists==len(dlist));
					dists.append(dlist);
					if float(pairSplit[0].strip())>1:
						print("Invalid Value: "+pairSplit[0].strip()+" -- "+mName+" "+rName)
				f.close();
			if ndists>0:
				recallTotal += ndists;
			objEnd.append(len(dists));
			objRecallTotal.append(ndists);
		
		#print(dists);
		if (len(dists)>0):
			drawStr += "nsteps="+str(nsteps)+";\n";
			# compute TP, FP, FN
			precisionTruePos = np.zeros(nsteps+1);
			precisionTotal = np.zeros(nsteps+1);
			recallTruePositive = np.zeros(nsteps+1);
			for i in range(0, nsteps+1):    	# for each threshold
				valThresh = float(i) / float(nsteps);
				for j in range(0, len(dists)):	# compute precision
					if vals[j] >= valThresh: 	# passes value threshold
						precisionTotal[i] += 1;
						if len(dists[j])>0 and min(dists[j]) < distThreshold:   # passes dist threshold
							precisionTruePos[i] += 1;
				for o in range(0, len(objBegin)):
					d1 = objBegin[o];
					d2 = objEnd[o];
					for d in range(0, objRecallTotal[o]):
						mindist = -1;
						for j in range(d1, d2):
							if vals[j] >= valThresh and (mindist==-1 or dists[j][d]<mindist):
								mindist = dists[j][d]; # passed value threshold -- find closest
						if (mindist < distThreshold and mindist>=0): # closest point is within threshold -- accept
							recallTruePositive[i] += 1;
			# create PR curve data
			precision = np.zeros(nsteps+1);
			recall = np.zeros(nsteps+1);
			for i in range(0, nsteps+1):
				if precisionTruePos[i]>0:
					precision[i] = precisionTruePos[i] / precisionTotal[i];
				if recallTruePositive[i]>0:
					recall[i] = recallTruePositive[i] / recallTotal;
			
			# render matrix
			drawStr+= "recall"+rName+"=[";
			for x in range(0, len(recall)):
				if precision[x]==0:
					break;
				drawStr += str(recall[x]);
				if x<len(recall)-1 and precision[x+1]>0:
					drawStr += ", ";
			drawStr+="];\n";
			drawStr+= "precision"+rName+"=[";
			for x in range(0, len(precision)):
				if precision[x]==0:
					break;
				drawStr += str(precision[x]);
				if x<len(precision)-1 and precision[x+1]>0:
					drawStr += ", ";
			drawStr+="];\n";
			
			colorStr = "";
			if rName=="pelvis":
				colorStr = ", color=(0, 1, 0)";
			elif rName=="right_palm":
				colorStr = ", color=(0, 0, 1)";
			elif rName=="right_toe":
				colorStr = ", color=(0, .8, .8)";
			elif rName=="right_leg1":
				colorStr = ", color=(.6, .6, 0)";
			elif rName=="back":
				colorStr = ", color=(.55, .11, .26)";

			drawStr+= "res = plt.plot(recall"+rName+"[:-1], precision"+rName+"[:-1]"+colorStr+", alpha=0.9);\n";
			drawStr+= "plt.scatter(recall"+rName+"[:-1], precision"+rName+"[:-1], s=10);\n";
			
			it = 0;
			nregionstrue = nregionstrue + 1;
			for vb in valBins:
				Rc[it] += recall[vb];
				Pr[it] += precision[vb];
				it = it + 1;
				drawStr+="vb="+str(vb)+";\n";
				drawStr+="plt.scatter(recall"+rName+"[vb], precision"+rName+"[vb], s=50, c='red');\n";

			
#			drawStr += "plt.scatter(recall"+rName+"[1], precision"+rName+"[1], s=50, c='green');\n";
#			drawStr += "plt.scatter(recall"+rName+"[nsteps-1], precision"+rName+"[nsteps-1], s=100, c='yellow');\n";
			if rName[0:5]=="right":
				legendNames.append(rName[6:]);
			else:
				legendNames.append(rName);


	#plt.xticks([0, nsteps-1], [0, 1]);
	#plt.yticks([0, nbins-1], [0, 1]);
	drawStr+="plt.xlim([0, 1]);\n";
	drawStr+="plt.ylim([0, 1]);\n";
	drawStr+="legendNames="+str(legendNames)+"\n";
	drawStr+="plt.legend(legendNames);\n";
	drawStr+="plt.xlabel('Recall');\n";
	drawStr+="plt.ylabel('Precision');\n";
	drawStr+="plt.text(0.05, 0.9, \""+plotName+"\", fontsize=titlefont);\n";
	drawStr+="plt.savefig(\""+outDir+"/PR_combined.pdf\");\n";
	drawStr+="plt.savefig(\""+outDir+"/PR_combined.png\");\n";
	with open(outDir+"/plotCombined.py", "w") as f:
		f.write(drawStr);

	file = outDir+"/plotCombined.py";
	os.system("chmod 777 "+file+"; "+file+";");

def EvalPairs2PRCurvesSinglePlotAllDatasets(datadir, mLists, distThreshold, valThreshold, evalDirs, evalDirNames, outDir, rName, plotName):
	nsteps = 20;
	counter = 0;
	Rc = [];
	Pr = [];
	nregionstrue = 0;
	valBins = [];
	valBins += [int(valThreshold * nsteps)];
	Rc += [0];
	Pr += [0];
	
	drawStr = InitDrawStr();
	drawStr += "plt.clf();\n";
	drawStr += "fig = plt.figure();\n";
	legendNames = [];
	counter = -1;
	for evalDir in evalDirs:
		if rName[0:4]=="left":
			continue;
		#print(evalDir);
		counter = counter + 1;
		dists = [];
		vals = [];
		recallTotal = 0;
		objBegin = [];
		objEnd = [];
		objRecallTotal = [];
		#print(rName+ " - "+str(len(mLists[counter])));
		for mName in mLists[counter]:
			evalFile = evalDir+"/"+mName+"/"+rName+".eval_pairs";
			ndists = -1;
			objBegin.append(len(dists));
			if os.path.isfile(evalFile):
				f = open(evalFile);
				for pair in f.readlines():
					pairSplit = pair.strip().split(" ");
					vals.append(float(pairSplit[0].strip()));
					#dists.append(float(pairSplit[1].strip()));
					dlist = [];
					for dStr in pairSplit[1:]:
						dlist.append(float(dStr.strip()));
					if ndists<0:
						ndists = len(dlist);
					else:
						assert(ndists==len(dlist));
					dists.append(dlist);
					if float(pairSplit[0].strip())>1:
						print("Invalid Value: "+pairSplit[0].strip()+" -- "+mName+" "+rName)
				f.close();
			if ndists>0:
				recallTotal += ndists;
			objEnd.append(len(dists));
			objRecallTotal.append(ndists);
#		assert(recallTotal>0);

		#print(dists);
		if (len(dists)>0):
			legendNames.append(GetPlotName("loo_"+evalDirNames[counter]));
			drawStr += "nsteps="+str(nsteps)+";\n";
			# compute TP, FP, FN
			precisionTruePos = np.zeros(nsteps+1);
			precisionTotal = np.zeros(nsteps+1);
			recallTruePositive = np.zeros(nsteps+1);
			for i in range(0, nsteps+1):    	# for each threshold
				valThresh = float(i) / float(nsteps);
				for j in range(0, len(dists)):	# compute precision
					if vals[j] >= valThresh: 	# passes value threshold
						precisionTotal[i] += 1;
						if len(dists[j])>0 and min(dists[j]) < distThreshold:   # passes dist threshold
							precisionTruePos[i] += 1;
				for o in range(0, len(objBegin)):
					d1 = objBegin[o];
					d2 = objEnd[o];
					if d1!=d2:
						for d in range(0, objRecallTotal[o]):
							mindist = -1;
							for j in range(d1, d2):
								if vals[j] >= valThresh and (mindist==-1 or dists[j][d]<mindist):
									mindist = dists[j][d]; # passed value threshold -- find closest
							if (mindist < distThreshold and mindist>=0): # closest point is within threshold -- accept
								recallTruePositive[i] += 1;
			# create PR curve data
			precision = np.zeros(nsteps+1);
			recall = np.zeros(nsteps+1);
			for i in range(0, nsteps+1):
				if precisionTruePos[i]>0:
					precision[i] = precisionTruePos[i] / precisionTotal[i];
				if recallTotal>0:
					recall[i] = recallTruePositive[i] / recallTotal;
			
			# render matrix
			drawStr+= "recall"+evalDirNames[counter]+"=[";
			for x in range(0, len(recall)):
				if precision[x]==0:
					break;
				drawStr += str(recall[x]);
				if x<len(recall)-1 and precision[x+1]>0:
					drawStr += ", ";
			drawStr+="];\n";
			drawStr+= "precision"+evalDirNames[counter]+"=[";
			for x in range(0, len(precision)):
				if precision[x]==0:
					break;
				drawStr += str(precision[x]);
				if x<len(precision)-1 and precision[x+1]>0:
					drawStr += ", ";
			drawStr+="];\n";
			
			colorStr = "";
			if evalDirNames[counter]=="bikes":
				colorStr = ", color=(1, 0, 0)";
			elif evalDirNames[counter]=="kaick":
				colorStr = ", color=(0, 0, 1)";
			elif evalDirNames[counter]=="cockpits":
				colorStr = ", color=(0, .7, 0.)";
			elif evalDirNames[counter]=="other":
				colorStr = ", color=(1, 0, 1)";
			elif evalDirNames[counter]=="chairsFC":
				colorStr = ", color=(0, 0, 0)";
			elif evalDirNames[counter]=="gym":
				colorStr = ", color=(.5, .5, .5)";

			drawStr+= "res = plt.plot(recall"+evalDirNames[counter]+"[:-1], precision"+evalDirNames[counter]+"[:-1]"+colorStr+", alpha=0.9);\n";
			drawStr+= "plt.scatter(recall"+evalDirNames[counter]+"[:-1], precision"+evalDirNames[counter]+"[:-1], s=10);\n";
			
			it = 0;
			nregionstrue = nregionstrue + 1;
			for vb in valBins:
				Rc[it] += recall[vb];
				Pr[it] += precision[vb];
				it = it + 1;
				drawStr+="vb="+str(vb)+";\n";
				drawStr+="plt.scatter(recall"+evalDirNames[counter]+"[vb], precision"+evalDirNames[counter]+"[vb], s=50, c='red');\n";
			
			#legendNames.append(evalDirNames[counter]);
			#			drawStr += "plt.scatter(recall"+rName+"[1], precision"+rName+"[1], s=50, c='green');\n";
			#			drawStr += "plt.scatter(recall"+rName+"[nsteps-1], precision"+rName+"[nsteps-1], s=100, c='yellow');\n";
			#if rName[0:5]=="right":
			#	legendNames.append(rName[6:]);
			#else:
			#	legendNames.append(rName);
	
	
	#plt.xticks([0, nsteps-1], [0, 1]);
	#plt.yticks([0, nbins-1], [0, 1]);
	drawStr+="plt.xlim([0, 1]);\n";
	drawStr+="plt.ylim([0, 1]);\n";
	drawStr+="legendNames="+str(legendNames)+"\n";
	if plotName=="palm" or plotName=="toe":
		drawStr+="plt.legend(legendNames, bbox_to_anchor=(1, 1), loc=\"upper right\");\n";
	else:
		drawStr+="plt.legend(legendNames, bbox_to_anchor=(0, 0), loc=\"lower left\");\n";
	drawStr+="plt.xlabel('Recall');\n";
	drawStr+="plt.ylabel('Precision');\n";
	if plotName=="palm":
		drawStr+="plt.text(0.4, 0.02, \""+plotName.capitalize()+"\", fontsize=titlefont);\n";
	elif plotName=="toe":
		drawStr+="plt.text(0.4, 0.07, \""+plotName.capitalize()+"\", fontsize=titlefont);\n";
	else:
		drawStr+="plt.text(0.4, 0.03, \""+plotName.capitalize()+"\", fontsize=titlefont);\n";
	drawStr+="plt.savefig(\""+outDir+"/PR_combined.pdf\");\n";
	drawStr+="plt.savefig(\""+outDir+"/PR_combined.png\");\n";
	with open(outDir+"/plotCombined.py", "w") as f:
		f.write(drawStr);
	
	file = outDir+"/plotCombined.py";
	os.system("chmod 777 "+file+"; "+file+";");

def Errs2CDFPlotWithNameAndFlags(errorArrays, errorNames, plotFile, minErr, maxErr, plotName, flags):
	drawStr = InitDrawStr();
	drawStr += "mpl.rc('lines', linewidth=3);\n";
	drawStr += "plt.clf()\n";
	drawStr += "fig = plt.figure();\n";
	colorStr = ["(1, 0, 0)", "(0, 0, 1)", "(0, .7, 0)", "(1, 0, 1)", "(0, 0, 0)"];
	colorStr += ["(.5, .5, .5)", "(0, 1, 1)"];
	
	
	id = -1;
	for errArray in errorArrays:
		id = id + 1;
		errArray = sorted(errArray);
		errors = [];
		percentage = [];
		currpercent = -1;
		curritem = 0;
		for val in errArray:
			if val > maxErr:
				break;
			percent = int(curritem*100 / len(errArray));
			curritem = curritem + 1;
			if percent>currpercent:
				currpercent = percent;
				errors = errors + [val*units2meters];
				percentage = percentage+[percent];
		drawStr += "errors="+str(errors)+";\n";
		drawStr += "percentage="+str(percentage)+";\n";
		if "DashedPairs" in flags:
			if id%2==0:
				drawStr += "res = plt.plot(errors, percentage, '--', color="+colorStr[int(id/2)]+");\n";
			else:
				drawStr += "res = plt.plot(errors, percentage, '-', color="+colorStr[int(id/2)]+");\n";
		else:
			drawStr += "res = plt.plot(errors, percentage, color="+colorStr[id]+");\n";
	
	drawStr += "plt.xlim(["+str(minErr)+", "+str(maxErr*units2meters)+"]);\n";
	drawStr += "plt.ylim([0, 100]);\n";
	drawStr += "plt.xticks([0, 0.05, 0.1, 0.15, 0.20, 0.25], [0, 5, 10, 15, 20, 25]);\n";
	drawStr += "errorNames="+str(errorNames)+";\n";
	#	drawStr += "plt.legend(errorNames);\n";
	if not "nolegend" in flags:
		if "tl" in flags:
			drawStr += "plt.legend(errorNames, bbox_to_anchor=(0, 1), loc=\"upper left\");\n";
		elif "fbr" in flags:	 # far bottom right
			drawStr += "plt.legend(errorNames, bbox_to_anchor=(1.1, 0.0), loc=\"lower right\");\n";
		else:
			drawStr += "plt.legend(errorNames, bbox_to_anchor=(1, 0.0), loc=\"lower right\");\n";
	drawStr += "plt.xlabel('Error (centimeters)');\n";
	drawStr += "plt.ylabel('% success');\n";
	if plotName!="none":
		drawStr+="plt.text(0.015, 90, \""+plotName+"\", fontsize=titlefont);\n";
	drawStr += "plt.savefig(\""+plotFile+".pdf\");\n";
	drawStr += "plt.savefig(\""+plotFile+".png\");\n";
	f = open(plotFile+".py", "w");
	f.write(drawStr);
	f.close();
	os.system("chmod 777 "+plotFile+".py; "+plotFile+".py; ");

def Errs2CDFPlotWithName(errorArrays, errorNames, plotFile, minErr, maxErr, plotName):
	Errs2CDFPlotWithNameAndFlags(errorArrays, errorNames, plotFile, minErr, maxErr, plotName, [])

def Errs2CDFPlot(errorArrays, errorNames, plotFile, minErr, maxErr):
	Errs2CDFPlotWithNameAndFlags(errorArrays, errorNames, plotFile, minErr, maxErr, "none", []);

def GetPlotName(expname):
	if expname[0:4]=="loo_":
		if expname[-3:] == "gym":
			return "Gym Eq.";
		elif expname[-5:] == "bikes":
			return "Bicycles";
		elif expname[-8:] == "chairsFC":
			return "Chairs";
		elif expname[-8:] == "cockpits":
			return "Cockpits";
		elif expname[-5:] == "kaick":
			return "Bipedals";
		elif expname[-5:] == "other":
			return "Carts";
		elif expname[-5:] == "mixed":
			return "Mixed";
		else:
			return "NULLObj";
	else:
		return "NULLObj";


#def EvalPairs2Hist(datadir, mList, evalDir, outDir, rList):
#	counter = 0;
#	for rName in rList:
#		counter = counter + 1;
#		dists = [];
#		vals = [];
#		for mName in mList:
#			evalFile = evalDir+"/"+mName+"/"+rName+".eval_pairs";
#			if os.path.isfile(evalFile):
#				f = open(evalFile);
#				for pair in f.readlines():
#					pairSplit = pair.split(" ");
#					vals.append(float(pairSplit[0].strip()));
#					dists.append(float(pairSplit[1].strip()));
#				f.close();
#		if (len(dists)>0):
#			print("   Region "+rName+": "+str(counter)+" / "+str(len(rList)));
#			plt.clf();
#			# scatter plot (depricated)
#			## plt.scatter(dists, vals, 1, c="b", alpha=0.5, marker='.', label="pnts");
#			
#			# create density matrix
#			maxDist = 1.;
#			nbins = 15;
#			negdistbin = nbins+1;
#			densityMtx = np.zeros((negdistbin+1,nbins));
#			distNorms = np.zeros((negdistbin+1,1));
#			for i in range(0, len(dists)):
#				distbin = negdistbin;
#				if dists[i]>=0:
#					distbin = dists[i] / maxDist * nbins;
#					if distbin>=nbins:
#						distbin = nbins-1;
#				valbin = vals[i] *nbins;
#				if valbin>=nbins:
#					valbin = nbins-1;
#				densityMtx[distbin, valbin] = densityMtx[distbin, valbin] + 1;
#				distNorms[distbin] = distNorms[distbin] + 1;
#			# normalize density matrix
#			for i in range(0, negdistbin+1):
#				for j in range(0, nbins):
#					if densityMtx[i, j]>0:
#						densityMtx[i, j] = densityMtx[i, j] / distNorms[i];
#
#			# render matrix
#			fig = plt.figure()
#			plt.clf()
#			ax = fig.add_subplot(111);
#			ax.set_aspect(1);
#			res = ax.imshow(densityMtx.T, cmap=plt.cm.jet, interpolation='nearest');
#			res.set_cmap('copper')
#			plt.xticks([0, nbins-1], [0, 1]);
#			plt.yticks([0, nbins-1], [0, maxDist]);
#
#			plt.savefig(outDir+"pdf/"+rName+".pdf");
#			plt.savefig(outDir+"png/"+rName+".png");







