#!/usr/bin/python

##########
#
#  Predict pose
#
##########
import os, string, glob, sys
sys.path.append(os.path.realpath("./"+os.path.dirname(sys.argv[0])))
import fas, exp

modelsDir = "none";
pntsDir = "none";
outDir = "none";
outStatsDir = "none";
testFile = "none";
forceRecompute = False;			# re-do results
predictDir = "none";
clusterWorkspace = "./TempDir";
randseed = -1;
disallowParallel = False;
readExternalPoseDir = "none";
readExternalPoseFile = "none";
densePntsDir = "none";
planesDir = "none";

params = "";

if len(sys.argv)<2:
	print("Usage: "+sys.argv[0]+" [options]");
	print("Options:");
	print("   -modelsDir dir");
	print("   -pntsDir dir");
	print("   -densePntsDir dir");
	print("   -origrotate dir");
	print("   -predictDir dir ");
	print("   -posePriorFile file ");
	print("   -outDir dir ");
	print("   -planesDir dir ");
	print("   -forceRecompute ");
	print("   -testFile file ");
	print("   -readExternalPose dir file ");
	print("   -readRegionNames regionNamesFile ");
	print("   -readSkeleton skeletonFile ");
	print("   -readRegionStats regionStatsFile ");
	print("   -clusterWorkspace directory ");
	print("   -param name values_separated_by_underscore ");
	print("   -randseed seed - (seed can be -1) ");
	print("   -disallowParallel");
	exit();
else:
	i = 1;
	while i<len(sys.argv):
		if (sys.argv[i]=="-modelsDir"):
			i=i+1;
			modelsDir = sys.argv[i];
		elif (sys.argv[i]=="-pntsDir"):
			i=i+1;
			pntsDir = sys.argv[i];
		elif (sys.argv[i]=="-densePntsDir"):
			i=i+1;
			densePntsDir = sys.argv[i];
		elif (sys.argv[i]=="-predictDir"):
			i=i+1;
			predictDir = sys.argv[i];
		elif (sys.argv[i]=="-posePriorFile"):
			i=i+1;
			posePriorFile = sys.argv[i];
		elif (sys.argv[i]=="-outDir"):
			i=i+1;
			outDir = sys.argv[i];
		elif (sys.argv[i]=="-planesDir"):
			i=i+1;
			planesDir = sys.argv[i];
		elif (sys.argv[i]=="-origrotate"):
			i=i+1;
			origrotate = sys.argv[i];
		elif (sys.argv[i]=="-param"):
			N = int(sys.argv[i+1]);
			params += "-param "+sys.argv[i+2]+" ";
			for j in range(0, N):
				params +=sys.argv[i+3+j]+" ";
			i = i + 2 + N;
		elif (sys.argv[i]=="-forceRecompute"):
			forceRecompute = True;
		elif (sys.argv[i]=="-readRegionStats"):
			i = i + 1;
			readRegionStats = sys.argv[i];
		elif (sys.argv[i]=="-testFile"):
			i=i+1;
			testFile = sys.argv[i];
		elif (sys.argv[i]=="-clusterWorkspace"):
			i=i+1;
			clusterWorkspace = sys.argv[i];
		elif (sys.argv[i]=="-disallowParallel"):
			disallowParallel = True;
		elif (sys.argv[i]=="-readRegionNames"):
			i=i+1;
			readRegionNames = sys.argv[i];
		elif (sys.argv[i]=="-readSkeleton"):
			i=i+1;
			readSkeleton = sys.argv[i];
		elif (sys.argv[i]=="-randseed"):
			i=i+1;
			randseed = sys.argv[i];
		elif (sys.argv[i]=="-readExternalPose"):
			readExternalPoseDir = sys.argv[i+1];
			readExternalPoseFile = sys.argv[i+2];
			i=i+2;
		else:
			print("[ERROR] Unknown argument "+sys.argv[i]+" in predictPose.py ");
			exit();
		i=i+1;


symmInv = "";

# Prepare files
mListTest = fas.ReadModelNames(testFile)

## Prepare region lists
rList = fas.ReadRegionNames(readRegionNames);
jobIDs = [];
removeFiles = [];

##
# Compute discrete poses I: no IK
counter = 0;
for mName in mListTest:
	counter = counter + 1;
	
	predictedPoseFile = outDir+"/"+mName;

	if not os.path.isfile(predictedPoseFile+"/bindings.bin") or forceRecompute:
		print(str(counter)+"/"+str(len(mListTest))+": "+mName);
		offFile = modelsDir+"/"+mName+".off";
		pntsFile = pntsDir+"/"+mName+".pts";
		densePntsFile = densePntsDir+"/"+mName+".pts";
		predictionFile = predictDir+"/"+mName+".arff";
		planesFile = planesDir+"/"+mName+".planes.txt";
		cmd = "";
		cmd += exp.exedir+"FnAlign -r object "+offFile+" -r object_points "+pntsFile+" -r object_predictions "+predictionFile+" ";
		cmd += "-r object_densepoints "+densePntsFile+" ";
		cmd += "-r actor_pnts -bind ";
		cmd += "-r regionNames "+readRegionNames+" "
		cmd += "-r skeleton "+readSkeleton+" ";
		cmd += "-r actor_poseprior "+posePriorFile+" ";
		cmd += "-r regionStats "+readRegionStats+" ";
		if planesDir!="none":
			cmd += "-refl_planes "+planesFile+" ";
		
		cmd += "-origrotate "+origrotate+"/"+mName+".xform ";
	
		# write results (would break if ground truth was read)
		cmd += "-w bindings "+predictedPoseFile+" ";
		cmd += "-w gt "+predictedPoseFile+"/poses.txt ";
		
		# read ground truth: for debugging only
		if readExternalPoseDir!="none" or readExternalPoseFile!="none":
			readFile = readExternalPoseDir+"/";
			if readExternalPoseDir=="none":
				readFile = "";
			if readExternalPoseFile=="default":
				readFile+=mName+".gt";
			elif readExternalPoseFile!="none":
				readFile += readExternalPoseFile;
			cmd += "-r actor_gtpose "+readFile+" ";

		# write timing and energy
		cmd += "-w energy "+outDir+"/"+mName+"/energy.txt ";
		#cmd += "-w timing "+outDir+"/"+mName+"/bindtime.txt ";

		cmd += params+" ";
		if exp.DEBUG_VIS:
			cmd += "-vis ";	# REMOVE IT !!!!
		cmd += " -randseed "+randseed+" ";
		
		# execute
		#print(cmd);
		if not disallowParallel:
			scriptFile = clusterWorkspace+"/"+mName;
			jobIDs += fas.ScheduleJob(cmd, "pose_"+mName, scriptFile);
			removeFiles += fas.TmpFilesNames(scriptFile);
		else:
			os.system(cmd);
fas.WaitForJobsInArray(jobIDs);
#for file in removeFiles:
#	if os.path.isfile(file):
#		os.remove(file);


