#!/usr/bin/python

##########
#
#  Predict regions (classify)
#
##########
import os, string, glob, sys
sys.path.append(os.path.realpath("./"+os.path.dirname(sys.argv[0])))
import fas, exp

modelsDir = "none";
pntsDir = "none";
featDir = "none";
outDir = "none";
testFile = "none";
featureNameFile = "none";
forceRecompute = False;			# re-train classifier even if exists
symmInvariant = False;
wekaWorkDir = "none";
claType = "none";
clusterWorkspace = "./TempDir";
randseed = -1;
disallowParallel = False;

if len(sys.argv)<2:
	print("Usage: "+sys.argv[0]+" [options]");
	print("Options:");
	print("   -modelsDir dir");
	print("   -pntsDir dir");
	print("   -featDir dir ");
	print("   -features region names ");
	print("   -claDir dir ");
	print("   -claType type ");
	print("   -outDir dir ");
	print("   -forceRecompute ");
	print("   -testFile file ");
	print("   -wekaWorkDir dir ");
	print("   -symmInvariant ");
	print("   -clusterWorkspace directory ");
	print("   -randseed seed - (seed can be -1) ");
	print("   -disallowParallel");
	exit();
else:
	i = 1;
	while i<len(sys.argv):
		if (sys.argv[i]=="-modelsDir"):
			i=i+1;
			modelsDir = sys.argv[i];
		elif (sys.argv[i]=="-pntsDir"):
			i=i+1;
			pntsDir = sys.argv[i];
		elif (sys.argv[i]=="-featDir"):
			i=i+1;
			featDir = sys.argv[i];
		elif (sys.argv[i]=="-features"):
			i=i+1;
			featureNameFile = sys.argv[i];
		elif (sys.argv[i]=="-claDir"):
			i=i+1;
			claDir = sys.argv[i];
		elif (sys.argv[i]=="-claType"):
			i=i+1;
			claType = sys.argv[i];
		elif (sys.argv[i]=="-outDir"):
			i=i+1;
			outDir = sys.argv[i];
		elif (sys.argv[i]=="-wekaWorkDir"):
			i=i+1;
			wekaWorkDir = sys.argv[i];
		elif (sys.argv[i]=="-forceRecompute"):
			forceRecompute = True;
		elif (sys.argv[i]=="-testFile"):
			i=i+1;
			testFile = sys.argv[i];
		elif (sys.argv[i]=="-symmInvariant"):
			symmInvariant = True;
		elif (sys.argv[i]=="-clusterWorkspace"):
			i=i+1;
			clusterWorkspace = sys.argv[i];
		elif (sys.argv[i]=="-disallowParallel"):
			disallowParallel = True;
		elif (sys.argv[i]=="-readRegionNames"):
			i=i+1;
			readRegionNames = sys.argv[i];
		elif (sys.argv[i]=="-readRegionSymmetry"):
			i=i+1;
			readRegionSymmetry = sys.argv[i];
		elif (sys.argv[i]=="-paramFile"):
			i=i+1;
			paramFile = sys.argv[i];
		elif (sys.argv[i]=="-randseed"):
			i=i+1;
			randseed = sys.argv[i];
		else:
			print("[ERROR] Unknown argument "+sys.argv[i]+" in predictRegions.py ");
			exit();
		i=i+1;


# Prepare files
mListTest = fas.ReadModelNames(testFile);


# Prepare files
regionsFile = readRegionNames;
rList = fas.ReadRegionNames(regionsFile);
jobIDs = [];
removeFiles = [];

##
# Classify regions
counter = 0;
if symmInvariant:
	symmInv = "-r symm_invariant "+readRegionSymmetry+" ";
for mName in mListTest:
	counter = counter + 1;
	offFile = modelsDir+"/"+mName+".off";
	pntsFile = pntsDir+"/"+mName+".pts";
	predictionFile = outDir+"/"+mName+".arff";
		
	if not os.path.isfile(predictionFile) or forceRecompute:
		print("    "+mName+": "+str(counter)+" / "+str(len(mListTest)));
		# prepare params: features to read etc...
		params = "-r featnames "+featureNameFile+" ";
		params += "-r gtnames "+ regionsFile+" ";
		params += "-ro "+mName+" mesh "+offFile+" ";
		params += "-ro "+mName+" featpnts "+pntsFile+" ";
		with open(featureNameFile, 'r') as f:
			featList = f.readlines();
		for featLine in featList:
			featName = featLine.split(" ")[0].strip();
			featValsFile = featDir+"/"+featName+"/"+mName+".arff";
			params += "-ro "+mName+" featvals "+featName+" "+featValsFile+" ";
		params += " -r classifier "+claDir+" "
		params +=symmInv+" -randseed "+randseed+" ";

		cmd = "";
		# if WEKA
		wekaResultsPfx = wekaWorkDir+"/"+mName+"_";
		if claType=="Weka":
			cmd += "starttime=$SECONDS; ";
			# Run code to prepare file to classify
			wekaUnclassified = wekaWorkDir+"/"+mName+".arff";
			cmd += exp.exedir+"FnRegionClassifier "+params+" -wo "+mName+" weka_unclassified "+wekaUnclassified+"; ";
			
			# Run external classifier (w.r.t. to each label separately)
			for rName in rList:
				wekaClassifier = claDir+"/weka_"+rName+".model";
				wekaResults = wekaResultsPfx+rName+".txt";
				if os.path.isfile(wekaClassifier):
					cmd += fas.SciKitClassify(wekaClassifier, wekaUnclassified, wekaResults)+";";
			# in the next run - classifier should read the resulting weka file
			params += "-ro "+mName+" weka_classified "+wekaResultsPfx+" ";
		elif claType[0:8]=="External":
			params += "-ro "+mName+" weka_classified "+wekaResultsPfx+" ";
		else:
			params += "-classify "+mName+" ";

		# Run code to parse results produced by external classifier (e.g. weka)
		cmd += exp.exedir+"FnRegionClassifier "+params+" -wo "+mName+" prediction "+predictionFile+" ";
		if exp.DEBUG_VIS:
			cmd += "-vis";
		cmd += "; ";
		
		# Record running time for external classifier
		if claType=="Weka":
			cmd += "echo $(($SECONDS-$starttime))s > "+wekaWorkDir+"/"+mName+".timing.txt; ";
		
		# Run jobs (parallel or sequential)
		if not disallowParallel:
			scriptFile = clusterWorkspace+"/"+mName;
			jobIDs += fas.ScheduleJob(cmd, "pred_"+mName, scriptFile)
			removeFiles += fas.TmpFilesNames(scriptFile);
		else:
			os.system(cmd);
fas.WaitForJobsInArray(jobIDs);
#for file in removeFiles:
#	if os.path.isfile(file):
#		os.remove(file);

