#!/usr/bin/python

##########
#
# Create ground truth for functionality analysis. 
#
##########
import os, string, decimal, glob, re, subprocess, shlex, sys, datetime
sys.path.append(os.path.realpath("./"+os.path.dirname(sys.argv[0])))
import fas, exp

datadir = exp.data;

# Get file list (collada directory)
renderMeshes = "none";
renderPoses = "none";
modelsDir = "none";
gtDir = "none";
forceRecompute = False;
clusterWorkspace = "./TempDir";
vis = False;
disallowParallel = False;

if len(sys.argv)<2:
	print("Usage: "+sys.argv[0]+" [options]");
	print("Options:");
	print("   -modelsDir dir");
	print("   -gtDir dir ");
	print("   -renderPoses dir ");
	print("   -renderMeshes dir ");
	print("   -forceRecompute ");
	print("   -vis  ");
	print("   -clusterWorkspace directory ");
	print("   -disallowParallel");
	exit();
else:
	i = 1;
	while i<len(sys.argv):
		if (sys.argv[i]=="-modelsDir"):
			i=i+1;
			modelsDir = sys.argv[i];
		elif (sys.argv[i]=="-gtDir"):
			i=i+1;
			gtDir = sys.argv[i];
		elif (sys.argv[i]=="-renderPoses"):
			i=i+1;
			renderPoses = sys.argv[i];
		elif (sys.argv[i]=="-renderMeshes"):
			i=i+1;
			renderMeshes = sys.argv[i];
		elif (sys.argv[i]=="-forceRecompute"):
			forceRecompute = True;
		elif (sys.argv[i]=="-clusterWorkspace"):
			i=i+1;
			clusterWorkspace = sys.argv[i];
		elif (sys.argv[i]=="-disallowParallel"):
			disallowParallel = True;
		elif (sys.argv[i]=="-readRegionNames"):
			i=i+1;
			readRegionNames = sys.argv[i];
		elif (sys.argv[i]=="-readSkeleton"):
			i=i+1;
			readSkeleton = sys.argv[i];
		elif (sys.argv[i]=="-vis"):
			vis = True;
		else:
			print("[ERROR] Unknown argument "+sys.argv[i]+" in render.py ");
			exit();
		i = i + 1;

mList = fas.ReadModelNames(modelsDir);

jobIDs = [];
removeFiles = [];

# Render everything
counter = 0;
for mName in mList:
	counter = counter + 1;
	imageFile = renderMeshes+"/"+mName+".jpg";
	imagePosesDir = renderPoses+"/"+mName+"/";
	gtFile = gtDir+"/"+mName+".gt";
	rMesh = not os.path.isfile(imageFile) and renderMeshes!="none";
	rPose = renderPoses!="none" and not os.path.isdir(imagePosesDir) and os.path.isfile(gtFile);

	if forceRecompute:
		rMesh = True;
		rPose = True;
	if rMesh or rPose:
		print("Render: "+mName+": "+str(counter)+" / "+str(len(mList)));
		offFile = modelsDir+mName+".off";

		cmd = exp.exedir+"FnBenchmark "+offFile+" "+gtFile+" -readRegionNames "+readRegionNames+" ";
		cmd += "-readSkeleton "+readSkeleton+" ";
		if vis:
			cmd += "-interact ";
		else:
			if rMesh:
				cmd += "-renderShape "+imageFile+" ";
			if rPose:
				#print("Render poses: "+imagePosesDir);
				cmd += "-renderPoses "+imagePosesDir+" ";
		if not disallowParallel:
			scriptFile = clusterWorkspace+"/"+mName;
			jobIDs += fas.ScheduleJob(cmd, "rend_"+mName, scriptFile)
			removeFiles += fas.TmpFilesNames(scriptFile);
		else:
			os.system(cmd);

fas.WaitForJobsInArray(jobIDs);
#for file in removeFiles:
#	if os.path.isfile(file):
#		os.remove(file);



