#!/usr/bin/python

##########
#
#  Train predictor for regions
#
##########
import os, string, glob, sys, time
sys.path.append(os.path.realpath("./"+os.path.dirname(sys.argv[0])))
import fas, exp

modelsDir = "none";
gtDir = "none";
pntsDir = "none";
featDir = "none";
outDir = "none";
trainFile = "none";
featFile = "none";
claType = "none";
densePntsDir = "none";
readSkeleton = "none"
metricDir = "none";
forceRecompute = False;			# re-train classifier even if exists
symmInvariant = False;
clusterWorkspace = "./TempDir";
randseed = -1;
disallowParallel = False;

params = "";

if len(sys.argv)<2:
	print("Usage: "+sys.argv[0]+" [options]");
	print("Options:");
	print("   -modelsDir dir");
	print("   -gtDir dir ");
	print("   -pntsDir dir");
	print("   -featDir dir ");
	print("   -outDir dir ");
	print("   -forceRecompute ");
	print("   -trainFile file ");
	print("   -features file ");
	print("   -param N [params for FnRegionClassifier] ");
	print("   -symmInvariant");
	print("   -claType [Dist2NearestGauss, Weka] ");
	print("   -clusterWorkspace directory ");
	print("   -randseed seed - (seed can be -1) ");
	print("   -disallowParallel");
	exit();
else:
	i = 1;
	while i<len(sys.argv):
		if (sys.argv[i]=="-modelsDir"):
			i=i+1;
			modelsDir = sys.argv[i];
		elif (sys.argv[i]=="-gtDir"):
			i=i+1;
			gtDir = sys.argv[i];
		elif (sys.argv[i]=="-pntsDir"):
			i=i+1;
			pntsDir = sys.argv[i];
		elif (sys.argv[i]=="-densePntsDir"):
			i=i+1;
			densePntsDir = sys.argv[i];
		elif (sys.argv[i]=="-metricDir"):
			i=i+1;
			metricDir = sys.argv[i];
		elif (sys.argv[i]=="-featDir"):
			i=i+1;
			featDir = sys.argv[i];
		elif (sys.argv[i]=="-outDir"):
			i=i+1;
			outDir = sys.argv[i];
		elif (sys.argv[i]=="-forceRecompute"):
			forceRecompute = True;
		elif (sys.argv[i]=="-trainFile"):
			i=i+1;
			trainFile = sys.argv[i];
		elif (sys.argv[i]=="-features"):
			i=i+1;
			featFile = sys.argv[i];
		elif (sys.argv[i]=="-symmInvariant"):
			symmInvariant = True;
		elif (sys.argv[i]=="-claType"):
			i=i+1;
			claType = sys.argv[i];
		elif (sys.argv[i]=="-param"):
			N = int(sys.argv[i+1]);
			params += "-"+sys.argv[i+2]+" ";
			for j in range(0, N):
				params +=sys.argv[i+3+j]+" ";
			i = i + 2 + N;
		elif (sys.argv[i]=="-clusterWorkspace"):
			i=i+1;
			clusterWorkspace = sys.argv[i];
		elif (sys.argv[i]=="-disallowParallel"):
			disallowParallel = True;
		elif (sys.argv[i]=="-readRegionNames"):
			i=i+1;
			readRegionNames = sys.argv[i];
		elif (sys.argv[i]=="-readSkeleton"):
			i=i+1;
			readSkeleton = sys.argv[i];
		elif (sys.argv[i]=="-readRegionSymmetry"):
			i=i+1;
			readRegionSymmetry = sys.argv[i];
		elif (sys.argv[i]=="-randseed"):
			i=i+1;
			randseed = sys.argv[i];
		else:
			print("[ERROR] Unknown argument "+sys.argv[i]+" in trainRegions.py ");
			exit();
		i=i+1;



# Prepare files
sys.stdout.flush()
mListTrain = fas.ReadModelNames(trainFile);

## Prepare region lists
regionsFile = readRegionNames;
rList = fas.ReadRegionNames(regionsFile);
jobIDs = [];
removeFiles = [];

##
# Create classifier
symmInv = "";
if symmInvariant:
	symmInv = "-r symm_invariant "+readRegionSymmetry+" "

featList = [];
with open(featFile) as ff:
	featList = ff.readlines();
assert(len(featList)>0);

outFile = outDir+"/info.txt";
if not os.path.isdir(outDir) or not os.path.isfile(outFile) or forceRecompute:
	if not os.path.isdir(outDir):
		os.makedirs(outDir);
	cmd = "";
	args = "";
	#args += "-w classifier "+outDir+" ";
	args += "-r featnames "+featFile+ " -r gtnames "+regionsFile+" ";
	args += "-r regionNames "+readRegionNames+" -r skeleton "+readSkeleton+" ";
	args += " -randseed "+randseed+" ";
	if len(claType)>8 and claType[0:8] == "External":
		args += "-trainCla External ";
	else:
		args += "-trainCla "+claType+" ";

	emptyArgs = "";

	useIntermediateDirectory = len(mListTrain)>40;# if too big

#	mListTrain = [mListTrain[0]]

	for mName in mListTrain:
		sys.stdout.flush()
		offFile = modelsDir+"/"+mName+".off";
		gtFile = gtDir+"/"+mName+".gt";
		pntsFile = pntsDir+"/"+mName+".pts";
		densePntsFile = densePntsDir + "/"+mName+".pts";
		metricFile = metricDir+"/"+mName+".metric";
		nameargs = "";
		nameargs += "-ro "+mName+" mesh "+offFile+" ";
		nameargs += "-ro "+mName+" gtregions "+gtFile+" ";
		nameargs += "-ro "+mName+" featpnts "+pntsFile+" ";
		nameargs += "-ro "+mName+" metric "+densePntsFile+" "+metricFile+" ";
		nameargs += symmInv+" ";
		for featLine in featList:
			featName = featLine.split(" ")[0].strip();
			featValsFile = featDir+"/"+featName+"/"+mName+".arff";
			nameargs += "-ro "+mName+" featvals "+featName+" "+featValsFile+" ";
		mnameoutdir = outDir+"/permodel/"+mName+"/";

		if useIntermediateDirectory:
			emptyArgs += "-ro "+mName+" empty ";
			if not os.path.isdir(mnameoutdir):
				cmd += exp.exedir+"FnRegionClassifier "+args+nameargs+" -w classifier "+mnameoutdir+" "+params+" -train;\n";
		else:
			emptyArgs += nameargs;


	if claType=="Weka" or claType[0:8]=="External":
		args1 = "-r featnames "+featFile+ " -r gtnames "+regionsFile+" ";
		args1 += "-r regionNames "+readRegionNames+" -r skeleton "+readSkeleton+" ";
		emptyArgs = args1+" -w classifier "+outDir+" "+emptyArgs+" ";
		if not useIntermediateDirectory:
			emptyArgs += "-trainCla Weka "+params+" ";
		else:
			emptyArgs += "-trainCla GroupWeka ";
		emptyArgs+="-train ";
		if exp.DEBUG_VIS and not useIntermediateDirectory:
			emptyArgs += "-vis "
		if not os.path.isdir(outDir):
			os.makedirs(outDir);
		argFile = outDir+"/args.txt";
		with open(argFile, "w") as f:
			f.write(emptyArgs);
		cmd += exp.exedir+"FnRegionClassifier "+argFile+";\n";
	else:
		assert(False);

	if claType=="Weka": # run weka on training data...
		for rName in rList:
			arffFile = outDir+"/train_"+rName+".arff";
			wekaModel = outDir+"/weka_"+rName+".model";
			wekaStats = outDir+"/weka_"+rName+".stats.txt";
			cmd += "if [  -f "+arffFile+" ]; then\n";
			cmd += "echo Training classifier: "+rName+";\n";
			#cmd += fas.WekaTrainClassifier(arffFile, wekaModel, wekaStats)+";\n";
			cmd += fas.SciKitTrainClassifier(arffFile, wekaModel, wekaStats)+";\n";
			#cmd += "rm "+arffFile+";\n";	# optional cleanup
			cmd += "fi;\n";

#		if useIntermediateDirectory:
#			cmd += "rm -r "+outDir+"/permodel;\n";

	if not disallowParallel:
		scriptFile = clusterWorkspace+"/trainCla";
		jobIDs += fas.ScheduleJob(cmd, "trainCla", scriptFile)
		removeFiles += fas.TmpFilesNames(scriptFile);
	else:
		scriptFile = clusterWorkspace+"/trainCla.sh";
		if not os.path.isdir(clusterWorkspace):
			os.makedirs(clusterWorkspace);
		with open(scriptFile, "w") as f:
			f.write(cmd);
		os.system("chmod 777 "+scriptFile+"; ./"+scriptFile);

fas.WaitForJobsInArray(jobIDs);
#for file in removeFiles:
#	if os.path.isfile(file):
#		os.remove(file);



